// model実装.
//
var DBUtil = pkg("org.maachang.dao.dbms.DbUtil") ;

////////////////////////////////////////////////////////////////////////////////
// _baseModel.
////////////////////////////////////////////////////////////////////////////////

// 基本モデルオブジェクト.
_baseModel = function(modelName) {
    if( isNull( modelName ) ) {
        return null ;
    }
    // model用データ格納条件.
    var baseMap = new java.util.HashMap() ;
    // モデル名整頓.
    modelName = trim( modelName ) ;
    // 変数セット.
    baseMap.put( "modelName",modelName ) ;  // モデル名.
    baseMap.put( "_thisObj",baseMap ) ;     // マップ情報.
    baseMap.put( "sequence",true ) ;        // シーケンスフラグ.
    baseMap.put( "isCacheMap",false ) ;     // 生成マップ.
    baseMap.put( "isDeleteFlag",false ) ;   // 削除フラグ.
    baseMap.put( "defaultColumns",null ) ;  // カラム一覧.
    baseMap.put( "join",null ) ;            // JOIN.
    baseMap.put( "srcJoin",null ) ;         // SRC_JOIN.
    // インスタンス生成用.
    baseMap.put( "bean",
        function( map ) {
            var md = getBaseModel() ;
            var meta = md.getMeta( this.modelName ) ;
            var len = meta.size() ;
            var ret = {} ;
            var idFlag = false ;
            for( var i = 0 ; i < len ; i ++ ) {
                var key = meta.getColumnName( true,i ) ;
                // IDがシーケンスの場合は処理しないようにする.
                if( key != "id" ) {
                    ret[ key ] = "" ;
                }
                else {
                    idFlag = true ;
                }
            }
            // シーケンスが無効な場合は、IDカラムも生成する.
            if( idFlag && !this.sequence ) {
                ret["id"] = "" ;
            }
            // デフォルト値が存在する場合は、それをbeanに設定する.
            if( isNull( this.defaultColumns ) == false ) {
                var df = this.defaultColumns ;
                for( var k in df ) {
                    ret[k] = df[k] ;
                }
            }
            // map情報が渡された場合、
            if( isNull( map ) == false && typeof( map ) == "map" ) {
                for( var i = 0 ; i < len ; i ++ ) {
                    var tName = meta.getColumnName( true,i ) ;
                    // IDがシーケンスの場合は処理しないようにする.
                    if( this.sequence && tName == "id" ) {
                        continue ;
                    }
                    var type = SqlType.typeOf( meta.getColumnType( i ) ) ;
                    // 日付情報の場合.
                    if( tName != "CreateTime" && tName != "UpdateTime" &&
                        ( type == "date" || type == "timestamp" ) &&
                        ( isEmpty( map[tName+"Year"] ) == false &&
                            isEmpty( map[tName+"Month"] ) == false &&
                            isEmpty( map[tName+"Day"] ) ) == false ) {
                        ret[ tName ] = convertMapByDate( tName,map ) ;
                    }
                    else {
                        var p = map[ tName ] ;
                        if( isEmpty( p ) == false ) {
                            var pType = typeof( p ) ;
                            // booleanの場合.
                            if( type == "boolean" ) {
                                if( pType == "boolean" ) {
                                    ret[ tName ] = p ;
                                }
                                else {
                                    p = p.toLowerCase() ;
                                    if( p == "true" ) {
                                        ret[ tName ] = true ;
                                    }
                                    else if( p == "false" ) {
                                        ret[ tName ] = false ;
                                    }
                                }
                            }
                            // 整数の場合.
                            else if( type == "integer" || type == "long" ) {
                                ret[ tName ] = p ;
                            }
                            // 日付の場合.
                            else if( type == "date" || type == "timestamp" || type == "time" ) {
                                ret[ tName ] = p ;
                            }
                            // 上記以外の内容.
                            else {
                                ret[ tName ] = p ;
                            }
                        }
                    }
                }
            }
            return ret ;
        }
    ) ;
    
    // インスタンス生成用.
    baseMap.put( "copy",
        function( src,dest ) {
            if( isNull( src ) || typeof( src ) != "map" ||
                isNull( dest ) || typeof( dest ) != "map" ) {
                return false ;
            }
            var md = getBaseModel() ;
            var meta = md.getMeta( this.modelName ) ;
            var len = meta.size() ;
            for( var i = 0 ; i < len ; i ++ ) {
                var tName = meta.getColumnName( true,i ) ;
                // IDがシーケンスの場合、処理しないようにする.
                if( this.sequence && tName == "id" ) {
                    continue ;
                }
                var type = SqlType.typeOf( meta.getColumnType( i ) ) ;
                // 日付情報の場合.
                if( tName != "CreateTime" && tName != "UpdateTime" &&
                    ( type == "date" || type == "timestamp" ) &&
                    ( isNull( src[tName+"Year"] ) == false &&
                        isNull( src[tName+"Month"] ) == false &&
                        isNull( src[tName+"Day"] ) == false ) ) {
                    dest[ tName ] = convertMapByDate( tName,src ) ;
                }
                else {
                    var p = src[ tName ] ;
                    if( isEmpty( p ) == false ) {
                        var pType = typeof( p ) ;
                        // 情報が存在しない場合は処理しない.
                        if( pType == "string" && useString( p ) == false ) {
                            continue ;
                        }
                        // booleanの場合.
                        if( type == "boolean" ) {
                            if( pType == "boolean" ) {
                                dest[ tName ] = p ;
                            }
                            else {
                                p = p.toLowerCase() ;
                                if( p == "true" ) {
                                    dest[ tName ] = true ;
                                }
                                else if( p == "false" ) {
                                    dest[ tName ] = false ;
                                }
                            }
                        }
                        // 整数の場合.
                        else if( type == "integer" || type == "long" ) {
                            if( pType == "number" ) {
                                dest[ tName ] = p ;
                            }
                            else {
                                dest[ tName ] = parseInt( p ) ;
                            }
                        }
                        // 日付の場合.
                        else if( type == "date" || type == "timestamp" || type == "time" ) {
                            dest[ tName ] = new JDate( p.getTime() ) ; ;
                        }
                        // 上記以外の内容.
                        else {
                            dest[ tName ] = p ;
                        }
                    }
                }
            }
            return true ;
        }
    ) ;
    
    // カラム名一覧を取得.
    baseMap.put( "columns",
        function(mode) {
            var md = getBaseModel() ;
            var meta = md.getMeta( this.modelName ) ;
            var ret = [] ;
            var len = meta.size() ;
            if( isNull( mode ) == false && mode == true ) {
                for( var i = 0 ; i < len ; i ++ ) {
                    ret.add( { "name": meta.getColumnName( true,i ),
                        "type":meta.meta.getColumnType( i ) } ) ;
                }
            }
            else {
                for( var i = 0 ; i < len ; i ++ ) {
                    ret.add( meta.getColumnName( true,i ) ) ;
                }
            }
            return ret ;
        }
    ) ;
    
    // テーブルを削除.
    baseMap.put( "truncate",
        function() {
            OperationTable.recreate( this.modelName ) ;
        }
    ) ;
    
    // テーブルExport(csv).
    // java.io.Writerオブジェクトをセット.
    baseMap.put( "exportCsv",
        function( writer ) {
            OperationTable.exportCsv( writer,this.modelName ) ;
        }
    ) ;
    
    // テーブルImport(csv).
    // java.io.Readerオブジェクトをセット.
    baseMap.put( "importCsv",
        function( reader ) {
            OperationTable.importCsv( reader,this.modelName ) ;
        }
    ) ;
    
    // 保存用.
    baseMap.put( "save",
        function(params) {
            if( ( params instanceof java.util.Map ) == false ) {
                throw new java.lang.IllegalArgumentException( "save[param]パラメータが不正です" ) ;
            }
            if( isNull( params ) ) {
                return false ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            //var map = deepCopy( params ) ;
            var map = copy( params ) ;
            if( isNull( map ) || map.size() <= 0 ) {
                return false ;
            }
            SubModel.removeManyColumn( map ) ;
            // デフォルト値が存在する場合は、それをmapに設定する.
            if( isNull( this.defaultColumns ) == false ) {
                for( var k in this.defaultColumns ) {
                    // 空の条件のときは、デフォルト条件を設定.
                    if( !useString( map.get( k ) ) ) {
                        map[k] = this.defaultColumns[k] ;
                    }
                }
            }
            var insertFlag = false ;
            var res = md.save( this.modelName,this.sequence,map ) ;
            if( params[SubModel.COLUMN_ID] == null && res != null ) {
                params[SubModel.COLUMN_ID]= res ;
                insertFlag = true ;
            }
            var createAt = map.get( SubModel.COLUMN_CREATE_TIME ) ;
            var updateAt = map.get( SubModel.COLUMN_UPDATE_TIME ) ;
            var versionValue = map.get( SubModel.COLUMN_OPTIMISTIC_LOCK ) ;
            if( !isNull( createAt ) && insertFlag ) {
                params[SubModel.COLUMN_CREATE_TIME] = new Date( createAt.getTime() ) ;
            }
            if( !isNull( updateAt ) ) {
                params[SubModel.COLUMN_UPDATE_TIME] = new Date( updateAt.getTime() ) ;
            }
            if( !isNull( versionValue ) ) {
                params[SubModel.COLUMN_OPTIMISTIC_LOCK] = versionValue ;
            }
            if( md.isDebug() ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            // 更新通知.
            md.setUpdateOrDeleteTable( this.modelName ) ;
            return true ;
        }
    ) ;
    // 指定更新用.
    baseMap.put( "update",
        function(where,params) {
            if( !useString( where ) ) {
                where = "" ;
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            md.update( this.modelName,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            // 更新通知.
            md.setUpdateOrDeleteTable( this.modelName ) ;
            return true ;
        }
    ) ;
    // 指定削除用(Join).
    baseMap.put( "delj",
        function(where,params) {
            return this._thisObj.get( "delete" )( where,params,true ) ;
        }
    ) ;
    // 指定削除用.
    baseMap.put( "del",
        function(where,params) {
            return this._thisObj.get( "delete" )( where,params,false ) ;
        }
    ) ;
    // 指定削除用.
    baseMap.put( "delete",
        function(where,params,join) {
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            var joinMap = null ;
            if( join == true ) {
                joinMap = SubModel.getModelBySrcJoinKeys( this.modelName,this._thisObj ) ;
            }
            md.remove( this.modelName,joinMap,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            // 更新通知.
            md.setUpdateOrDeleteTable( this.modelName ) ;
            return true ;
        }
    ) ;
    // 指定削除用.
    baseMap.put( "djoin",
        function(join,where,params) {
            if( typeof( join ) != "map" ) {
                join = null ;
            }
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            md.remove( this.modelName,join,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            // 更新通知.
            md.setUpdateOrDeleteTable( this.modelName ) ;
            return true ;
        }
    ) ;
    // 削除用(Join).
    baseMap.put( "rmvj",
        function(params) {
            return this._thisObj.get( "remove" )( params,true ) ;
        }
    ) ;
    // 削除用.
    baseMap.put( "rmv",
        function(params) {
            return this._thisObj.get( "remove" )( params,true ) ;
        }
    ) ;
    // 削除用
    baseMap.put( "remove",
        function(params,join) {
            if( ( params instanceof java.util.Map ) == false ) {
                throw new java.lang.IllegalArgumentException( "remove[params]パラメータが不正です" ) ;
            }
            if( isNull( params ) ) {
                return false ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            //var map = deepCopy( params ) ;
            var map = copy( params ) ;
            if( isNull( map ) || map.size() <= 0 ) {
                return false ;
            }
            SubModel.removeManyColumn( map ) ;
            var joinMap = null ;
            if( join == true ) {
                joinMap = SubModel.getModelBySrcJoinKeys( this.modelName,this._thisObj ) ;
            }
            md.removeData( this.modelName,joinMap,map ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            // 更新通知.
            md.setUpdateOrDeleteTable( this.modelName ) ;
            return true ;
        }
    ) ;
    // 削除用
    baseMap.put( "rjoin",
        function(join,params) {
            if( ( params instanceof java.util.Map ) == false ) {
                throw new java.lang.IllegalArgumentException( "remove[params]パラメータが不正です" ) ;
            }
            if( isNull( params ) ) {
                return false ;
            }
            if( typeof( join ) != "map" ) {
                join = null ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            //var map = deepCopy( params ) ;
            var map = copy( params ) ;
            if( isNull( map ) || map.size() <= 0 ) {
                return false ;
            }
            SubModel.removeManyColumn( map ) ;
            md.removeData( this.modelName,join,map ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            // 更新通知.
            md.setUpdateOrDeleteTable( this.modelName ) ;
            return true ;
        }
    ) ;
    // 指定取得用(Join).
    baseMap.put( "fndj",
        function(where,params,sort) {
            return this._thisObj.get( "find" )( where,params,sort,true,false ) ;
        }
    ) ;
    // 指定取得用(Many).
    baseMap.put( "fndm",
        function(where,params,sort) {
            return this._thisObj.get( "find" )( where,params,sort,false,true ) ;
        }
    ) ;
    // 指定取得用.
    baseMap.put( "fnd",
        function(where,params,sort) {
            return this._thisObj.get( "find" )( where,params,sort,false,false ) ;
        }
    ) ;
    // 指定取得用.
    baseMap.put( "find",
        function(where,params,sort,join,many) {
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            var joinMap = null ;
            if( join == true ) {
                joinMap = SubModel.getModelByJoinKeys( this._thisObj ) ;
                many = false ;
            }
            var manyMap = null ;
            if( many == true ) {
                manyMap = SubModel.getModelBySrcJoinKeys( this.modelName,this._thisObj ) ;
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.find( this.modelName,joinMap,manyMap,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定取得用(join用).
    baseMap.put( "fjoin",
        function(join,where,params,sort) {
            if( typeof( join ) != "map" ) {
                join = null ;
            }
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.find( this.modelName,join,null,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定取得用(many用).
    baseMap.put( "fmany",
        function(many,where,params,sort) {
            if( typeof( many ) != "map" ) {
                many = null ;
            }
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.find( this.modelName,null,many,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定取得用(Join).
    baseMap.put( "lmtj",
        function(offset,limit,where,params,sort) {
            return this._thisObj.get( "limit" )( offset,limit,where,params,sort,true,false ) ;
        }
    ) ;
    // 指定取得用(many).
    baseMap.put( "lmtm",
        function(offset,limit,where,params,sort) {
            return this._thisObj.get( "limit" )( offset,limit,where,params,sort,false,true ) ;
        }
    ) ;
    // 指定取得用(Join).
    baseMap.put( "lmt",
        function(offset,limit,where,params,sort) {
            return this._thisObj.get( "limit" )( offset,limit,where,params,sort,false,false ) ;
        }
    ) ;
    // 指定取得用(リミット指定).
    baseMap.put( "limit",
        function(offset,limit,where,params,sort,join,many ) {
            if( isNull( offset ) ) {
                offset = 0 ;
            }
            if( isNull( limit ) ) {
                limit = -1 ;
            }
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            var joinMap = null ;
            if( join == true ) {
                joinMap = SubModel.getModelByJoinKeys( this._thisObj ) ;
            }
            var manyMap = null ;
            if( many == true ) {
                manyMap = SubModel.getModelBySrcJoinKeys( this.modelName,this._thisObj ) ;
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.limit( this.modelName,joinMap,manyMap,offset,limit,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定取得用(join-リミット指定).
    baseMap.put( "ljoin",
        function(join,offset,limit,where,params,sort ) {
            if( typeof( join ) != "map" ) {
                join = null ;
            }
            if( isNull( offset ) ) {
                offset = 0 ;
            }
            if( isNull( limit ) ) {
                limit = -1 ;
            }
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.limit( this.modelName,join,null,offset,limit,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定取得用(many-リミット指定).
    baseMap.put( "lmany",
        function(many,offset,limit,where,params,sort ) {
            if( typeof( many ) != "map" ) {
                many = null ;
            }
            if( isNull( offset ) ) {
                offset = 0 ;
            }
            if( isNull( limit ) ) {
                limit = -1 ;
            }
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.limit( this.modelName,null,many,offset,limit,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定取得用(最初の１件目だけを取得)(Join).
    baseMap.put( "fstj",
        function(where,params,sort) {
            return this._thisObj.get( "first" )( where,params,sort,true,false ) ;
        }
    ) ;
    // 指定取得用(最初の１件目だけを取得)(Many).
    baseMap.put( "fstm",
        function(where,params,sort) {
            return this._thisObj.get( "first" )( where,params,sort,false,true ) ;
        }
    ) ;
    // 指定取得用(最初の１件目だけを取得).
    baseMap.put( "fst",
        function(where,params,sort) {
            return this._thisObj.get( "first" )( where,params,sort,false,false ) ;
        }
    ) ;
    // 指定取得用(最初の１件目だけを取得).
    baseMap.put( "first",
        function(where,params,sort,join,many) {
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            var joinMap = null ;
            if( join == true ) {
                joinMap = SubModel.getModelByJoinKeys( this._thisObj ) ;
            }
            var manyMap = null ;
            if( many == true ) {
                manyMap = SubModel.getModelBySrcJoinKeys( this.modelName,this._thisObj ) ;
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.first( this.modelName,joinMap,manyMap,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定取得用(join-最初の１件目だけを取得).
    baseMap.put( "sjoin",
        function(join,where,params,sort) {
            if( typeof( join ) != "map" ) {
                join = null ;
            }
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.first( this.modelName,join,null,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定取得用(many-最初の１件目だけを取得).
    baseMap.put( "smany",
        function(many,where,params,sort) {
            if( typeof( many ) != "map" ) {
                many = null ;
            }
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var ret = md.first( this.modelName,null,many,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    // 指定数取得.
    baseMap.put( "read",
        function(where,params,sort) {
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            if( !useString( sort ) ) {
                sort = this.default_sort ;
            }
            if( useString( ( sort = SubModel.orderBy( sort ) ) ) ) {
                where += " " + sort ;
            }
            var rline = md.readLine( this.modelName,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            var ret = new _resultReadLine() ;
            ret.create( rline ) ;
            return ret ;
        }
    ) ;
    // 指定数取得.
    baseMap.put( "count",
        function(where,params) {
            if( !useString( where ) ) {
                where = "" ;
            }
            else {
                if( where.length > 0 && startsWith( where.toLowerCase(),"where " ) == false ) {
                    where = "where " + where ;
                }
            }
            if( isNull( params ) ) {
                params = null ;
            }
            else if( typeof( params ) != "array" ) {
                params = [ params ] ;
            }
            var time = null ;
            var md = getBaseModel() ;
            if( md.isDebug() == true ) {
                time = currentTime() ;
            }
            var list = null ;
            if( params != null ) {
                list = SubModel.convertParamByList( params ) ;
                if( isNull( list ) || list.size() <= 0 ) {
                    list = null ;
                }
            }
            var ret = md.count( this.modelName,where,list ) ;
            if( md.isDebug() == true ) {
                debugLog( "## [" + ( currentTime() - time ) + "ms]" ) ;
            }
            return ret ;
        }
    ) ;
    
    // 楽観ロックチェック.
    baseMap.put( "optimisticLock",
        function(wparam,modelOneLine) {
            // MAPでない場合は処理しない.
            if( typeof( modelOneLine ) != "map" ) {
                return false ;
            }
            // 条件にIDと楽観ロックIDが格納されていない場合は、処理しない.
            if( !isNumeric( modelOneLine[ "id" ] ) ||
                !isNumeric( modelOneLine( SubModel.COLUMN_OPTIMISTIC_LOCK ) ) ) {
                return false ;
            }
            // 検索条件が存在しない場合は、検索条件なし.
            if( typeof( wparam ) != "map" || wparam.length <= 0 ) {
                wparam = null ;
            }
            var md = getBaseModel() ;
            md.optimisticLock( this.modelName,wparam,modelOneLine ) ;
            return true ;
        }
    ) ;
    
    // missing_method定義(通常利用可能な命令一覧).
    baseMap.put( "_db_missing_methods_normal",{
        delj:null,del:null,delete:null,
        fndj:null,fndm:null,fnd:null,find:null,
        fstj:null,fstm:null,fst:null,first:null,
        fjoin:null,fmany:null,sjoin:null,smany:null,
        djoin:null,read:null,count:null
    } ) ;
    // missing_method定義(通常利用可能なlimit命令一覧).
    baseMap.put( "_db_missing_methods_limit",{
        lmtj:null,lmt:null,limit:null,ljoin:null,lmany:null
    } ) ;
    // missing_method定義(通常利用可能なselect処理系一覧).
    baseMap.put( "_db_missing_method_select",{
        fndj:null,fndm:null,fnd:null,find:null,
        fstj:null,fstm:null,fst:null,first:null,
        fjoin:null,fmany:null,sjoin:null,smany:null,
        read:null
    } ) ;
    // 直接Join/many系定義.
    baseMap.put( "_db_missing_method_joins",{
        fjoin:null,fmany:null,sjoin:null,smany:null,
        ljoin:null,lmany:null,djoin:null
    } ) ;
    
    // missing_method処理系.
    var _resultMap = new java.util.HashMap() ;
    
    _resultMap["targetMap"] = baseMap ;
    
    ////////////////////////////////////////////////////////////////////////////
    // JSAdapter定義.
    ////////////////////////////////////////////////////////////////////////////
    _resultMap["__put__"] = function(name,value) {
        if( useString( name ) ) {
            if( !this.name != "__put__" &&
                !this.name != "__get__" &&
                !this.name != "__has__" &&
                !this.name != "__delete__" &&
                !this.name != "__getIds__" &&
                !this.name != "__length__" ) {
                !this.targetMap.put( name,value ) ;
            }
        }
    } ;
    
    _resultMap["__get__"] = function (name) {
        if( !useString( name ) ) {
            return null ;
        }
        if( this.targetMap.containsKey( name ) ) {
            ret = this.targetMap.get( name ) ;
            if( ret instanceof pnuts.lang.PnutsFunction ) {
                return ret.setThisObject( this.targetMap ) ;
            }
            return ret ;
        }
        if( name.toLowerCase().indexOf( "_by_" ) <= -1 ) {
            return null ;
        }
        var missing = null ;
        var pms = name.split( "_" ) ;
        if( pms.length >= 3 ) {
            if( pms[ 1 ].toLowerCase() == "by" ) {
                var funcName = pms[ 0 ].toLowerCase() ;
                if( this.targetMap._db_missing_methods_normal.containsKey( funcName ) ) {
                    // join系処理.
                    if( this.targetMap._db_missing_method_joins.containsKey( funcName ) ) {
                        missing = function(join,params,sort) {
                            if( isNull( params ) ) {
                                params = null ;
                            }
                            var where = "" ;
                            var len = pms.length ;
                            for( var i = 2 ; i < len ; i ++ ) {
                                var columnName = DBUtil.convertJavaNameByDBName( pms[ i ] ) ;
                                pms[ i ] = pms[ i ].toLowerCase() ;
                                if( ( i & 0x00000001 ) == 0 ) {
                                    //if( pms[ i ] == "and" || pms[ i ] == "or" ) {
                                    if( pms[ i ] == "and" ) {
                                        return null ;
                                    }
                                    where += " $"+columnName+"=?" ;
                                }
                                else {
                                    //if( pms[ i ] == "and" || pms[ i ] == "or" ) {
                                    if( pms[ i ] == "and" ) {
                                        where += " " + pms[ i ] ;
                                    }
                                    else {
                                        return null ;
                                    }
                                }
                            }
                            // select系の処理.
                            if( this._db_missing_method_select.containsKey( funcName ) ) {
                                return this.get( funcName )( join,where,params,sort ) ;
                            }
                            // select系以外の処理.
                            return this.get( funcName )( join,where,params ) ;
                        } ;
                    }
                    // 通常処理.
                    else {
                        missing = function(params,sort) {
                            if( isNull( params ) ) {
                                params = null ;
                            }
                            var where = "" ;
                            var len = pms.length ;
                            for( var i = 2 ; i < len ; i ++ ) {
                                var columnName = DBUtil.convertJavaNameByDBName( pms[ i ] ) ;
                                pms[ i ] = pms[ i ].toLowerCase() ;
                                if( ( i & 0x00000001 ) == 0 ) {
                                    //if( pms[ i ] == "and" || pms[ i ] == "or" ) {
                                    if( pms[ i ] == "and" ) {
                                        return null ;
                                    }
                                    where += " $"+columnName+"=?" ;
                                }
                                else {
                                    //if( pms[ i ] == "and" || pms[ i ] == "or" ) {
                                    if( pms[ i ] == "and" ) {
                                        where += " " + pms[ i ] ;
                                    }
                                    else {
                                        return null ;
                                    }
                                }
                            }
                            // select系の処理.
                            if( this._db_missing_method_select.containsKey( funcName ) ) {
                                return this.get( funcName )( where,params,sort ) ;
                            }
                            // select系以外の処理.
                            return this.get( funcName )( where,params ) ;
                        } ;
                    }
                }
                else if( this.targetMap._db_missing_methods_limit.containsKey( funcName ) ) {
                    // join系.
                    if( this.targetMap._db_missing_method_joins.containsKey( funcName ) ) {
                        missing = function(join,offset,limit,params,sort) {
                            if( isNull( offset ) ) {
                                offset = 0 ;
                            }
                            if( isNull( limit ) ) {
                                limit = -1 ;
                            }
                            if( isNull( params ) ) {
                                params = null ;
                            }
                            var where = "" ;
                            var len = pms.length ;
                            for( var i = 2 ; i < len ; i ++ ) {
                                var columnName = DBUtil.convertJavaNameByDBName( pms[ i ] ) ;
                                pms[ i ] = pms[ i ].toLowerCase() ;
                                if( ( i & 0x00000001 ) == 0 ) {
                                    //if( pms[ i ] == "and" || pms[ i ] == "or" ) {
                                    if( pms[ i ] == "and" ) {
                                        return null ;
                                    }
                                    where += " $"+columnName+"=?" ;
                                }
                                else {
                                    //if( pms[ i ] == "and" || pms[ i ] == "or" ) {
                                    if( pms[ i ] == "and" ) {
                                        where += " " + pms[ i ] ;
                                    }
                                    else {
                                        return null ;
                                    }
                                }
                            }
                            return this.get( funcName )( join,offset,limit,where,params,sort ) ;
                        } ;
                    }
                    // 通常系.
                    else {
                        missing = function(offset,limit,params,sort) {
                            if( isNull( offset ) ) {
                                offset = 0 ;
                            }
                            if( isNull( limit ) ) {
                                limit = -1 ;
                            }
                            if( isNull( params ) ) {
                                params = null ;
                            }
                            var where = "" ;
                            var len = pms.length ;
                            for( var i = 2 ; i < len ; i ++ ) {
                                var columnName = DBUtil.convertJavaNameByDBName( pms[ i ] ) ;
                                pms[ i ] = pms[ i ].toLowerCase() ;
                                if( ( i & 0x00000001 ) == 0 ) {
                                    //if( pms[ i ] == "and" || pms[ i ] == "or" ) {
                                    if( pms[ i ] == "and" ) {
                                        return null ;
                                    }
                                    where += " $"+columnName+"=?" ;
                                }
                                else {
                                    //if( pms[ i ] == "and" || pms[ i ] == "or" ) {
                                    if( pms[ i ] == "and" ) {
                                        where += " " + pms[ i ] ;
                                    }
                                    else {
                                        return null ;
                                    }
                                }
                            }
                            return this.get( funcName )( offset,limit,where,params,sort ) ;
                        } ;
                    }
                }
            }
        }
        return missing.setThisObject( this.targetMap ) ;
    } ;
    
    _resultMap["__has__"] = function (name) {
        if( !useString( name ) ) {
            return false ;
        }
        if( name.toLowerCase().indexOf( "_by_" ) > 0 ) {
            return true ;
        }
        return this.targetMap.containsKey(name);
    } ;
    
    _resultMap["__getIds__"] = function () {
        return null ;
    } ;
    
    _resultMap["__length__"] = function() {
        return 0 ;
    } ;
    return createJSAdapter(_resultMap) ;
} ;

