/**
 * Json型式の外部定義をキャッシュ化.
 */

CacheJSON = {
    OBJECT : pkg( "org.maachang.seabass.httpd.engine.script.cache.CacheJson" ).getInstance(),
    // Json型式のファイルをロード.
    load : function( name,mode ) {
        if( !useString( name ) ) {
            throw new Exception( "JSONキャッシュファイル名を指定してください" ) ;
        }
        if( name.startsWith( "/" ) ) {
            name = Io.getFullPath( name.substring( 1 ) ) ;
        }
        else if( name.startsWith( "./" ) || name.startsWith( "../" ) ) {
            name = Io.getFullPath( toFullPath( name ) ) ;
        }
        // ファイルが存在する場合.
        if( !Io.isFileExists( name ) ) {
            throw new Exception( "JSONキャッシュファイルは存在しません:" + name ) ;
        }
        mode = ( mode == "true" || mode == true ) ;
        // キャッシュから情報を取得.
        var ret = this.OBJECT.get( name ) ;
        // キャッシュから情報を取得出来ない場合は、ファイルを読み込んでキャッシュ化.
        if( ret == null ) {
            var b = Io.readFileByBinary( name ) ;
            var lst = this.OBJECT.jsonHeader( b ) ;
            if( lst == null ) {
                throw new Exception( "JSONキャッシュファイル型式ではありません" ) ;
            }
            var len = lst.length ;
            eval( new String( b,"UTF8" ) ) ;
            var ret = new org.maachang.seabass.httpd.engine.script.cache.CacheJsonChild() ;
            ret.keyName = name ;
            ret.createChild( len ) ;
            for( var i = 0 ; i < len ; i ++ ) {
                var n = lst[ i ] ;
                ret.child[ i ].key = n ;
                ret.child[ i ].value = lockCopy( getBindings( n ) ) ;
                if( mode ) {
                    putBindings( n,deepCopy( ret.child[ i ].value ) ) ;
                }
                else {
                    putBindings( n,ret.child[ i ].value ) ;
                }
            }
            this.OBJECT.put( name,ret ) ;
            return ;
        }
        else {
            // キャッシュが再読み込みされた場合は、キャッシュ内容を更新.
            ret.update() ;
        }
        // JSONキャッシュ情報をメインメモリに展開.
        var len = ret.child.length ;
        for( var i = 0 ; i < len ; i ++ ) {
            if( mode ) {
                putBindings( ret.child[i].key,deepCopy( ret.child[i].value ) ) ;
            }
            else {
                putBindings( ret.child[i].key,ret.child[i].value ) ;
            }
        }
    },
    // コピーした内容をロード.
    loadCopy : function( name ) {
        this.load( name,true ) ;
    },
    // １つのファイルに１つのJSON条件を保存.
    oneSave : function( name,keyName,value ) {
        if( !useString( name ) ) {
            throw new Exception( "JSONキャッシュファイル名を指定してください" ) ;
        }
        else if( value == null ) {
            throw new Exception( "JSONキャッシュファイル内容を指定してください" ) ;
        }
        else if( keyName == null ) {
            throw new Exception( "JSONキャッシュキー名を指定してください" ) ;
        }
        if( name.startsWith( "/" ) ) {
            name = Io.getFullPath( name.substring( 1 ) ) ;
        }
        else if( name.startsWith( "./" ) || name.startsWith( "../" ) ) {
            name = Io.getFullPath( toFullPath( name ) ) ;
        }
        var buf = new StrBuf() ;
        buf.ad( "// " ).ad( keyName ).ad( "\n" ).
            ad( keyName ).ad( "=" ).ad( encodeJSON( value ) ).ad( "\n" ) ;
        Io.writeFileByString( true,name,buf.ts(),"UTF8" ) ;
        buf = null ;
        this.OBJECT.remove( name ) ;
    },
    // １つのファイルに複数のJSON条件を保存.
    arraySave : function( name,map ) {
        if( !useString( name ) ) {
            throw new Exception( "JSONキャッシュファイル名を指定してください" ) ;
        }
        else if( map == null ) {
            throw new Exception( "JSONキャッシュファイル内容を指定してください" ) ;
        }
        else if( typeof( map ) != "map" ) {
            throw new Exception( "JSONキャッシュファイル内容はMap指定してください" ) ;
        }
        else if( map.size() <= 0 ) {
            throw new Exception( "JSONキャッシュファイル内容を指定してください" ) ;
        }
        if( name.startsWith( "/" ) ) {
            name = Io.getFullPath( name.substring( 1 ) ) ;
        }
        else if( name.startsWith( "./" ) || name.startsWith( "../" ) ) {
            name = Io.getFullPath( toFullPath( name ) ) ;
        }
        var buf = new StrBuf() ;
        var nList = map.keySet().toArray() ;
        var len = nList.length ;
        buf.ad( "// " ) ;
        for( var i = 0 ; i < len ; i ++ ) {
            if( i != 0 ) {
                buf.ad( "|" ) ;
            }
            buf.ad( nList[ i ] ) ;
        }
        buf.ad( "\n" ) ;
        for( var i = 0 ; i < len ; i ++ ) {
            buf.ad( nList[ i ] ).ad( "=" ).ad( encodeJSON( map[nList[i]] ) ).ad( "\n" ) ;
        }
        Io.writeFileByString( true,name,buf.ts(),"UTF8" ) ;
        buf = null ;
        this.OBJECT.remove( name ) ;
    }
}
