// Bdb(DBM)操作用スクリプト.
//

Dbm = {
    // オブジェクト生成処理.
    prototype : {
        name : null,
        bdb : null,
        keyValue : null,
        // コンストラクタ.
        _constructor : function( name ) {
            if( useString( name ) ) {
                this.create( name ) ;
            }
        },
        // DBMオブジェクト生成処理.
        create : function( name ) {
            if( !useString( name ) ) {
                throw new java.lang.IllegalArgumentException( "引数は不正です" ) ;
            }
            BdbFactory = pkg( "org.maachang.seabass.dbm.BdbFactory" ) ;
            if( isRequest() ) {
                this.keyValue = getRequest().getBdbKeyValue() ;
            }
            else {
                this.keyValue = BdbFactory.getBdbKeyValue() ;
            }
            this.name = name ;
            this.bdb = BdbFactory.get( name ) ;
            return true ;
        },
        // DBMオブジェクトクリア処理.
        close : function() {
            this.name = null ;
            this.bdb = null ;
            this.keyValue = null ;
        },
        // データ追加.
        put : function( key,value ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || isNull( value ) ) {
                throw new java.lang.IllegalArgumentException( "引数は不正です" ) ;
            }
            if( typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            value = encodeJSON( value ) ;
            this.bdb.put( this.keyValue.create( key.getBytes( "UTF8" ),value.getBytes( "UTF8" ) ) ) ;
            this.keyValue.clear() ;
            return true ;
        },
        // データ削除.
        remove : function( key ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            if( this.bdb.remove( this.keyValue.create( key.getBytes( "UTF8" ),null ) ) ) {
                this.keyValue.clear() ;
                return true ;
            }
            this.keyValue.clear() ;
            return false ;
        },
        // データ削除して取得.
        removeToGet : function( key ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            if( this.bdb.remove( this.keyValue.create( key.getBytes( "UTF8" ),null ) ) ) {
                var val = this.keyValue.getValue() ;
                this.keyValue.clear() ;
                var ret = null ;
                if( isNull( val ) == false ) {
                    ret = decodeJSON( new java.lang.String( val,"UTF8" ) ) ;
                }
                if( isNull( ret ) ) {
                    return null ;
                }
                return ret ;
            }
            return null ;
        },
        // データ取得.
        get : function( key ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            if( this.bdb.get( this.keyValue.create( key.getBytes( "UTF8" ),null ) ) ) {
                var val = this.keyValue.getValue() ;
                this.keyValue.clear() ;
                var ret = null ;
                if( isNull( val ) == false ) {
                    ret = decodeJSON( new java.lang.String( val,"UTF8" ) ) ;
                }
                if( isNull( ret ) ) {
                    return null ;
                }
                return ret ;
            }
            return null ;
        },
        // データ存在確認.
        containsKey : function( key ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            var ret = this.bdb.containsKey( this.keyValue.create( key.getBytes( "UTF8" ),null ) ) ;
            this.keyValue.clear() ;
            return ret ;
        },
        // 格納数を取得.
        getLength : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.bdb.size() ;
        },
        // オープンDBM名を取得.
        getName : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.name ;
        },
        // クローズチェック.
        isClose : function() {
            return ( this.bdb == null || this.bdb.isClose() ) ;
        }
    }
}

// トランザクション用DBM.
TransactionDbm = {
    prototype : {
        trans:null,
        name:null,
        // コンストラクタ.
        _constructor : function( name ) {
            if( !useString( name ) ) {
                throw new java.lang.IllegalArgumentException( "引数は不正です" ) ;
            }
            BdbFactory = pkg( "org.maachang.seabass.dbm.BdbFactory" ) ;
            if( isRequest() ) {
                var keyValue = getRequest().getBdbKeyValue() ;
            }
            else {
                var keyValue = BdbFactory.getBdbKeyValue() ;
            }
            this.name = name ;
            this.trans = BdbFactory.getTransaction( keyValue,name ) ;
            return true ;
        },
        // DBMオブジェクトクリア処理.
        close : function() {
            if( this.trans != null ) {
                trans.close() ;
            }
            this.name = null ;
            this.trans = null ;
        },
        // コミット処理.
        commit : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            this.trans.commit() ;
            return true ;
        },
        // ロールバック.
        rollback : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            this.trans.rollback() ;
            return true ;
        },
        // データ追加.
        put : function( key,value ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || isNull( value ) ) {
                throw new java.lang.IllegalArgumentException( "引数は不正です" ) ;
            }
            if( typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            value = encodeJSON( value ) ;
            this.trans.put( key.getBytes( "UTF8" ),value.getBytes( "UTF8" ) ) ;
            return true ;
        },
        // データ削除.
        remove : function( key ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            this.trans.remove( key.getBytes( "UTF8" ) ) ;
            return true ;
        },
        // データ削除して取得.
        removeToGet : function( key ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            var val = this.trans.remove( key.getBytes( "UTF8" ) ) ;
            if( val != null ) {
                return decodeJSON( new java.lang.String( val,"UTF8" ) ) ;
            }
            return null ;
        },
        // データ取得.
        get : function( key ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            var val = this.trans.get( key.getBytes( "UTF8" ) ) ;
            if( val != null ) {
                return decodeJSON( new java.lang.String( val,"UTF8" ) ) ;
            }
            return null ;
        },
        // データ存在確認.
        containsKey : function( key ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( key ) || typeof( key ) != "string" ) {
                if( typeof( key ) == "number" ) {
                    key = convertString( key ) ;
                }
                else {
                    throw new java.lang.IllegalArgumentException( "キーは文字列及び、数値で指定してください" ) ;
                }
            }
            return this.trans.containsKey( key.getBytes( "UTF8" ) ) ;
        },
        // 格納数を取得.
        getLength : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.trans.size() ;
        },
        // オープンDBM名を取得.
        getName : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.name ;
        },
        // クローズチェック.
        isClose : function() {
            return ( this.trans == null || this.trans.isClose() ) ;
        }
    }
}

// BQueueオブジェクト.
BQueue = {
    // オブジェクト生成処理.
    prototype : {
        name : null,
        queue : null,
        keyValue : null,
        // コンストラクタ.
        _constructor : function( name,max,warning ) {
            if( useString( name ) ) {
                this.create( name,max,warning ) ;
            }
        },
        // DBMオブジェクト生成処理.
        create : function( name,max,warning ) {
            if( !this.isClose() ) {
                this.close() ;
            }
            if( !useString( name ) ) {
                throw new java.lang.IllegalArgumentException( "引数は不正です" ) ;
            }
            BQueueFactory = pkg( "org.maachang.seabass.dbm.BQueueFactory" ) ;
            if( isRequest() ) {
                this.keyValue = getRequest().getBdbKeyValue() ;
            }
            else {
                this.keyValue = BQueueFactory.getBdbKeyValue() ;
            }
            if( !isNumeric( max ) ) {
                max = pkg( "java.lang.Long" ).MAX_VALUE ;
            }
            if( !isDouble( warning ) ) {
                warning = 0.75d ;
            }
            this.name = name ;
            this.queue = BQueueFactory.get( name,max,warning ) ;
            return true ;
        },
        // DBMオブジェクトクリア処理.
        close : function() {
            this.name = null ;
            this.queue = null ;
            this.keyValue = null ;
        },
        // データ追加.
        offer : function( value ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( value ) ) {
                throw new java.lang.IllegalArgumentException( "引数は不正です" ) ;
            }
            value = encodeJSON( value ) ;
            this.queue.offer( this.keyValue,value.getBytes( "UTF8" ) ) ;
            this.keyValue.clear() ;
            return true ;
        },
        // データ取得.
        // 取得後、キュー先頭情報は削除される.
        poll : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            var ret = this.queue.poll( this.keyValue ) ;
            this.keyValue.clear() ;
            if( ret == null ) {
                return null ;
            }
            return decodeJSON( new java.lang.String( ret,"UTF8" ) ) ;
        },
        // データ取得.
        // 取得後、キュー先頭情報は削除されない.
        peek : function() {
            if( this.queue == null ) {
                return null ;
            }
            var ret = this.queue.peek( this.keyValue ) ;
            this.keyValue.clear() ;
            if( ret == null ) {
                return null ;
            }
            return decodeJSON( new java.lang.String( ret,"UTF8" ) ) ;
        },
        // 格納数を取得.
        getLength : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.queue.length() ;
        },
        // Iteratorを取得.
        iterator : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return new JSONIterator( this.queue.iterator(),false ) ;
        },
        // 最大格納数を取得.
        getMaxLength : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.queue.getMaxLength() ;
        },
        // 警告パーセント値を取得.
        getWarning : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.queue.getWarning() ;
        },
        // キュー格納数が警告水域に達している場合.
        isWarning : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.queue.isWarning() ;
        },
        // オープンBQueue名を取得.
        getName : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.name ;
        },
        // クローズチェック.
        isClose : function() {
            return ( this.queue == null || this.queue.isClose() ) ;
        }
    }
}

// トランザクション用Queue.
TransactionQueue = {
    // オブジェクト生成処理.
    prototype : {
        name : null,
        queue : null,
        // コンストラクタ.
        _constructor : function( name,max,warning ) {
            if( !this.isClose() ) {
                this.close() ;
            }
            if( !useString( name ) ) {
                throw new java.lang.IllegalArgumentException( "引数は不正です" ) ;
            }
            BQueueFactory = pkg( "org.maachang.seabass.dbm.BQueueFactory" ) ;
            if( isRequest() ) {
                var keyValue = getRequest().getBdbKeyValue() ;
            }
            else {
                var keyValue = BQueueFactory.getBdbKeyValue() ;
            }
            if( !isNumeric( max ) ) {
                max = (long)pkg( "java.lang.Integer" ).MAX_VALUE ;
            }
            if( !isDouble( warning ) ) {
                warning = 0.75d ;
            }
            this.name = name ;
            this.queue = BQueueFactory.getTransaction( keyValue,name,max,warning ) ;
            return true ;
        },
        // DBMオブジェクトクリア処理.
        close : function() {
            if( this.queue != null ) {
                this.queue.close() ;
            }
            this.name = null ;
            this.queue = null ;
        },
        // コミット処理.
        commit : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            this.queue.commit() ;
            return true ;
        },
        // ロールバック.
        rollback : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            this.queue.rollback() ;
            return true ;
        },
        // データ追加.
        offer : function( value ) {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            if( isNull( value ) ) {
                throw new java.lang.IllegalArgumentException( "引数は不正です" ) ;
            }
            this.queue.offer( encodeJSON( value ).getBytes( "UTF8" ) ) ;
            return true ;
        },
        // データ取得.
        // 取得後、キュー先頭情報は削除される.
        poll : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            var ret = this.queue.poll() ;
            if( ret == null ) {
                return null ;
            }
            return decodeJSON( new java.lang.String( ret,"UTF8" ) ) ;
        },
        // データ取得.
        // 取得後、キュー先頭情報は削除されない.
        peek : function() {
            if( this.queue == null ) {
                return null ;
            }
            var ret = this.queue.peek() ;
            if( ret == null ) {
                return null ;
            }
            return decodeJSON( new java.lang.String( ret,"UTF8" ) ) ;
        },
        // 格納数を取得.
        getLength : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.queue.length() ;
        },
        // Iteratorを取得.
        iterator : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return new JSONIterator( this.queue.iterator(),false ) ;
        },
        // 最大格納数を取得.
        getMaxLength : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.queue.getMaxLength() ;
        },
        // 警告パーセント値を取得.
        getWarning : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.queue.getWarning() ;
        },
        // キュー格納数が警告水域に達している場合.
        isWarning : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.queue.isWarning() ;
        },
        // オープンBQueue名を取得.
        getName : function() {
            if( this.isClose() ) {
                throw new java.io.IoException( "オブジェクトは既にクローズしています" ) ;
            }
            return this.name ;
        },
        // クローズチェック.
        isClose : function() {
            return ( this.queue == null || this.queue.isClose() ) ;
        }
    }
}

// JSON変換用Iterator.
JSONIterator = {
    prototype : {
        mode : false,
        // コンストラクタ.
        _constructor : function( it,mode ) {
            if( !isNull( it,mode ) ) {
                this.create( it,mode ) ;
            }
        },
        // 情報生成.
        create : function( it,mode ) {
            this.it = it ;
            this.mode = ( mode == true ) ;
            // modeがtrueで、nextの戻り値がBdbKeyValueの場合は、
            // key情報を返す.
        },
        // 次の情報が存在するかチェック.
        hasNext : function() {
            return this.it.hasNext() ;
        },
        // 次の情報を取得.
        next : function() {
            var s = this.it.next() ;
            if( s == null ) {
                return null ;
            }
            if( s instanceof byte[] ) {
                if( this.mode ) {
                    return new java.lang.String( s,"UTF8" ) ;
                }
                else {
                    return decodeJSON( new java.lang.String( s,"UTF8" ) ) ;
                }
            }
            if( s instanceof java.lang.String ) {
                if( this.mode ) {
                    return s ;
                }
                else {
                    return decodeJSON( s ) ;
                }
            }
            else if( s instanceof org.maachang.bdb.BdbKeyValue ) {
                if( this.mode ) {
                    s = s.getKey() ;
                }
                else {
                    s = s.getValue() ;
                }
                if( s == null ) {
                    return null ;
                }
                if( this.mode ) {
                    return new java.lang.String( s,"UTF8" ) ;
                }
                else {
                    return decodeJSON( new java.lang.String( s,"UTF8" ) ) ;
                }
            }
            return null ;
        },
        // 次の情報を削除.
        remove : function() {
            this.it.remove() ;
        }
    }
}

