// マスターDBキャッシュ用.
//
DbUtil = pkg("org.maachang.dao.dbms.DbUtil") ;

// マスタキャッシュ管理.
MasterCache = {
    // マスターキャッシュ情報を生成.
    create : function(name) {
        if( isNull( _$cachedb$params ) || useString( name ) == false ) {
            return null ;
        }
        var model = Models[name] ;
        if( isNull( model ) ||
            isNull( model.isCacheMap ) ||
            model.isCacheMap != true ) {
            return null ;
        }
        if( _$cachedb$params.containsKey( name ) ) {
            var o = _$cachedb$params.get( name ) ;
            if( o instanceof org.maachang.seabass.httpd.engine.script.dao.MasterCacheChild ) {
                if( o.isNoValue() == false ) {
                    return o.getValue() ;
                }
            }
            _$cachedb$params.remove( name ) ;
        }
        nameMasterMode = model.namesMaster ; // 名前管理マスターモードを取得.
        var ary = null ;
        if( model.isDeleteFlag == true ) {
            ary = model.find_by_deleteFlag( false ) ;
        }
        else {
            ary = model.find( null,null ) ;
        }
        if( isNull( ary ) == false && ary.length > 0 ) {
            var cmodel = {} ;
            // 名前管理マスターの場合.
            if( nameMasterMode ) {
                var map = {} ;
                var list = {} ;
                var def = {} ;
                var groupMap = {}
                var len = ary.length ;
                // グループ名で区分け.
                for( var i = 0 ; i < len ; i ++ ) {
                    var group = ary[ i ].groupName ;
                    var o = map.get( group ) ;
                    if( o == null ) {
                        o = [ ary[ i ].childId,ary[ i ].childName ] ;
                    }
                    else {
                        o.add( ary[ i ].childId ) ;
                        o.add( ary[ i ].childName ) ;
                    }
                    map.put( group,o ) ;
                    var childId = ary[ i ].childId ;
                    o = list[ group ] ;
                    if( o == null ) {
                        o = {} ;
                        list.put( group,o ) ;
                    }
                    o[childId] = ary[ i ].childName ;
                    o = def[ group ] ;
                    if( o == null ) {
                        o = {} ;
                        def.put( group,o ) ;
                    }
                    o[ary[ i ].childName] = childId ;
                    groupMap[ group ] = group ;
                }
                var glist = [] ;
                for( var k in groupMap ) {
                    glist.add( groupMap[ k ] ) ;
                    glist.add( k ) ;
                }
                groupMap.clear() ;
                groupMap = null ;
                glist = sortList( glist ) ;
                cmodel["map"] = map ;
                cmodel["list"] = list ;
                cmodel["groups"] = glist ;
                cmodel["def"] = def ;
            }
            // 通常マスターの場合.
            else {
                var map = {} ;
                var len = ary.length ;
                for( var i = 0 ; i < len ; i ++ ) {
                    map.put( ary[ i ]["id"],ary[ i ] ) ;
                }
                cmodel["map"] = map ;
                cmodel["list"] = ary ;
            }
            _$cachedb$params.putByCache( name,cmodel ) ;
            return cmodel ;
        }
        else {
            var cmodel = {list:[],map:{}} ;
            _$cachedb$params.putByCache( name,cmodel ) ;
            return cmodel ;
        }
    },
    // マスターキャッシュ情報をMap形式で取得.
    getCacheDB : function(name) {
        var ret = this.create(name) ;
        if( isNull( ret ) ) {
            return {} ;
        }
        return ret["map"] ;
    },
    // マスターキャッシュ情報をリスト形式で取得.
    getCacheDBByList : function(name) {
        var ret = this.create(name) ;
        if( isNull( ret ) ) {
            return [] ;
        }
        return ret["list"] ;
    },
    // 名前管理マスターのグループ名一覧を取得.
    getNameManagerByGroupList : function( name ) {
        if( !MasterCache.isNamesMaster( name ) ) {
            return [] ;
        }
        var ret = this.create(name) ;
        if( isNull( ret ) ) {
            return [] ;
        }
        return ret["groups"] ;
    },
    // マスターキャッシュから、指定項目を除外した条件で取得.
    getCacheDBByListByNotColumns : function(name,colList) {
        if( isNull( _$cachedb$params ) || isNull( name ) ||
            isNull( colList ) ) {
            return [] ;
        }
        // 名前管理マスターの場合は、この処理は利用できない.
        if( MasterCache.isNamesMaster( name ) ) {
            return [] ;
        }
        if( typeof( colList ) == "string" ) {
            colList = [ colList ] ;
        }
        else if( typeof( colList ) != "array" ) {
            return [] ;
        }
        var cmodel = this.create( name ) ;
        if( cmodel == null ) {
            return [] ;
        }
        if( colList.length <= 0 ) {
            return cmodel['list'] ;
        }
        var len = cmodel['list'].length ;
        if( len <= 0 ) {
            return [] ;
        }
        var buf = new StrBuf() ;
        buf.ad( "list-" ) ;
        for( var i = 0 ; i < colList.length ; i ++ ) {
            if( i != 0 ) {
                buf.ad( "-" ) ;
            }
            buf.ad( colList[ i ] ) ;
        }
        var key = buf.ts() ;
        buf = null ;
        if( cmodel.containsKey( key ) ) {
            return cmodel[ key ] ;
        }
        var ret = [] ;
        var lst = cmodel['list'] ;
        for( var i = 0 ; i < len ; i ++ ) {
            var m = {} ;
            var tm = lst[ i ] ;
            for( var k in tm ) {
                if( colList.contains( k ) ) {
                    continue ;
                }
                m[k] = tm[k] ;
            }
            ret.add( m ) ;
        }
        cmodel[ key ] = ret ;
        return ret ;
    },
    // マスターキャッシュ削除.
    remove : function( name ) {
        if( isNull( _$cachedb$params ) || isNull( name ) ) {
            return ;
        }
        _$cachedb$params.remove( name ) ;
    },
    
    // 指定したDB名がキャッシュ化されているかチェック.
    containsKey : function( name ) {
        if( isNull( _$cachedb$params ) || isNull( name ) ) {
            return false ;
        }
        return _$cachedb$params.containsKey( name ) ;
    },
    
    // 現在存在するキャッシュ情報をリセット.
    resetCacheDB : function() {
        if( isNull( _$cachedb$params ) ) {
            return ;
        }
        _$cachedb$params.clear() ;
    },
    
    // マスターキャッシュ名一覧を取得.
    useCacheName : function() {
        if( isNull( _$cachedb$params ) ) {
            return [] ;
        }
        ret = _$cachedb$params.getKeys() ;
        if( isNull( ret ) ) {
            return [] ;
        }
        return ret ;
    },
    
    // 単体内容チェック.
    _chkToLike : function( c1,c2 ) {
        c1 = convertString( c1 ) ;
        c2 = convertString( c2 ) ;
        // like検索のような感じで行う.
        if( startsWith( c2,"%" ) ) {
            if( endsWith( c2,"%" ) ) {
                if( c1.indexOf( c2.substring( 1,c2.length-1 ) ) <= -1 ) {
                    return false ;
                }
            }
            else {
                if( startsWith( c1,c2.substring( 1 ) ) == false ) {
                    return false ;
                }
            }
        }
        else if( endsWith( c2,"%" ) ) {
            if( endsWith( c1,c2.substring( 0,c2.length-1 ) ) == false ) {
                return false ;
            }
        }
        else {
            if( c1 != c2 ) {
                return false ;
            }
        }
        return true ;
    },
    
    // データベース名に変換.
    _convertDatabaseName : function( name ) {
        return name.substring( 0,1 ).toUpperCase() + name.substring( 1 ) ;
    },
    
    // MasterCacheから、一致する条件を検索.
    find : function( name,where,offset,limit ) {
        // 名前管理マスターの場合は、この処理は利用できない.
        if( MasterCache.isNamesMaster( name ) ) {
            return [] ;
        }
        var res = this.create( name ) ;
        if( isNull( res ) || isNull( res["list"] ) || res["list"].length <= 0 ) {
            return [] ;
        }
        if( isNull( offset ) || isNumeric( offset ) == false || offset < 0 ) {
            offset = 0 ;
        }
        if( isNull( limit ) || isNumeric( limit ) == false || limit < 0 ) {
            limit = -1 ;
        }
        
        // 検索条件が存在しない場合.
        if( isNull( where ) || typeof( where ) != "map" ) {
            // 範囲取得の場合は、部分ごとで取得する.
            if( offset >= 0 || limit >= 0 ) {
                var lst = res["list"] ;
                var len = lst.length ;
                var ret = [] ;
                for( var i = offset ; i < len ; i ++ ) {
                    ret.add( deepCopy( lst[ i ] ) ) ;
                    if( limit > 0 && ret.length >= limit ) {
                        return ret ;
                    }
                }
                return ret ;
            }
            return res["list"] ;
        }
        // 検索条件が存在する場合.
        var ret = [] ;
        var lst = res["list"] ;
        var len = lst.length ;
        for( var i = offset ; i < len ; i ++ ) {
            var x = lst[ i ] ;
            var f = true ;
            for( var k in where ) {
                var w = where[ k ] ;
                var t = typeof( w ) ;
                // orでチェック.
                if( t == "array" ) {
                    var lenJ = w.length ;
                    var fj = false ;
                    for( var j = 0 ; j < lenJ ; j ++ ) {
                        if( this._chkToLike( x[ k ],w[ j ] ) == false ) {
                            fj = true ;
                            break ;
                        }
                    }
                    if( fj == false ) {
                        f = false ;
                        break ;
                    }
                }
                // 範囲チェック.
                else if( t == "map" ) {
                    var min = w["min"] ;
                    var max = w["max"] ;
                    // 範囲指定の用件でない場合は無視する.
                    if( isNull( min ) || isNull( max ) ) {
                        continue ;
                    }
                    if( (x[ k ] >= min && x[ k ] <= max) == false ) {
                        f = false ;
                        break ;
                    }
                }
                else {
                    if( this._chkToLike( x[ k ],w ) == false ) {
                        f = false ;
                        break ;
                    }
                }
            }
            if( f ) {
                ret.add( deepCopy( x ) ) ;
                if( limit > 0 && ret.length >= limit ) {
                    break ;
                }
            }
        }
        return ret ;
    },
    
    // MasterCacheから、一致する条件の数を検索.
    count : function( name,where ) {
        // 名前管理マスターの場合は、この処理は利用できない.
        if( MasterCache.isNamesMaster( name ) ) {
            return [] ;
        }
        var res = this.create( name ) ;
        if( isNull( res ) || isNull( res["list"] ) || res["list"].length <= 0 ) {
            return 0 ;
        }
        
        if( isNull( where ) || typeof( where ) != "map" ) {
            return res["list"].length ;
        }
        var ret = 0 ;
        var lst = res["list"] ;
        var len = lst.length ;
        for( var i = 0 ; i < len ; i ++ ) {
            var x = lst[ i ] ;
            var f = true ;
            for( var k in where ) {
                var w = where[ k ] ;
                var t = typeof( w ) ;
                // orでチェック.
                if( t == "array" ) {
                    var lenJ = w.length ;
                    var fj = false ;
                    for( var j = 0 ; j < lenJ ; j ++ ) {
                        if( this._chkToLike( x[ k ],w[ j ] ) == false ) {
                            fj = true ;
                            break ;
                        }
                    }
                    if( fj == false ) {
                        f = false ;
                        break ;
                    }
                }
                // 範囲チェック.
                else if( t == "map" ) {
                    var min = w["min"] ;
                    var max = w["max"] ;
                    // 範囲指定の用件でない場合は無視する.
                    if( isNull( min ) || isNull( max ) ) {
                        continue ;
                    }
                    if( (x[ k ] >= min && x[ k ] <= max) == false ) {
                        f = false ;
                        break ;
                    }
                }
                // 通常チェック.
                else {
                    if( this._chkToLike( x[ k ],w ) == false ) {
                        f = false ;
                        break ;
                    }
                }
            }
            if( f ) {
                ret ++ ;
            }
        }
        return ret ;
    },
    // 名前管理マスターであるか調べる.
    isNamesMaster : function( name ) {
        name = MasterCache._convertDatabaseName( name ) ;
        if( Models[name] != null && Models[name].namesMaster ) {
            return true ;
        }
        return false ;
    }
}

// マスターキャッシュパラメータ.
// ☆使い方☆
// cacheDB["model名"]
// <通常>
//   対象model名の全idをキーとしたmapが返されます.
// <名前管理テーブル>
//   groupNameカラムをキーとした、map(key=groupName,value=[childId,childName])が返されます.
//
// cacheDB["$model名"]
// <通常>
//   対象model名の全データ(list)が返されます.
// <名前管理テーブル>
//   groupNameカラムをキーとした、map(key=groupName,value={childId:childName})が返されます.
// 
// cacheDB["$$model名"]
// <通常>
//   対象model名の全nameをキーとしたmapが返されます.
// <名前管理テーブル>
//   一意のgroupNameを昇順ソートされた内容がlist型式で返されます.
//
// cacheDB["$find_model名"]
// <通常>
//   通常のModelと同じような使い方ができる、find系function( where,offset,limit )が返されます.
// <名前管理テーブル>
//   空のリストが返されます(未対応).
//
// cacheDB["$count_model名"]
// <通常>
//   通常のModelと同じような使い方ができる、count系function( where,offset,limit )が返されます.
// <名前管理テーブル>
//   空のリストが返されます(未対応).
//
// cacheDB['@model名']
// <通常>
//  未対応.
// <名前管理テーブル>
//  指定名の定義名に対するIDが返されます.
//  例えばこんな感じ.
//  >cacheDB.NameMaster.Test
//  {A:1,B:2]
//  >cacheDB.NameMaster.Test.A
//  1
//
cacheDBParams = function() {
    return createJSAdapter({
        __get__ : function(name) {
            // 先頭に＄が付加されている.
            if( startsWith( name,"$" ) ) {
                // 先頭が[$find_]の場合は、Find処理を行う.
                if( startsWith( name,"$find_" ) ) {
                    var dbName = MasterCache._convertDatabaseName( name.substring( "$find_".length ) ) ;
                    return (function( where,offset,limit ) {
                        return MasterCache.find( arguments[0],where,offset,limit ) ;
                    }).pushArgs( dbName )
                }
                // 先頭が[$count_]の場合は、count処理を行う.
                else if( startsWith( name,"$count_" ) ) {
                    var dbName = MasterCache._convertDatabaseName( name.substring( "$count_".length ) ) ;
                    return (function( where ) {
                        return MasterCache.count( arguments[0],where ) ;
                    }).pushArgs( dbName ) ;
                }
                // 先頭に＄が２つ存在する場合は、名前をキーとして処理する.
                else if( startsWith( name,"$$" ) || startsWith( name,"$key_" ) ) {
                    if( startsWith( name,"$$" ) ) {
                        name = MasterCache._convertDatabaseName( name.substring( "$$".length ) ) ;
                    }
                    else {
                        name = MasterCache._convertDatabaseName( name.substring( "$key_".length ) ) ;
                    }
                    if( Models[name] != null && Models[name].namesMaster ) {
                        var res = MasterCache.create( name ) ;
                        if( isNull( res ) || isNull( res["groups"] ) || res["groups"].length <= 0 ) {
                            return [] ;
                        }
                        return res["groups"] ;
                    }
                    var res = MasterCache.create( name ) ;
                    if( isNull( res ) || isNull( res["list"] ) ) {
                        return {} ;
                    }
                    var lst = res["list"] ; 
                    var len = lst.length ;
                    var ret = {} ;
                    for( var i = 0 ; i < len ; i ++ ) {
                        ret.put( lst[ i ].name,lst[ i ].id ) ;
                    }
                    return ret ;
                }
                // 先頭に＄が存在する場合は、Array配列で返す.
                else if( startsWith( name,"$" ) ) {
                    name = MasterCache._convertDatabaseName( name.substring( 1 ) ) ;
                    var res = MasterCache.create( name ) ;
                    if( isNull( res ) || isNull( res["list"] ) || res["list"].length <= 0 ) {
                        return [] ;
                    }
                    return res["list"] ;
                }
            }
            // 開始文字列が＠の場合は、名前管理マスター定義処理を行う.
            else if( startsWith( name,"@" ) ) {
                name = MasterCache._convertDatabaseName( name.substring( 1 ) ) ;
                var model = Models[name] ;
                if( model != null ) {
                    // 名前管理マスターの場合.
                    if( model.namesMaster ) {
                        var res = MasterCache.create( name ) ;
                        if( res == null ) {
                            return {} ;
                        }
                        return res["def"] ;
                    }
                }
            }
            // ＄が先頭にない場合はMAP形式で返す.
            var ret = MasterCache.getCacheDB( MasterCache._convertDatabaseName( name ) ) ;
            if( isNull( ret ) ) {
                return {} ;
            }
            return ret ;
        },
        __has__ :  function(name) {
            // 全て存在することにする.
            return true ;
        },
        __delete__ : function (name) {
            return null
        },
        __put__ : function(name, value) {
            return null ;
        },
        __getIds__ : function() {
            return null ;
        },
        __length__ : function() {
            return 0 ;
        }
    });
} ;

// 実際に使われるcacheDB.
cacheDB = cacheDBParams() ;

////////////////////////////////////////////////////////////////////////////////
// マスターキャッシュ対応タグ.
////////////////////////////////////////////////////////////////////////////////
MasterTag = {
    // 対象パラメータ名のマスター内容を出力.
    viewListToParams : function(name,model,sview,eview,vname,chain,targetLine) {
        if( isNull( name ) || isNull( model ) ) {
            return "" ;
        }
        if( typeof( name ) == "string" && !useString( name ) ) {
            return "" ;
        }
        if( typeof( model ) == "map" && Models[model.name] == model ) {
            model = model.name ;
        }
        if( typeof( model ) != "string" ) {
            return "" ;
        }
        if( !useString( vname ) ) {
            vname = MasterTag._viewColumns( model ) ;
        }
        var query = super._$script$query.get( name ) ;
        if( !isNumeric( query ) ) {
            return "" ;
        }
        if( !useString( sview ) ) {
            sview = "" ;
        }
        if( !useString( eview ) ) {
            eview = "" ;
        }
        var e = MasterTag._viewNameMaster( model,query ) ;
        if( e == null ) {
            var m = MasterCache.getCacheDB(model) ;
            var no = null ;
            if( isNumeric( query ) ) {
                no = parseInt( query ) ;
            }
            else {
                no = query ;
            }
            if( m != null ) {
                if( chain != null && typeof( chain ) == "map" && targetLine != null &&
                    typeof( targetLine ) == "map" ) {
                    e = MasterTag._notCacheById( model,no,vname,chain,targetLine ) ;
                }
                else {
                    e = m[ no ] ;
                }
            }
            if( isNull( e ) ) {
                e = MasterTag._notCacheById( model,no,vname,chain,targetLine ) ;
                if( e == null ) {
                    e = "" ;
                }
            }
            e = MasterTag._valueColumns( e,vname ) ;
        }
        return createStrBuf().
            ad( sview ).ad( convertViewHtml( e ) ).ad( eview ).ts() ;
    },
    // 指定パラメータ内容のマスター内容を出力.
    viewToParams : function(name,model,vname,chain,targetLine) {
        if( isNull( name ) || isNull( model ) ) {
            return "" ;
        }
        if( typeof( name ) == "string" && !useString( name ) ) {
            return "" ;
        }
        if( typeof( model ) == "map" && Models[model.name] == model ) {
            model = model.name ;
        }
        if( typeof( model ) != "string" ) {
            return "" ;
        }
        if( !useString( vname ) ) {
            vname = MasterTag._viewColumns( model ) ;
        }
        var query = super._$script$query.get( name ) ;
        var e = MasterTag._viewNameMaster( model,query ) ;
        if( e == null ) {
            var m = MasterCache.getCacheDB(model) ;
            var no = null ;
            if( isNumeric( query ) ) {
                no = parseInt( query ) ;
            }
            else {
                no = query ;
            }
            if( m != null ) {
                if( chain != null && typeof( chain ) == "map" && targetLine != null &&
                    typeof( targetLine ) == "map" ) {
                    e = MasterTag._notCacheById( model,no,vname,chain,targetLine ) ;
                }
                else {
                    e = m[ no ] ;
                }
            }
            if( isNull( e ) ) {
                e = MasterTag._notCacheById( model,no,vname,chain,targetLine ) ;
                if( e == null ) {
                    return "" ;
                }
            }
            e = MasterTag._valueColumns( e,vname ) ;
        }
        return convertViewHtml( e ) ;
    },
    // 指定IDのマスター内容を出力.
    viewToId : function(name,model,vname,chain,targetLine) {
        if( isNull( name ) || isNull( model ) ) {
            return "" ;
        }
        if( typeof( name ) == "string" && !useString( name ) ) {
            return "" ;
        }
        if( typeof( model ) == "map" && Models[model.name] == model ) {
            model = model.name ;
        }
        if( typeof( model ) != "string" ) {
            return "" ;
        }
        if( !useString( vname ) ) {
            vname = MasterTag._viewColumns( model ) ;
        }
        var e = MasterTag._viewNameMaster( model,name ) ;
        if( e == null ) {
            var m = MasterCache.getCacheDB(model) ;
            var no = null ;
            if( isNumeric( name ) ) {
                no = parseInt( name ) ;
            }
            else {
                no = name ;
            }
            if( m != null ) {
                if( chain != null && typeof( chain ) == "map" && targetLine != null &&
                    typeof( targetLine ) == "map" ) {
                    e = MasterTag._notCacheById( model,no,vname,chain,targetLine ) ;
                }
                else {
                    e = m[ no ] ;
                }
            }
            if( e == null ) {
                e = MasterTag._notCacheById( model,no,vname,chain,targetLine ) ;
                if( e == null ) {
                    return "" ;
                }
            }
            e = MasterTag._valueColumns( e,vname ) ;
        }
        return convertViewHtml( e ) ;
    },
    // マスターデータを用いて、Selectタグを生成.
    selectTag : function(name,model,vname,size,multiple,id,clazz,etc,blankFlag) {
        if( typeof( model ) == "map" && Models[model.name] == model ) {
            model = model.name ;
        }
        if( typeof( model ) != "string" ) {
            return "" ;
        }
        if( blankFlag == null ) {
            blankFlag = true ;
        }
        if( !useString( vname ) ) {
            vname = MasterTag._viewColumns( model ) ;
        }
        var m = MasterTag._selectNameMaster( model ) ;
        if( m == null ) {
            model = MasterCache.getCacheDBByList( model ) ;
            var m = [] ;
            var len = model.length ;
            for( var i = 0 ; i < len ; i ++ ) {
                c = model[ i ] ;
                m.add( c.id ) ;
                m.add( MasterTag._valueColumns( c,vname ) ) ;
            }
        }
        return selectTag( name,m,size,multiple,id,clazz,etc,blankFlag ) ;
    },
    // マスターデータを用いて、チェックボックスタグを生成.
    checkBoxTag : function( name,model,vname,parse,line,clazz,etc ) {
        if( typeof( model ) == "map" && Models[model.name] == model ) {
            model = model.name ;
        }
        if( typeof( model ) != "string" ) {
            return "" ;
        }
        if( !useString( vname ) ) {
            vname = MasterTag._viewColumns( model ) ;
        }
        var m = MasterTag._selectNameMaster( model ) ;
        if( m == null ) {
            model = MasterCache.getCacheDBByList( model ) ;
            var m = [] ;
            var len = model.length ;
            for( var i = 0 ; i < len ; i ++ ) {
                c = model[ i ] ;
                m.add( c.id ) ;
                m.add( MasterTag._valueColumns( c,vname ) ) ;
            }
        }
        return gCheckBoxTag( name,m,parse,line,clazz,etc ) ;
    },
    // マスターデータを用いて、ラジオボタンタグを生成.
    radioTag : function( name,model,vname,parse,line,clazz,etc ) {
        if( typeof( model ) == "map" && Models[model.name] == model ) {
            model = model.name ;
        }
        if( typeof( model ) != "string" ) {
            return "" ;
        }
        if( !useString( vname ) ) {
            vname = MasterTag._viewColumns( model ) ;
        }
        var m = MasterTag._selectNameMaster( model ) ;
        if( m == null ) {
            model = MasterCache.getCacheDBByList( model ) ;
            var m = [] ;
            var len = model.length ;
            for( var i = 0 ; i < len ; i ++ ) {
                c = model[ i ] ;
                m.add( c.id ) ;
                m.add( MasterTag._valueColumns( c,vname ) ) ;
            }
        }
        return gRadioTag( name,m,parse,line,clazz,etc ) ;
    },
    // 名前管理マスターの表示支援.
    _viewNameMaster : function(model,target) {
        var p ;
        if( ( p = model.indexOf( "." ) ) != -1 ) {
            var m = null ;
            var v = "" ;
            var n = model.substring( 0,p ) ;
            var t = model.substring( p+1 ) ;
            // モデル名が名前管理マスターの場合.
            if( MasterCache.isNamesMaster( n ) ) {
                // 名前管理マスターの子名設定が、グループ名指定の場合.
                if( t == "*" ) {
                    m = MasterCache.getNameManagerByGroupList( n ) ;
                    if( m != null ) {
                        var len = m.length ;
                        for( var i = 0 ; i < len ; i ++ ) {
                            if( m[ i ] == target ) {
                                v = target ;
                                break ;
                            }
                        }
                    }
                    return v ;
                }
                // 名前管理マスターの子名設定が行われている場合.
                var m = MasterCache.getCacheDBByList( n ) ;
                if( m != null ) {
                    var n = m[ t ] ;
                    if( n != null ) {
                        v = n[target] ;
                        if( v == null ) {
                            v = "" ;
                        }
                    }
                }
                return v ;
            }
        }
        // モデル名が名前管理マスターの場合.
        else if( MasterCache.isNamesMaster( model ) ) {
            var v = "" ;
            m = MasterCache.getNameManagerByGroupList( model ) ;
            if( m != null ) {
                var len = m.length ;
                for( var i = 0 ; i < len ; i ++ ) {
                    if( m[ i ] == target ) {
                        v = target ;
                        break ;
                    }
                }
            }
            return v ;
        }
        // 名前管理マスターでない場合.
        return null ;
    },
    // 名前管理マスターの選択支援.
    _selectNameMaster : function( model ) {
        var p ;
        if( ( p = model.indexOf( "." ) ) != -1 ) {
            var m = null ;
            var n = model.substring( 0,p ) ;
            var t = model.substring( p+1 ) ;
            if( MasterCache.isNamesMaster( n ) ) {
                if( t == "*" ) {
                    m = MasterCache.getNameManagerByGroupList( n ) ;
                    if( m != null ) {
                        return m ;
                    }
                    return null ;
                }
                var z = MasterCache.getCacheDB( n ) ;
                if( z != null ) {
                    m = z[ t ] ;
                }
                if( m != null ) {
                    return m ;
                }
            }
            return null ;
        }
        else if( MasterCache.isNamesMaster( model ) ) {
            m = MasterCache.getNameManagerByGroupList( model ) ;
            if( m != null ) {
                return m ;
            }
            return null ;
        }
        return null ;
    },
    // マスターキャッシュがOFF場合の条件取得.
    _notCacheById : function( model,id,vname,chain,targetLine ) {
        if( typeof( id ) == "string" && !useString( id ) ) {
            return null ;
        }
        model = Models[ model ] ;
        if( model != null ) {
            var res = null ;
            if( chain != null && typeof( chain ) == "map" && targetLine != null &&
                typeof( targetLine ) == "map" ) {
                var wlist = [ id ] ;
                var buf = new StrBuf() ;
                buf.ad( "where id=?" ) ;
                for( var k in chain ) {
                    buf.ad( " and " ).ad( DbUtil.convertJavaNameByDBName( chain[ k ] ) ).ad( "=?" ) ;
                    wlist.add( targetLine[ DbUtil.convertDBNameByJavaName( false,k ) ] ) ;
                }
                buf.ad( " " ) ;
                var sql = buf.ts() ;
                buf = null ;
                res = model.first( sql,wlist ) ;
            }
            else {
                res = model.first_by_id( id ) ;
            }
            if( res != null && res.length > 0 ) {
                res = res[ 0 ] ;
                if( res != null ) {
                    return res ;
                }
            }
        }
        return null ;
    },
    // Model定義の表示カラム名を取得.
    _viewColumns : function( model ) {
        var m = Models[model] ;
        if( m != null ) {
            var name = m.view_columns ;
            if( typeof( name ) == "array" ) {
                if( name.length == 1 ) {
                    name = name[ 0 ] ;
                }
            }
        }
        if( name == null || ( typeof( name ) == "string" && !useString( name ) ) ) {
            name = "name" ;
        }
        return name ;
    },
    // 指定表示カラム名の情報を取得.
    _valueColumns : function( value,cols ) {
        if( typeof( cols ) == "array" ) {
            var len = cols.length ;
            var buf = new StrBuf() ;
            for( var i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.ad( " " ) ;
                }
                buf.ad( value[ cols[ i ] ] ) ;
            }
            return buf.ts() ;
        }
        return value[ cols ] ;
    }
} ;

///////////////////////////////////////////////////////////////////////////////
// マスターメンテナンス関連.
///////////////////////////////////////////////////////////////////////////////

MaintenanceTag = {
    // マスターテーブル名一覧を取得.
    list : function( types,desc ) {
        if( desc == false || desc == "false" ) {
            desc = false ;
        }
        else {
            desc = true ;
        }
        var names = new org.maachang.util.ObjectArray() ;
        for( var k in Models ) {
            if( k.endsWith( "Master" ) ) {
                names.add( k ) ;
            }
        }
        var len = names.size() ;
        if( len <= 0 ) {
            return [] ;
        }
        names.sort() ;
        var ret = [] ;
        if( !isNull( types ) && typeof( types ) == "string" ) {
            types = [ types ] ;
        }
        if( isNull( types ) || typeof( types ) != "array" || types.length <= 0 ) {
            if( !desc ) {
                for( var i = 0 ; i < len ; i ++ ) {
                    ret.add( Models[ names.get( i ) ] ) ;
                }
            }
            else {
                for( var i = len-1 ; i >= 0 ; i -- ) {
                    ret.add( Models[ names.get( i ) ] ) ;
                }
            }
        }
        else {
            var model = null ;
            var lenJ = types.length ;
            if( !desc ) {
                for( var j = 0 ; j < lenJ ; j ++ ) {
                    for( var i = 0 ; i < len ; i ++ ) {
                        model = Models[ names.get( i ) ] ;
                        if( model.division == types[ j ] ) {
                            ret.add( Models[ names.get( i ) ] ) ;
                        }
                    }
                }
            }
            else {
                for( var j = 0 ; j < lenJ ; j ++ ) {
                    for( var i = len-1 ; i >= 0 ; i -- ) {
                        model = Models[ names.get( i ) ] ;
                        if( model.division == types[ j ] ) {
                            ret.add( Models[ names.get( i ) ] ) ;
                        }
                    }
                }
            }
        }
        return ret ;
    },
    // マスター編集が可能かチェック.
    isEdit : function( master ) {
        if( typeof( master ) == "string" ) {
            master = Models[master] ;
        }
        if( master == null ) {
            return false ;
        }
        return ( master.isMasterEdit != null && master.isMasterEdit == true ) ;
    },
    // マスター新規作成が可能かチェック.
    isCreate : function( master ) {
        if( typeof( master ) == "string" ) {
            master = Models[master] ;
        }
        if( master == null ) {
            return false ;
        }
        return ( master.isMasterCreate != null && master.isMasterCreate == true ) ;
    }
}

