// record実装.
//

////////////////////////////////////////////////////////////////////////////////
// 基本JavaObject定義.
////////////////////////////////////////////////////////////////////////////////
DbUtil = pkg("org.maachang.dao.dbms.DbUtil") ;
ExecutionDao = pkg("org.maachang.dao.ExecutionDao") ;
OperationModel = pkg( "org.maachang.seabass.httpd.engine.script.dao.OperationModel" ) ;
TableSort = pkg( "org.maachang.dao.dbms.TableSort" ) ;

////////////////////////////////////////////////////////////////////////////////
// データベースタイプ.
////////////////////////////////////////////////////////////////////////////////
SqlType = {
    Types : pkg( "java.sql.Types" ),
    typeOf: function( type ) {
        if( isNumeric( type ) == false ) {
            return "unknown" ;
        }
        switch( type ) {
            case this.Types.BOOLEAN :// Boolean.
            case this.Types.BIT :
            case this.Types.TINYINT :
            case this.Types.CHAR :
                return "boolean" ;
            case this.Types.SMALLINT :// Integer.
            case this.Types.INTEGER :
                return "integer" ;
            case this.Types.BIGINT :// Long.
                return "long" ;
            case this.Types.DOUBLE :// Double.
                return "double" ;
            case this.Types.FLOAT :// Float.
            case this.Types.REAL :// Float.
                return "float" ;
            case this.Types.VARCHAR :// String.
            case this.Types.LONGVARCHAR :
            case this.Types.DATALINK :// URL.
                return "string" ;
            case this.Types.DATE :// Date.
                return "date" ;
            case this.Types.TIME :// Time.
                return "time" ;
            case this.Types.TIMESTAMP :// Timestamp.
                return "timestamp" ;
            case this.Types.BINARY :// byte[].
            case this.Types.VARBINARY :
            case this.Types.LONGVARBINARY :
            case this.Types.BLOB :
                return "binary" ;
        }
        return "unknown" ;
    },
    toType: function( str ) {
        if( useString( str ) == false ) {
            return -1 ;
        }
        str = trim( str ).toLowerCase() ;
        if( str == "boolean" ) {
            return this.Types.BOOLEAN ;
        }
        if( str == "integer" ) {
            return this.Types.INTEGER ;
        }
        if( str == "long" ) {
            return this.Types.BIGINT ;
        }
        if( str == "double" ) {
            return this.Types.DOUBLE ;
        }
        if( str == "float" ) {
            return this.Types.FLOAT ;
        }
        if( str == "string" ) {
            return this.Types.VARCHAR ;
        }
        if( str == "date" ) {
            return this.Types.DATE ;
        }
        if( str == "time" ) {
            return this.Types.TIME ;
        }
        if( str == "timestamp" ) {
            return this.Types.TIMESTAMP ;
        }
        if( str == "binary" ) {
            return this.Types.BINARY ;
        }
        return -1 ;
    }
} ;

////////////////////////////////////////////////////////////////////////////////
// Record系.
////////////////////////////////////////////////////////////////////////////////

// モデル情報を取得.
script_model = function() {
    return _$script$base_model ;
} ;

// BaseModel生成用.
create_base_model = function() {
    // dummy-baseModelの場合は、生成されたことにする.
    if( _$script$base_model.isType() == false ) {
        return true ;
    }
    var pool = null ;
    if( isNull( ( pool = _$$seabass_manager.get( "DBMS_POOL" ) ) ) ) {
        return false ;
    }
    if( _$script$base_model.isCreate() == false ) {
        _$script$base_model.create( context,pool.getRecord() ) ;
    }
    return true ;
} ;

// ベースモデル情報を取得.
getBaseModel = function() {
    if( !_$script$base_model.isType() ) {
        throw new IOException( "モデル情報は利用可能ではありません" ) ;
    }
    if( !_$script$base_model.isCreate() ) {
        create_base_model() ;
        if( !_$script$base_model.isCreate() ) {
            throw new IOException( "コネクション情報の生成に失敗しました." ) ;
        }
    }
    return _$script$base_model ;
}

// レコードオブジェクトを取得.
getRecordObject = function() {
    return getBaseModel().getRecord() ;
}

// データベースコミット.
commit = function() {
    if( _$script$base_model.isCreate() ) {
        _$script$base_model.commit() ;
    }
} ;

// データベースロールバック.
rollback = function() {
    if( _$script$base_model.isCreate() ) {
        _$script$base_model.rollback() ;
    }
} ;

// select系SQL直接実行処理.
selectSQL = function( sql,params,offset,limit ) {
    if( !_$script$base_model.isCreate() ) {
        create_base_model() ;
        if( !_$script$base_model.isCreate() ) {
            return null ;
        }
    }
    if( sql == null || !useString( sql ) ) {
        return null ;
    }
    if( params == null ) {
        params = null ;
    }
    else if( typeof( params ) != "array" ) {
        params = [ params ] ;
    }
    if( offset == null || offset <= -1 ) {
        offset = -1 ;
    }
    if( limit == null || limit <= 0 ) {
        limit = -1 ;
    }
    return _$script$base_model.selectSQL( sql,params,offset,limit ) ;
}
ssql = selectSQL ;

////////////////////////////////////////////////////////////////////////////////
// Model-Sub定義.
////////////////////////////////////////////////////////////////////////////////

// サブModelメソッド群用オブジェクト.
SubModel = {} ;

// 予約カラム名.
SubModel.COLUMN_ID = DbUtil.convertDBNameByJavaName(
    false,ExecutionDao.SEQ_COLUMN ) ;
SubModel.COLUMN_CREATE_TIME = DbUtil.convertDBNameByJavaName(
    false,ExecutionDao.CREATE_TIME ) ;
SubModel.COLUMN_UPDATE_TIME = DbUtil.convertDBNameByJavaName(
    false,ExecutionDao.UPDATE_TIME ) ;
SubModel.COLUMN_OPTIMISTIC_LOCK = DbUtil.convertDBNameByJavaName(
    false,ExecutionDao.OPTIMISTIC_LOCK_COLUMN ) ;

// 楽観的ロックエラーパラメータ格納名.
SubModel.ERROR_OPTIMISTIC_LOCK = "~error_optimisticLock" ;

// 新規パラメータに対して、予約パラメータ情報をコピー.
SubModel.copyRemColumns = function( src,dest ) {
    if( src.containsKey( SubModel.COLUMN_ID ) ) {
        dest[SubModel.COLUMN_ID] = src[SubModel.COLUMN_ID] ;
    }
    if( src.containsKey( SubModel.COLUMN_CREATE_TIME ) ) {
        dest[SubModel.COLUMN_CREATE_TIME] = src[SubModel.COLUMN_CREATE_TIME] ;
    }
    if( src.containsKey( SubModel.COLUMN_OPTIMISTIC_LOCK ) ) {
        dest[SubModel.COLUMN_OPTIMISTIC_LOCK] = src[SubModel.COLUMN_OPTIMISTIC_LOCK] ;
    }
}

// 指定リストをListに変換.
SubModel.convertParamByList = function( params ) {
    if( isNull( params ) || params.length <= 0 ) {
        return null ;
    }
    t = valueof( params ) ;
    if( t == "array" ) {
        return params ;
    }
    else if( t == "object[]" ) {
        var ret = [] ;
        var len = params.length ;
        for( var i = 0 ; i < len ; i ++ ) {
            ret.add( params[ i ] ) ;
        }
        return ret ;
    }
    throw new java.io.IOException( "不正なパラメータです" ) ;
}

// joinキー群を取得.
SubModel.getModelByJoinKeys = function( map ) {
    if( isNull( map ) ) {
        return null ;
    }
    var join = map.get( "join" ) ;
    if( isNull( join ) || typeof( join ) != "map" ) {
        return null ;
    }
    return join ;
}

// 対象テーブルのSrcJoinキー群を取得.
SubModel.getModelBySrcJoinKeys = function( thisModel,map ) {
    var cnt = 0 ;
    var tables = map.get( "srcJoin" ) ;
    if( isNull( tables ) == false && tables.length > 0 ) {
        var len = tables.length ;
        var ret = {} ;
        for( var i = 0 ; i < len ; i ++ ) {
            var name = tables[i] ;
            var model = context.get( name ) ;
            if( isNull( model ) ) {
                continue ;
            }
            var join = model.join ;
            if( isNull( join ) == false ) {
                for( var k in join ) {
                    if( join[k] == thisModel ) {
                        // １対他外部結合条件は１テーブル内で１つだけ
                        // 取得対象とする.
                        ret.put( name,k ) ;
                        cnt ++ ;
                        break ;
                    }
                }
            }
        }
    }
    if( cnt <= 0 ) {
        return null ;
    }
    return ret ;
}

// １対他の外部結合条件を削除.
// findManyなどで取得された、データには、結合を示すカラム名($結合テーブル名)となります。
SubModel.removeManyColumn = function( map ) {
    if( typeof( map ) != "map" ) {
        return ;
    }
    var remList = [] ;
    for( var k in map ) {
        if( k.startsWith( "$" ) ) {
            remList = k ;
        }
    }
    var len = remList.length ;
    for( var i = 0 ; i < len ; i ++ ) {
        map.remove( remList[ i ] ) ;
    }
}

// order by条件を処理.
SubModel.orderBy = function( sort ) {
    if( useString( sort ) && ( sort = trim( sort ) ).length > 0 ) {
        if( !startsWith( sort.toLowerCase(),"order by" ) ) {
            sort = ( sort.startsWith( "$" ) || sort.startsWith( ":" ) ) ? sort.substring( 1 ) : sort ;
            sort = DbUtil.convertJavaNameByDBName( sort ) ;
            sort = "order by "+sort ;
        }
        else {
            var c = split( true,true,sort,", 　\t" ) ;
            var len = c.length ;
            var buf = new StrBuf() ;
            for( var i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.ad( "," ) ;
                }
                if( c[i].startsWith( "$" ) || c[i].startsWith( ":" ) ) {
                    buf.ad( DbUtil.convertJavaNameByDBName( c[i].substring( 1 ).trim() ) ) ;
                }
                else {
                    buf.ad( c[i] ) ;
                }
            }
            sort = buf.ts() ;
            buf = null ;
            c = null ;
        }
        return sort ;
    }
    return null ;
}

////////////////////////////////////////////////////////////////////////////////
// baseModel.read戻りオブジェクト.
////////////////////////////////////////////////////////////////////////////////
_resultReadLine = {
    prototype : {
        res : null,
        create : function( res ) {
            this.res = res ;
        },
        clear : function() {
            if( this.res != null ) {
                this.res.clear() ;
            }
            this.res = null ;
        },
        next : function() {
            if( this.res == null ) {
                return null ;
            }
            var ret = this.res.next() ;
            if( ret == null ) {
                return null ;
            }
            return ret ;
        },
        count : function() {
            if( this.res == null ) {
                return -1 ;
            }
            return this.res.count() ;
        },
        name : function() {
            if( this.res == null ) {
                return null ;
            }
            return this.res.getModelName() ;
        }
    }
}

////////////////////////////////////////////////////////////////////////////////
// テーブル問い合わせBean.
////////////////////////////////////////////////////////////////////////////////
SelectBean = {
    prototype : {
        buf : null,
        list : null,
        // コンストラクタ.
        _constructor : function() {
            this.create() ;
        },
        // 情報生成.
        create : function() {
            this.buf = new StrBuf() ;
            this.list = [] ;
        },
        // 情報クリア.
        clear : function() {
            this.buf = null ;
            this.list = null ;
        },
        // wordだけを追加.
        append : function( word ) {
            if( !useString( word ) ) {
                return ;
            }
            if( this.buf.length() > 0 ) {
                this.buf.ad( " and " ) ;
            }
            this.buf.ad( word ) ;
        },
        // 情報追加.
        add : function( word,value ) {
            if( !useString( word ) ) {
                return ;
            }
            if( this.buf.length() > 0 ) {
                this.buf.ad( " and " ) ;
            }
            this.buf.ad( word ) ;
            if( value != null ) {
                this.list.add( value ) ;
            }
        },
        // or情報追加.
        orStart : function( word,value ) {
            if( this.buf.length() > 0 ) {
                this.buf.ad( " and " ) ;
            }
            this.buf.ad( "(" ) ;
            this.buf.ad( word ) ;
            if( value != null ) {
                this.list.add( value ) ;
            }
        },
        // or情報追加.
        orEnd : function( word,value ) {
            if( this.buf.length() > 0 ) {
                this.buf.ad( " or " ) ;
            }
            this.buf.ad( word ) ;
            this.buf.ad( ")" ) ;
            if( value != null ) {
                this.list.add( value ) ;
            }
        },
        // or情報追加.
        or : function( word,value ) {
            if( this.buf.length() > 0 ) {
                this.buf.ad( " or " ) ;
            }
            this.buf.ad( word ) ;
            if( value != null ) {
                this.list.add( value ) ;
            }
        },
        // データ追加長を取得.
        length : function() {
            return this.buf.length() ;
        },
        // パラメータ取得.
        params : function() {
            return this.list ;
        },
        // SQL取得.
        sql : function() {
            return this.buf.ts() ;
        }
    }
}

////////////////////////////////////////////////////////////////////////////////
// テーブル操作.
////////////////////////////////////////////////////////////////////////////////
OperationTable = {
    // テーブル再構築.
    recreate : function( model ) {
        OperationModel.recreateTable( model ) ;
    },
    // テーブルExport.
    exportCsv : function( writer,model ) {
        var record = getRecordObject() ;
        return OperationModel.exportCsv( writer,record,model ) ;
    },
    // テーブルImport.
    importCsv : function( reader,model ) {
        var record = getRecordObject() ;
        return OperationModel.importCsv( model,record,reader ) ;
    }
}


////////////////////////////////////////////////////////////////////////////////
// 指定カラムをJSON変換.
////////////////////////////////////////////////////////////////////////////////
TableJSON = {
    // エンコード処理.
    encode : function( columns,target ) {
        if( columns == null || target == null ) {
            return target ;
        }
        var v = typeof( columns ) ;
        if( v == "string" ) {
            columns = [columns] ;
        }
        else if( v != "list" ) {
            return target ;
        }
        var v = typeof( target ) ;
        if( v == "map" ) {
            target = [target] ;
        }
        else if( v != "array" ) {
            return target ;
        }
        var len = target.length ;
        var lenJ = columns.length ;
        for( var i = 0 ; i < len ; i ++ ) {
            var m = target[ i ] ;
            if( typeof( m ) != "map" ) {
                continue ;
            }
            for( var j = 0 ; j < lenJ ; j ++ ) {
                var b = m[columns[ j ]] ;
                if( b != null ) {
                    m.put( columns[ j ],encodeJSON( b ) ) ;
                }
            }
        }
        return target ;
    },
    // デコード処理.
    decode : function( columns,target ) {
        if( columns == null || target == null ) {
            return target ;
        }
        var v = typeof( columns ) ;
        if( v == "string" ) {
            columns = [columns] ;
        }
        else if( v != "list" ) {
            return target ;
        }
        var v = typeof( target ) ;
        if( v == "map" ) {
            target = [target] ;
        }
        else if( v != "array" ) {
            return target ;
        }
        var len = target.length ;
        var lenJ = columns.length ;
        for( var i = 0 ; i < len ; i ++ ) {
            var m = target[ i ] ;
            if( typeof( m ) != "map" ) {
                continue ;
            }
            for( var j = 0 ; j < lenJ ; j ++ ) {
                var b = m[columns[ j ]] ;
                if( b != null ) {
                    m.put( columns[ j ],decodeJSON( b ) ) ;
                }
            }
        }
        return target ;
    }
}

////////////////////////////////////////////////////////////////////////////////
// record補助.
////////////////////////////////////////////////////////////////////////////////

// １つのデータ行を、Mapにコピーする.
copyRecord = function( map,recordBean ) {
    if( isNull( map ) || isNull( recordBean ) || typeof( recordBean ) != "map" ) {
        return false ;
    }
    for( var key in recordBean ) {
        var val = recordBean[ key ] ;
        if( isNull( val ) ) {
            map[ key ] = null ;
        }
        else if( key != SubModel.COLUMN_CREATE_TIME &&
            key != SubModel.COLUMN_UPDATE_TIME &&
            typeof( val ) == "date" ) {
            convertDateByMap( map,key,val ) ;
            map[ key ] = val ;
        }
        else {
            map[ key ] = val ;
        }
    }
    return true ;
}

// テーブルソートオブジェクトを取得.
createTableSort = function( value ) {
    if( typeof( value ) != "array" ) {
        return null ;
    }
    return new org.maachang.dao.dbms.TableSort( value ) ;
}

////////////////////////////////////////////////////////////////////////////////
// システムモニター系.
////////////////////////////////////////////////////////////////////////////////

// プーリングRDBMS最大値.
MSys.getDbMax = function(){
    return this.MSystem.getDbMax() ;
}
// プーリングRDBMSアクティブ値.
MSys.getDbActives = function(){
    return this.MSystem.getDbActives() ;
}
// プーリングRDBMSアイドル値.
MSys.getDbIdle = function(){
    return this.MSystem.getDbIdle() ;
}
