// (テーブル定義)検索タグ.
// 

// (アイテム)検索タグ.
SearchTag = {
    // 基本テキスト表示サイズ.
    DEFAULT_TEXT_SIZE:50,
    // 基本範囲検索テキスト表示サイズ.
    DEFAULT_RANGE_SIZE:21,
    // 検索アイテムヘッダ名.
    ITEM_HEAD:"_$search$_$_",
    POPUP_GROUP:"$search",
    // AZ表示リスト.
    AZ_LIST:[
        "ア","アイウエオ",
        "カ","カキクケコ",
        "サ","サシスセソ",
        "タ","タチツテト",
        "ナ","ナニヌネノ",
        "ハ","ハヒフヘホ",
        "マ","マミムメモ",
        "ヤ","ヤユヨ",
        "ラ","ラリルレロ",
        "ワ","ワヲン" ],
    // 半角入力判別.
    HANKAKU : function( value ) { return ( typeof( value ) == "string" && value.startsWith( "#" ) ) ; },
    // シンプル検索判別.
    SIMPLE : function( value ) { return ( typeof( value ) == "string" && value.trim().indexOf( "simple" ) != -1 ) ; },
    // シンプル検索判別.
    CHECK : function( value ) { return ( typeof( value ) == "string" && value.trim().indexOf( "check" ) != -1 ) ; },
    // カナ頭文字検索.
    AZ : function( value ) { return ( typeof( value ) == "string" && value.trim().indexOf( "a-z" ) != -1 ) ; },
    // (数値)範囲検索.
    RANGE : function( value ) { return ( typeof( value ) == "string" && value.trim() == "range" ) ; },
    // 日付検索.
    DATE : function( value ) { return ( typeof( value ) == "string" && value.trim().indexOf( "date" ) != -1 ) ; },
    // セレクトボックス検索.
    SELECT : function( value ) {
        if( typeof( value ) == "map" ) {
            for( var k in value ) {
                if( "select" == k.trim() ) {
                    return true ;
                }
            }
        }
        else if( typeof( value ) == "string" ) {
            return value.trim() == "select" ;
        }
        return false ;
    },
    // ポップアップ検索.
    POPUP : function( value ) {
        if( typeof( value ) == "map" ) {
            for( var k in value ) {
                if( "popup" == k.trim() ) {
                    return true ;
                }
            }
        }
        return false ;
    },
    // 計算実行で検索.
    CALC : function( em ) {
        type = em.type ;
        if( typeof( type ) == "map" ) {
            for( var k in type ) {
                if( "calc" == k.trim() ) {
                    return true ;
                }
            }
        }
        return false ;
    },
    // SeabassSearch名として取得.
    seabassSearchName : function( name ) {
        if( useString( name ) ) {
            if( name.startsWith( SearchTag.ITEM_HEAD ) ) {
                return name ;
            }
            return SearchTag.ITEM_HEAD + name ;
        }
        return "" ;
    },
    // SeabassSearchに必要なパラメータ群を取得.
    paramList : function( mode,addPms ) {
        var cnt = 0 ;
        var map = {} ;
        var columns = null ;
        var type = null ;
        var stypeLen = "_$stype.".length ;
        if( addPms != null && typeof( addPms ) == "map" ) {
            for( var k in addPms ) {
                map[ k ] = addPms[ k ] ;
            }
        }
        // パラメータから、検索条件入力枠に条件設定.
        for( var k in params ) {
            if( k.startsWith( "_$" ) ) {
                // テーブル定義の場合.
                // マスタータイプが存在する場合は、その条件も含む.
                if( k == "_$table" || k == "_$masterType" ) {
                    map[ k ] = params[ k ] ;
                    cnt ++ ;
                }
                // パラメータ定義の場合.
                else if( k.startsWith( "_$stype." ) ) {
                    columns = k.substring( stypeLen ) ;
                    type = params[ k ] ;
                    map[ k ] = params[ k ] ;
                    if( type == "range" ) {
                        map[ "_$skey_S." + columns ] = params[ "_$skey_S." + columns ] ;
                        map[ "_$skey_E." + columns ] = params[ "_$skey_E." + columns ] ;
                        cnt ++ ;
                    }
                    else if( type == "date" ) {
                        map[ "_$skey_Y." + columns ] = params[ "_$skey_Y." + columns ] ;
                        map[ "_$skey_M." + columns ] = params[ "_$skey_M." + columns ] ;
                        map[ "_$skey_D." + columns ] = params[ "_$skey_D." + columns ] ;
                        cnt ++ ;
                    }
                    else {
                        map[ "_$skey." + columns ] = kanaZen( params[ "_$skey." + columns ] ) ;
                        cnt ++ ;
                    }
                }
            }
        }
        if( !( mode == true || mode == "true" ) ) {
            if( params['_$off'] != null ) {
                map['_$off'] = params['_$off'] ;
            }
        }
        return map ;
    },
    // 検索リスト初期化処理.
    init : function() {
        var HEAD = SearchTag.ITEM_HEAD ;
        var columns = null ;
        var type = null ;
        var stypeLen = "_$stype.".length ;
        // パラメータから、検索条件入力枠に条件設定.
        var addPms = [] ;
        for( var k in params ) {
            if( k.startsWith( "_$" ) ) {
                // パラメータ定義の場合.
                if( k.startsWith( "_$stype." ) ) {
                    columns = k.substring( stypeLen ) ;
                    type = params[ k ] ;
                    if( type == "range" ) {
                        addPms.add( HEAD+columns+"_S" ) ;
                        addPms.add( params[ "_$skey_S." + columns ] ) ;
                        addPms.add( HEAD+columns+"_E" ) ;
                        addPms.add( params[ "_$skey_E." + columns ] ) ;
                    }
                    else {
                        addPms.add( HEAD+columns ) ;
                        addPms.add( kanaZen( params[ "_$skey." + columns ] ) ) ;
                    }
                }
            }
        }
        // 追加対象のパラメータが存在する場合は、処理する.
        var len = addPms.length ;
        for( var i = 0 ; i < len ; i += 2 ) {
            params[ addPms[ i ] ] = addPms[ i+1 ] ;
        }
        return "" ;
    },
    // 検索リストページに渡す共通パラメータ定義.
    chain : function() {
        map = SearchTag.paramList() ;
        // 検索パラメータが存在する場合は、デフォルトパラメータにセット.
        if( map.length > 0 ) {
            DefaultParams.create( map ) ;
        }
        return "" ;
    },
    // hidden処理.
    hidden : function() {
        // ※この命令を使う場合は、必ず[chain]メソッドを呼び出す必要がある.
        if( DefaultParams.empty() ) {
            return DefaultParams.hidden() ;
        }
        return "" ;
    },
    // 検索結果出力処理.
    hiddenScript : function() {
        // ※この命令を使う場合は、必ず[chain]メソッドを呼び出す必要がある.
        if( DefaultParams.empty() ) {
            var m = DefaultParams.map() ;
            var buf = new StrBuf() ;
            buf.ad( "<script>\n" ).
                ad( "_$searchList = [ " ).ad( encodeJSON( m ) ).ad( " ] ;" ).
                ad( "\n</script>\n" ) ;
            return buf.ts() ;
        }
        return "" ;
    },
    // 指定条件に対する検索表題を取得.
    title : function( value ) {
        if( typeof( value ) == "string" ) {
            value = value.toLowerCase() ;
            if( value == "simple" ) {
                return "検索" ;
            }
            else if( value == "#simple" ) {
                return "検索[ｶﾅ]" ;
            }
            else if( value == "a-z" ) {
                return "五十音順" ;
            }
            else if( value == "#a-z" ) {
                return "五十音順[ｶﾅ]" ;
            }
            else if( value == "range" ) {
                return "範囲選択" ;
            }
        }
        return "検索" ;
    },
    // 検索枠一覧名を取得.
    list : function( master ) {
        if( typeof( master ) == "string" ) {
            master = Models[master] ;
        }
        if( master == null ||  master._$detail$ == null ) {
            return [] ;
        }
        var nms = master._$detail$.maintenanceDef ;
        var em = master._$detail$.element ;
        var len = em.length ;
        var sval = null ;
        var ret = [] ;
        for( var i = 0 ; i < len ; i ++ ) {
            if( ( sval = em[i].search ) != null ) {
                if( typeof( sval ) == "array" ) {
                    var lenJ = sval.length ;
                    for( var j = 0 ; j < lenJ ; j ++ ) {
                        ret.add( nms[ i ] ) ;
                        ret.add( sval[ j ] ) ;
                    }
                }
                else {
                    ret.add( nms[ i ] ) ;
                    ret.add( sval ) ;
                }
            }
        }
        return ret ;
    },
    // 検索ボタンを表示.
    button : function( name,url,searchTable,callbackId,clazz,params,value,func ) {
        if( !useString( url ) ||
            !useString( callbackId ) ) {
            return "" ;
        }
        // 検索テーブルカラム名が存在しない場合は、_をセット.
        if( isNull( searchTable ) ) {
            searchTable = "_" ;
        }
        else if( typeof( searchTable ) == "map" || typeof( searchTable ) == "jsadapter" ) {
            searchTable = searchTable.name ;
        }
        if( !useString( clazz ) ) {
            clazz = "btn1_class" ;
        }
        if( !useString( value ) ) {
            value = "検&nbsp;&nbsp;索" ;
        }
        var buf = new StrBuf() ;
        buf.ad( "_$SSearch$_.search(\"" ).ad( url ).ad( "\",\"" ).ad( searchTable ).ad( "\",\"" ).
            ad( callbackId ).ad( "\",\"" ).ad( convertMapToGetParams( params ) ).ad( "\"" ) ;
        if( useString( func ) ) {
            buf.ad( "," ).ad( func ) ;
        }
        buf.ad( " )" ) ;
        return button( value,Event.onclick( buf.ts() ),name,null,clazz ) ;
    },
    // 検索データをクリア.
    clear : function( name,clazz,value ) {
        if( !useString( clazz ) ) {
            clazz = "btn1_class" ;
        }
        if( !useString( value ) ) {
            value = "クリア" ;
        }
        return button( value,Event.onclick( "_$SSearch$_.clear()" ),name,null,clazz ) ;
    },
    // ページング処理(Ajax).
    page : function( url,callbackId,pos,max,space,targetClazz,etcClazz,viewSize,vpage,checkFunc ) {
        if( !useString( url ) || max <= 0 ) {
            return "" ;
        }
        if( !useString( checkFunc ) ) {
            checkFunc = null ;
        }
        max = parseInt( max ) ;
        if( useString( targetClazz ) == false ) {
            targetClazz = "page_target_view" ;
        }
        if( useString( etcClazz ) == false ) {
            etcClazz = "page_link_view" ;
        }
        if( isNull( viewSize ) ) {
            viewSize = 10 ;
        }
        if( isNull( vpage ) ) {
            vpage = 13 ;
        }
        else if( ( vpage & 0x00000001 ) == 0 ) {
            vpage ++ ;
        }
        if( max <= viewSize ) {
            return "" ;
        }
        if( isNull( space ) ) {
            space = 1 ;
        }
        var targetPos = parseInt( pos / viewSize ) ;
        var len = parseInt( vpage ) ;
        var maxPos = parseInt( ( max / viewSize ) + ( ( max % viewSize != 0 ) ? 1 : 0 ) ) ;
        var startPos = 0 ;
        if( len < maxPos ) {
            var half = parseInt( vpage / 2 ) ;
            startPos = targetPos - half ;
            if( startPos <= 0 ) {
                startPos = 0 ;
            }
            if( startPos + len > maxPos ) {
                startPos = maxPos - len ;
            }
        }
        else {
            len = maxPos ;
        }
        var spaceStr = "" ;
        if( isNumeric( space ) ) {
            for( var i = 0 ; i < space ; i ++ ) {
                spaceStr += "&nbsp;" ;
            }
        }
        else {
            spaceStr = space ;
        }
        // Seabass検索パラメータを設定.
        var pms = SearchTag.paramList( true ) ;
        var buf = createStrBuf() ;
        startPos = parseInt( startPos ) ;
        var defParams = convertMapToGetParams( pms ) ;
        if( !useString( defParams ) ) {
            defParams = null ;
        }
        var func = "function(res){document.getElementById(\""+callbackId+"\").innerHTML=res;}" ;
        if( targetPos > 0 ) {
            var pms = ( defParams == null ) ? "_$off="+(targetPos-1) : createStrBuf().ad(defParams).ad("&").ad("_$off=").ad(targetPos-1).ts() ;
            buf.ad( "<span class=\"" ).ad( etcClazz ).ad( "\">" ) ;
            buf.ad( "<a href='javascript:void(0)' onclick='javascript:" ) ;
            if( checkFunc != null ) {
                buf.ad( "if( !" ).ad( checkFunc ).ad( "() ) { return false ; };" ) ;
            }
            buf.ad( "accessAjaxToJsExecution(null,\"").
                ad(url).ad("\",\"").ad(pms).ad("\",null,null,").ad(func).ad(");return false;'>" ) ;
            buf.ad( "＜＜" ).ad( "</a></span>&nbsp;&nbsp;" ) ;
        }
        else {
            buf.ad( "<span class=\"" ).ad( etcClazz ).ad( "\">" ) ;
            buf.ad( "<a href='javascript:void(0)'>＜＜" ).ad( "</a></span>&nbsp;&nbsp;" ) ;
        }
        for( var i = 0 ; i < len ; i ++ ) {
            if( i != 0 ) {
                buf.ad( spaceStr ) ;
            }
            var pageNo = ( startPos + i ) ;
            var pg = pageNo+1 ;
            pg = ( pg <= 9 ) ? createStrBuf().ad("0").ad(pg).ts() : ""+pg ;
            if( pageNo == targetPos ) {
                buf.ad( "<span class=\"" ).ad( targetClazz ).ad( "\">" ).ad(pg).ad( "</span>" ) ;
            }
            else {
                buf.ad( "<span class=\"" ).ad( etcClazz ).ad( "\">" ) ;
                var pms = ( defParams == null ) ? "_$off="+pageNo : createStrBuf().ad(defParams).ad("&").ad("_$off=").ad(pageNo).ts() ;
                buf.ad( "<a href='javascript:void(0)' onclick='javascript:" ) ;
                if( checkFunc != null ) {
                    buf.ad( "if( !" ).ad( checkFunc ).ad( "() ) { return false ; };" ) ;
                }
                buf.ad( "accessAjaxToJsExecution(null,\"").
                    ad(url).ad("\",\"").ad(pms).ad("\",null,null,").ad(func).ad(");return false;'>" ) ;
                buf.ad(pg).ad( "</a>" ).ad( "</span>" ) ;
            }
        }
        if( (targetPos+1) < maxPos ) {
            var pms = ( defParams == null ) ? "_$off="+(targetPos+1) : createStrBuf().ad(defParams).ad("&").ad("_$off=").ad(targetPos+1).ts() ;
            buf.ad( "<span class=\"" ).ad( etcClazz ).ad( "\">" ) ;
            buf.ad( "&nbsp;&nbsp;<a href='javascript:void(0)' onclick='javascript:" ) ;
            if( checkFunc != null ) {
                buf.ad( "if( !" ).ad( checkFunc ).ad( "() ) { return false ; };" ) ;
            }
            buf.ad( "accessAjaxToJsExecution(null,\"").
                ad(url).ad("\",\"").ad(pms).ad("\",null,null,").ad(func).ad(");return false;'>" ) ;
            buf.ad( "＞＞" ).ad( "</a></span>" ) ;
        }
        else {
            buf.ad( "&nbsp;&nbsp;<a href='javascript:void(0)'>＞＞" ).ad( "</a></span>" ) ;
        }
        var ret = buf.ts() ;
        buf = null ;
        return ret ;
    },
    // 再描画用Javascriptを生成.
    reviewJS : function( name,url,callbackId ) {
        if( !useString( url ) || !useString( callbackId ) ) {
            return "" ;
        }
        if( !useString( name ) ) {
            name = "review" ;
        }
        var offset = super.offset ;
        var pms = SearchTag.paramList( true ) ;
        var defParams = convertMapToGetParams( pms ) ;
        if( !useString( defParams ) ) {
            defParams = null ;
        }
        var pms = ( defParams == null ) ? "_$off="+(offset) : createStrBuf().ad(defParams).ad("&").ad("_$off=").ad(offset).ts() ;
        var func = "function(res){document.getElementById(\""+callbackId+"\").innerHTML=res;}" ;
        var buf = new StrBuf() ;
        buf.ad( name ).ad( " = function() {\n" ) ;
            buf.ad( "    " ) ;
            buf.ad( "accessAjaxToJsExecution(null,\"").
                ad(url).ad("\",\"").ad(pms).ad("\",null,null,").ad(func).ad(");\nreturn false;\n" ) ;
        buf.ad( "}\n" ) ;
    },
    // 検索処理が実行可能かチェック.
    useResult : function() {
        if( Models[ params['_$table' ] ] == null ) {
            return false ;
        }
        return true ;
    },
    // 検索パラメータ群を取得.
    getSearchParam : function() {
        var ret = {} ;
        // パラメータから、検索条件を生成.
        for( var k in params ) {
            if( k.startsWith( "_$stype." ) ) {
                var t = params[k] ;
                var n = k.substring( "_$stype.".length ) ;
                if( t == 'range' ) {
                    var s = params[ "_$skey_S." + n ] ;
                    var e = params[ "_$skey_E." + n ] ;
                    ret[n] = [ s,e ] ;
                }
                else if( t == 'date' ) {
                    var y = params[ "_$skey_Y." + n ] ;
                    var m = params[ "_$skey_M." + n ] ;
                    var d = params[ "_$skey_D." + n ] ;
                    if( !useString( y ) || !useString( m ) || !useString( d ) ||
                        !isNumeric( y ) || !isNumeric( m ) || !isNumeric( d ) ) {
                        // 日付はどれかが入っていない場合は、検索対象としない.
                        continue ;
                    }
                    var date = new JDate( y,m-1,d ) ;
                    ret[n] = new JDate( date.getTime()+((60*60*24*1000)-1) ) ;
                }
                else {
                    var v = params[ "_$skey." + n ] ;
                    ret[n] = v ;
                }
            }
        }
        return ret ;
    },
    // 検索結果を生成.
    result : function( length,offset,tableName ) {
        if( !isNumeric( length ) || length <= 0 ) {
            return {state:"error",count:0,result:[],message:"検索パラメータが足りない"} ;
        }
        if( !isNumeric( offset ) || offset <= 0 ) {
            offset = params['_$off'] ;
            if( !isNumeric( offset ) || offset <= 0 ) {
                offset = 0 ;
            }
        }
        var columns = null ;
        var value = null ;
        var kana = false ;
        var tel = false ;
        var stypeLen = "_$stype.".length ;
        var cnt = 0 ;
        var sql = new StrBuf() ;
        var values = [] ;
        var dateColumns = null ;
        // パラメータから、検索条件を生成.
        for( var k in params ) {
            if( k.startsWith( "_$" ) ) {
                // テーブル名.
                if( k == "_$table" ) {
                    if( params[k] != "_" ) {
                        tableName = params[k] ;
                    }
                }
                // テーブルタイプ.
                else if( k.startsWith( "_$stype." ) ) {
                    columns = k.substring( stypeLen ) ;
                    value = params[ k ] ;
                    // データ内容が不正.
                    if( !useString( value ) ) {
                        continue ;
                    }
                    kana = value.startsWith( "#" ) ;
                    if( kana ) {
                        value = value.substring( 1 ) ;
                    }
                    else {
                        tel = value.startsWith( "+" ) ;
                        if( tel ) {
                            value = value.substring( 1 ) ;
                        }
                    }
                    value = value.toLowerCase() ;
                    // 実行命令が、頭文字検索の場合.
                    if( value == "az" || value == "head" ) {
                        var v = trim( params[ "_$skey." + columns ] ) ;
                        if( !useString( v ) ) {
                            continue ;
                        }
                        if( cnt != 0 ) {
                            sql.ad( "and" ) ;
                        }
                        sql.ad( " " ).ad( columns ).ad( " like ?" ) ;
                        if( kana ) {
                            values.add( kanaHan( v ) + "%" ) ;
                        }
                        else if( tel ) {
                            values.add( SearchTag.telToNumber( v ) + "%" ) ;
                        }
                        else {
                            values.add( v + "%" ) ;
                        }
                    }
                    // 実行命令が、範囲検索の場合.
                    else if( value == "range" ) {
                        var s = trim( params[ "_$skey_S." + columns ] ) ;
                        var e = trim( params[ "_$skey_E." + columns ] ) ;
                        if( !useString( s ) || !useString( e ) ) {
                            if( !useString( s ) && !useString( e ) ) {
                                continue ;
                            }
                            else if( !useString( s ) ) {
                                if( cnt != 0 ) {
                                    sql.ad( "and" ) ;
                                }
                                sql.ad( " " ).ad( columns ).ad( " <= ? " ) ;
                                values.add( kanaHan( e ) ) ;
                            }
                            else {
                                if( cnt != 0 ) {
                                    sql.ad( "and" ) ;
                                }
                                sql.ad( " " ).ad( columns ).ad( " >= ? " ) ;
                                values.add( kanaHan( s ) ) ;
                            }
                        }
                        else {
                            if( cnt != 0 ) {
                                sql.ad( "and" ) ;
                            }
                            sql.ad( " " ).ad( columns ).ad( " >= ? and " ).ad( columns ).ad( " <= ? " ) ;
                            values.add( kanaHan( s ) ) ;
                            values.add( kanaHan( e ) ) ;
                        }
                    }
                    // 実行命令が、日付設定の場合.
                    else if( value == 'date' ) {
                        var y = trim( params[ "_$skey_Y." + columns ] ) ;
                        var m = trim( params[ "_$skey_M." + columns ] ) ;
                        var d = trim( params[ "_$skey_D." + columns ] ) ;
                        if( !useString( y ) || !useString( m ) || !useString( d ) ||
                            !isNumeric( y ) || !isNumeric( m ) || !isNumeric( d ) ) {
                            // 日付はどれかが入っていない場合は、検索対象としない.
                            continue ;
                        }
                        var date = new JDate( y,m-1,d ) ;
                        date = new JDate( date.getTime()+((60*60*24*1000)-1) ) ;
                        if( cnt != 0 ) {
                            sql.ad( "and" ) ;
                        }
                        sql.ad( " " ).ad( columns ).ad( "<=?" ) ;
                        values.add( date ) ;
                        dateColumns = columns ;
                    }
                    // 実行命令が、通常命令の場合.
                    else {
                        var v = trim( params[ "_$skey." + columns ] ) ;
                        if( !useString( v ) ) {
                            continue ;
                        }
                        if( cnt != 0 ) {
                            sql.ad( "and" ) ;
                        }
                        sql.ad( " " ).ad( columns ).ad( " = ? " ) ;
                        if( kana ) {
                            values.add( kanaHan( v ) ) ;
                        }
                        else if( tel ) {
                            values.add( SearchTag.telToNumber( v ) ) ;
                        }
                        else {
                            values.add( v ) ;
                        }
                    }
                    cnt ++ ;
                }
            }
        }
        // 検索条件が不正.
        if( tableName == null || !useString( tableName ) ) {
            return {state:"error",count:0,result:[],message:"テーブルパラメータが存在しない"} ;
        }
        // modelオブジェクトが存在しない場合.
        var model = Models[ tableName ] ;
        if( model == null ) {
            return {state:"error",count:0,result:[],message:"テーブル名が不正:" + tableName + "]"} ;
        }
        var ret = {} ;
        // 検索条件なし.
        if( cnt == 0 ) {
            if( useString( dateColumns ) ) {
                ret.result = model.limit( offset,length,null,null,dateColumns+" desc" ) ;
            }
            else {
                ret.result = model.limit( offset,length,null,null ) ;
            }
            ret.count = model.count() ;
            ret.state = "all" ;
            ret.message = "検索成功" ;
        }
        else {
            sql = sql.ts() ;
            if( useString( dateColumns ) ) {
                ret.result = model.limit( offset,length,sql,values,dateColumns+" desc" ) ;
            }
            else {
                ret.result = model.limit( offset,length,sql,values ) ;
            }
            ret.count = model.count( sql,values ) ;
            ret.state = "custom" ;
            ret.message = "検索成功" ;
        }
        return ret ;
    },
    // 検索データ初期値をセット.
    initParams : function( name,initInfo ) {
        if( initInfo != null ) {
            params[SearchTag.ITEM_HEAD+name] = initInfo ;
        }
    },
    // 検索データ初期化(date)をセット.
    initDate : function( name,year,month,day ) {
        if( isNumeric( year ) ) {
            params[SearchTag.ITEM_HEAD+name+"_YMDYear"] = year ;
        }
        if( isNumeric( month ) ) {
            params[SearchTag.ITEM_HEAD+name+"_YMDMonth"] = month ;
        }
        if( isNumeric( day ) ) {
             params[SearchTag.ITEM_HEAD+name+"_YMDDay"] = day ;
        }
    },
    // 検索データ初期化(range)をセット.
    initRange : function( name,s,e ) {
        if( useString( s ) ) {
            params[SearchTag.HEAD+name+"_S"] = s ;
        }
        if( useString( e ) ) {
            params[SearchTag.HEAD+name+"_E"] = e ;
        }
    },
    // シンプル(半角)検索.
    simple : function( name,option,hankaku,textMode,mode,telMode ) {
        if( !useString( name ) ) {
            return "" ;
        }
        if( isNull( option ) || typeof( option ) != "map" ) {
            var inputclazz = "text_class" ;
        }
        else {
            var inputclazz = option.input ;
            var btn1clazz = option.btn1 ;
            if( !useString( inputclazz ) ) {
                inputclazz = "text_class" ;
            }
        }
        if( hankaku == true || hankaku == "true" ) {
            hankaku = true ;
        }
        else {
            hankaku = false ;
        }
        if( telMode == true || telMode == "true" ) {
            telMode = true ;
        }
        else {
            telMode = false ;
        }
        if( mode == true || mode == "true" ) {
            mode = true ;
        }
        else {
            mode = false ;
        }
        var HEAD = SearchTag.ITEM_HEAD ;
        buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define(\"simple\",\"" ).ad( name ).ad( "\"," ).ad( hankaku ).ad( "," ).ad( telMode ).ad(" );" ).
                ad( "</script>\n" ) ;
        }
        if( hankaku ) {
            buf.ad( textTag( HEAD+name,null,inputclazz,SearchTag.DEFAULT_TEXT_SIZE,SearchTag.DEFAULT_TEXT_SIZE,null,TextMode.KATA,null ) ) ;
        }
        else {
            buf.ad( textTag( HEAD+name,null,inputclazz,SearchTag.DEFAULT_TEXT_SIZE,SearchTag.DEFAULT_TEXT_SIZE,null,textMode,null ) ) ;
        }
        return buf.ts() ;
    },
    // チェックボックス.
    check : function( name,mode ) {
        var HEAD = SearchTag.ITEM_HEAD ;
        buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define(\"check\",\"" ).ad( name ).ad( "\"," ).ad( false ).ad( " );" ).
                ad( "</script>\n" ) ;
        }
        return buf.ad( selectTag( HEAD+name,["","検索しない","true","ON","false","OFF"] ) ).ts() ;
    },
    // アカサタナ検索.
    az : function( name,option,hankaku,mode ) {
        if( !useString( name ) ) {
            return "" ;
        }
        var a_zView_List = SearchTag.AZ_LIST ;
        if( isNull( option ) || typeof( option ) != "map" ) {
            var inputclazz = "text_class" ;
            var btn1clazz = "bun1_class" ;
            var btn2clazz = "bun2_class" ;
        }
        else {
            var inputclazz = option.input ;
            var btn1clazz = option.btn1 ;
            var btn2clazz = option.btn2 ;
            if( !useString( inputclazz ) ) {
                inputclazz = "text_class" ;
            }
            if( !useString( btn1clazz ) ) {
                btn1clazz = "bun1_class" ;
            }
            if( !useString( btn2clazz ) ) {
                btn2clazz = "bun2_class" ;
            }
        }
        if( hankaku == true || hankaku == "true" ) {
            hankaku = true ;
        }
        else {
            hankaku = false ;
        }
        if( mode == true || mode == "true" ) {
            mode = true ;
        }
        else {
            mode = false ;
        }
        // 頭文字検索のパラメータが渡された場合.
        var HEAD = SearchTag.ITEM_HEAD ;
        var hd = trim( params[ "_$skey."+name ] ) ;
        var hdMode = false ;
        if( useString( hd ) && hd.length == 1 ) {
            hd = kanaZen( hd ) ;
            hdMode = true ;
        }
        var childId = "_$azChild_" + name + "_Id" ;
        buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define(\"az\",\"" ).ad( name ).ad( "\"," ).ad( hankaku ).ad( " );" ).
                ad( "</script>\n" ) ;
        }
        var len = a_zView_List.length ;
        for( var i = 0 ; i < len ; i += 2 ) {
            var c = a_zView_List[i+1] ;
            var chLen = c.length ;
            buf.ad( "\n<script>\n _$mm_az_$" ).ad( name ).ad( i ).ad( "$child=function(){" ).ad( "return '" ) ;
            for( var j = 0 ; j < chLen ; j ++ ) {
                buf.ad( "&nbsp;&nbsp;<input type=button class=\"" ).ad( btn2clazz ).ad( "\" value=\"" ).ad( c.substring( j,j+1 ) ).
                    ad( "\" onclick=javascript:" ).
                    ad( "$(\"" ).ad( HEAD ).ad( name ).ad( "\").value=\"" ).ad( c.substring( j,j+1 ) ).ad( "\";" ).
                    ad( " >" ) ;
            }
            buf.ad( "';};\n</script>" ).
                ad( "\n<input type=button class=\"" ).ad( btn1clazz ).ad( "\" value=\"" ).ad( a_zView_List[i] ).
                ad( "\" onclick=\"javascript:" ).
                    ad( "$('" ).ad( HEAD ).ad( name ).ad( "').value='" ).ad( a_zView_List[i] ).ad( "';" ).
                    ad( "$('" ).ad( childId ).ad( "').innerHTML=_$mm_az_$" ).
                    ad( name ).ad( i ).ad( "$child();" ).
                    ad( "return false;\">" ) ;
        }
        buf.ad( "\n<div id=\"" ).ad( childId ).ad( "\"></div>" ) ;
        buf.ad( "\n<div>" ).
            ad( textTag( HEAD+name,null,inputclazz,SearchTag.DEFAULT_TEXT_SIZE,SearchTag.DEFAULT_TEXT_SIZE,null,TextMode.KATA,null ) ).
            ad( "</div>" ) ;
        // 頭文字検索パラメータが有効な場合.
        if( hdMode ) {
            for( var i = 0 ; i < len ; i += 2 ) {
                if( a_zView_List[i+1].indexOf( hd ) != -1 ) {
                    buf.ad( "<script> $('" ).ad( childId ).ad( "').innerHTML=_$mm_az_$" ).
                    ad( name ).ad( i ).ad( "$child(); </script>" ) ;
                    break ;
                }
            }
        }
        return buf.ts() ;
    },
    // 範囲検索(数値のみ).
    range : function( name,option,mode ) {
        if( !useString( name ) ) {
            return "" ;
        }
        if( isNull( option ) || typeof( option ) != "map" ) {
            var inputclazz = "text_class" ;
        }
        else {
            var inputclazz = option.input ;
            if( !useString( inputclazz ) ) {
                inputclazz = "text_class" ;
            }
        }
        if( mode == true || mode == "true" ) {
            mode = true ;
        }
        else {
            mode = false ;
        }
        var HEAD = SearchTag.ITEM_HEAD ;
        var buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define(\"range\",\"" ).ad( name ).ad( "\"," ).ad( false ).ad( " );" ).
                ad( "</script>\n" ) ;
        }
        buf.ad( textTag( HEAD+name+"_S",null,inputclazz,SearchTag.DEFAULT_RANGE_SIZE,SearchTag.DEFAULT_RANGE_SIZE,null,TextMode.NUMBER,null ) ).
            ad( "&nbsp;～&nbsp;" ).
            ad( textTag( HEAD+name+"_E",null,inputclazz,SearchTag.DEFAULT_RANGE_SIZE,SearchTag.DEFAULT_RANGE_SIZE,null,TextMode.NUMBER,null ) ) ;
        return buf.ts() ;
    },
    // 日付検索.
    date : function( name,year,startYear,mode,initFlag ) {
        if( !useString( name ) ) {
            return "" ;
        }
        if( initFlag == true || initFlag == "true" ) {
            initFlag = true ;
        }
        else {
            initFlag = false ;
        }
        if( mode == true || mode == "true" ) {
            mode = true ;
        }
        else {
            mode = false ;
        }
        var HEAD = SearchTag.ITEM_HEAD ;
        var buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define(\"date\",\"" ).ad( name ).ad( "\"," ).ad( false ).ad( " );" ).
                ad( "</script>\n" ) ;
        }
        if( year == null || !isNumeric( year ) ) {
            year = 1 ;
        }
        if( startYear == null || !isNumeric( startYear ) ) {
            startYear = 0 ;
        }
        buf.ad( selectYMD( HEAD+name+"_YMD",year,initFlag,null,startYear ) ) ;
        return buf.ts() ;
    },
    // マスター指定のセレクト処理.
    select : function( name,masterInfo,mode ) {
        if( !useString( name ) || ( typeof( masterInfo ) != "map" && typeof( masterInfo ) != "jsadapter" ) ) {
            return "" ;
        }
        if( masterInfo.containsKey( "search" ) ) {
            masterInfo = masterInfo.search ;
            if( masterInfo == null ) {
                return "" ;
            }
        }
        if( !useString( masterInfo.view ) ) {
            var vname = "name" ;
        }
        else {
            var vname = masterInfo.view ;
        }
        var master = masterInfo.select ;
        if( !useString( master ) ) {
            masterInfo.master ;
        }
        if( !useString( master ) ) {
            return "" ;
        }
        if( mode == true || mode == "true" ) {
            mode = true ;
        }
        else {
            mode = false ;
        }
        var HEAD = SearchTag.ITEM_HEAD ;
        var buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define( \"select\",\"" ).ad( name ).ad( "\"," ).ad( false ).ad( " );" ).
                ad( "</script>\n" ) ;
        }
        buf.ad( MasterTag.selectTag( HEAD+name,master,vname ) ) ;
        return buf.ts() ;
    },
    // 配列指定のセレクト処理.
    uselect : function( name,masterInfo,mode ) {
        if( !useString( name ) || ( typeof( masterInfo ) != "map" && typeof( masterInfo ) != "jsadapter" ) ) {
            return "" ;
        }
        if( masterInfo.containsKey( "search" ) ) {
            masterInfo = masterInfo.search ;
            if( masterInfo == null ) {
                return "" ;
            }
        }
        var list = masterInfo.select ;
        if( typeof( list ) != "array" ) {
            return "" ;
        }
        if( mode == true || mode == "true" ) {
            mode = true ;
        }
        else {
            mode = false ;
        }
        var HEAD = SearchTag.ITEM_HEAD ;
        var buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define( \"select\",\"" ).ad( name ).ad( "\"," ).ad( false ).ad( " );" ).
                ad( "</script>\n" ) ;
        }
        buf.ad( selectTag( HEAD+name,list ) ) ;
        return buf.ts() ;
    },
    // popup画面表示での検索処理.
    popup : function( name,option,popupInfo,mode ) {
        if( !useString( name ) || ( typeof( popupInfo ) != "map" && typeof( popupInfo ) != "jsadapter" ) ) {
            return "" ;
        }
        if( popupInfo.containsKey( "search" ) ) {
            popupInfo = popupInfo.search ;
            if( popupInfo == null ) {
                return "" ;
            }
        }
        var popupOp = popupInfo ;
        if( popupInfo.containsKey( "popup" ) ) {
            popupInfo = popupInfo.popup ;
            if( typeof( popupInfo ) != "map" && typeof( popupInfo ) != "jsadapter" ) {
                return "" ;
            }
        }
        if( isNull( option ) || typeof( option ) != "map" ) {
            var inputclazz = "text_class" ;
            var btnclazz = "bun1_class" ;
        }
        else {
            var inputclazz = option.input ;
            if( !useString( inputclazz ) ) {
                inputclazz = "text_class" ;
            }
            var btnclazz = option.btn ;
            if( !useString( btnclazz ) ) {
                btnclazz = "bun1_class" ;
            }
        }
        if( mode == true || mode == "true" ) {
            mode = true ;
        }
        else {
            mode = false ;
            if( popupInfo.readOnly == true || popupInfo.readOnly == "true" ) {
                mode = true ;
            }
        }
        var popupParams = popupInfo.params ;
        var url = popupInfo.url ;
        var title = popupInfo.title ;
        var width = popupInfo.width ;
        var height = popupInfo.height ;
        var group = popupInfo.group ;
        if( !useString( title ) ) {
            title = "検索" ;
        }
        if( typeof( group ) != "string" || !useString( group ) ) {
            group = SearchTag.POPUP_GROUP ;
        }
        if( !isNumeric( width ) ) {
            width = -1 ;
        }
        if( !isNumeric( height ) ) {
            height = -1 ;
        }
        var HEAD = SearchTag.ITEM_HEAD ;
        if( typeof( popupParams ) == "map" ) {
            popupParams = convertMapToGetParams( popupParams ) ;
        }
        if( useString( popupParams ) ) {
            popupParams += "&_src="+HEAD+name+"&_$masterType="+params['_$masterType'] ;
        }
        else if( !useString( popupParams ) ) {
            popupParams = "_src="+HEAD+name+"&_$masterType="+params['_$masterType'] ;
        }
        var viewInfo = SearchTag.popupView( true,name,popupOp ) ;
        var buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define( \"popup\",\"" ).ad( name ).ad( "\"," ).ad( false ).ad( " );" ).
                ad( "</script>\n" ) ;
        }
        buf.ad( textTag( HEAD+name,null,inputclazz,SearchTag.DEFAULT_RANGE_SIZE,SearchTag.DEFAULT_RANGE_SIZE,null,TextMode.NUMBER,null,false ) ) ;
        buf.ad( "\n<script>\n _$mm_popup$" ).ad( name ).ad( "$btn_func = function() { " ).
            ad( "var pms=\"" ).ad( popupParams ).ad( "\"; " ).
            ad( "popup.ajax( \"" ).ad( url ).ad( "\",pms,\"" ).
                ad( title ).ad( "\"," ).ad( width ).ad( "," ).ad( height ).ad( ",\"" ).ad( group ).ad( "\",null," ).
                ad( "function( r ){" ).
                ad( "$(\"" ).ad( HEAD+name ).ad( "\").value=r;" ).
                ad( "} ); }\n</script>" ).
        ad( "\n&nbsp;<input type=button class=\"" ).ad( btnclazz ).ad( "\" value=\"索&nbsp;&nbsp;引\"" ).
            ad( " onclick='javascript:_$mm_popup$" ).ad( name ).ad( "$btn_func();' / >" ) ;
        buf.ad( "&nbsp;&nbsp;<span id=" ).ad( HEAD+name ).ad( "_ViewId" ).ad( ">" ).ad( viewInfo ).ad( "</span>" ) ;
        return buf.ts() ;
    },
    // 計算指定.
    calc : function( name,option,calcInfo,mode ) {
        if( !useString( name ) || ( typeof( calcInfo ) != "map" && typeof( calcInfo ) != "jsadapter" ) ) {
            return "" ;
        }
        if( calcInfo.containsKey( "search" ) ) {
            calcInfo = calcInfo.search ;
            if( calcInfo == null ) {
                return "" ;
            }
        }
        var calcOp = calcInfo ;
        if( calcInfo.containsKey( "calc" ) ) {
            calcInfo = calcInfo.calc ;
            if( typeof( calcInfo ) != "map" && typeof( calcInfo ) != "jsadapter" ) {
                return "" ;
            }
        }
        var exec = calcInfo.execution ;
        if( !useString( exec ) ) {
            exec = calcInfo.exec ;
            if( !useString( exec ) ) {
                return "" ;
            }
        }
        if( isNull( option ) || typeof( option ) != "map" ) {
            var inputclazz = "text_class" ;
            var btnclazz = "bun1_class" ;
        }
        else {
            var inputclazz = option.input ;
            if( !useString( inputclazz ) ) {
                inputclazz = "text_class" ;
            }
            var btnclazz = option.btn ;
            if( !useString( btnclazz ) ) {
                btnclazz = "bun1_class" ;
            }
        }
        if( mode == true || mode == "true" ) {
            mode = true ;
        }
        else {
            mode = false ;
            if( calcOp.readOnly == true || calcOp.readOnly == "true" ) {
                mode = true ;
            }
        }
        var buttonInfo = calcInfo.button ;
        if( !useString( buttonInfo ) ) {
            buttonInfo = "計&nbsp;&nbsp;算" ;
        }
        var type = TextMode.NUMBER ;
        if( calcOp.type != null ) {
            type = calcOp.type ;
        }
        var HEAD = SearchTag.ITEM_HEAD ;
        var buf = new StrBuf() ;
        if( mode ) {
            buf.ad( "<script>" ).
                ad( "_$SSearch$_.define( \"calc\",\"" ).ad( name ).ad( "\"," ).ad( false ).ad( " );" ).
                ad( "</script>\n" ) ;
        }
        buf.ad( textTag( HEAD+name,null,inputclazz,SearchTag.DEFAULT_RANGE_SIZE,SearchTag.DEFAULT_RANGE_SIZE,null,type,null,true ) ) ;
        buf.ad( "\n<script>\n _$mm_calc_i$" ).ad( name ).ad( "$btn_func = function()" ).ad( exec ).ad( "\n</script>\n" ) ;
        buf.ad( "\n&nbsp;<input type=button class=\"" ).ad( btnclazz ).ad( "\" value=\"" ).ad( buttonInfo ).ad( "\"" ).
            ad( " onclick='javascript:_$mm_calc_i$" ).ad( name ).ad( "$btn_func();' / >" ) ;
        return buf.ts() ;
    },
    // POPUP選択条件を取得.
    popupView : function( mode,name,popupOp ) {
        var HEAD = SearchTag.ITEM_HEAD ;
        if( mode ) {
            name = HEAD + name ;
        }
        var viewInfo = null ;
        if( useString( params[name] ) && popupOp['master'] != null ) {
            var master = Models[popupOp['master']] ;
            if( master != null ) {
                var vname = popupOp['vname'] ;
                if( !useString( vname ) ) {
                    vname = master.view_columns ;
                    if( vname == null ) {
                        vname = "name" ;
                    }
                    else if( typeof( vname ) == "array" && vname.length == 1 ) {
                        vname = vname[ 0 ] ;
                    }
                }
                if( master.isCacheMap ) {
                    var cs = MasterCache.getCacheDB( popupOp['master'] ) ;
                    var ret = cs[ params[name] ] ;
                    if( ret != null && ret.containsKey( vname ) ) {
                        viewInfo = ret[vname] ;
                    }
                }
                else {
                    var bean = new SelectBean() ;
                    var chain = popupOp['chain'] ;
                    if( chain != null ) {
                        if( mode ) {
                            for( var k in chain ) {
                                if( params.containsKey( HEAD+k ) ) {
                                    bean.add( chain[ k ]+"=?",params[ HEAD+k ] ) ;
                                }
                            }
                        }
                        else {
                            for( var k in chain ) {
                                if( params.containsKey( k ) ) {
                                    bean.add( chain[ k ]+"=?",params[ k ] ) ;
                                }
                            }
                        }
                    }
                    bean.add( "id=?",params[name] ) ;
                    var lst = master.first( bean.sql(),bean.params(),"id" ) ;
                    var len = lst.length ;
                    if( len > 0 ) {
                        lst = lst[ 0 ] ;
                        if( typeof( vname ) == "array" ) {
                            var b = new StrBuf() ;
                            var len = vname.length ;
                            for( var i = 0 ; i < len ; i ++ ) {
                                if( lst.containsKey( vname[ i ] ) ) {
                                    b.ad( lst[ vname[ i ] ] ) ;
                                }
                                viewInfo = b.ts() ;
                            }
                        }
                        else {
                            if( lst.containsKey( vname ) ) {
                                viewInfo = lst[vname] ;
                            }
                        }
                    }
                }
            }
        }
        return viewInfo ;
    },
    // 電話番号型式を数値のみの電話番号に変換.
    telToNumber : function( tel ) {
        if( typeof( tel ) == "string" && useString( tel ) ) {
            var s = tel.split( "-" ) ;
            var ret = new StrBuf() ;
            var len = s.length ;
            for( var i = 0 ; i < len ; i ++ ) {
                ret.ad( s[ i ] ) ;
            }
            return ret.ts() ;
        }
        return tel ;
    }
}

