// Javascript(client)で動作するValidateタグ.
// VTag-Frameworkと比べれた場合、以下の点が行えない.
// ・ServerでのValidateTag継承は非対応.
//
// 利点としては、VTag-Frameworkの欠点である、URL単位でのValidate条件を
// 保持する点(静的)の問題がある.
// TLTag-Frameworkの場合は、このような問題点はなく、URL単位が１つの条件で
// 動的に内容が変わるので、たとえばMaster-Maintenance-Framework
// のような場合に利用できる.
// 

VLTag = {
    // エラーメッセージ出力先.
    outError : function( name,id,no ) {
        if( useString( name ) ) {
            if( !useString( id ) ) {
                id = getId( name,no ) ;
            }
            return errorTag( id ) ;
        }
        return "" ;
    },
    // Validate-Lite情報を出力.
    output : function() {
        var script = BaseVLTag.output() ;
        if( useString( script ) ) {
            return new StrBuf().ad( "\n<script language=\"javascript\">\n" ).
                ad( "executionVL = function( name ) {\n" ).
                ad( "validate_clear_all_message() ;\n" ).
                ad( "if( $base.useString( name ) ) {\n" ).
                ad( "switch( name ){\n" ).
                ad( script ).
                ad( "\n}}}\n" ).
                ad( "</script>\n" ).ts() ;
        }
        return "" ;
    },
    // 開始フォーム.
    startForm : function( action,name,method,target,etc ) {
        var ret = startForm( action,name,method,target,etc ) ;
        if( !useString( name ) ) {
            v = UnknownFormCache.get() ;
            name = "uform_name_" + v ;
        }
        BaseVLTag._startForm( name ) ;
        return ret ;
    },
    // 開始MultiPartフォーム.
    multiForm : function( action,name,target,etc ) {
        var ret = multiForm( action,name,target,etc ) ;
        if( !useString( name ) ) {
            v = UnknownFormCache.get() ;
            name = "uform_name_" + v ;
        }
        BaseVLTag._startForm( name ) ;
        return ret ;
    },
    // 終了フォーム.
    endForm : function( mode ) {
        var ret = endForm( mode ) ;
        BaseVLTag._endForm() ;
        return ret ;
    },
    // submitボタン.
    submit : function( formName,buttonName,id,clazz,message ) {
        if( !useString( formName ) ) {
            formName = FormNameCache.get() ;
        }
        if( useString( message ) ) {
            Event.onclick( "validate_lite_submit_conf(\"" + formName + "\",\"" + message + "\")" ) ;
        }
        else {
            Event.onclick( "validate_lite_submit(\""+formName+"\");" ) ;
        }
        return button( buttonName,null,null,id,clazz ) ;
    },
    // 画像submitボタン.
    imgSubmit : function( formName,name,src,alt,border,width,height,cmode,id,clazz,message ) {
        if( !useString( formName ) ) {
            formName = FormNameCache.get() ;
        }
        if( useString( message ) ) {
            Event.onclick( "validate_lite_submit_conf(\"" + formName + "\",\"" + message + "\")" ) ;
        }
        else {
            Event.onclick( "validate_lite_submit(\""+formName+"\");" ) ;
        }
        return imgButton( name,src,alt,border,width,height,null,cmode,id,clazz ) ;
    },
    // 確認ダイアログsubmitボタン.
    submitByConfirmation : function( formName,message,buttonName,id,clazz ) {
        if( !useString( formName ) ) {
            formName = FormNameCache.get() ;
        }
        Event.onclick( "validate_lite_submit_conf(\"" + formName + "\",\"" + message + "\")" ) ;
        return button( buttonName,null,null,id,clazz ) ;
    },
    // 確認ダイアログ画像submitボタン.
    imgSubmitByConfirmation : function( formName,message,name,src,alt,border,width,height,cmode,id,clazz ) {
        if( !useString( formName ) ) {
            formName = FormNameCache.get() ;
        }
        Event.onclick( "validate_lite_submit_conf(\"" + formName + "\",\"" + message + "\")" ) ;
        return imgButton( name,src,alt,border,width,height,null,cmode,id,clazz ) ;
    },
    // 必須入力チェック.
    toRequired : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_no_use",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // 汎用文字列チェック.
    toStringTag : function( message,args,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_string",args,name,message,id,no ) ;
        }
        return "" ;
    },
    // 数値チェック.
    toNumberTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_string",STR_NUMBER,name,message,id,no ) ;
        }
        return "" ;
    },
    // アルファベットチェック.
    toAlphabetTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_string",STR_ALPHABET,name,message,id,no ) ;
        }
        return "" ;
    },
    // ASCIIチェック.
    toAsciiTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_string",STR_ASCII,name,message,id,no ) ;
        }
        return "" ;
    },
    // カナチェック.
    toKanaTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_string",STR_KANA,name,message,id,no ) ;
        }
        return "" ;
    },
    // ひらがなチェック.
    toHiraTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_string",STR_HIRA,name,message,id,no ) ;
        }
        return "" ;
    },
    // 正規表現チェック.
    toMatchTag : function( message,match,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_match",match,name,message,id,no ) ;
        }
        return "" ;
    },
    // 日付フォーマットチェック.
    toDateTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_date",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // 時間フォーマットチェック.
    toTimeTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_time",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // 郵便番号チェック.
    toZipTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_zip",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // 電話番号チェック.
    toTelTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_tel",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // ISBN番号チェック.
    toIsbnTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_isbn",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // IPアドレスチェック.
    toIpTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_ip",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // URLチェック.
    toUrlTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_url",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // EMailチェック.
    toEmailTag : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_email",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // 文字列長最小値判別.
    toMinString : function( message,size,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_min",size,name,message,id,no ) ;
        }
        return "" ;
    },
    // 文字列長最大値判別.
    toMaxString : function( message,size,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_max",size,name,message,id,no ) ;
        }
        return "" ;
    },
    // 文字列長範囲判別.
    toMinMaxString : function( message,min,max,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_minMax",[min,max],name,message,id,no ) ;
        }
        return "" ;
    },
    // 指定日付が現在時間より前かチェック.
    toNowDate : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validagte_nowDate",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // 指定日付が、対象名の内容より前かチェック.
    toTagetDate : function( message,target,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_dateBySrcDest",name,target,message,id,no ) ;
        }
        return "" ;
    },
    // 画像拡張子判別.
    toImage : function( message,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_image",null,name,message,id,no ) ;
        }
        return "" ;
    },
    // 拡張子判別.
    toPlus : function( message,plus,name,id,no ) {
        if( useString( name ) ) {
            BaseVLTag.addValidate( "validate_fileNamePlus",plus,name,message,id,no ) ;
        }
        return "" ;
    }
}
