/**
 * ユーザ管理情報処理.
 * (client用).
 */

// ユーザ管理情報アクセスAPI(http).
UserManagerAPI = {
    USERMANAGER_SERVICE : pkg( "org.maachang.userManager.client.UserManagerService" ),
    // ログイン処理.
    login : function( host,user,passwd ) {
        // 成功の場合、新しいセッションIDが返される.
        var v = this.USERMANAGER_SERVICE.login( host,user,passwd ) ;
        var ret = decodeJSON( v[1] ) ;
        ret["session"] = v[0] ;
        return ret ;
    },
    // ログアウト処理.
    logout : function( host,session,user ) {
        return this.USERMANAGER_SERVICE.logout( host,session,user ) ;
    },
    // ユーザ情報を取得.
    userInfo : function( host,session,user ) {
        var ret = this.USERMANAGER_SERVICE.getUserInfo( host,session,user ) ;
        return decodeJSON( ret ) ;
    },
    // ログイン情報更新処理.
    update : function( host,session,user ) {
        return this.USERMANAGER_SERVICE.update( host,session,user ) ;
    },
    // セッション存在チェック.
    isUse : function( host,session ) {
        return this.USERMANAGER_SERVICE.use( host,session ) ;
    },
    // 権限マスターを取得.
    authorityMaster : function( host ) {
        var ret = this.USERMANAGER_SERVICE.getAuthorityMaster( host ) ;
        return decodeJSON( ret ) ;
    },
    // 会社マスターを取得.
    companyMaster : function( host ) {
        var ret = this.USERMANAGER_SERVICE.getCompanyMaster( host ) ;
        return decodeJSON( ret ) ;
    },
    // 部署マスターを取得.
    partMaster : function( host ) {
        var ret = this.USERMANAGER_SERVICE.getPartMaster( host ) ;
        return decodeJSON( ret ) ;
    },
    // 社員マスターを取得.
    personMaster : function( host ) {
        var ret = this.USERMANAGER_SERVICE.getPersonMaster( host ) ;
        return decodeJSON( ret ) ;
    }
}

// ユーザ管理情報アクセスAPI(seabass32).
UserManager32API = {
    USERMANAGER_SERVICE : pkg( "org.maachang.userManager.client.UserManagerService32" ),
    BATCH_RAND : new org.maachang.util.atomic.AtomicXor128Random(),
    // ログイン処理.
    login : function( host,user,passwd ) {
        // 成功の場合、新しいセッションIDが返される.
        if( isRequest() ) {
            var r = getRequest() ;
            var v = this.USERMANAGER_SERVICE.login( r.getRandom(),r.getSession().getSessionId(),host,user,passwd ) ;
            var ret = decodeJSON( v[1] ) ;
            ret["session"] = v[0] ;
            return ret ;
        }
        return null ;
    },
    // ログイン処理(Batch処理).
    loginBatch : function( sessionId,host,user,passwd ) {
        // 成功の場合、新しいセッションIDが返される.
        return this.USERMANAGER_SERVICE.login( BATCH_RAND,sessionId,host,user,passwd ) ;
    },
    // ログアウト処理.
    logout : function( host,session,user ) {
        if( isRequest() ) {
            var r = getRequest() ;
            var v = this.USERMANAGER_SERVICE.logout( r.getRandom(),r.getSession().getSessionId(),host,session,user ) ;
            var ret = decodeJSON( v[1] ) ;
            ret["session"] = v[0] ;
            return ret ;
        }
        return null ;
    },
    // ログアウト処理(Batch処理).
    logoutBatch : function( host,session,user ) {
        return this.USERMANAGER_SERVICE.logout( BATCH_RAND,session,host,session,user ) ;
    },
    // ユーザ情報を取得.
    userInfo : function( host,session,user ) {
        if( isRequest() ) {
            var r = getRequest() ;
            var ret = this.USERMANAGER_SERVICE.getUserInfo( r.getRandom(),r.getSession().getSessionId(),host,session,user ) ;
            return decodeJSON( ret ) ;
        }
        return null ;
    },
    // ユーザ情報を取得(Batch処理).
    userInfoBatch : function( host,session,user ) {
        var ret = this.USERMANAGER_SERVICE.getUserInfo( BATCH_RAND,session,host,session,user ) ;
        return decodeJSON( ret ) ;
    },
    // ログイン情報更新処理.
    update : function( host,session,user ) {
        if( isRequest() ) {
            var r = getRequest() ;
            return this.USERMANAGER_SERVICE.update( r.getRandom(),r.getSession().getSessionId(),host,session,user ) ;
        }
        return false ;
    },
    // ログイン情報更新処理(Batch処理).
    updateBatch : function( host,session,user ) {
        return this.USERMANAGER_SERVICE.update( BATCH_RAND,session,host,session,user ) ;
    },
    // セッション存在チェック.
    isUse : function( host,session ) {
        if( isRequest() ) {
            var r = getRequest() ;
            return this.USERMANAGER_SERVICE.use( r.getRandom(),r.getSession().getSessionId(),host,session ) ;
        }
        return false ;
    },
    // セッション存在チェック(Batch処理).
    isUseBatch : function( host,session ) {
        return this.USERMANAGER_SERVICE.use( BATCH_RAND,session,host,session ) ;
    },
    // 権限マスターを取得.
    authorityMaster : function( host ) {
        if( isRequest() ) {
            var r = getRequest() ;
            var ret = this.USERMANAGER_SERVICE.getAuthorityMaster( r.getRandom(),r.getSession().getSessionId(),host ) ;
            return decodeJSON( ret ) ;
        }
        return null ;
    },
    // 権限マスターを取得(Batch処理).
    authorityMasterBatch : function( session,host ) {
        var ret = this.USERMANAGER_SERVICE.getAuthorityMaster( BATCH_RAND,session,host ) ;
        return decodeJSON( ret ) ;
    },
    // 会社マスターを取得.
    companyMaster : function( host ) {
        if( isRequest() ) {
            var r = getRequest() ;
            var ret = this.USERMANAGER_SERVICE.getCompanyMaster( r.getRandom(),r.getSession().getSessionId(),host ) ;
            return decodeJSON( ret ) ;
        }
        return null ;
    },
    // 会社マスターを取得(Batch処理).
    companyMasterBatch : function( session,host ) {
        var ret = this.USERMANAGER_SERVICE.getCompanyMaster( BATCH_RAND,session,host ) ;
        return decodeJSON( ret ) ;
    },
    // 部署マスターを取得.
    partMaster : function( host ) {
        if( isRequest() ) {
            var r = getRequest() ;
            var ret = this.USERMANAGER_SERVICE.getPartMaster( r.getRandom(),r.getSession().getSessionId(),host ) ;
            return decodeJSON( ret ) ;
        }
        return null ;
    },
    // 部署マスターを取得(Batch処理).
    partMasterBatch : function( session,host ) {
        var ret = this.USERMANAGER_SERVICE.getPartMaster( BATCH_RAND,session,host ) ;
        return decodeJSON( ret ) ;
    },
    // 社員マスターを取得.
    personMaster : function( host ) {
        if( isRequest() ) {
            var r = getRequest() ;
            var ret = this.USERMANAGER_SERVICE.getPersonMaster( r.getRandom(),r.getSession().getSessionId(),host ) ;
            return decodeJSON( ret ) ;
        }
        return null ;
    },
    // 社員マスターを取得(Batch処理).
    personMasterBatch : function( session,host ) {
        var ret = this.USERMANAGER_SERVICE.getPersonMaster( BATCH_RAND,session,host ) ;
        return decodeJSON( ret ) ;
    }
}

