package org.maachang.userManager.server ;

import java.io.IOException;

import org.maachang.util.ConvertParam;

/**
 * １つのユーザを管理するオブジェクト.
 *
 * @version 2010/01/11
 * @author masahito suzuki
 * @since UserManager 1.0.0
 */
public class UserBean {
    
    /** format **
     * 0-7(8) : lastAccessTime
     * 8-15(8) : count
     * 16-16(1) : userNameLength
     * 17-63(47) : userName
     **/
    
    /** データ長 **/
    private static final int BINARY_LENGTH = 64 ;
    
    /** 最終アクセスタイム. **/
    private long lastAccess ;
    
    /** アクセスカウント. **/
    private long count ;
    
    /** ログインユーザ名 **/
    private String userName ;
    
    /** コンストラクタ. **/
    public UserBean() {
        
    }
    
    /**
     * コンストラクタ.
     * @param name ログインユーザ名を設定します.
     */
    public UserBean( String name ) {
        this.create( name ) ;
    }
    
    /**
     * コンストラクタ.
     * @param binary 復元対象のオブジェクトを設定します.
     * @exception Exception 例外.
     */
    public UserBean( byte[] binary ) throws Exception {
        this.create( binary ) ;
    }
    
    /**
     * 情報生成.
     * @param name ログインユーザ名を設定します.
     */
    public void create( String name ) {
        if( name == null || name.length() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        lastAccess = System.currentTimeMillis() ;
        count = 1L ;
        userName = name ;
    }
    
    /**
     * 情報生成.
     * @param binary 復元対象のオブジェクトを設定します.
     * @exception Exception 例外.
     */
    public void create( byte[] binary ) throws Exception {
        if( binary == null || binary.length != BINARY_LENGTH ) {
            throw new IOException( "不正なバイナリ" ) ;
        }
        int c = 0 ;
        long a = ConvertParam.convertLong( c,binary ) ;
        c += 8 ;
        long b = ConvertParam.convertLong( c,binary ) ;
        c += 8 ;
        int m = (int)binary[ c ] ;
        c ++ ;
        String n = new String( binary,c,m,"UTF8" ) ;
        this.lastAccess = a ;
        this.count = b ;
        this.userName = n ;
    }
    
    /**
     * 情報クリア.
     */
    public void clear() {
        lastAccess = -1L ;
        count = -1L ;
        userName = null ;
    }
    
    /**
     * 更新処理.
     */
    public void update() {
        lastAccess = System.currentTimeMillis() ;
        count ++ ;
    }
    
    /**
     * 最終アクセス時間を取得.
     * @return long 最終アクセス時間が返されます.
     */
    public long getLastAccess() {
        return lastAccess ;
    }
    
    /**
     * アクセスカウントを取得.
     * @return long アクセスカウントが返されます.
     */
    public long getCount() {
        return count ;
    }
    
    /**
     * ログインユーザ名を取得.
     * @return String ログインユーザ名が返されます.
     */
    public String getUserName() {
        return userName ;
    }
    
    /**
     * バイナリに変換.
     * @return byte[] バイナリ情報が返されます.
     * @exception Exception 例外.
     */
    public byte[] toBinary() throws Exception {
        if( userName == null ) {
            throw new IOException( "このオブジェクト内容は無効です" ) ;
        }
        byte[] b = userName.getBytes( "UTF8" ) ;
        int len = b.length ;
        byte[] ret = new byte[ BINARY_LENGTH ] ;
        int c = 0 ;
        ConvertParam.convertLong( ret,c,lastAccess ) ;
        c += 8 ;
        ConvertParam.convertLong( ret,c,count ) ;
        c += 8 ;
        ret[ c ] = ( byte )len ;
        c ++ ;
        System.arraycopy( b,0,ret,c,len ) ;
        return ret ;
    }
}
