// キャッシュオブジェクト.
$cache = {
    Low: 1,
    Normal: 2,
    High: 4,
    prototype : {
        create : function( maxSize ) {
            this.items = {};
            this.count = 0;
            if (maxSize == null)
                maxSize = -1;
            this.maxSize = maxSize;
            this.fillFactor = .75;
            this.purgeSize = Math.round(this.maxSize * this.fillFactor);
            this.stats = {}
            this.stats.hits = 0;
            this.stats.misses = 0;
        },
        clear : function() {
            for (var key in this.items) {
              this._removeItem(key);
            }  
        },
        getItem : function(key) {
            var item = this.items[key];
            if (item != null) {
                if (!this._isExpired(item)) {
                    item.lastAccessed = new Date().getTime();
                } else {
                    this._removeItem(key);
                    item = null;
                }
            }
            var returnVal = null;
            if (item != null) {
                returnVal = item.value;
                this.stats.hits++;
            } else {
                this.stats.misses++;
            }
            return returnVal;
        },
        setItem : function(key, value, options) {
            function CacheItem(k, v, o) {
                if ((k == null) || (k == ''))
                    throw new Error("key cannot be null or empty");
                this.key = k;
                this.value = v;
                if (o == null)
                    o = {};
                if (o.expirationAbsolute != null)
                    o.expirationAbsolute = o.expirationAbsolute.getTime();
                if (o.priority == null)
                    o.priority = $cache.Normal;
                this.options = o;
                this.lastAccessed = new Date().getTime();
            }
            if (this.items[key] != null)
                this._removeItem(key);
            this._addItem(new CacheItem(key, value, options));
            if ((this.maxSize > 0) && (this.count > this.maxSize)) {
                this._purge();
            }
        },
        _purge : function() {
            var tmparray = new Array();
            for (var key in this.items) {
                var item = this.items[key];
                if (this._isExpired(item)) {
                    this._removeItem(key);
                } else {
                    tmparray.push(item);
                }
            }
            if (tmparray.length > this.purgeSize) {
                tmparray = tmparray.sort(function(a, b) { 
                    if (a.options.priority != b.options.priority) {
                        return b.options.priority - a.options.priority;
                    } else {
                        return b.lastAccessed - a.lastAccessed;
                    }
                });
                while (tmparray.length > this.purgeSize) {
                    var ritem = tmparray.pop();
                    this._removeItem(ritem.key);
                }
            }
        },
        _addItem : function(item) {
            this.items[item.key] = item;
            this.count++;
        },
        _removeItem : function(key) {
            var item = this.items[key];
            delete this.items[key];
            this.count--;
            if (item.options.callback != null) {
                var callback = function() {
                    item.options.callback(item.key, item.value);
                }
                setTimeout(callback, 0);
            }
        },
        _isExpired : function(item) {
            var now = new Date().getTime();
            var expired = false;
            if ((item.options.expirationAbsolute) && (item.options.expirationAbsolute < now)) {
                expired = true;
            } 
            if ((expired == false) && (item.options.expirationSliding)) {
                var lastAccess = item.lastAccessed + (item.options.expirationSliding * 1000);
                if (lastAccess < now) {
                    expired = true;
                }
            }
            return expired;
        },
        toHtmlString : function() {
            var returnStr = this.count + " item(s) in cache<br /><ul>";
            for (var key in this.items) {
                var item = this.items[key];
                returnStr = returnStr + "<li>" + item.key.toString() + " = " + item.value.toString() + "</li>";
            }
            returnStr = returnStr + "</ul>";
            return returnStr;
        },
    }
}
