////////////////////////////////////////////////////////////////////////////////
// seabass32暗号／解析処理.
// Copyright (c) 2009 masahito suzuki, Inc. All Rights Reserved.
////////////////////////////////////////////////////////////////////////////////

Seabass32 = {
    rand : new Xor128(),
    createPrivateKey : function( dmin,sessionId ) {
        if( $base.isNull( dmin ) || $base.isNull( sessionId ) ) {
            //throw "Seabass32#createPrivateKey:<br>引数は不正です" ;
            throw "エラー:処理に失敗しました[error:1]<br>再度ページ読み込みを行ってください." ;
        }
        var domainBin = Seabass32._md5( dmin ) ;
        var sessionIdBin = Seabass32._md5( sessionId ) ;
        var ret = domainBin.concat( sessionIdBin ) ;
        for( var i = 0 ; i < 16 ; i ++ ) {
            ret[ i ] = Seabass32._convert( ret,i,sessionIdBin[ i ] ) ;
        }
        for( var i = 15,j = 0 ; i >= 0 ; i --,j ++ ) {
            ret[ i+16 ] = Seabass32._convert( ret,i+16,domainBin[ j ] ) ;
        }
        return ret ;
    },
    encTo : function( binary,dmin,sessionId ) {
        return Seabass32.enc( binary,Seabass32.createPrivateKey( dmin,sessionId ) ) ;
    },
    enc : function( binary,pKey ) {
        if( $base.isNull( binary ) || binary.length <= 0 ||
            $base.isNull( pKey ) || pKey.length != 32 ) {
            //throw "Seabass32#enc:<br>引数は不正です" ;
            throw "エラー:処理に失敗しました[erorr:2]<br>再度ページ読み込みを行ってください." ;
        }
        var bin ;
        if( valueof( binary ) == "array" ) {
            bin = binary ;
        }
        else {
            bin = $base.strToArray( ""+binary ) ;
        }
        binary = null ;
        var pubKey = Seabass32._randKey() ;
        var key32 = Seabass32._convertKey( pKey,pubKey ) ;
        var key256 = Seabass32._key256( key32 ) ;
        key32 = null ;
        var stepNo = ( Seabass32._getStepNo( pKey,bin ) & 0x0000007f ) ;
        var nowStep = Seabass32._convert256To( key256,pubKey,stepNo ) ;
        Seabass32._ed( true,bin,key256,nowStep ) ;
        var eb = new Array( parseInt(33+bin.length) ) ;
        eb[ 0 ] = (~stepNo) ;
        $base.arraycopy( pubKey,0,eb,1,32 ) ;
        $base.arraycopy( bin,0,eb,33,bin.length ) ;
        return "@sb32" + Base64.encode( eb ) ;
    },
    decTo : function( encString,dmin,sessionId ) {
        return $base.aryToString( Seabass32.decArray( encString,Seabass32.createPrivateKey( dmin,sessionId ) ) ) ;
    },
    dec : function( encString,pKey ) {
        return $base.aryToString( Seabass32.decArray( encString,pKey ) ) ;
    },
    decArrayTo : function( encString,dmin,sessionId ) {
        return Seabass32.decArray( encString,Seabass32.createPrivateKey( dmin,sessionId ) ) ;
    },
    decArray : function( encString,pKey ) {
        if( $base.isNull( encString ) || valueof( encString ) != "string" ||
            !$base.startsWith( encString,"@sb32" ) ||
            pKey == null || pKey.length != 32 ) {
            //throw "Seabass32#decArray:<br>引数は不正です" ;
            throw "エラー:処理に失敗しました[error:3]<br>再度ページ読み込みを行ってください." ;
        }
        encString = encString.substring( 5 ) ;
        var bin = Base64.decodeArray( encString ) ;
        encString = null ;
        if( bin.length <= 33 ) {
            //throw "Seabass32#decArray:<br>不正なバイナリ長です" ;
            throw "エラー:処理に失敗しました[error:4]<br>再度ページ読み込みを行ってください." ;
        }
        var stepNo = ((~bin[ 0 ])&0x0000007f) ;
        var pubKey = new Array( 32 ) ;
        $base.arraycopy( bin,1,pubKey,0,32 ) ;
        var bodyLen = parseInt(bin.length - 33) ;
        var body = new Array( bodyLen ) ;
        $base.arraycopy( bin,33,body,0,bodyLen ) ;
        bin = null ;
        var key32 = Seabass32._convertKey( pKey,pubKey ) ;
        var key256 = Seabass32._key256( key32 ) ;
        key32 = null ;
        var nowStep = Seabass32._convert256To( key256,pubKey,stepNo ) ;
        Seabass32._ed( false,body,key256,nowStep ) ;
        if( ( Seabass32._getStepNo( pKey,body ) & 0x0000007f ) != stepNo ) {
            //throw "Seabass32#decArray:<br>Seabass32復号処理に失敗しました" ;
            throw "エラー:処理に失敗しました[error:5]<br>再度ページ読み込みを行ってください." ;
        }
        return body ;
    },
    _convert : function(key,no,pause) {
        switch ((no & 0x00000001)) {
            case 0:
                return ((~(pause ^ key[no])) & 0x000000ff) ;
            case 1:
                return ((pause ^ key[no]) & 0x000000ff) ;
        }
        return 0 ;
    },
    _randKey : function() {
        var bin = new Array(32) ;
        for( var i = 0 ; i < 32 ; i ++ ) {
            bin[ i ] = ( Seabass32.rand.next() & 0x000000ff ) ;
        }
        return bin ;
    },
    _md5 : function( s ) {
        return $md5.hash( s ) ;
    },
    _convertKey : function( pKey,key ) {
        var low = Seabass32._md5(pKey);
        var hight = Seabass32._md5(key);
        var ret = new Array(32);
        for (var i = 0,j = 0,k = 15; i < 16; i++, j += 2, k--) {
            ret[j] = Seabass32._convert(low, i, key[j]);
            ret[j + 1] = Seabass32._convert(hight, i, low[k]);
        }
        return ret;
    },
    _key256 : function( key32 ) {
        var ret = new Array( 256 ) ;
        var b = new Array( 4 ) ;
        var o ;
        var n = 0 ;
        var s,e ;
        for( var i = 0,j = 0 ; i < 31 ; i += 2,j += 16 ) {
            s = ( key32[i] & 0x000000ff ) ;
            e = ( key32[i+1] & 0x000000ff ) ;
            if( ( n & 0x00000001 ) != 0 ) {
                n += s ^ (~ e ) ;
            }
            else {
                n -= (~s) ^ e ;
            }
            b[0] = (n & 0x000000ff) ;
            b[1] = (((n & 0x0000ff00)>>8)&0x000000ff) ;
            b[2] = (((n & 0x00ff0000)>>16)&0x000000ff) ;
            b[3] = (((n & 0xff000000)>>24)&0x000000ff) ;
            o = Seabass32._md5(b) ;
            $base.arraycopy( o,0,ret,j,16 ) ;
        }
        return ret ;
    },
    _getStepNo : function( pubKey,binary ) {
        var i, j;
        var bin;
        var ret = 0;
        var len = binary.length ;
        var addCd = parseInt((pubKey[(binary[parseInt(len / 2)] & 0x0000001f)] & 0x00000003) + 1);
        for (i = 0, j = 0; i < len; i += addCd, j += addCd) {
            bin = ((~binary[i]) & 0x000000ff);
            ret = ((bin & 0x00000001) + ((bin & 0x00000002) >> 1)
                    + ((bin & 0x00000004) >> 2) + ((bin & 0x00000008) >> 3)
                    + ((bin & 0x00000010) >> 4) + ((bin & 0x00000020) >> 5)
                    + ((bin & 0x00000040) >> 6) + ((bin & 0x00000080) >> 7))
                    + (j & 0x000000ff) + ret;
        }
        if ((ret & 0x00000001) == 0) {
            for (i = 0; i <32; i++) {
                bin = (((pubKey[i] & 0x00000001) == 0) ? ((~pubKey[i]) & 0x000000ff)
                        : (pubKey[i] & 0x000000ff));
                ret += ((bin & 0x00000001) + ((bin & 0x00000002) >> 1)
                        + ((bin & 0x00000004) >> 2) + ((bin & 0x00000008) >> 3)
                        + ((bin & 0x00000010) >> 4) + ((bin & 0x00000020) >> 5)
                        + ((bin & 0x00000040) >> 6) + ((bin & 0x00000080) >> 7));
            }
        } else {
            for (i = 0; i < 32; i++) {
                bin = (((pubKey[i] & 0x00000001) == 0) ? ((~pubKey[i]) & 0x000000ff)
                        : (pubKey[i] & 0x000000ff));
                ret -= ((bin & 0x00000001) + ((bin & 0x00000002) >> 1)
                        + ((bin & 0x00000004) >> 2) + ((bin & 0x00000008) >> 3)
                        + ((bin & 0x00000010) >> 4) + ((bin & 0x00000020) >> 5)
                        + ((bin & 0x00000040) >> 6) + ((bin & 0x00000080) >> 7));
            }
        }
        return ((~ret) & 0x000000ff);
    },
    _flip : function(pause, step) {
        switch (step & 0x00000007) {
        case 1:
            return ((((pause & 0x00000003) << 6) & 0x000000c0) | (((pause & 0x000000fc) >> 2) & 0x0000003f)) & 0x000000ff ;
        case 2:
            return ((((pause & 0x0000003f) << 2) & 0x000000fc) | (((pause & 0x000000c0) >> 6) & 0x00000003)) & 0x000000ff ;
        case 3:
            return ((((pause & 0x00000001) << 7) & 0x00000080) | (((pause & 0x000000fe) >> 1) & 0x0000007f)) & 0x000000ff ;
        case 4:
            return ((((pause & 0x0000000f) << 4) & 0x000000f0) | (((pause & 0x000000f0) >> 4) & 0x0000000f)) & 0x000000ff ;
        case 5:
            return ((((pause & 0x0000007f) << 1) & 0x000000fe) | (((pause & 0x00000080) >> 7) & 0x00000001)) & 0x000000ff ;
        case 6:
            return ((((pause & 0x00000007) << 5) & 0x000000e0) | (((pause & 0x000000f8) >> 3) & 0x0000001f)) & 0x000000ff ;
        case 7:
            return ((((pause & 0x0000001f) << 3) & 0x000000f8) | (((pause & 0x000000e0) >> 5) & 0x00000007)) & 0x000000ff ;
        }
        return pause & 0x000000ff ;
    },
    _nflip : function(pause, step) {
        switch (step & 0x00000007) {
        case 1:
            return ((((pause & 0x0000003f) << 2) & 0x000000fc) | (((pause & 0x000000c0) >> 6) & 0x00000003)) & 0x000000ff ;
        case 2:
            return ((((pause & 0x00000003) << 6) & 0x000000c0) | (((pause & 0x000000fc) >> 2) & 0x0000003f)) & 0x000000ff ;
        case 3:
            return ((((pause & 0x0000007f) << 1) & 0x000000fe) | (((pause & 0x00000080) >> 7) & 0x00000001)) & 0x000000ff ;
        case 4:
            return ((((pause & 0x0000000f) << 4) & 0x000000f0) | (((pause & 0x000000f0) >> 4) & 0x0000000f)) & 0x000000ff ;
        case 5:
            return ((((pause & 0x00000001) << 7) & 0x00000080) | (((pause & 0x000000fe) >> 1) & 0x0000007f)) & 0x000000ff ;
        case 6:
            return ((((pause & 0x0000001f) << 3) & 0x000000f8) | (((pause & 0x000000e0) >> 5) & 0x00000007)) & 0x000000ff ;
        case 7:
            return ((((pause & 0x00000007) << 5) & 0x000000e0) | (((pause & 0x000000f8) >> 3) & 0x0000001f)) & 0x000000ff ;
        }
        return pause & 0x000000ff ;
    },
    _convert256To : function( key256, pKey, step) {
        var ns = step ;
        for (var i = 0, j = 0; i < 256; i++, j = ((j + 1) & 0x0000001f)) {
            ns = (ns ^ (~(key256[i]))) ;
            if( (ns & 0x00000001 ) == 0 ) {
                ns = ~ns ;
            }
            key256[i] = Seabass32._convert(pKey, j, key256[i]);
            key256[i] = Seabass32._flip(key256[i], ns);
        }
        return ns;
    },
    _ed : function(mode,binary,key256,step) {
        var len = binary.length ;
        var ns = step ;
        if( mode ) {
            for (var i = 0, j = 0; i < len; i++, j = ((j + 1) & 0x000000ff)) {
                ns = (ns ^ (~( key256[j]))) ;
                if( (ns & 0x00000001 ) != 0 ) {
                    ns = ~ns ;
                }
                binary[i] = Seabass32._convert(key256, j, binary[i]);
                binary[i] = Seabass32._flip( binary[ i ],ns ) ;
            }
        }
        else {
            for (var i = 0, j = 0; i < len; i++, j = ((j + 1) & 0x000000ff)) {
                ns = (ns ^ (~( key256[j]))) ;
                if( (ns & 0x00000001 ) != 0 ) {
                    ns = ~ns ;
                }
                binary[i] = Seabass32._nflip( binary[ i ],ns ) ;
                binary[i] = Seabass32._convert(key256, j, binary[i]);
            }
        }
    }
}

// Seabass32暗号化送信Ajax処理.
seabass32Ajax = function( url,sendParams ) {
    var sid = md5( $base.getSessionId() + randomId( 32 ) ) ;
    var secKey = "" ;
    //var key = $base.getDomain() ;
    //var key = $base.getServerDomain() ;
    var key = __VAL_GLOVAL__._$SEABASS32 ;
    if( !$base.isNull( sendParams ) ) {
        if( valueof( sendParams ) == "map" ) {
            var buf = new StrBuf() ;
            var f = false ;
            for( var k in sendParams ) {
                if( f ) {
                    buf.ad( "&" ) ;
                }
                buf.ad( k ).ad( "=" ).ad( encodeURIComponent( sendParams[k] ) ) ;
                f = true ;
            }
            sendParams = buf.ts() ;
        }
        var burl = url ;
        var p = burl.indexOf( "?" ) ;
        if( p != -1 ) {
            burl = burl.substring( 0,p ) ;
        }
        var pLen = 0
        p = burl.indexOf( "http://" ) ;
        pLen = 7 ;
        if( p <= -1 ) {
            p = burl.indexOf( "https://" ) ;
            pLen = 8 ;
        }
        if( p >= 0 ) {
            p = burl.indexOf( "/",p+pLen ) ;
            if( p >= 0 ) {
                burl = burl.substring( p ) ;
            }
        }
        secKey = md5(burl)+sid ;
        var v = null ;
        try {
            v = Seabass32.encTo( sendParams,key,secKey ) ;
            sendParams = "_seabass32=" + encodeURIComponent(v) + "&_sid=" + encodeURIComponent(sid) ;
        } catch( e ) {
            return "@error:" + e ;
        }
    }
    else {
        sendParams = "_seabass32=" + "&_sid=" + encodeURIComponent(sid) ;
    }
    var res = accessAjax( url,sendParams ) ;
    try {
        res = Seabass32.decTo( res,key,secKey ) ;
    } catch( e ) {
        return "@error:" + e ;
    }
    return res ;
} ;

// Seabass32暗号化処理を行い、
// scriptタグが含まれている場合にJavascript実行するAjax.
seabass32AjaxToJsExecution = function( id,url,sendParams,scriptId,innerId ) {
    var s = seabass32Ajax( url,sendParams ) ;
    return resultAjaxScript( id,s,scriptId,innerId ) ;
} ;

// Seabass32暗号化処理を行い、
// JTemplateとscriptタグ内容を実行するAjax.
seabass32ByJTemplate = function( id,url,sendParams,scriptId,innerId ) {
    var s = seabass32Ajax( url,sendParams ) ;
    s = JTemplate.execution( s ) ;
    return resultAjaxScript( id,s,scriptId,innerId ) ;
} ;

// Seabass32Ajax用:エラー判別.
isSeabass32Error = function( result ) {
    if( !$base.useString( result ) || $base.startsWith( result,"@error" ) ) {
        return true ;
    }
    return false ;
}

// Seabass32Ajax用:エラーメッセージを取得.
getSeabass32Error = function( result ) {
    if( isSeabass32Error( result ) ) {
        if( $base.startsWith( result,"@error:" ) ) {
            return result.substring( "@error:".length ) ;
        }
    }
    return "" ;
}

