package org.maachang.util;

/**
 * 最速文字連結オブジェクト.
 * 
 * @version 2009/10/14
 * @author masahito suzuki
 * @since MaachangBase 1.0B
 */
public class FastStringBuffer {
    private static final int _LENGTH = 4096 ;
    private class _$FastStringBuffer_StackList {
        public char[] c ;
        public _$FastStringBuffer_StackList next = null ;
    }
    private _$FastStringBuffer_StackList first = null ;
    private _$FastStringBuffer_StackList list = null ;
    private char[] last = null ;
    private int length ;
    private int stackLength ;
    
    /** コンストラクタ. **/
    protected FastStringBuffer() {
        list = new _$FastStringBuffer_StackList() ;
        last = new char[ _LENGTH ] ;
        list.c = last ;
        first = list ;
        length = 0 ;
        stackLength = 0 ;
    }
    
    /**
     * デストラクタ.
     */
    protected void finalize() throws Exception {
        FastStringBufferFactory.release( this ) ;
    }
    
    /**
     * クリア処理.
     */
    protected void clear() {
        first.next = null ;
        list = first ;
        last = first.c ;
        length = 0 ;
        stackLength = 0 ;
    }
    
    /**
     * 情報追加.
     * @param o 追加対象のオブジェクトを設定します.
     */
    public void add( Object o ) {
        if( o == null ) {
            return ;
        }
        String s ;
        if( o instanceof java.lang.String ) {
            s = (String)o ;
        }
        else {
            s = o.toString() ;
        }
        int len ;
        if( ( len = s.length() ) <= 0 ) {
            return ;
        }
        int off = 0 ;
        int pls ;
        _$FastStringBuffer_StackList n ;
        while( true ) {
            if( _LENGTH+off < len+stackLength) {
                pls = (_LENGTH-stackLength) ;
                s.getChars( off,off+pls,last,stackLength ) ;
                n = new _$FastStringBuffer_StackList() ;
                last = new char[ _LENGTH ] ;
                n.c = last ;
                list = list.next = n ;
                stackLength = 0 ;
                off += pls ;
                if( off >= len ) {
                    length += len ;
                    break ;
                }
            }
            else {
                s.getChars( off,len,last,stackLength ) ;
                stackLength += len - off ;
                length += len ;
                break ;
            }
        }
    }
    
    /**
     * 現在の文字列長を取得.
     * @return int 現在の文字列長が返されます.
     */
    public int length() {
        return length ;
    }
    
    /**
     * クリアされているかチェック.
     * @return boolean [true]の場合、クリアされています.
     */
    public boolean isClear() {
        return length == 0 ;
    }
    
    /**
     * 文字列を結合して出力.
     * @return String 結合された文字列が返されます.
     * @exception Exception 例外.
     */
    public String get() throws Exception {
        if( length > 0 ) {
            _$FastStringBuffer_StackList n = first ;
            if( n.next == null ) {
                return new String( n.c,0,length ) ;
            }
            int off = 0 ;
            char[] c = new char[ length ] ;
            while( true ) {
                if( n.next == null ) {
                    System.arraycopy( n.c,0,c,off,stackLength ) ;
                    break ;
                }
                else {
                    System.arraycopy( n.c,0,c,off,_LENGTH ) ;
                    off += _LENGTH ;
                }
                n = n.next ;
            }
            return new String( c ) ;
        }
        return "" ;
    }
    
    /**
     * 文字列を連結して出力.
     * @return String 結合された文字列が返されます.
     */
    public String toString() {
        String ret ;
        try {
            ret = get() ;
        } catch( Exception e ) {
            ret = "" ;
        }
        return ret ;
    }
}
