package org.maachang.util;

import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;

import org.maachang.util.atomic.AtomicBOOL;
import org.maachang.util.atomic.AtomicFLOAT;

/**
 * JavaCpu負荷を取得.
 *
 * @version 2009/08/29
 * @author masahito suzuki
 * @since MaachangBase 1.00
 */
public class JavaCpu {
    private static final JavaCpu SNGL = new JavaCpu() ;
    private JavaCpuThread thread = null ;
    private final AtomicFLOAT cpu = new AtomicFLOAT( 0.0f ) ;
    
    private JavaCpu(){
        this.thread = new JavaCpuThread( cpu ) ;
    }
    
    /**
     * オブジェクトを取得.
     * @return JavaCpu オブジェクトが返されます.
     */
    public static final JavaCpu getInstance() {
        return SNGL ;
    }
    
    /**
     * オブジェクト破棄.
     */
    protected void destroy() {
        if( thread != null ) {
            thread.stopThread() ;
        }
        thread = null ;
    }
    
    /**
     * JavaのCPU利用率を取得.
     * @return float JavaのCPU利用率が返されます.
     */
    public float cpu() {
        return cpu.get() ;
    }
    
    private static class JavaCpuThread extends Thread {
        private final com.sun.management.OperatingSystemMXBean result =
            ( com.sun.management.OperatingSystemMXBean )ManagementFactory.getOperatingSystemMXBean() ;
        private final RuntimeMXBean bean = ManagementFactory.getRuntimeMXBean();
        private long upTime = 0L ;
        private long processTime = 0L ;
        private AtomicFLOAT _cpu = null ;
        private final AtomicBOOL stopFlag = new AtomicBOOL( false ) ;
        
        protected JavaCpuThread( AtomicFLOAT cpu ) {
            this._cpu = cpu ;
            checkCpu() ;
            super.setDaemon( true ) ;
            super.start() ;
        }
        
        protected void stopThread() {
            stopFlag.set( true ) ;
        }
        
        protected static final int WAIT = 500 ;
        
        public void run() {
            ThreadDeath threadDeach = null ;
            boolean endFlag = false ;
            while( true ) {
                if( endFlag ) {
                    if( threadDeach != null ) {
                        throw threadDeach ;
                    }
                    break ;
                }
                if( stopFlag.get() ) {
                    break ;
                }
                try {
                    Thread.sleep( WAIT ) ;
                    checkCpu() ;
                } catch( InterruptedException ite ) {
                    endFlag = true ;
                } catch ( NullPointerException ne ) {
                } catch( OutOfMemoryError mem ) {
                } catch( Exception e ) {
                } catch( ThreadDeath td ) {
                    endFlag = true ;
                    threadDeach = td ;
                }
            }
        }
        
        private void checkCpu() {
            if (bean.getUptime() > upTime) {
                long elapsedCpu = result.getProcessCpuTime() - processTime;
                long elapsedTime = bean.getUptime() - upTime;
                _cpu.set( Math.min(99F,elapsedCpu / (elapsedTime * 10000F * result.getAvailableProcessors())) ) ;
                processTime = result.getProcessCpuTime();
            }
            upTime = bean.getUptime();
        }
    }
}

