package org.maachang.util;

import java.net.InetAddress;

/**
 * メイン処理用ユーティリティ.
 * 
 * @version 2009/09/20
 * @author masahito suzuki
 * @since MaachangBase 1.0B
 */
public abstract class MainUtil {
    
    /**
     * パラメータタイプ : 文字列.
     */
    public static final int CONVERT_TYPE_STRING = 0 ;
    
    /**
     * パラメータタイプ : 数値(int).
     */
    public static final int CONVERT_TYPE_INTEGER = 1 ;
    
    /**
     * パラメータタイプ : 数値(long).
     */
    public static final int CONVERT_TYPE_LONG = 2 ;
    
    /**
     * パラメータタイプ : 浮動小数点(float).
     */
    public static final int CONVERT_TYPE_FLOAT = 3 ;
    
    /**
     * パラメータタイプ : 浮動小数点(double).
     */
    public static final int CONVERT_TYPE_DOUBLE = 4 ;
    
    /**
     * パラメータタイプ : IPアドレス.
     */
    public static final int CONVERT_TYPE_IP_ADDR = 5 ;
    
    /**
     * Main引数のパラメータから、任意文字以降の内容が存在するかチェック.
     * @param target 検索対象の任意文字を設定します.
     * @param args パラメータを設定します.
     * @return boolean [true]の場合、情報は存在します.
     */
    public static final boolean isArgs( String target,String[] args ) {
        if( target == null || ( target = target.trim() ).length() <= 0 || args == null || args.length <= 0 ) {
            return false ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String o = args[ i ] ;
            if( o == null ) {
                continue ;
            }
            o = o.trim() ;
            if( o.startsWith( target ) ) {
                return true ;
            }
        }
        return false ;
    }
    
    /**
     * Main引数のパラメータから、任意文字以降の内容を取得.
     * @param target 検索対象の任意文字を設定します.
     * @param type 変換結果のパラメータタイプを設定します.
     * @param args パラメータを設定します.
     * @return Object パラメータが返されます.
     * @exception Exception 例外.
     */
    public static final Object getArgs( String target,int type,String[] args )
        throws Exception {
        if( target == null || ( target = target.trim() ).length() <= 0 || args == null || args.length <= 0 ) {
            return null ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String o = args[ i ] ;
            if( o == null ) {
                continue ;
            }
            o = o.trim() ;
            if( o.startsWith( target ) ) {
                o = o.substring( target.length() ) ;
                switch( type ) {
                    case CONVERT_TYPE_STRING : return o ;
                    case CONVERT_TYPE_INTEGER : return new Integer( o ) ;
                    case CONVERT_TYPE_LONG : return new Long( o ) ;
                    case CONVERT_TYPE_FLOAT : return new Float( o ) ;
                    case CONVERT_TYPE_DOUBLE : return new Double( o ) ;
                    case CONVERT_TYPE_IP_ADDR : return InetAddress.getByName( o ) ;
                }
                return o ;
            }
        }
        return null ;
    }
    
    /**
     * 指定ファイル名の拡張子を変更.
     * @param name 対象のファイル名を設定します.
     * @param cpls 変更対象のパラメータを設定します.
     * @return String 変更されたファイル名が返されます.
     */
    public static final String changePlus( String name,String cpls ) {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            return null ;
        }
        if( cpls == null || ( cpls = cpls.trim() ).length() <= 0 ) {
            cpls = "" ;
        }
        else if( cpls.startsWith( "." ) ) {
            cpls = cpls.substring( 1 ).trim() ;
        }
        int p = name.lastIndexOf( "\\" ) ;
        int p2 = name.lastIndexOf( "/" ) ;
        if( p < p2 ) {
            p = p2 ;
        }
        p2 = name.lastIndexOf( "." ) ;
        if( p < p2 ) {
            name = name.substring( 0,p2 ) ;
        }
        return name + "." + cpls ;
    }
    
}

