package org.maachang.util ;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;

/**
 * 単純Map.
 * 
 * @version 2009/01/05
 * @author masahito suzuki
 * @since Maachang 1.00
 */
public class SimpleObjectMap implements Map<Object,Object>,Serializable {
    static class SimpleObjectMapChild implements Serializable {
        private static final long serialVersionUID = 4794280953591512935L;
        private Object key = null ;
        private Object value = null ;
        private SimpleObjectMapChild chain = null ;
    }
    
    private static final long serialVersionUID = -6757738116593682045L;
    
    private static final int MAX_CAPACITY = 0x00040000 ;
    private static final int MIN_BIT = 4 ;
    private SimpleObjectMapChild[] maps = null ;
    private int defMaps = -1 ;
    private int[] count = new int[ 1 ] ;
    private int mask = -1 ;
    
    /**
     * コンストラクタ.
     */
    public SimpleObjectMap() {
        this( -1 ) ;
    }
    
    /**
     * コンストラクタ.
     * @param def 初期キャパシティ値として、ビットシフト値を設定します.<br>
     *            [4]以下の場合、4ビット値(16)となります.
     */
    public SimpleObjectMap( int def ) {
        if( def <= MIN_BIT ) {
            def = MIN_BIT ;
        }
        def = 1 << def ;
        this.maps = new SimpleObjectMapChild[ def ] ;
        this.defMaps = def ;
        this.mask = def - 1 ;
        this.count[0] = 0 ;
    }
    
    protected void finalize() throws Exception {
        destroy() ;
    }
    
    /** オブジェクト破棄 **/
    protected void destroy() {
        maps = null ;
        defMaps = -1 ;
        count[0] = 0 ;
        mask = -1 ;
    }
    
    /**
     * クリア.
     */
    public void clear() {
        this.maps = new SimpleObjectMapChild[ defMaps ] ;
        this.mask = defMaps - 1 ;
        this.count[0] = 0 ;
    }
    
    /** 追加データ長が一定値以上の場合、キャパシティを増加. **/
    private void addCapacity() {
        int len = maps.length ;
        int next = ( len << 1 ) ;
        int newMask = next-1 ;
        SimpleObjectMapChild[] newNodes = new SimpleObjectMapChild[ next ] ;
        SimpleObjectMapChild[] befNode = maps ;
        SimpleObjectMapChild temp = null ;
        for( int i = 0 ; i < len ; i ++ ) {
            SimpleObjectMapChild node = befNode[ i ] ;
            while( node != null ) {
                temp = node.chain ;
                int pos = ( node.key.hashCode() & newMask ) ;
                if( newNodes[ pos ] == null ) {
                    newNodes[ pos ] = node ;
                    node.chain = null ;
                }
                else {
                    node.chain = newNodes[ pos ] ;
                    newNodes[ pos ] = node ;
                }
                node = temp ;
            }
        }
        int cnt = this.count[ 0 ] ;
        this.maps = newNodes ;
        this.mask = newMask ;
        this.count = new int[]{cnt} ;
    }
    
    /** 現在のキャパシティ数を取得. **/
    public int capacityLength() {
        return maps.length ;
    }
    
    public Object put(Object name, Object value) {
        int pos = (name.hashCode() & mask ) ;
        SimpleObjectMapChild node = maps[ pos ] ;
        while( node != null ) {
            if( node.key == name || node.key.equals( name ) ) {
                node.value = value ;
                return value ;
            }
            node = node.chain ;
        }
        node = new SimpleObjectMapChild() ;
        node.key = name ;
        node.value = value ;
        count[0] ++ ;
        if( count[0] >= maps.length && maps.length < MAX_CAPACITY ) {
            addCapacity() ;
        }
        if( maps[ pos ] == null ) {
            maps[ pos ] = node ;
        }
        else {
            node.chain = maps[ pos ] ;
            maps[ pos ] = node ;
        }
        return value ;
    }
    public boolean containsKey(Object key) {
        SimpleObjectMapChild node = maps[ (key.hashCode() & mask ) ] ;
        while( node != null ) {
            if( node.key == key || node.key.equals( key ) ) {
                return true ;
            }
            node = node.chain ;
        }
        return false ;
    }
    public Object get(Object key) {
        int nodePos = ( key.hashCode() & mask ) ;
        SimpleObjectMapChild node = maps[ nodePos ] ;
        SimpleObjectMapChild bef = null ;
        while( node != null ) {
            if( node.key == key || node.key.equals( key ) ) {
                if( bef == null ) {
                    return node.value ;
                }
                else {
                    bef.chain = node.chain ;
                    bef = maps[ nodePos ] ;
                    node.chain = bef ;
                    maps[ nodePos ] = node ;
                    return node.value ;
                }
            }
            bef = node ;
            node = node.chain ;
        }
        return null ;
    }
    public Object remove(Object key) {
        int nodePos = ( key.hashCode() & mask ) ;
        SimpleObjectMapChild node = maps[ nodePos ] ;
        SimpleObjectMapChild bef = null ;
        while( node != null ) {
            if( node.key == key || node.key.equals( key ) ) {
                if( bef == null ) {
                    maps[ nodePos ] = null ;
                    count[0] -- ;
                    return node.value ;
                }
                else {
                    bef.chain = node.chain ;
                    count[0] -- ;
                    return node.value ;
                }
            }
            bef = node ;
            node = node.chain ;
        }
        return null ;
    }
    public boolean isEmpty() {
        return this.count[0] <= 0 ;
    }
    public void putAll(Map toMerge) {
        if( toMerge == null ) {
            return ;
        }
        Iterator it = toMerge.keySet().iterator() ;
        while( it.hasNext() ) {
            Object key = it.next() ;
            if( key != null ) {
                put( key,toMerge.get( key ) ) ;
            }
        }
    }
    public boolean containsValue(Object value) {
        if( value == null ) {
            SimpleObjectMapChild[] nds = maps ;
            int len = nds.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                SimpleObjectMapChild node = nds[ i ] ;
                while( node != null ) {
                    if( node.value == null ) {
                        return true ;
                    }
                    node = node.chain ;
                }
            }
        }
        else {
            SimpleObjectMapChild[] nds = maps ;
            int len = nds.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                SimpleObjectMapChild node = nds[ i ] ;
                while( node != null ) {
                    if( node.value.equals( value ) ) {
                        return true ;
                    }
                    node = node.chain ;
                }
            }
        }
        return false ;
    }
    public Set entrySet() {
        return null ;
    }
    public int size() {
        return count[0] ;
    }
    public String toString() {
        StringBuilder buf = new StringBuilder() ;
        SimpleObjectMapChild[] nds = maps ;
        int len = nds.length ;
        buf.append( "{" ) ;
        boolean nextFlag = false ;
        for( int i = 0 ; i < len ; i ++ ) {
            SimpleObjectMapChild node = nds[ i ] ;
            while( node != null ) {
                if( node.value == this ) {
                    node = node.chain ;
                    continue ;
                }
                if( nextFlag ) {
                    buf.append( "," ) ;
                }
                buf.append( node.key ).append( ":" ).append( node.value ) ;
                node = node.chain ;
                nextFlag = true ;
            }
        }
        return buf.append( "}" ).toString() ;
    }
    public Collection values() {
        ArrayList<Object> ret = new ArrayList<Object>() ;
        SimpleObjectMapChild[] nds = maps ;
        int len = nds.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            SimpleObjectMapChild node = nds[ i ] ;
            while( node != null ) {
                ret.add( node.value ) ;
                node = node.chain ;
            }
        }
        return ret ;
    }
    
    static class SimpleObjectIterator implements Iterator<Object> {
        private SimpleObjectMapChild[] maps ;
        private SimpleObjectMapChild nowNode ;
        private int nowPos ;
        private SimpleObjectMapChild target ;
        
        private SimpleObjectIterator() {
        }
        protected SimpleObjectIterator( SimpleObjectMapChild[] maps ) {
            this.maps = maps ;
            this.nowNode = null ;
            this.nowPos = -1 ;
            this.target = null ;
        }
        private boolean getNext() {
            if( target == null ) {
                SimpleObjectMapChild[] nds = maps ;
                int len = nds.length ;
                if( nowNode != null ) {
                    nowNode = nowNode.chain ;
                    if( nowNode != null ) {
                        target = nowNode ;
                        return true ;
                    }
                }
                nowPos ++ ;
                for( int i = nowPos ; i < len ; i ++ ) {
                    SimpleObjectMapChild node = nds[ i ] ;
                    while( node != null ) {
                        nowNode = node ;
                        target = nowNode ;
                        nowPos = i ;
                        return true ;
                    }
                }
                return false ;
            }
            return true ;
        }
        public boolean hasNext() {
            return getNext() ;
        }
        public Object next() {
            if( getNext() == false ) {
                throw new NoSuchElementException() ;
            }
            Object ret = target.key ;
            target = null ;
            return ret ;
        }
        public void remove() {
            return ;
        }
    }
    static class SimpleObjectSet implements Set<Object> {
        private SimpleObjectMapChild[] maps = null ;
        private int[] count = null ;
        private SimpleObjectSet() {
        }
        protected SimpleObjectSet( SimpleObjectMapChild[] maps,int[] count ) {
            this.maps = maps ;
            this.count = count ;
        }
        public int size() {
            return count[0] ;
        }
        public boolean isEmpty() {
            return count[0] <= 0 ;
        }
        public boolean contains(Object o) {
            return false ;
        }
        public Iterator<Object> iterator() {
            return new SimpleObjectIterator( maps ) ;
        }
        public Object[] toArray() {
            ArrayList<Object> ret = new ArrayList<Object>() ;
            SimpleObjectMapChild[] nds = maps ;
            int len = nds.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                SimpleObjectMapChild node = nds[ i ] ;
                while( node != null ) {
                    ret.add( node.value ) ;
                    node = node.chain ;
                }
            }
            if( ret.size() <= 0 ) {
                return new Object[0] ;
            }
            return ret.toArray() ;
        }
        public <Object> Object[] toArray(Object[] a) {
            return null ;
        }
        public boolean add(Object e) {
            return false ;
        }
        public boolean remove(Object o) {
            return false ;
        }
        public boolean containsAll(Collection<?> c) {
            return false ;
        }
        public boolean addAll(Collection<? extends Object> c) {
            return false ;
        }
        public boolean retainAll(Collection<?> c) {
            return false ;
        }
        public boolean removeAll(Collection<?> c) {
            return false ;
        }
        public void clear() {
            maps = null ;
            count = null ;
        }
        public boolean equals(Object o) {
            return this == o ;
        }
        public int hashCode() {
            return -1 ;
        }
    }
    
    
    public Set keySet() {
        return new SimpleObjectSet( maps,count ) ;
    }
}

