package org.maachang.util.atomic ;

import java.io.Serializable;
import java.util.concurrent.atomic.AtomicReference;

import org.maachang.util.SerializableUtil;

/**
 * AtomicBeans.
 * 
 * @version 2008/12/01
 * @author masahito suzuki
 * @since MaachangBase 1.09
 */
public class AtomicBEANS implements Serializable {
    private static final long serialVersionUID = 8257074739650323863L;
    private final AtomicReference<Serializable> ato = new AtomicReference<Serializable>() ;
    
    /**
     * コンストラクタ.
     */
    public AtomicBEANS() {
        
    }
    
    /**
     * コンストラクタ.
     * @param b 初期値を設定します.
     */
    public AtomicBEANS( Serializable b ) {
        setting( b ) ;
    }
    
    /**
     * オブジェクトを取得.
     * @return Serializable オブジェクトが返されます.
     * @exception Exception 例外.
     */
    public Serializable get() {
        try {
            return cloneObject( ato.get() ) ;
        } catch( Exception e ) {
            throw new RuntimeException( e ) ;
        }
    }
    
    /**
     * オブジェクトを更新.
     * @param b 対象オブジェクトを設定します.
     */
    public void update( Serializable b ) {
        setting( b ) ;
    }
    
    /**
     * オブジェクトを設定.
     * @param b 対象オブジェクトを設定します.
     */
    public void set( Serializable b ) {
        setting( b ) ;
    }
    
    private void setting( Serializable b ) {
        while( !ato.compareAndSet( ato.get(),b ) ) {}
    }
    
    private Serializable cloneObject( Serializable b ) throws Exception {
        if( b == null ) {
            return null ;
        }
        return SerializableUtil.toObject( SerializableUtil.toBinary( b ) ) ;
    }
    
}
