package org.maachang.util.atomic ;

import java.io.Serializable;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Atomic(int)シーケンス発行.
 * 
 * @version 2008/12/01
 * @author masahito suzuki
 * @since MaachangBase 1.09
 */
public class AtomicIntSeq implements Serializable {
    private static final long serialVersionUID = 2146413862872622800L;
    private final AtomicInteger ato = new AtomicInteger( 0 ) ;
    private int max = 0 ;
    
    /**
     * コンストラクタ.
     */
    public AtomicIntSeq() {
        this(-1);
    }
    
    /**
     * コンストラクタ.
     * @param n 初期値を設定します.
     */
    public AtomicIntSeq( int n ) {
        if( n <= 0 ) {
            n = Integer.MAX_VALUE ;
        }
        max = n ;
    }
    
    /**
     * シーケンスIDを取得.
     * @return int シーケンスIDが返されます.
     */
    public int get() {
        int n ;
        int r ;
        while( true ) {
            n = ato.get() ;
            r = n + 1 ;
            if( r >= max ) { r = 0; }
            if( ato.compareAndSet( n,r ) ) {
                return r ;
            }
        }
    }
    
    /**
     * 最大値を取得.
     * @return int 最大値が返されます.
     */
    public int max() {
        return max ;
    }
    
}
