package org.maachang.util.atomic ;

import java.io.Serializable;
import java.util.concurrent.atomic.AtomicLong;

/**
 * Atomic(long)シーケンス発行.
 * 
 * @version 2008/12/01
 * @author masahito suzuki
 * @since MaachangBase 1.09
 */
public class AtomicLongSeq implements Serializable {
    private static final long serialVersionUID = 2146413862872622799L;
    private final AtomicLong ato = new AtomicLong( 0 ) ;
    private long max = 0 ;
    
    /**
     * コンストラクタ.
     */
    public AtomicLongSeq() {
        this(-1L);
    }
    
    /**
     * コンストラクタ.
     * @param n 初期値を設定します.
     */
    public AtomicLongSeq( long n ) {
        if( n <= 0L ) {
            n = Long.MAX_VALUE ;
        }
        max = n ;
    }
    
    /**
     * シーケンスIDを取得.
     * @return long シーケンスIDが返されます.
     */
    public long get() {
        long n ;
        long r ;
        while( true ) {
            n = ato.get() ;
            r = n + 1L ;
            if( r >= max ) { r = 0L; }
            if( ato.compareAndSet( n,r ) ) {
                return r ;
            }
        }
    }
    
    /**
     * 最大値を取得.
     * @return long 最大値が返されます.
     */
    public long max() {
        return max ;
    }
    
}
