package org.maachang.util.atomic;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

/**
 * スタック処理.
 * 
 * @version 2009/09/01
 * @author masahito suzuki
 * @since MaachangAtomic 1.00
 */
public class StackToLock<E> {
    private static class _stackToLockChild<E> {
        public E object ;
        public _stackToLockChild<E> next = null ;
    }
    
    private final ReentrantLock sync = new ReentrantLock() ;
    private _stackToLockChild<E> c = null ;
    private int size = 0 ;
    
    protected void finalize() throws Exception {
        clear() ;
    }
    
    /**
     * 情報をクリア.
     */
    public void clear() {
        sync.lock() ; try {
            c = null ;
            size = 0 ;
        } finally {
            sync.unlock() ;
        }
    }
    
    /**
     * 情報追加.
     * @param e 追加対象の情報を設定します.
     */
    public void add( E e ) {
        if( e == null ) {
            return ;
        }
        _stackToLockChild<E> a = new _stackToLockChild<E>() ;
        a.object = e ;
        sync.lock() ; try {
            if( c == null ) {
                c = a ;
            }
            else {
                a.next = c ;
                c = a ;
            }
            size ++ ;
        } finally {
            sync.unlock() ;
        }
    }
    
    /**
     * 先頭情報を削除して取得.
     * @return E 対象の情報が返されます.
     */
    public E poll() {
        E ret = null ;
        sync.lock() ; try {
            if( c != null ) {
                ret = c.object ;
                c = c.next ;
                size -- ;
            }
        } finally {
            sync.unlock() ;
        }
        return ret ;
    }
    
    /**
     * 先頭情報を削除せず取得.
     * @return E 対象の情報が返されます.
     */
    public E peek() {
        E ret = null ;
        sync.lock() ; try {
            if( c != null ) {
                ret = c.object ;
            }
        } finally {
            sync.unlock() ;
        }
        return ret ;
    }
    
    /**
     * サイズを取得.
     * @return int サイズが返されます.
     */
    public int size() {
        int ret ;
        sync.lock() ; try {
            ret = size ;
        } finally {
            sync.unlock() ;
        }
        return ret ;
    }
    
    /**
     * 全内容を取得.
     * @return List<E> 全内容が返されます.
     */
    public List<E> getAll() {
        List<E> ret = new ArrayList<E>() ;
        _stackToLockChild<E> n ;
        sync.lock() ; try {
            for( n = c ; n != null ; n = n.next ) {
                ret.add( n.object ) ;
            }
        } finally {
            sync.unlock() ;
        }
        return ret ;
    }
}
