package org.maachang.util.atomic.lock ;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * spinLock(read/write).
 *  
 * @version 2009/03/17
 * @author  masahito suzuki
 * @since MaachangBase 1.09
 */
public final class RWLock {
    private static final int DEF_SPIN_COUNT = 10000 ;
    private static final int READ_COUNT = 0x00000002 ;
    private static final int READ_MASK =  0xfffffffe ;
    private static final int WRITE_MASK = 0x00000001 ;
    private static final int WRITE_ON =   0x00000001 ;
    private final AtomicInteger rw = new AtomicInteger( 0 ) ;
    private int spin = 0 ;
    
    /**
     * コンストラクタ.
     */
    public RWLock() {
        this( 0 ) ;
    }
    
    /**
     * コンストラクタ.
     * @param spin spinロック値を設定します.
     */
    public RWLock( int spin ) {
        if( spin <= 0 ) {
            spin = DEF_SPIN_COUNT ;
        }
        this.spin = spin ;
    }
    
    /**
     * 読み込みロック.
     */
    public void readLock() {
        int n ;
        if( ( (n = rw.get()) & WRITE_MASK ) == 0 && rw.compareAndSet( n,n+READ_COUNT ) ) {
            return ;
        }
        for( int i = 0 ; i < spin ; i ++ ) {
            if( ( (n = rw.get()) & WRITE_MASK ) == 0 && rw.compareAndSet( n,n+READ_COUNT ) ) {
                return ;
            }
        }
        try {
            while( true ) {
                Thread.sleep( 1 ) ;
                if( ( (n = rw.get()) & WRITE_MASK ) == 0 && rw.compareAndSet( n,n+READ_COUNT ) ) {
                    return ;
                }
            }
        } catch( Exception e ) {
        }
    }
    
    /**
     * 読み込みアンロック.
     */
    public void readUnlock() {
        int n ;
        if( ((n = rw.get())&READ_MASK) > 0 && rw.compareAndSet( n,n-READ_COUNT ) ) {
            return ;
        }
        if( n <= 0 ) {
            throw new IllegalStateException( "[read]アンロックエラー" ) ;
        }
        while( true ) {
            if( ((n = rw.get())&READ_MASK) <= 0 ) {
                throw new IllegalStateException( "[read]アンロックエラー" ) ;
            }
            if( rw.compareAndSet( n,n-READ_COUNT ) ) {
                return ;
            }
        }
    }
    
    /**
     * 書き込みロック.
     */
    public void writeLock() {
        int n ;
        if( (n = rw.get()) == 0L && rw.compareAndSet( n,n|WRITE_ON ) ) {
            return ;
        }
        for( int i = 0 ; i < spin ; i ++ ) {
            if( (n = rw.get()) == 0L && rw.compareAndSet( n,n|WRITE_ON ) ) {
                return ;
            }
        }
        try {
            while( true ) {
                Thread.sleep( 1 ) ;
                if( (n = rw.get()) == 0L && rw.compareAndSet( n,n|WRITE_ON ) ) {
                    return ;
                }
            }
        } catch( Exception e ) {
        }
    }
    
    /**
     * 書き込みアンロック.
     */
    public void writeUnlock() {
        int n = rw.get() ;
        if( !rw.compareAndSet( n,n & READ_MASK ) ) {
            while( true ) {
                n = rw.get() ;
                if( rw.compareAndSet( n,n & READ_MASK ) ) {
                    return ;
                }
            }
        }
    }
    
    /**
     * spinロック値を取得.
     * @return int spinロック値が返されます.
     */
    public int getSpin() {
        return spin ;
    }
}

