package org.maachang.util.atomic.lock ;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * spinLock.
 *  
 * @version 2009/03/17
 * @author  masahito suzuki
 * @since MaachangBase 1.09
 */
public final class SpinLock {
    private static final int DEF_SPIN_COUNT = 10000 ;
    private final AtomicInteger c = new AtomicInteger( 0 ) ;
    private int spin = 0 ;
    
    /**
     * コンストラクタ.
     */
    public SpinLock() {
        this( 0 ) ;
    }
    
    /**
     * コンストラクタ.
     * @param spin spinロック値を設定します.
     */
    public SpinLock( int spin ) {
        if( spin <= 0 ) {
            spin = DEF_SPIN_COUNT ;
        }
        this.spin = spin ;
    }
    
    /**
     * ロック処理.
     */
    public void lock() {
        if( !c.compareAndSet( 0,1 ) ) {
            // spinlock.
            for( int i = 0 ; i < spin ; i ++ ) {
                if( c.compareAndSet( 0,1 ) ) {
                    return ;
                }
            }
            // spinlockが一定ループ以上の場合は、WaitしたLock解除待ち.
            try {
                while( true ) {
                    Thread.sleep( 1 ) ;
                    if( c.compareAndSet( 0,1 ) ) {
                        return ;
                    }
                }
            } catch( Exception e ) {
            }
        }
    }
    
    /**
     * アンロック処理.
     */
    public void unlock() {
        if( !c.compareAndSet( c.get(),0 ) ) {
            while( true ) {
                if( c.compareAndSet( c.get(),0 ) ) {
                    return ;
                }
            }
        }
    }
    
    /**
     * spinロック値を取得.
     * @return int spinロック値が返されます.
     */
    public int getSpin() {
        return spin ;
    }
}

