package org.maachang.util.image;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.io.OutputStream;

import javax.imageio.ImageIO;

/**
 * 高速画像解像度変換処理.
 *
 * @version 2009/02/17
 * @author  masahito suzuki
 * @since   MaachangBase 1.0A
 */
public class FastConvertImage {
    
    /**
     * 画像変換.
     * @param output 変換された画像情報を保持するオブジェクトを設定します.
     * @param input 変換対象の画像情報を保持するオブジェクトを設定します.
     * @param type 変換画像タイプを設定します.
     * @param width 変換画像サイズ(横)を設定します.
     * @param height 変換画像サイズ(縦)を設定します.
     * @exception Exception 例外.
     */
    public static final void coonvert( OutputStream output,InputStream input,String type,int width,int height )
        throws Exception {
        if( output == null || input == null || width <= 0 || height <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        if( type == null ) {
            type = "jpg" ;
        }
        BufferedImage ret = convert( ImageIO.read( input ),width,height ) ;
        ImageIO.write(ret, type, output);
    }
    
    /**
     * 画像を拡大／縮小変換.
     * @param image 変換対象の画像イメージを設定します.
     * @param width 変換画像サイズ(横)を設定します.
     * @param height 変換画像サイズ(縦)を設定します.
     * @return Image 変換されたイメージ情報が返されます.
     * @exception Exception 例外.
     */
    public static final BufferedImage convert( BufferedImage image,int width,int height )
        throws Exception {
        BufferedImage shrinkImage = new BufferedImage(width,height,image.getType());
        Graphics2D g2d = shrinkImage.createGraphics();
        g2d.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION,
        RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING,
        RenderingHints.VALUE_COLOR_RENDER_QUALITY);
        g2d.setRenderingHint(RenderingHints.KEY_DITHERING,
        RenderingHints.VALUE_DITHER_ENABLE);
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
        RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
        RenderingHints.VALUE_RENDER_QUALITY);
        g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
        RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g2d.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,
        RenderingHints.VALUE_FRACTIONALMETRICS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
        RenderingHints.VALUE_STROKE_NORMALIZE);
        g2d.drawImage(image, 0, 0, width, height, null);
        return shrinkImage ;
    }
    
}