package org.maachang.util.thread ;

import org.maachang.util.atomic.AtomicBOOL;
import org.maachang.util.atomic.AtomicINT;

/**
 * スレッド雛形.
 *  
 * @version 2010/01/07
 * @author  masahito suzuki
 * @since  MaachangBase 1.0B
 */
public abstract class AbstractThread extends Thread {
    
    /**
     * スレッド停止管理フラグ.
     */
    protected final AtomicBOOL stopFlag = new AtomicBOOL( true ) ;
    
    /**
     * スレッドID.
     */
    protected final AtomicINT threadId = new AtomicINT( -1 ) ;
    
    /**
     * ローカルI/O.
     */
    protected final LocalThreadIO local = new LocalThreadIO() ;
    
    /**
     * スレッド開始.
     * @exception Exception 例外.
     */
    public void startThread() throws Exception {
        this.stopFlag.set( false ) ;
        this.setDaemon( true ) ;
        this.start() ;
    }
    
    /**
     * スレッドIDを取得.
     * @return int スレッドIDが返されます.
     */
    public int getThreadId() {
        return threadId.get() ;
    }
    
    /**
     * スレッド停止.
     */
    public void stopThread() {
        this.stopFlag.set( true ) ;
        local.clear() ;
    }
    
    /**
     * スレッドが停止しているかチェック.
     * @return boolean [true]の場合、スレッド停止しています.
     */
    public boolean isStop() {
        return stopFlag.get() ;
    }
    
    /**
     * ローカルIDを取得.
     * @return long ローカルIDが返されます.
     */
    public long getLocalId() {
        return local.getId() ;
    }
    
    /**
     * ローカルI/Oオブジェクトを取得.
     * @return LocalThreadIO ローカルI/Oオブジェクトが返されます.
     */
    public LocalThreadIO getLocalThreadIO() {
        return local ;
    }
}
