package org.maachang.util.thread ;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.maachang.util.atomic.AtomicLongSeq;

/**
 * スレッド雛形.
 *  
 * @version 2010/01/07
 * @author  masahito suzuki
 * @since  MaachangBase 1.0B
 */
public class LocalThreadIO {
    
    /**
     * スレッドセーフ管理.
     */
    protected static final ConcurrentHashMap<Long,Map<String,Object>> _HASH = new ConcurrentHashMap<Long,Map<String,Object>>() ;
    
    /**
     * スレッドセーフ項番発行用.
     */
    protected static final AtomicLongSeq _SEQ = new AtomicLongSeq() ;
    
    /**
     * スレッドセーフ項番管理.
     */
    private long no ;
    
    /**
     * コンストラクタ.
     */
    protected LocalThreadIO() {
        no = _SEQ.get() ;
    }
    
    /**
     * ローカル情報をすべてクリア.
     */
    public void clear() {
        _HASH.remove( no ) ;
    }
    
    /**
     * ローカル情報に追加.
     * @param key 対象のキー名を設定します.
     * @param value 対象の要素を設定します.
     */
    public void put( String key,Object value ) {
        Map<String,Object> m = _HASH.get( no ) ;
        if( m == null ) {
            m = new HashMap<String,Object>() ;
            _HASH.put( no,m ) ;
        }
        m.put( key,value ) ;
    }
    
    /**
     * ローカル情報から削除.
     * @param key 対象のキー名を設定します.
     */
    public void remove( String key ) {
        Map<String,Object> m = _HASH.get( no ) ;
        if( m != null ) {
            m.remove( key ) ;
        }
    }
    
    /**
     * ローカル情報から取得.
     * @param key 対象のキーを設定します.
     * @return Object 要素情報が返されます.
     */
    public Object get( String key ) {
        Map<String,Object> m = _HASH.get( no ) ;
        if( m != null ) {
            return m.get( key ) ;
        }
        return null ;
    }
    
    /**
     * ローカル情報に保管した情報数を取得.
     * @return int 情報数が返されます.
     */
    public int size() {
        Map<String,Object> m = _HASH.get( no ) ;
        if( m != null ) {
            return m.size() ;
        }
        return 0 ;
    }
    
    /**
     * 現在のローカルIDを取得.
     * @return long ローカルIDが返されます.
     */
    public long getId() {
        return no ;
    }
}
