package org.maachang.util.thread ;


/**
 * ループスレッド用オブジェクト.
 *  
 * @version 2008/05/25
 * @author  masahito suzuki
 * @since  MaachangBase 1.00
 */
public abstract class LoopThread extends AbstractThread {
    
    /**
     * オブジェクトクリア処理.
     * <p>スレッドが停止したときに呼び出されます.</p>
     */
    protected abstract void clear() ;
    
    /**
     * 実行処理.
     * <p>スレッド実行を行う実装を行います.</p>
     * @return boolean [false]の場合、Thread.sleep(30)が発生します.
     * @exception Exception 例外.
     */
    protected abstract boolean execution() throws Exception ;
    
    /**
     * 例外発生時の処理を実装.
     * <p>スレッド内で例外が発生した場合の実装です.</p>
     * @param e Throwable 発生した例外が設定されます.
     */
    protected void toException( Throwable e ) {
        
    }
    
    /**
     * スレッド実行処理.
     */
    public void run() {
        boolean endFlag = false ;
        ThreadDeath threadDeach = null ;
        while( true ) {
            if( endFlag == true || stopFlag.get() == true ) {
                break ;
            }
            try {
                if( !execution() ) {
                    Thread.sleep( 30L ) ;
                }
            } catch( InterruptedException ite ) {
                toException( ite ) ;
                endFlag = true ;
            } catch ( NullPointerException ne ) {
                toException( ne ) ;
                try {
                    Thread.sleep( 1L ) ;
                }
                catch( InterruptedException ite2 ) {
                    endFlag = true ;
                }
            } catch( OutOfMemoryError mem ) {
                toException( mem ) ;
                try {
                    Thread.sleep( 1L ) ;
                }
                catch( InterruptedException ite2 ) {
                    endFlag = true ;
                }
            } catch( Exception e ) {
                toException( e ) ;
                try {
                    Thread.sleep( 1L ) ;
                }
                catch( InterruptedException ite2 ) {
                    endFlag = true ;
                }
            } catch( ThreadDeath td ) {
                toException( td ) ;
                endFlag = true ;
                threadDeach = td ;
            }
        }
        clear() ;
        this.stopFlag.set( true ) ;
        if( threadDeach != null ) {
            throw threadDeach ;
        }
    }
    
}
