package org.maachang.dao.dbms;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.maachang.dao.dbms.DbUtil;
import org.maachang.dao.dbms.MetaColumn;
import org.maachang.dao.dbms.MetaFactory;

/**
 * SQLをパース/結合します.
 * 
 * @version 2010/04/06
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class ParseSQL {
    
    /** コンストラクタ **/
    private ParseSQL() {}
    
    /**
     * SQLをパース.
     * @param sql 対象のSQLを設定します.
     * @return List<String> パースされたSQLが返されます.
     * @exception Exception 例外.
     */
    public static final List<String> parse( String sql ) throws Exception {
        if( sql == null || ( sql = sql.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        List<String> ret = new ArrayList<String>() ;
        int len = sql.length() ;
        char c,b ;
        int cote = -1 ;
        int st = -1 ;
        b = 0 ;
        boolean yenFlag = false ;
        for( int i = 0 ; i < len ; i ++ ) {
            c = sql.charAt( i ) ;
            // コーテーション中.
            if( cote != -1 ) {
                // コーテーション終了.
                if( !yenFlag && cote == c ) {
                    cote = -1 ;
                    ret.add( sql.substring( st,i+1 ) ) ;
                    st = -1 ;
                }
            }
            // コーテーション開始.
            else if( !yenFlag && ( c == '\'' || c == '\"' ) ) {
                if( st != -1 ) {
                    ret.add( sql.substring( st,i ) ) ;
                }
                st = i ;
                cote = c ;
            }
            // コーテーション以外.
            else {
                if( c == '!' || c == '<' || c == '>' ) {
                    if( st != -1 && st != i ) {
                        ret.add( sql.substring( st,i ) ) ;
                    }
                    st = -1 ;
                    b = c ;
                    yenFlag = false ;
                    continue ;
                }
                else if( b == '!' || b == '<' || b == '>' ) {
                    if( st != -1 && st != i-1 ) {
                        System.out.println( sql.substring( st,i-1 ) ) ;
                    }
                    if( c == '=' ) {
                        ret.add( new String( new char[]{ b,c } ) ) ;
                    }
                    else {
                        ret.add( new String( new char[]{ b } ) ) ;
                        st = i ;
                        b = 0 ;
                        yenFlag = false ;
                        i -- ;
                        continue ;
                    }
                    st = -1 ;
                }
                else if( c == ' ' || c == '　' || c == '\t' || c == '\r' || c == '\n' ) {
                    if( st != -1 && st != i ) {
                        ret.add( sql.substring( st,i ) ) ;
                    }
                    st = -1 ;
                }
                else if( c == '=' || c == '.' || c == ',' || c == '?' || c == '*' ||
                    c == '(' || c == ')' || c == '[' || c == ']' || c == '{' || c == '}' || c == '}' ) {
                    if( st != -1 && st != i ) {
                        ret.add( sql.substring( st,i ) ) ;
                    }
                    ret.add( new String( new char[]{ c } ) ) ;
                    st = -1 ;
                }
                else if( ( b == '&' && c == '&' ) ||
                    ( b == '|' && c == '|' ) ) {
                    if( st != -1 && st != i-1 ) {
                        ret.add( sql.substring( st,i-1 ) ) ;
                    }
                    if( b == '&' ) {
                        ret.add( "and" ) ;
                    }
                    else {
                        ret.add( "or" ) ;
                    }
                    st = -1 ;
                }
                else if( st == -1 ) {
                    st = i ;
                }
            }
            b = c ;
            if( b == '\\' ) {
                yenFlag = true ;
            }
            else {
                yenFlag = false ;
            }
        }
        // 後処理.
        if( st != -1 && len > st ) {
            ret.add( sql.substring( st ).toLowerCase() ) ;
        }
        return ret ;
    }
    
    /**
     * select/update/delete用パラメータ判別.
     * @param list パースされたSQL文を設定します.
     * @param model 対象のモデル名を設定します.
     * @return List<String> パラメータ判別された内容が返されます.
     * @exception Exception 例外.
     */
    public static final List<String> addParams( List<String> list,String model )
        throws Exception {
        if( list == null || list.size() <= 0 ||
            model == null || ( model = model.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        MetaColumn meta = MetaFactory.getInstance().getMetaColumn( true,model ) ;
        if( meta == null ) {
            throw new IOException( "指定テーブル名[" + model + "]のModelは存在しません" ) ;
        }
        return addParams( list,meta ) ;
    }
    
    /**
     * select/update/delete用パラメータ判別.
     * @param parse パースされたSQL文を設定します.
     * @param meta 対象のメタカラムを設定します.
     * @return List<String> パラメータ判別された内容が返されます.
     * @exception Exception 例外.
     */
    public static final List<String> addParams( List<String> parse,MetaColumn meta )
        throws Exception {
        if( parse == null || parse.size() <= 0 || meta == null ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        int st = -1 ;
        int parCode = -1 ;
        int parCount = 0 ;
        String s,lo ;
        boolean params = false ;
        boolean check ;
        int len = parse.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            check = false ;
            if( st == -1 ) {
                st = i ;
            }
            lo = parse.get( i ).toLowerCase() ;
            if( parCount > 0 ) {
                if( ( parCode == 1 && ")".equals( lo ) ) ||
                    ( parCode == 2 && "]".equals( lo ) ) ||
                    ( parCode == 3 && "}".equals( lo ) ) ) {
                    parCount -- ;
                    if( parCount <= 0 ) {
                        parCode = -1 ;
                        check = true ;
                    }
                }
            }
            else {
                if( "(".equals( lo ) ) {
                    parCode = 1 ;
                    parCount ++ ;
                }
                else if( "[".equals( lo ) ) {
                    parCode = 2 ;
                    parCount ++ ;
                }
                else if( "{".equals( lo ) ) {
                    parCode = 3 ;
                    parCount ++ ;
                }
                else if( "into".equals( lo ) || "where".equals( lo ) ||
                    "order".equals( lo ) || "group".equals( lo ) || "having".equals( lo ) ||
                    "by".equals( lo ) || "desc".equals( lo ) || 
                    "and".equals( lo ) || "or".equals( lo ) ||
                    ",".equals( lo ) || ".".equals( lo ) || "?".equals( lo ) ) {
                    check = true ;
                }
            }
            if( check == true ) {
                params = false ;
                for( int j = st ; j <= i ; j ++ ) {
                    if( "?".equals( parse.get( j ) ) ) {
                        params = true ;
                    }
                }
                if( params ) {
                    for( int j = st ; j <= i ; j ++ ) {
                        s = parse.get( j ) ;
                        if( "?".equals( s ) ) {
                            break ;
                        }
                        if( meta.getColumnNameByNumber( false,s ) >= 0 ) {
                            parse.set( j,"$"+s ) ;
                            break ;
                        }
                        else if( meta.getColumnNameByNumber( true,s ) >= 0 ) {
                            parse.set( j,"$"+DbUtil.convertJavaNameByDBName( s ) ) ;
                            break ;
                        }
                    }
                }
                st = -1 ;
            }
        }
        return parse ;
    }
    
    /**
     * パースしたSQL文を結合.
     * @param parse パースされたSQLを設定します.
     * @return String 結合されたSQL文が返されます.
     * @exception Exception 例外.
     */
    public static final String toSQL( List<String> parse ) throws Exception {
        if( parse == null || parse.size() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        StringBuilder buf = new StringBuilder() ;
        int len = parse.size() ;
        String s ;
        for( int i = 0 ; i < len ; i ++ ) {
            s = parse.get( i ) ;
            if( isValue( s ) ) {
                s = s.substring( 1 ) ;
                buf.append( " " ).append( DbUtil.convertJavaNameByDBName( s ) ) ;
            }
            else if( ".".equals( s ) ) {
                buf.append( s ) ;
            }
            else {
                buf.append( " " ).append( s ) ;
            }
        }
        String ret = buf.toString() ;
        buf = null ;
        if( ret.endsWith( ";" ) == true ) {
            ret.substring( 0,ret.length()-1 ) ;
        }
        return ret.trim() ;
    }
    
    /**
     * 変数文字列一覧を取得.
     * @param parse パースされたSQLを設定します.
     * @return List<String> 変数文字列が返されます.
     * @exception Exception 例外.
     */
    public static final List<String> getValues( List<String> parse )
        throws Exception {
        if( parse == null || parse.size() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        List<String> ret = new ArrayList<String>() ;
        int len = parse.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            String code = parse.get( i ) ;
            if( isValue( code ) == true ) {
                ret.add( code.substring( 1,code.length() ) ) ;
            }
        }
        if( ret.size() > 0 ) {
            return ret ;
        }
        return null ;
    }
    
    /**
     * 変数文字列が存在するかチェック.
     * @param parse パースされたSQLを設定します.
     * @return boolean [true]の場合は存在する.
     * @exception Exception 例外.
     */
    public static final boolean isValues( List<String> parse )
        throws Exception {
        if( parse == null || parse.size() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        int len = parse.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            String code = parse.get( i ) ;
            if( isValue( code ) == true ) {
                return true ;
            }
        }
        return false ;
    }
    
    private static final char[] VALUE_HEAD = { ':','$' } ;
    
    /**
     * 指定文字コードが変数文字の場合.
     */
    private static final boolean isValue( char c ) {
        int len = VALUE_HEAD.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( c == VALUE_HEAD[ i ] ) {
                return true ;
            }
        }
        return false ;
    }
    
    /**
     * 指定文字列が変数であるかチェック.
     */
    private static final boolean isValue( String code ) {
        if( isValue( code.charAt( 0 ) ) == true ) {
            if( code.length() > 1 ) {
                char c = code.charAt( 1 ) ;
                if( ( c >= 'a' && c <= 'z' ) ||
                    ( c >= 'A' && c <= 'Z' ) ||
                    ( c >= '0' && c <= '9' ) ||
                    c == '_' ) {
                    return true ;
                }
            }
        }
        return false ;
    }
    
    /** test **/
    public static final void main( String[] args ) throws Exception {
        String s = "select * from test where a <? ;" ;
        List<String> list = parse( s ) ;
        for( int i = 0 ; i < list.size() ; i ++ ) {
            System.out.println( list.get( i ) ) ;
        }
        
    }
}

