package org.maachang.dao.dbms;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.maachang.util.StringUtil;

/**
 * テーブルソート.
 * 
 * @version 2007/10/18
 * @author masahito suzuki
 * @since MaachangDao 1.00
 */
public class TableSort {
    private static class TableSortChild implements Comparable {
        public int number ;
        public Object target ;
        public int type ;
        public TableSortChild() {
            number = -1 ;
            target = null ;
            type = 0 ;
        }
        public TableSortChild( int n,Object t ) {
            create( n,t ) ;
        }
        public void create( int n,Object t ) {
            number = n ;
            if( t == null ) {
                type = 0 ;// NULL.
                target = null ;
            }
            else if( t instanceof byte[] ) {
                type = 5 ;// binary.
                target = t ;
            }
            else if( t instanceof Boolean ) {
                type = 6 ;// boolean.
                target = t ;
            }
            else if( t instanceof Long ) {
                type = 1 ;// 整数.
                target = t ;
            }
            else if( t instanceof Double ) {
                type = 2 ;// 小数点.
                target = t ;
            }
            else if( t instanceof Byte || t instanceof Character ||
                t instanceof Short || t instanceof Integer ) {
                type = 1 ;// 整数.
                target = ((Number)t).longValue() ;
            }
            else if( t instanceof Float || t instanceof BigDecimal ) {
                type = 2 ;// 小数点.
                target = ((Number)t).doubleValue() ;
            }
            else if( t instanceof java.util.Date ) {
                type = 3 ;// 日付.
                target = ((java.util.Date)t).getTime() ;
            }
            else if( t instanceof String ) {
                String s = ((String)t).trim() ;
                if( s.equals( "true" ) ) {
                    type = 6 ;// boolean.
                    target = Boolean.TRUE ;
                    return ;
                }
                if( s.equals( "false" ) ) {
                    type = 6 ;// boolean.
                    target = Boolean.FALSE ;
                    return ;
                }
                int m = StringUtil.isNumberType( s ) ;
                if( m == 0 ) {
                    type = 1 ; // 数値.
                    target = new Long( s ) ;
                    return ;
                }
                if( m == 1 ) {
                    type = 2 ; // 小数点.
                    target = new Double( s ) ;
                    return ;
                }
                if( m == -1 ) {
                    type = 4 ; // 文字列.
                    target = t ;
                    return ;
                }
            }
            else {
                throw new ClassCastException( "非対応のクラス:" + t.getClass().getName() ) ;
            }
        }
        public int compareTo( Object o ) {
            TableSortChild c = (TableSortChild)o ;
            switch( type ) {
                case 0 :// NULL.
                    if( c.type == 0 ) {
                        return 0 ;
                    }
                    return -1 ;
                case 1 :// 整数.
                    if( c.type == 0 ) {
                        return 1 ;
                    }
                    else if( c.type == 1 || c.type == 2 ) {
                        double x = (Long)target - ((Number)c.target).longValue() ;
                        if( x == 0 ) {
                            return 0 ;
                        }
                        else if( x > 0 ) {
                            return 1 ;
                        }
                        else if( x < 0 ) {
                            return -1 ;
                        }
                    }
                    throw new ClassCastException( "整数オブジェクトと一致しない条件:" + c.type ) ;
                case 2 :// 小数点.
                    if( c.type == 0 ) {
                        return 1 ;
                    }
                    else if( c.type == 1 || c.type == 2 ) {
                        double x = (Double)target - ((Number)c.target).doubleValue() ;
                        if( x == 0 ) {
                            return 0 ;
                        }
                        else if( x > 0 ) {
                            return 1 ;
                        }
                        else if( x < 0 ) {
                            return -1 ;
                        }
                    }
                    throw new ClassCastException( "小数点オブジェクトと一致しない条件:" + c.type ) ;
                case 3 :// 日付.
                    if( c.type == 0 ) {
                        return 1 ;
                    }
                    else if( c.type == 3 ) {
                        long x = (Long)target - (Long)c.target ;
                        if( x == 0 ) {
                            return 0 ;
                        }
                        else if( x > 0 ) {
                            return 1 ;
                        }
                        else if( x < 0 ) {
                            return -1 ;
                        }
                    }
                    throw new ClassCastException( "java.lang.Dateオブジェクトと一致しない条件:" + c.type ) ;
                case 4 :// 文字列.
                    if( c.type == 0 ) {
                        return 1 ;
                    }
                    else if( c.type == 6 ) {
                        
                    }
                    else if( c.type == 4 ) {
                        return ((String)target).compareTo( (String)c.target ) ;
                    }
                    else if( c.type == 1 || c.type == 2 ) {
                        return ((String)target).compareTo( String.valueOf( c.target ) ) ;
                    }
                    throw new ClassCastException( "文字列と一致しない条件:" + c.type ) ;
                case 5 :// binary.
                    if( c.type == 0 ) {
                        return 1 ;
                    }
                    if( c.type == 5 ) {
                        int x = ((byte[])target).length - ((byte[])target).length ;
                        if( x == 0 ) {
                            return 0 ;
                        }
                        else if( x > 0 ) {
                            return 1 ;
                        }
                        else if( x < 0 ) {
                            return -1 ;
                        }
                    }
                    throw new ClassCastException( "バイナリトと一致しない条件:" + c.type ) ;
                case 6 :// Boolean.
                    if( c.type == 0 ) {
                        return 1 ;
                    }
                    else if( c.type == 6 ) {
                        if( (Boolean)target == true ) {
                            return 1 ;
                        }
                        return -1 ;
                    }
                    else if( c.type == 4 ) {
                        if( (Boolean)target == true ) {
                            return "true".compareTo( (String)c.target ) ;
                        }
                        return "false".compareTo( (String)c.target ) ;
                    }
                    throw new ClassCastException( "Booleanと一致しない条件:" + c.type ) ;
            }
            throw new ClassCastException( "不正な条件:" + c.type ) ;
        }
    }
    
    private Object[] table = null ;
    private Object[] back = null ;
    private int length = -1 ;
    private TableSortChild[] sortList = null ;
    
    /**
     * コンストラクタ.
     */
    public TableSort() {
        
    }
    
    /**
     * コンストラクタ.
     * @param t 対象のリスト情報を設定します.
     */
    public TableSort( List<Map<String,Object>> t ) {
        create( t ) ;
    }
    
    /**
     * 情報クリア.
     */
    public void clear() {
        table = null ;
        back = null ;
        length = -1 ;
        sortList = null ;
    }
    
    /**
     * 情報生成.
     * @param t 対象のリスト情報を設定します.
     */
    public void create( List<Map<String,Object>> t ) {
        clear() ;
        if( t == null || t.size() <= 0 ) {
            table = null ;
            length = 0 ;
        }
        else {
            length = t.size() ;
            TableSortChild[] s = new TableSortChild[ length ] ;
            for( int i = 0 ; i < length ; i ++ ) {
                s[ i ] = new TableSortChild() ;
            }
            sortList = s ;
            Object[] o = new Object[ length ] ;
            for( int i = 0 ; i < length ; i ++ ) {
                o[ i ] = t.get( i ) ;
            }
            table = o ;
            back = new Object[ length ] ;
        }
    }
    
    /**
     * ソート処理.
     * 降順でソートします.
     * @param column 対象のカラム名を設定します.
     */
    public void sort( String column ) {
        sort( column,false ) ;
    }
    
    /**
     * ソート処理.
     * @param column 対象のカラム名を設定します.
     * @param desc 昇順の場合はtrueを設定します.
     */
    public void sort( String column,boolean desc ) {
        if( column == null || ( column = column.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "カラム名が設定されていません" ) ;
        }
        if( length <= 0 ) {
            return ;
        }
        for( int i = 0 ; i < length ; i ++ ) {
            back[ i ] = table[ i ] ;
            sortList[ i ].create( i,((Map)table[ i ]).get( column ) ) ;
        }
        Arrays.sort( sortList ) ;
        int n ;
        if( desc ) {
            for( int i = 0,j = length-1 ; i < length ; i ++,j-- ) {
                n = sortList[ j ].number ;
                table[ i ] = back[ n ] ;
            }
        }
        else {
            for( int i = 0 ; i < length ; i ++ ) {
                n = sortList[ i ].number ;
                table[ i ] = back[ n ] ;
            }
        }
    }
    
    /**
     * リスト変換して取得.
     * @return List<Map<Object,Object>> 変換されたリスト情報が返されます.
     */
    public List<Map<Object,Object>> getList() {
        if( length <= 0 ) {
            return new ArrayList<Map<Object,Object>>() ;
        }
        List<Map<Object,Object>> ret = new ArrayList<Map<Object,Object>>() ;
        for( int i = 0 ; i < length ; i ++ ) {
            ret.add( (Map<Object,Object>)table[ i ] ) ;
        }
        return ret ;
    }
    
    /**
     * 情報を取得.
     * @param num 対象の項番を設定します.
     * @return Object 取得された情報が返されます.
     */
    public Object get( int num ) {
        if( num <= -1 || length <= num ) {
            return null ;
        }
        return table[ num ] ;
    }
    
    /**
     * 情報長を取得.
     * @return int length 情報長が返されます.
     */
    public int size() {
        return length ;
    }
    
}

