package org.maachang.dao.dbms.ctbl;

import gnu.trove.THashMap;
import gnu.trove.THashSet;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.conf.Config;
import org.maachang.conf.ReadIni;
import org.maachang.dao.DaoSessionFactory;
import org.maachang.dao.dbms.DbUtil;
import org.maachang.dao.dbms.Record;
import org.maachang.dao.dbms.RecordFactory;
import org.maachang.dao.dbms.kind.SupportKind;
import org.maachang.util.Digest;
import org.maachang.util.FileUtil;
import org.maachang.util.StringUtil;

/**
 * テーブル生成オブジェクト.
 * 
 * @version 2007/10/18
 * @author masahito suzuki
 * @since MaachangDao 1.00
 */
public abstract class CreateTable {
    protected CreateTable() {}
    
    /**
     * LOG.
     */
    protected static final Log LOG = LogFactory.getLog( CreateTable.class ) ;
    
    /**
     * バージョン管理ディレクトリ.
     */
    protected static final String VERSION_DIR = "_version" ;
    
    /**
     * 取込除外情報格納ファイル名.
     */
    protected static final String NON_ACCESS_DB_MANAGER_FILE = "_non_access_db.list" ;
    
    /**
     * 結合元テーブルを示す条件.
     */
    protected static final String SRC_JOIN = TableMaster.SRC_JOIN ;
    
    /**
     * シーケンス条件.
     */
    protected static final String SEQUENCE_MODE = TableMaster.SEQUENCE_MODE ;
    
    /**
     * デフォルトソート条件.
     */
    protected static final String DEFAULT_SORT_MODE = TableMaster.DEFAULT_SORT_MODE ;
    
    /**
     * 表示カラム条件.
     */
    protected static final String VIEW_COLUMN_NAMES = TableMaster.VIEW_COLUMN_NAMES ;
    
    /**
     * 名前管理マスター条件.
     */
    protected static final String NAMES_MASTER_MODE = TableMaster.NAMES_MASTER_MODE ;
    
    /**
     * マスターキャッシュ条件.
     */
    protected static final String MASTER_CACHE_MODE = TableMaster.MASTER_CACHE_MODE ;
    
    /**
     * マスター編集条件.
     */
    protected static final String MASTER_EDIT_MODE = TableMaster.MASTER_EDIT_MODE ;
    
    /**
     * マスター編集条件.
     */
    protected static final String MASTER_CREATE_MODE = TableMaster.MASTER_CREATE_MODE ;
    
    /**
     * テーブル名を示す定義.
     */
    protected static final String TABLE_NAME = TableMaster.TABLE_NAME ;
    
    /**
     * テーブル区分を示す定義.
     */
    protected static final String TABLE_DIVISION = TableMaster.TABLE_DIVISION ;
    
    /**
     * データ削除確認フラグ(テーブルカラム名).
     */
    protected static final String DELETE_FLAG_COLUMN = TableMaster.DELETE_FLAG_COLUMN ;
    
    /**
     * 検索表示フラグ(テーブルカラム名).
     */
    protected static final String SEARCH_FLAG_COLUMN = TableMaster.SEARCH_FLAG_COLUMN ;
    
    /**
     * テーブル生成モード : ヘルプ.
     */
    protected static final String PARAM_HELP = "?" ;
    
    /**
     * テーブル生成モード : 生成.
     */
    protected static final String PARAM_CREATE = "create" ;
    
    /**
     * テーブル生成モード : テーブル削除.
     */
    protected static final String PARAM_DROP = "drop" ;
    
    /**
     * テーブル生成モード : Model再生性.
     */
    protected static final String PARAM_MODEL = "model" ;
    
    /**
     * テーブル生成モード : マスター情報セット.
     */
    protected static final String PARAM_MASTER = "master" ;
    
    /**
     * テーブル生成モード : バージョン表示.
     */
    protected static final String PARAM_VERSION = "version" ;
    
    /**
     * バージョンコンフィグ拡張子.
     */
    protected static final String TABLE_CONF = ".conf" ;
    
    /**
     * モデルファイルフッダ名.
     */
    protected static final String MODEL_NAME_FOODER = "Model.ms" ;
    
    /**
     * トランザクションモード.
     * [true]の場合、autoCommit.
     */
    public static boolean TRANSACTION_MODE = true ;
    
    /**
     * テーブル生成.
     * <BR><BR>
     * テーブル生成処理を実行します.
     * <BR>
     * @param mode [false]の場合、バッチ処理で、テーブル生成を行います.
     * @param basePath 対象のテーブルコンフィグディレクトリを設定します.
     * @param outDir モデルファイル出力先ディレクトリ名を設定します.
     * @param factory 対象のレコードファクトリを生成します.
     * @exception Exception 例外.
     */
    public static final void create( boolean mode,String basePath,String outDir,String param,RecordFactory factory )
        throws Exception {
        if( param == null || ( param = param.trim() ).length() <= 0 ) {
            param = PARAM_CREATE ;
        }
        else {
            param = param.toLowerCase() ;
        }
        // ヘルプ表示.
        if( PARAM_HELP.equals( param ) ) {
            viewHelp() ;
            return ;
        }
        if( basePath == null || ( basePath = basePath.trim() ).length() <= 0 ) {
            if( mode == false ) {
                System.out.println( "テーブルコンフィグディレクトリは存在しません" ) ;
                return ;
            }
            throw new IllegalArgumentException( "テーブルコンフィグディレクトリは存在しません" ) ;
        }
        if( factory == null ) {
            if( mode == false ) {
                System.out.println( "データベース定義は存在しません." ) ;
                return ;
            }
            throw new IllegalArgumentException( "データベース定義は存在しません." ) ;
        }
        SupportKind dbms = factory.getSupportKind() ;
        if( dbms == null ) {
            if( mode == false ) {
                System.out.println( "アダプタ名が非サポートのデータベースです." ) ;
                return ;
            }
            throw new IOException( "アダプタ名が非サポートのデータベースです." ) ;
        }
        if( mode == false ) {
            System.out.println( "## startup create table" ) ;
        }
        else {
            LOG.info( "## startup create table" ) ;
        }
        Record record = null ;
        
        try {
            Config tableConfig = null ;
            Config lastVersionConfig = null ;
            boolean execVersion = false ;
            
            // アダプタ名から、対象のテーブルオブジェクトを取得.
            Table table = TableMaster.get( dbms.getAdapter() ) ;
            
            // 現在の最大バージョンの内容を取得.
            int version = getVersion( basePath ) ;
            
            // バージョン指定で実行する場合.
            if( isNumber( param ) ) {
                tableConfig = getVersionConfig( basePath,param ) ;
                if( tableConfig == null || tableConfig.size() <= 0 ) {
                    if( mode == false ) {
                        System.out.println( "読み込み可能なコンフィグファイルは存在しません." ) ;
                    }
                    else {
                        LOG.info( "読み込み可能なコンフィグファイルは存在しません." ) ;
                    }
                    return ;
                }
                execVersion = true ;
            }
            // バージョン表示.
            else if( param.endsWith( PARAM_VERSION ) ) {
                System.out.println( "version " + version ) ;
                return ;
            }
            // マスターファイルをDBに書き込む.
            else if( param.endsWith( PARAM_MASTER ) ) {
                if( mode == false ) {
                    System.out.println( "masterファイルをDBに書き込みます" ) ;
                }
                else {
                    LOG.info( "masterファイルをDBに書き込みます" ) ;
                }
                // コンフィグ内容を読み込む.
                tableConfig = readTableConfig( mode,false,basePath ) ;
                removeAllEnhancingDefine( true,tableConfig ) ;
                record = factory.getRecord() ;
                record.getConnection().setAutoCommit( TRANSACTION_MODE ) ;
                InsertTable.insert( mode,getNonAccessList(),table,tableConfig,record,basePath ) ;
                return ;
            }
            // 通常実行.
            else {
                // テーブルを削除する場合.
                if( param.equals( PARAM_DROP ) ) {
                    if( mode == false ) {
                        System.out.println( "テーブルを削除." ) ;
                    }
                    else {
                        LOG.info( "テーブルを削除." ) ;
                    }
                    if( version > 0 ) {
                        tableConfig = getVersionConfig( basePath,String.valueOf( version ) ) ;
                    }
                    if( tableConfig == null ) {
                        // バージョン管理のテーブルコンフィグが読み込めない場合は、現在のテーブル内容を
                        // 読み込む.
                        tableConfig = readTableConfig( mode,false,basePath ) ;
                        if( tableConfig == null || tableConfig.size() <= 0 ) {
                            if( mode == false ) {
                                System.out.println( "読み込み可能なコンフィグファイルは存在しません." ) ;
                            }
                            else {
                                LOG.info( "読み込み可能なコンフィグファイルは存在しません." ) ;
                            }
                            return ;
                        }
                    }
                    record = factory.getRecord() ;
                    record.getConnection().setAutoCommit( TRANSACTION_MODE ) ;
                    
                    dropTable( mode,table,record,tableConfig ) ;
                    deleteVersionInfo( basePath ) ;
                    if( DaoSessionFactory.getInstance().isJavaSequence() ) {
                        DaoSessionFactory.getInstance().deleteJavaSequence() ;
                        if( mode == false ) {
                            System.out.println( "Javaシーケンスファイルの削除." ) ;
                        }
                        else {
                            LOG.info( "Javaシーケンスファイルの削除." ) ;
                        }
                    }
                    return ;
                }
                // テーブル名に合わせたModelファイルを出力.
                else if( param.equals( PARAM_MODEL ) ) {
                    // コンフィグ内容を読み込む.
                    tableConfig = readTableConfig( mode,true,basePath ) ;
                    if( tableConfig == null || tableConfig.size() <= 0 ) {
                        if( mode == false ) {
                            System.out.println( "読み込み可能なコンフィグファイルは存在しません." ) ;
                        }
                        else {
                            LOG.info( "読み込み可能なコンフィグファイルは存在しません." ) ;
                        }
                        return ;
                    }
                    if( mode == false ) {
                        System.out.println( "modelファイルのみを出力." ) ;
                    }
                    else {
                        LOG.info( "modelファイルのみを出力." ) ;
                    }
                    // Modelファイルを全削除.
                    removeModelFile( outDir ) ;
                    
                    // Modelファイルを強制出力.
                    fileOutModel( table,dbms,outDir,tableConfig ) ;
                    return ;
                }
                
            }
            
            // 最終バージョンファイルを取得.
            if( version > 0 ) {
                lastVersionConfig = getVersionConfig( basePath,String.valueOf( version ) ) ;
            }
            
            if( table == null ) {
                if( mode == false ) {
                    System.out.println( "対象アダプタ名[" + dbms.getAdapter() + "]はサポートされていません" ) ;
                }
                else {
                    LOG.info( "対象アダプタ名[" + dbms.getAdapter() + "]はサポートされていません" ) ;
                }
                return ;
            }
            
            // Modelファイルを全削除.
            removeModelFile( outDir ) ;
            
            // Modelファイルを出力.
            fileOutModel( table,dbms,outDir,readTableConfig( mode,true,basePath ) ) ;
            
            String sql = null ;
            tableConfig = readTableConfig( mode,false,basePath ) ;
            // バージョンファイルが存在しない場合は、全SQLを生成.
            if( lastVersionConfig == null ) {
                sql = table.createTable( tableConfig ) ;
            }
            // 最後のバージョン条件と比較して、同じ場合は処理しない.
            else if( equlasConfig( tableConfig,lastVersionConfig ) == true ) {
                if( mode == false ) {
                    System.out.println( "テーブル内容に変更点はありません" ) ;
                }
                else {
                    LOG.info( "テーブル内容に変更点はありません" ) ;
                }
                return ;
            }
            // 最後のバージョン条件と比較して、同じでない場合は、部分実行.
            else {
                sql = partSQL( table,tableConfig,lastVersionConfig ) ;
            }
            
            // 実行可能なSQL文が存在しない場合.
            if( sql == null || ( sql = sql.trim() ).length() <= 0 ) {
                if( mode == false ) {
                    System.out.println( "実行可能なSQL文は存在しません" ) ;
                }
                else {
                    LOG.info( "実行可能なSQL文は存在しません" ) ;
                }
                return ;
            }
            
            // テーブル生成.
            record = factory.getRecord() ;
            record.getConnection().setAutoCommit( TRANSACTION_MODE ) ;
            executionBySql( mode,record,sql ) ;
            
            // バージョンファイルを実行した場合.
            if( execVersion == true ) {
                if( mode == false ) {
                    System.out.println( "## version " + param + TABLE_CONF + " を実行." ) ;
                }
                else {
                    LOG.info( "## version " + param + TABLE_CONF + " を実行." ) ;
                }
            }
            
            // バージョンファイルを出力.
            outputVersionConfig( mode,basePath,version,tableConfig ) ;
            
        } catch( Exception e ) {
            if( record != null ) {
                try {
                    record.rollback() ;
                } catch( Exception ee ) {
                }
            }
            if( mode == false ) {
                System.out.println( "## error" ) ;
                e.printStackTrace() ;
            }
            else {
                LOG.info( "## error",e ) ;
            }
        } finally {
            if( record != null ) {
                try {
                    record.close() ;
                } catch( Exception ee ) {
                }
            }
        }
    }
    
    /**
     * テーブル再定義.
     * @param mode モードを設定します.
     * @param sectionName 対象のセクション名(テーブル名)を設定します.
     * @param table 対象のテーブルオブジェクトを設定します.
     * @param config 対象のテーブルコンフィグオブジェクトを設定します.
     * @param record データベース接続(JDBC)オブジェクトを設定します.
     */
    public static final void recreateTable( boolean mode,String sectionName,Table table,Config config,Record record )
        throws Exception {
        StringBuilder buf = new StringBuilder() ;
        String recreate = table.dropPrimaryKey( sectionName ) ;
        if( recreate != null ) {
            buf.append( recreate ) ;
        }
        recreate = table.dropSequence( sectionName ) ;
        if( recreate != null ) {
            buf.append( recreate ) ;
        }
        recreate = table.createTable( sectionName,config ) ;
        if( recreate != null ) {
            buf.append( recreate ) ;
            recreate = buf.toString() ;
            buf = null ;
            executionBySql( mode,record,recreate ) ;
        }
    }
    
    /**
     * tableディレクトリ以下の定義ファイルを読み込む.
     * @param mode モードを設定します.
     * @param all trueの場合は、全データを取得対象とします.
     * @param path テーブルディレクトリ名を設定します.
     * @return Config コンフィグ情報が返されます.
     */
    public static final Config readTableConfig( boolean mode,boolean all,String path )
        throws Exception {
        String readPath = FileUtil.getFullPath( path )+FileUtil.FILE_SPACE ;
        Config conf = new Config() ;
        Set<String> nonAccessList = null ;
        if( all ) {
            nonAccessList = new HashSet<String>() ;
        }
        else {
            nonAccessList = getNonAccessList() ;
        }
        int cnt = readByDir( mode,all,nonAccessList,conf,0,readPath ) ;
        if( !all ) {
            if( mode == false ) {
                System.out.println( "@@@@ read config [END:" + cnt + "]" ) ;
            }
            else {
                LOG.info( "@@@@ read config [END:" + cnt + "]" ) ;
            }
        }
        return conf ;
    }
    
    /**
     * SQL文を実行.
     */
    protected static final void executionBySql( boolean mode,Record record,String sql )
        throws Exception {
        BufferedReader buf = null ;
        try {
            long tm = System.currentTimeMillis() ;
            buf = new BufferedReader( new StringReader( sql ) ) ;
            while( true ) {
                String one = buf.readLine() ;
                if( one == null ) {
                    break ;
                }
                if( mode == false ) {
                    System.out.println( ">"+one ) ;
                }
                else {
                    LOG.info( ">"+one ) ;
                }
                try {
                    record.executeUpdate( one ) ;
                    try {
                        record.commit() ;
                    } catch( Exception ee ) {
                        try {
                            record.rollback() ;
                        } catch( Exception eee ) {
                        }
                    }
                    if( mode == false ) {
                        System.out.println( ".....OK." ) ;
                    }
                    else {
                        LOG.info( ".....OK." ) ;
                    }
                } catch( Exception e ) {
                    try {
                        record.rollback() ;
                    } catch( Exception ee ) {
                    }
                    String x = one.trim().toLowerCase() ;
                    if( x.startsWith( "create " ) ||
                        ( x.startsWith( "alter " ) && x.indexOf( " add " ) != -1 ) ) {
                        throw e ;
                    }
                    if( mode == false ) {
                        System.out.println( ".....>>"+e ) ;
                    }
                    else {
                        LOG.info( ".....>>"+e ) ;
                    }
                }
                if( mode == false ) {
                    System.out.println( "" ) ;
                }
            }
            try {
                record.commit() ;
            } catch( Exception ee ) {
                try {
                    record.rollback() ;
                } catch( Exception eee ) {
                }
            }
            if( mode == false ) {
                System.out.println( "## success:" + (System.currentTimeMillis()-tm)+"ms" ) ;
            }
            else {
                LOG.info( "## success:" + (System.currentTimeMillis()-tm)+"ms" ) ;
            }
        } finally {
            if( buf != null ) {
                try {
                    buf.close() ;
                } catch( Exception e ) {
                }
            }
        }
    }
    
    /**
     * テーブルを削除.
     */
    protected static final void dropTable( boolean mode,Table table,Record record,Config config )
        throws Exception {
        Object[] sections = config.getSections() ;
        if( sections != null && sections.length > 0 ) {
            int len = sections.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( sections[ i ] != null &&
                    ( ( String )( sections[ i ] = ( ( String )sections[ i ] ).trim() ) ).length() > 0 ) {
                    for( int j = 0 ; j < 3 ; j ++ ) {
                        String dropSQL = null ;
                        try {
                            switch( j ) {
                                case 0 :
                                dropSQL = "drop table " + sections[ i ] +  table.getSemicolon() + "\n" ;
                                break ;
                                case 1 :
                                dropSQL = "drop sequence " + sections[ i ] + SupportKind.SEQ_NAME + " " + table.getSemicolon() + "\n" ;
                                break ;
                                case 2 :
                                dropSQL = "alter table " + sections[ i ] + " drop constraint " + sections[ i ] + "_pk " + table.getSemicolon() + "\n" ;
                                break ;
                            }
                            if( dropSQL != null ) {
                                if( mode == false ) {
                                    System.out.println( ">" + dropSQL ) ;
                                }
                                else {
                                    LOG.info( ">" + dropSQL ) ;
                                }
                                record.executeUpdate( dropSQL ) ;
                                try {
                                    record.commit() ;
                                } catch( Exception ee ) {
                                    try {
                                        record.rollback() ;
                                    } catch( Exception eee ) {
                                    }
                                }
                            }
                        } catch( Exception e ) {
                            if( mode == false ) {
                                System.out.println( "error:" + e.getMessage() ) ;
                            }
                            else {
                                LOG.info( "error:" + e.getMessage() ) ;
                            }
                            try {
                                record.rollback() ;
                            } catch( Exception ee ) {
                            }
                        }
                    }
                    if( mode == false ) {
                        System.out.println( "drop table " + sections[ i ] ) ;
                    }
                    else {
                        LOG.info( "drop table " + sections[ i ] ) ;
                    }
                }
            }
        }
    }
    
    /**
     * 指定ディレクトリ以下のコンフィグファイルを読み込む.
     */
    protected static final int readByDir( boolean mode,boolean all,Set<String> nonAccessList,Config conf,int cnt,String base )
        throws Exception {
        String dir = base ;
        if( dir.endsWith( "/" ) == false && dir.endsWith( "\\" ) == false ) {
            dir += FileUtil.FILE_SPACE ;
        }
        String[] list = FileUtil.getFileDirList( dir ) ;
        if( list != null && list.length > 0 ) {
            int len = list.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                String c = dir + list[ i ] ;
                if( FileUtil.isDirExists( c ) == true ) {
                    if( VERSION_DIR.equals( list[ i ] ) ) {
                        continue ;
                    }
                    cnt = readByDir( mode,all,nonAccessList,conf,cnt,c ) ;
                }
                else if( FileUtil.isRead( c ) && c.endsWith( TABLE_CONF ) ) {
                    if( nonAccessList.contains( list[ i ].substring( 0,list[ i ].length() - 5 ) ) ) {
                        if( mode == false ) {
                            System.out.println( "-- non-access:[" + list[ i ] + "]" ) ;
                        }
                        else {
                            LOG.info( "-- non-access:[" + list[ i ] + "]" ) ;
                        }
                        continue ;
                    }
                    if( !all ) {
                        if( mode == false ) {
                            System.out.println( "-- readFile:[" + list[ i ] + "]" ) ;
                        }
                        else {
                            LOG.info( "-- readFile:[" + list[ i ] + "]" ) ;
                        }
                    }
                    getIniParams( conf,c ) ;
                    cnt ++ ;
                }
                list[ i ] = null ;
            }
        }
        return cnt ;
    }
    
    /**
     * Configファイルを取得.
     */
    protected static final void getIniParams( Config out,String name )
        throws Exception {
        BufferedReader buf = null ;
        try {
            buf = new BufferedReader( new InputStreamReader( new FileInputStream( name ),"UTF8" ) ) ;
            ReadIni.analisys( out,buf ) ;
            buf.close() ;
            buf = null ;
        } finally {
            if( buf != null ) {
                try {
                    buf.close() ;
                } catch( Exception e ) {
                }
            }
            buf = null ;
        }
    }
    
    /**
     * 全モデルファイルを削除.
     */
    protected static final void removeModelFile( String outDir )
        throws Exception {
        String[] list = FileUtil.getList( outDir ) ;
        if( list == null || list.length < 0 ) {
            return ;
        }
        int len = list.length ;
        String s ;
        outDir = FileUtil.getFullPath( outDir ) ;
        if( outDir.endsWith( "/" ) ) {
            outDir = outDir.substring( 0,outDir.length()-1 ) ;
        }
        for( int i = 0 ; i < len ; i ++ ) {
            s = list[ i ] ;
            if( s.endsWith( MODEL_NAME_FOODER ) ) {
                FileUtil.removeFile( outDir + "/" + s ) ;
            }
        }
    }
    
    /**
     * テーブル一覧に対するモデルファイルを生成.
     */
    protected static final void fileOutModel( Table table,SupportKind dbms,String outDir,Config config )
        throws Exception {
        Object[] o = config.getSections() ;
        if( o == null && o.length <= 0 ) {
            return ;
        }
        if( outDir.endsWith( "\\" ) || outDir.endsWith( "/" ) ) {
            outDir = outDir.substring( 0,outDir.length() - 1 ) ;
        }
        int len = o.length ;
        String date = new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss" ).format( new Date() ) ;
        for( int i = 0 ; i < len ; i ++ ) {
            String s = ( String )o[ i ] ;
            if( s == null || ( s = s.trim() ).length() <= 0 ) {
                continue ;
            }
            String section = s ;
            boolean seqFlag = isSequence( config,section ) ;
            String javaName = DbUtil.convertDBNameByJavaName( true,s ) ;
            s = outDir + "/" + javaName + MODEL_NAME_FOODER ;
            // テーブルマスターオブジェクトファイルは、毎回再定義する.
            StringBuilder buf = new StringBuilder() ;
            buf.append( "// " ).append( javaName ).append( " モデル." ).append( "\n" ) ;
            buf.append( "// " ).append( date ).append( "\n" ) ;
            buf.append( "// " ).append( "\n" ) ;
            
            // 表示用テーブル名を設定.
            buf.append( "\n" ).
                append( "// ** テーブル名 **\n" ).
                append( javaName ).append( "." ).append( TableMaster.TABLE_NAME_DEFINE ).
                append( "=\"" ) ;
            if( config.size( section,TABLE_NAME ) > 0 ) {
                buf.append( config.get( section,TABLE_NAME,0 ) ) ;
            }
            else {
                buf.append( javaName ) ;
            }
            buf.append( "\";\n" ) ;
            
            // オブジェクト名を設定.
            buf.append( "\n" ).
                append( "// ** オブジェクト名. **\n" ).
                append( javaName ).append( ".name=\"" ).append( javaName ).append( "\";\n" ) ;
            
            // テーブルカラム一覧を定義.
            String res = getTableColumns( config,section,seqFlag ) ;
            if( res != null ) {
                buf.append( "\n" ).
                    append( res ) ;
            }
            
            // テーブルカラムタイプ群を定義.
            res = getTableTypes( config,section,seqFlag ) ;
            if( res != null ) {
                buf.append( "\n" ).
                    append( res ) ;
            }
            
            // テーブル区分定義を設定.
            if( config.size( section,TABLE_DIVISION ) > 0 ) {
                buf.append( "\n" ).
                    append( "// ** テーブル区分 **\n" ).
                    append( javaName ).append( "." ).append( TableMaster.TABLE_DIVISION_DEFINE ).
                    append( "=\"" ) ;
                    buf.append( config.get( section,TABLE_DIVISION,0 ) ).
                    append( "\";\n" ) ;
            }
            
            // src_join条件を出力.
            res = srcJoinInfo( config,section ) ;
            if( res != null ) {
                buf.append( "\n" ) ;
                buf.append( res ) ;
            }
            
            // join条件を出力.
            res = joinInfo( config,section ) ;
            if( res != null ) {
                buf.append( "\n" ) ;
                buf.append( res ) ;
            }
            
            // シーケンス条件を出力.
            res = sequenceInfo( config,section,seqFlag ) ;
            if( res != null ) {
                buf.append( "\n" ) ;
                buf.append( res ) ;
            }
            
            // デフォルトソート条件を出力.
            res = defaultSort( config,section ) ;
            if( res != null ) {
                buf.append( "\n" ) ;
                buf.append( res ) ;
            }
            
            // 表示用カラム名群を出力.
            res = viewColumns( config,section ) ;
            if( res != null ) {
                buf.append( "\n" ) ;
                buf.append( res ) ;
            }
            
            // デフォルト条件を出力.
            res = defaultParam( config,section,table,dbms ) ;
            if( res != null ) {
                buf.append( "\n" ) ;
                buf.append( res ) ;
            }
            res = null ;
            
            // 削除フラグカラムが存在する場合.
            if( config.isKeys( section,DELETE_FLAG_COLUMN ) == true ) {
                buf.append( "\n" ) ;
                buf.append( "// ** 削除フラグ. **\n" ) ;
                buf.append( javaName ) ;
                buf.append( "." ).append( TableMaster.IS_DELETE_DEFINE ).append( "=true ;\n" ) ;
            }
            
            // 検索フラグカラムが存在する場合.
            if( config.isKeys( section,SEARCH_FLAG_COLUMN ) == true ) {
                buf.append( "\n" ) ;
                buf.append( "// ** 検索フラグ. **\n" ) ;
                buf.append( javaName ) ;
                buf.append( "." ).append( TableMaster.IS_SEARCH_DEFINE ).append( "=true ;\n" ) ;
            }
            
            // マスターキャッシュ条件を出力.
            if( section.endsWith( "master" ) || config.getBoolean( section,MASTER_CACHE_MODE,0 ) == true ) {
                
                // 名前管理マスター条件を出力.
                res = namesMasterInfo( config,section ) ;
                if( res != null ) {
                    buf.append( "\n" ) ;
                    buf.append( res ) ;
                }
                
                // マスターキャッシュ条件を出力.
                res = masterCache( config,section ) ;
                if( res != null ) {
                    buf.append( "\n" ) ;
                    buf.append( res ) ;
                }
                res = null ;
                
            }
            // マスター編集条件を出力.
            if( section.endsWith( "master" ) ) {
                
                // マスター編集条件を出力.
                res = masterEdit( config,section ) ;
                if( res != null ) {
                    buf.append( "\n" ) ;
                    buf.append( res ) ;
                }
                res = null ;
                
                // マスター新規作成条件を出力.
                res = masterCreate( config,section ) ;
                if( res != null ) {
                    buf.append( "\n" ) ;
                    buf.append( res ) ;
                }
                res = null ;
                
            }
            
            // テーブル詳細用定義.
            DetailTable.create( buf,config,section,javaName,seqFlag ) ;
            
            // 定義終了.
            buf.append( "\n" ) ;
            FileUtil.setFileByString( s,buf.toString() ) ;
            buf = null ;
            // 後始末.
            removeEnhancingDefine( false,config,section ) ;
        }
    }
    
    /**
     * 全てのテーブル拡張定義を削除.
     */
    protected static final void removeAllEnhancingDefine( boolean mode,Config config ) {
        Object[] o = config.getSections() ;
        if( o != null && o.length > 0 ) {
            int len = o.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                removeEnhancingDefine( mode,config,(String)o[i] ) ;
            }
        }
    }
    
    /**
     * テーブル拡張定義を削除.
     */
    protected static final void removeEnhancingDefine( boolean mode,Config config,String section ) {
        String[] keys = config.getKeys( section ) ;
        if( keys != null && keys.length > 0 ) {
            int len = keys.length ;
            String s ;
            for( int i = 0 ; i < len ; i ++ ) {
                s = keys[ i ] ;
                if( !isSpecialColumnName( s ) ) {
                    if( mode && s.startsWith( TableMaster.SPECIAL_DEFINE ) ) {
                        continue ;
                    }
                    config.removeKey( section,s ) ;
                }
            }
        }
    }
    
    /**
     * テーブルカラム一覧を取得.
     */
    protected static final String getTableColumns( Config config,String section,boolean seqFlag ) {
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        if( names != null && names.size() > 0 ) {
            int len = names.size() ;
            String javaName = DbUtil.convertDBNameByJavaName( true,section ) ;
            StringBuilder buf = new StringBuilder() ;
            buf.append( "// ** カラム一覧 **\n" ).
                append( javaName ).append( "." ).append( TableMaster.TABLE_COLUMNS_DEFINE ).
                append( "=[" ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                buf.append( "\"" ).append( names.get( i ) ).append( "\"" ) ;
            }
            buf.append( "];\n" ) ;
            return buf.toString() ;
        }
        return null ;
    }
    
    /**
     * テーブルタイプ一覧を取得.
     */
    protected static final String getTableTypes( Config config,String section,boolean seqFlag ) {
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        String s ;
        if( len > 0 ) {
            String javaName = DbUtil.convertDBNameByJavaName( true,section ) ;
            StringBuilder buf = new StringBuilder() ;
            buf.append( "// ** カラムタイプ定義. **" ).
                append( "\n" ).
                append( javaName ).append( "." ).append( TableMaster.TABLE_TYPES_DEFINE ).
                append( "=[" ) ;
            int lenJ ;
            String v ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                s = names.get( i ) ;
                // シーケンスID発行が行われるテーブルの場合は、強制的にlong.
                if( s.equals( "id" ) && isSequence( config,section ) ) {
                    buf.append( "\"" ).append( "long" ).append( "\"" ) ;
                    continue ;
                }
                lenJ = config.size( section,s ) ;
                for( int j = 0 ; j < lenJ ; j ++ ) {
                    v = config.get( section,s,j ) ;
                    if( Table.BOOLEAN.equals( v ) ) {
                        buf.append( "\"" ).append( "boolean" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.INTEGER.equals( v ) ) {
                        buf.append( "\"" ).append( "int" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.LONG.equals( v ) ) {
                        buf.append( "\"" ).append( "long" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.FLOAT.equals( v ) ) {
                        buf.append( "\"" ).append( "float" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.DOUBLE.equals( v ) ) {
                        buf.append( "\"" ).append( "double" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.BINARY.equals( v ) ) {
                        buf.append( "\"" ).append( "binary" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.CHAR.equals( v ) || v.startsWith( Table.CHAR ) ) {
                        buf.append( "\"" ).append( "string" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.TEXT.equals( v ) ) {
                        buf.append( "\"" ).append( "text" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.DATE.equals( v ) ) {
                        buf.append( "\"" ).append( "date" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.TIME.equals( v ) ) {
                        buf.append( "\"" ).append( "time" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.TIMESTAMP.equals( v ) ) {
                        buf.append( "\"" ).append( "timestamp" ).append( "\"" ) ;
                        break ;
                    }
                }
            }
            return buf.append( "];\n" ).toString() ;
        }
        return null ;
    }
    
    /**
     * src_join条件を取得.
     */
    protected static final String srcJoinInfo( Config config,String section ) {
        String[] srcJoin = config.getAll( section,SRC_JOIN ) ;
        if( srcJoin != null && srcJoin.length > 0 ) {
            StringBuilder buf = new StringBuilder() ;
            int len = srcJoin.length ;
            String javaName = DbUtil.convertDBNameByJavaName( true,section ) ;
            buf.append( "// ** 結合元カラム. **\n" ) ;
            buf.append( javaName ).
                append( "." ).append( TableMaster.SRC_JOIN_DEFINE ).append( " = [] ;\n" ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                buf.append( javaName ).append( "." ).
                    append( "srcJoin.add(" ).
                    append( " \"" ).
                    append( DbUtil.convertDBNameByJavaName( true,srcJoin[i] ) ).
                    append( "\" ) ;\n" ) ;
            }
            return buf.toString() ;
        }
        return null ;
    }
    
    /**
     * Join条件を取得.
     */
    protected static final String joinInfo( Config config,String section ) {
        String[] keys = config.getKeys( section ) ;
        if( keys != null && keys.length > 0 ) {
            int len = keys.length ;
            String javaName = DbUtil.convertDBNameByJavaName( true,section ) ;
            StringBuilder buf = new StringBuilder() ;
            buf.append( "// ** 結合カラム. **\n" ) ;
            buf.append( javaName ).append( TableMaster.JOIN ).append( " = {} ;\n" ) ;
            int cnt = 0 ;
            for( int i = 0 ; i < len ; i ++ ) {
                String key = keys[ i ].toLowerCase() ;
                if( key.endsWith( TableMaster.JOIN ) ) {
                    String value = config.get( section,keys[ i ],0 ) ;
                    if( value != null && ( value = value.trim() ).length() > 0 ) {
                        String joinKey = key.substring( 0,key.length() - TableMaster.JOIN.length() ) ;
                        joinKey = DbUtil.convertDBNameByJavaName( false,joinKey ) ;
                        buf.append( javaName ).append( TableMaster.JOIN ).append( "[\"" ).append( joinKey ).append( "\"] = " ).
                            append( "\"" ).append( DbUtil.convertDBNameByJavaName( true,value ) ).
                            append( "\" ;\n" ) ;
                        cnt ++ ;
                    }
                }
            }
            keys = null ;
            if( cnt != 0 ) {
                return buf.toString() ;
            }
        }
        return null ;
    }
    
    /** シーケンス条件を取得. **/
    protected static final boolean isSequence( Config config,String section ) {
        if( config.size( section,SEQUENCE_MODE ) > 0 ) {
            if( config.getBoolean( section,SEQUENCE_MODE,0 ) == false ) {
                return false ;
            }
            else {
                return true ;
            }
        }
        return false ;
    }
    
    /**
     * シーケンス条件を取得.
     */
    protected static final String sequenceInfo( Config config,String section,boolean seq ) {
        StringBuilder buf = new StringBuilder() ;
        buf.append( "// ** シーケンスID条件. **\n" ) ;
        buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
            append( "." ).append( TableMaster.SEQUENCE_DEFINE ).append( "=" ).
            append( seq ).append( ";" ).append( "\n" ) ;
        return buf.toString() ;
    }
    
    /**
     * デフォルトソート条件を取得.
     */
    protected static final String defaultSort( Config config,String section ) {
        String value = "id" ;
        if( config.size( section,SEQUENCE_MODE ) > 0 ) {
            String sort = config.get( section,DEFAULT_SORT_MODE,0 ) ;
            if( sort != null && ( sort = sort.trim() ).length() > 0 ) {
                value = sort ;
            }
        }
        StringBuilder buf = new StringBuilder() ;
        buf.append( "// ** デフォルトソートID条件. **\n" ) ;
        buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
            append( "." ).append( TableMaster.DEFAULT_SORT_DEFINE ).append( "=\"" ).
            append( value ).append( "\";" ).append( "\n" ) ;
        return buf.toString() ;
    }
    
    /**
     * 表示用カラム名条件を取得.
     */
    protected static final String viewColumns( Config config,String section ) {
        String value = "[\"name\"]" ;
        if( config.size( section,SEQUENCE_MODE ) > 0 ) {
            String viewNames = config.get( section,VIEW_COLUMN_NAMES,0 ) ;
            if( viewNames != null && ( viewNames = viewNames.trim() ).length() > 0 ) {
                value = StringUtil.trim( viewNames ) ;
                if( !value.startsWith( "[" ) || !value.endsWith( "]" ) ) {
                    value = "[\"" + value + "\"]" ;
                }
            }
        }
        StringBuilder buf = new StringBuilder() ;
        buf.append( "// ** 表示カラム名群条件. **\n" ) ;
        buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
            append( "." ).append( TableMaster.VIEW_COLUMNS_DEFINE ).append( "=" ).
            append( value ).append( ";" ).append( "\n" ) ;
        return buf.toString() ;
    }
    
    /**
     * 名前管理マスター条件を取得.
     */
    protected static final String namesMasterInfo( Config config,String section ) {
        StringBuilder buf = new StringBuilder() ;
        buf.append( "// ** 名前管理マスター条件. **\n" ) ;
        if( config.size( section,NAMES_MASTER_MODE ) > 0 ) {
            if( config.getBoolean( section,NAMES_MASTER_MODE,0 ) == true ) {
                buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                    append( "." ).append( TableMaster.NAMES_MASTER_DEFINE ).append( "=true;" ).append( "\n" ) ;
            }
            else {
                buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                    append( "." ).append( TableMaster.NAMES_MASTER_DEFINE ).append( "=false;" ).append( "\n" ) ;
            }
        }
        else {
            buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                append( "." ).append( TableMaster.NAMES_MASTER_DEFINE ).append( "=false;" ).append( "\n" ) ;
        }
        return buf.toString() ;
    }
    
    /**
     * マスターキャッシュ条件を取得.
     */
    protected static final String masterCache( Config config,String section ) {
        StringBuilder buf = new StringBuilder() ;
        buf.append( "// ** マスターキャッシュ条件. **\n" ) ;
        if( config.size( section,MASTER_CACHE_MODE ) > 0 ) {
            if( config.getBoolean( section,MASTER_CACHE_MODE,0 ) == false ) {
                buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                    append( "." ).append( TableMaster.MASTER_CACHE_DEFINE ).append( "=false;" ).append( "\n" ) ;
            }
            else {
                buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                    append( "." ).append( TableMaster.MASTER_CACHE_DEFINE ).append( "=true;" ).append( "\n" ) ;
            }
        }
        else {
            buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                append( "." ).append( TableMaster.MASTER_CACHE_DEFINE ).append( "=true;" ).append( "\n" ) ;
        }
        return buf.toString() ;
    }
    
    /**
     * マスター編集条件を取得.
     */
    protected static final String masterEdit( Config config,String section ) {
        StringBuilder buf = new StringBuilder() ;
        buf.append( "// ** マスター編集条件. **\n" ) ;
        if( config.size( section,MASTER_EDIT_MODE ) > 0 ) {
            if( config.getBoolean( section,MASTER_EDIT_MODE,0 ) == false ) {
                buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                    append( "." ).append( TableMaster.MASTER_EDIT_DEFINE ).append( "=false;" ).append( "\n" ) ;
            }
            else {
                buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                    append( "." ).append( TableMaster.MASTER_EDIT_DEFINE ).append( "=true;" ).append( "\n" ) ;
            }
        }
        else {
            buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                append( "." ).append( TableMaster.MASTER_EDIT_DEFINE ).append( "=true;" ).append( "\n" ) ;
        }
        return buf.toString() ;
    }
    
    /**
     * マスター新規作成条件を取得.
     */
    protected static final String masterCreate( Config config,String section ) {
        boolean flg = true ;
        if( config.size( section,MASTER_EDIT_MODE ) > 0 ) {
            if( config.getBoolean( section,MASTER_EDIT_MODE,0 ) == false ) {
                // マスター編集がOFFの場合は、マスター新規作成もOFFに設定.
                flg = false ;
            }
        }
        StringBuilder buf = new StringBuilder() ;
        buf.append( "// ** マスター新規作成条件. **\n" ) ;
        if( config.size( section,MASTER_CREATE_MODE ) > 0 ) {
            if( config.getBoolean( section,MASTER_CREATE_MODE,0 ) == false ) {
                buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                    append( "." ).append( TableMaster.MASTER_CREATE_DEFINE ).append( "=false;" ).append( "\n" ) ;
            }
            else {
                buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                    append( "." ).append( TableMaster.MASTER_CREATE_DEFINE ).append( "=" ).append( flg ).append( ";" ).append( "\n" ) ;
            }
        }
        else {
            buf.append( DbUtil.convertDBNameByJavaName( true,section ) ).
                append( "." ).append( TableMaster.MASTER_CREATE_DEFINE ).append( "=" ).append( flg ).append( ";" ).append( "\n" ) ;
        }
        return buf.toString() ;
    }
    
    /**
     * デフォルトパラメータを取得.
     */
    protected static final String defaultParam( Config config,String section,Table table,SupportKind dbms ) {
        String[] keys = config.getKeys( section ) ;
        if( keys != null && keys.length > 0 ) {
            int len = keys.length ;
            Map<String,String> map = new THashMap<String,String>() ;
            int chkPos = Table.COLUMN_DEFAULT.length() ;
            for( int i = 0 ; i < len ; i ++ ) {
                int lenJ = config.size( section,keys[ i ] ) ;
                for( int j = 0 ; j < lenJ ; j ++ ) {
                    String value = config.get( section,keys[ i ],j ) ;
                    if( value == null || ( value = value.trim() ).length() <= 0 ) {
                        continue ;
                    }
                    if( value.startsWith( Table.COLUMN_DEFAULT ) ) {
                        char c = value.charAt( chkPos ) ;
                        if( c == '\'' || c == ' ' || c == '\"' ) {
                            String def = TableUtil.getDefault( value ) ;
                            map.put( keys[ i ],def ) ;
                        }
                    }
                }
            }
            keys = null ;
            Object[] o = map.keySet().toArray() ;
            if( o != null && o.length > 0 ) {
                String javaName = DbUtil.convertDBNameByJavaName( true,section ) ;
                StringBuffer buf = new StringBuffer() ;
                buf.append( "// ** デフォルト値. **\n" ) ;
                buf.append( javaName ).append( "." ).append( TableMaster.DEFAULT_COLUMN_DEFINE ).append( "={" ) ;
                len = o.length ;
                for( int i = 0 ; i < len ; i ++ ) {
                    if( i != 0 ) {
                        buf.append( ", " ) ;
                    }
                    String v = map.get( ( String )o[ i ] ) ;
                    if( v == null || v.length() <= 0 ) {
                        continue ;
                    }
                    buf.append( "\"" ).append( DbUtil.convertDBNameByJavaName( false,( String )o[ i ] ) ).append( "\":" ) ;
                    if( v.equals( "true" ) || v.equals( "false" ) || checkNumber( v ) == true ) {
                        buf.append( v ) ;
                    }
                    else {
                        buf.append( "\"" ).append( v ).append( "\"" ) ;
                    }
                }
                buf.append( " } ;\n" ) ;
                o = null ;
                map = null ;
                return buf.toString() ;
            }
        }
        return null ;
    }
    
    /**
     * 数値判別.
     */
    protected static final boolean checkNumber( String s ) {
        boolean ret = false ;
        try {
            Double.parseDouble( s ) ;
            ret = true ;
        } catch( Exception e ) {
            ret = false ;
        }
        return ret ;
    }
    
    /**
     * バージョンファイル一覧を削除.
     */
    protected static final void deleteVersionInfo( String baseDir )
        throws Exception {
        if( baseDir.endsWith( "\\" ) || baseDir.endsWith( "/" ) ) {
            baseDir = baseDir.substring( 0,baseDir.length()-1 ) ;
        }
        String dir = new StringBuilder().
            append( baseDir ).append( FileUtil.FILE_SPACE ).
            append( VERSION_DIR ).append( FileUtil.FILE_SPACE ).toString() ;
        if( FileUtil.isDirExists( dir ) == false ) {
            return ;
        }
        String[] tmp = FileUtil.getFileDirList( dir ) ;
        if( tmp == null || tmp.length <= 0 ) {
            return ;
        }
        int len = tmp.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( tmp[ i ] == null || tmp[ i ].length() <= 0 || tmp[ i ].endsWith( TABLE_CONF ) == false ) {
                continue ;
            }
            FileUtil.removeFile( dir + tmp[ i ] ) ;
        }
    }
    
    /**
     * 現在のバージョン値を取得.
     */
    protected static final int getVersion( String baseDir )
        throws Exception {
        if( baseDir.endsWith( "\\" ) || baseDir.endsWith( "/" ) ) {
            baseDir = baseDir.substring( 0,baseDir.length()-1 ) ;
        }
        String dir = new StringBuilder().
            append( baseDir ).append( FileUtil.FILE_SPACE ).
            append( VERSION_DIR ).append( FileUtil.FILE_SPACE ).toString() ;
        if( FileUtil.isDirExists( dir ) == false ) {
            return -1 ;
        }
        String[] tmp = FileUtil.getFileDirList( dir ) ;
        if( tmp == null || tmp.length <= 0 ) {
            return -1 ;
        }
        ArrayList<Integer> lst = new ArrayList<Integer>() ;
        int len = tmp.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( tmp[ i ] == null || tmp[ i ].length() <= 0 || tmp[ i ].endsWith( TABLE_CONF ) == false ) {
                continue ;
            }
            String s = tmp[ i ].substring( 0,tmp[ i ].length() - TABLE_CONF.length() ) ;
            if( isNumber( s ) == false ) {
                continue ;
            }
            lst.add( new Integer( s ) ) ;
        }
        if( lst.size() <= 0 ) {
            return -1 ;
        }
        tmp = null ;
        len = lst.size() ;
        int ret = 0 ;
        for( int i = 0 ; i < len ; i ++ ) {
            Integer t = lst.get( i ) ;
            if( ret < t.intValue() ) {
                ret = t.intValue() ;
            }
        }
        return ret ;
    }
    
    /**
     * 指定バージョンのコンフィグファイルを取得.
     */
    protected static final Config getVersionConfig( String baseDir,String version )
        throws Exception {
        if( baseDir.endsWith( "\\" ) || baseDir.endsWith( "/" ) ) {
            baseDir = baseDir.substring( 0,baseDir.length()-1 ) ;
        }
        String file = new StringBuilder().
            append( baseDir ).append( FileUtil.FILE_SPACE ).
            append( VERSION_DIR ).append( FileUtil.FILE_SPACE ).
            append( version ).append( TABLE_CONF ).toString() ;
        if( FileUtil.isFileExists( file ) == false ) {
            return null ;
        }
        Config conf = new Config() ;
        getIniParams( conf,file ) ;
        // 操作除外条件内容を反映.
        Set<String> nonAccessList = getNonAccessList() ;
        Iterator it = nonAccessList.iterator() ;
        while( it.hasNext() ) {
            conf.removeSection( (String)it.next() ) ;
        }
        return conf ;
    }
    
    /**
     * 指定バージョン+1でコンフィグファイルを出力.
     */
    protected static final void outputVersionConfig( boolean mode,String baseDir,int version,Config config )
        throws Exception {
        if( baseDir.endsWith( "\\" ) || baseDir.endsWith( "/" ) ) {
            baseDir = baseDir.substring( 0,baseDir.length()-1 ) ;
        }
        String dir = new StringBuilder().
            append( baseDir ).append( FileUtil.FILE_SPACE ).
            append( VERSION_DIR ).append( FileUtil.FILE_SPACE ).toString() ;
        if( FileUtil.isDirExists( dir ) == false ) {
            version = -1 ;
            FileUtil.mkdirs( dir ) ;
        }
        String versionName = null ;
        if( version <= -1 ) {
            versionName = "1" + TABLE_CONF ;
        }
        else {
            versionName = new StringBuilder().append( version+1 ).append( TABLE_CONF ).toString() ;
        }
        String data = allConfig( config ) ;
        if( data == null || data.length() <= 0 ) {
            if( mode == false ) {
                System.out.println( "## version " + versionName + " を出力に失敗." ) ;
            }
            else {
                LOG.error( "## version " + versionName + " を出力に失敗." ) ;
            }
            return ;
        }
        FileUtil.setFileByString( dir + versionName,data ) ;
        if( mode == false ) {
            System.out.println( "## version " + versionName + " を出力." ) ;
        }
        else {
            LOG.info( "## version " + versionName + " を出力." ) ;
        }
    }
    
    /**
     * 現在のコンフィグファイル内容を１コンフィグファイル形式にまとめる.
     */
    protected static final String allConfig( Config config )
        throws Exception {
        Object[] o = config.getSections() ;
        if( o == null || o.length <= 0 ) {
            return null ;
        }
        Arrays.sort( o ) ;
        int len = o.length ;
        StringBuilder buf = new StringBuilder() ;
        int cnt = 0 ;
        for( int i = 0 ; i < len ; i ++ ) {
            String section = ( String )o[ i ] ;
            if( section == null || ( section = section.trim() ).length() <= 0 ) {
                continue ;
            }
            section = section.toLowerCase() ;
            String[] keys = config.getKeys( section ) ;
            if( keys == null || keys.length <= 0 ) {
                continue ;
            }
            Arrays.sort( keys ) ;
            int lenJ = keys.length ;
            if( cnt != 0 ) {
                buf.append( "\n" ) ;
            }
            buf.append( "[" ).append( section ).append( "]\n" ) ;
            String v ;
            for( int j = 0 ; j < lenJ ; j ++ ) {
                if( keys[ j ] == null || ( keys[ j ] = keys[ j ].trim() ).length() <= 0 ) {
                    continue ;
                }
                String key = keys[ j ] ;
                if( !isSpecialColumnName( key ) ) {
                    continue ;
                }
                String[] all = config.getAll( section,key ) ;
                if( all == null || all.length <= 0 ) {
                    continue ;
                }
                trimArray( all ) ;
                Arrays.sort( all ) ;
                int lenK = all.length ;
                for( int k = 0 ; k < lenK ; k ++ ) {
                    v = all[ k ] ;
                    if( v == null || isValueToDefaultInfo( v ) ) {
                        continue ;
                    }
                    buf.append( key ).append( "=" ).append( "\"" ).append( v ).append( "\"\n" ) ;
                }
            }
            cnt ++ ;
        }
        return buf.toString() ;
    }
    
    /**
     * 指定データがデフォルト値の場合は、条件として含めない.
     */
    protected static final boolean isValueToDefaultInfo( String value ) {
        return value.trim().startsWith( "default" ) ;
    }
    
    /**
     * 2つのコンフィグ内容を比較.
     */
    protected static final boolean equlasConfig( Config src,Config dest )
        throws Exception {
        return Digest.convert( "SHA1",allConfig( src ).getBytes( "UTF8" ) ).equals(
            Digest.convert( "SHA1",allConfig( dest ).getBytes( "UTF8" ) ) ) ;
    }
    
    /**
     * 差分情報を取得.
     */
    protected static final String partSQL( Table table,Config src,Config dest )
        throws Exception {
        if( src.size() <= 0 || dest.size() <= 0 ) {
            return null ;
        }
        StringBuilder buf = new StringBuilder() ;
        Set<String> createKeyTable = new THashSet<String>() ;
        // 追加条件を取得.
        Object[] srcTable = src.getSections() ;
        int len = srcTable.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String tableName = ( String )srcTable[ i ] ;
            if( table == null || ( tableName = tableName.trim() ).length() <= 0 ||
                src.size( tableName ) <= 0 ) {
                continue ;
            }
            // 追加対象テーブルが存在する場合.
            if( dest.isSection( tableName ) == false ) {
                buf.append( table.createTable( tableName,src ) ) ;
                continue ;
            }
            // 追加対象要素が存在するかチェック.
            String[] keys = src.getKeys( tableName ) ;
            if( keys == null || keys.length <= 0 ) {
                continue ;
            }
            boolean sequenceFlag = true ;
            if( src.size( tableName,SEQUENCE_MODE ) > 0 ) {
                sequenceFlag = src.getBoolean( tableName,SEQUENCE_MODE,0 ) ;
            }
            int lenJ = keys.length ;
            for( int j = 0 ; j < lenJ ; j ++ ) {
                String key = keys[ j ] ;
                if( key == null || ( key = key.trim() ).length() <= 0 ||
                    TableUtil.isType( key ) == true || !isSpecialColumnName( key ) ||
                    ( "id".equals( key.toLowerCase() ) && sequenceFlag ) ) {
                    continue ;
                }
                // キー追加を検知した場合.
                if( dest.isKeys( tableName,key ) == false ) {
                    buf.append( table.addColumn( tableName,key,src ) ) ;
                    if( isPrimaryKey( src,tableName,key ) == true ) {
                        createKeyTable.add( tableName ) ;
                    }
                }
            }
        }
        srcTable = null ;
        
        // 削除条件を取得.
        Object[] destTable = dest.getSections() ;
        len = destTable.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String tableName = ( String )destTable[ i ] ;
            if( table == null || ( tableName = tableName.trim() ).length() <= 0 ||
                dest.size( tableName ) <= 0 ) {
                continue ;
            }
            // 削除対象テーブルが存在する場合.
            if( src.isSection( tableName ) == false ) {
                String recreate = table.dropPrimaryKey( tableName ) ;
                if( recreate != null ) {
                    buf.append( recreate ) ;
                }
                recreate = table.dropTable( tableName ) ;
                if( recreate != null ) {
                    buf.append( recreate ) ;
                }
                recreate = table.dropSequence( tableName ) ;
                if( recreate != null ) {
                    buf.append( recreate ) ;
                }
                continue ;
            }
            // 削除対象要素が存在するかチェック.
            String[] keys = dest.getKeys( tableName ) ;
            if( keys == null || keys.length <= 0 ) {
                continue ;
            }
            int lenJ = keys.length ;
            for( int j = 0 ; j < lenJ ; j ++ ) {
                String key = keys[ j ] ;
                if( key == null || ( key = key.trim() ).length() <= 0 ||
                    "id".equals( key.toLowerCase() ) || TableUtil.isType( key ) == true ) {
                    continue ;
                }
                // キー削除を検知した場合.
                if( src.isKeys( tableName,key ) == false ) {
                    buf.append( table.dropColumn( tableName,key ) ) ;
                    if( isPrimaryKey( dest,tableName,key ) == true ) {
                        createKeyTable.add( tableName ) ;
                    }
                }
            }
        }
        destTable = null ;
        
        // 生成/削除を検出したプライマリーキーを処理.
        if( createKeyTable.size() > 0 ) {
            String addSql = buf.toString() ;
            buf = new StringBuilder() ;
            Object[] pks = createKeyTable.toArray() ;
            len = pks.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                String pk = ( String )pks[ i ] ;
                if( pk == null || ( pk = pk.trim() ).length() <= 0 ) {
                    continue ;
                }
                buf.append( table.dropPrimaryKey( pk ) ) ;
                if( src.isSection( pk ) == true ) {
                    addSql += table.addPrimaryKey( pk,src ) ;
                }
            }
            buf.append( addSql ) ;
        }
        return buf.toString() ;
    }
    
    /**
     * 対象テーブルにキー条件が存在するかチェック.
     */
    protected static final boolean isPrimaryKey( Config config,String table,String key ) {
        String[] one = config.getAll( table,key ) ;
        int len = one.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( TableUtil.isIndex( one[ i ] ) == true || TableUtil.isKey( one[ i ] ) == true ) {
                return true ;
            }
        }
        return false ;
    }
    
    /**
     * 指定条件が数値であるかチェック.
     */
    protected static final boolean isNumber( String param ) {
        try {
            Integer.parseInt( param ) ;
            return true ;
        } catch( Exception e ) {
        }
        return false ;
    }
    
    /**
     * 指定文字列配列にTrim.
     */
    protected static final void trimArray( String[] all ) {
        int len = all.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( all[ i ] == null || ( all[ i ] = all[ i ].trim() ).length() <= 0 ) {
                all[ i ] = null ;
            }
        }
    }
    
    /**
     * ヘルプ内容を表示.
     */
    protected static final void viewHelp() {
        System.out.println( "maachangDao テーブル生成ヘルプ." ) ;
        System.out.println() ;
        System.out.println( "create     : テーブル生成処理を行います." ) ;
        System.out.println( "             また、引数なしでも同様の条件を実行します." ) ;
        System.out.println( "drop       : テーブル破棄およびバージョン破棄します." ) ;
        System.out.println( "model      : モデルファイルを再生性します." ) ;
        System.out.println( "master     : マスターファイル(csv)をマスターテーブルに格納します" ) ;
        System.out.println( "version    : 現在のモデルバージョンを表示します." ) ;
    }
    
    /** 特殊割り当てカラム定義かチェック. **/
    protected static final boolean isSpecialColumnName( String name ) {
        name = name.toLowerCase() ;
        return ( !name.startsWith( TableMaster.SPECIAL_DEFINE ) &&
                !name.endsWith( TableMaster.JOIN ) &&
                !name.endsWith( TableMaster.ELEMENT_DEFINE ) ) ;
    }
    
    /** 有効なカラム名を取得. **/
    protected static final List<String> getColumns( Config config,String section,boolean seqFlag ) {
        List<String> ret = new ArrayList<String>() ;
        int len = config.size( section ) ;
        if( len > 0 ) {
            String[] all = config.getKeys( section ) ;
            String s ;
            if( seqFlag ) {
                ret.add( "id" ) ;
            }
            for( int i = 0 ; i < len ; i ++ ) {
                if( CreateTable.isSpecialColumnName( ( s = all[ i ] ) ) ) {
                    ret.add( s ) ;
                }
            }
        }
        return ret ;
    }
    
    /** 非アクセスDBリストを取得. **/
    protected static final Set<String> getNonAccessList() throws Exception {
        if( !FileUtil.isFileExists( "./table/" + NON_ACCESS_DB_MANAGER_FILE ) ) {
            return new HashSet<String>() ;
        }
        Set<String> ret = new HashSet<String>() ;
        BufferedReader buf = null ;
        try {
            buf = new BufferedReader( new InputStreamReader( new FileInputStream(
                "./table/" + NON_ACCESS_DB_MANAGER_FILE ),"UTF8" ) ) ;
            String s ;
            while( ( s = buf.readLine() ) != null ) {
                s = StringUtil.trim( s ) ;
                if( s.length() == 0 ) {
                    continue ;
                }
                if( s.startsWith( "#" ) ) {
                    continue ;
                }
                ret.add( s ) ;
            }
            buf.close() ;
            buf = null ;
        } finally {
            if( buf != null ) {
                try {
                    buf.close() ;
                } catch( Exception e ) {
                }
            }
        }
        return ret ;
    }
}
