package org.maachang.dao.dbms.ctbl;

import java.util.List;

import org.maachang.conf.Config;
import org.maachang.dao.dbms.DbUtil;
import org.maachang.util.StringUtil;

/**
 * テーブル詳細定義の生成処理.
 * 
 * @version 2009/11/09
 * @author masahito suzuki
 * @since MaachangDao 1.14
 */
public abstract class DetailTable {
    private DetailTable() {}
    
    /**
     * 詳細定義用Map変数名頭文字.
     */
    private static final String DETAIL_MAP = "_$detail$" ;
    
    /**
     * 変数定義:詳細メンテナンス用表示順定義.
     */
    public static final String MAINTENANCE_DEFINE = "maintenanceDef" ;
    
    /**
     * 変数定義:カラム名順番定義.
     */
    public static final String COLUMN_MAP_DEFINE = "colsMap" ;
    
    /**
     * 変数定義:Key定義.
     */
    public static final String KEY_DEFINE = "key" ;
    
    /**
     * 変数定義:NotNull定義.
     */
    public static final String NOT_NULL_DEFINE = "n_null" ;
    
    /**
     * 変数定義:入力長定義.
     */
    public static final String NOT_INPUT_SIZE_DEFINE = "size" ;
    
    /**
     * 変数定義:要素定義.
     */
    public static final String ELEMENT_DEFINE = "element" ;
    
    
    /**
     * 詳細用定義.
     */
    public static final void create( StringBuilder buf,Config config,String section,String javaTableName,boolean seqFlag ) throws Exception {
        
        // 詳細定義用オブジェクトを定義.
        buf.append( "\n" ).
            append( "// ** 詳細定義用. **\n" ).
            append( javaTableName ).append( "." ).
            append( DETAIL_MAP ).
            append( "={};\n" ) ;
        
        // 詳細メンテナンス用カラム名定義.
        detailMaintenanceDef( buf,config,section,javaTableName,seqFlag ) ;
        
        // 詳細カラム順序条件を設定.
        detailColumnsMap( buf,config,section,javaTableName,seqFlag ) ;
        
        // key定義を設定.
        detailKey( buf,config,section,javaTableName,seqFlag ) ;
        
        // not_null定義を設定.
        detailNotNull( buf,config,section,javaTableName,seqFlag ) ;
        
        // 入力サイズを設定.
        detailInputSize( buf,config,section,javaTableName,seqFlag ) ;
        
        // 要素定義を設定.
        detailElementDefine( buf,config,section,javaTableName,seqFlag ) ;
    }
    
    /**
     * 詳細メンテナンス用:詳細メンテナンス生成順カラム名定義.
     */
    private static final void detailMaintenanceDef( StringBuilder buf,Config config,String section,String javaTableName,boolean seqFlag )
        throws Exception {
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        if( len > 0 ) {
            buf.append( "\n" ).
                append( "// ** 詳細メンテナンス生成順カラム名定義. **" ).
                append( "\n" ).
                append( javaTableName ).append( "." ).
                append( DETAIL_MAP ).
                append( "." ).append( MAINTENANCE_DEFINE ).append( "=[" ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                buf.append( "\"" ).append( DbUtil.convertDBNameByJavaName( false,names.get( i ) ) ).append( "\"" ) ;
            }
            buf.append( "];\n" ) ;
        }
    }
    
    /**
     * 詳細メンテナンス用:カラム順序を設定.
     */
    private static final void detailColumnsMap( StringBuilder buf,Config config,String section,String javaTableName,boolean seqFlag )
        throws Exception {
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        if( len > 0 ) {
            buf.append( "\n" ).
                append( "// ** 詳細カラム順序定義. **" ).
                append( "\n" ).
                append( javaTableName ).append( "." ).
                append( DETAIL_MAP ).
                append( "." ).append( COLUMN_MAP_DEFINE ).append( "={" ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                buf.append( "\"" ).append( DbUtil.convertDBNameByJavaName( false,names.get( i ) ) ).append( "\"" ).
                    append( ":" ).append( i ) ;
            }
            buf.append( "};\n" ) ;
        }
    }
    
    /**
     * 詳細メンテナンス用:key表示名を設定.
     */
    private static final void detailKey( StringBuilder buf,Config config,String section,String javaTableName,boolean seqFlag )
        throws Exception {
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        String s ;
        if( len > 0 ) {
            buf.append( "\n" ).
                append( "// ** 詳細カラムKey定義. **" ).
                append( "\n" ).
                append( javaTableName ).append( "." ).
                append( DETAIL_MAP ).
                append( "." ).append( KEY_DEFINE ).append( "=[" ) ;
            int lenJ ;
            String v ;
            boolean keyFlag ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                s = names.get( i ) ;
                if( s.equals( "id" ) ) {
                    buf.append( true ) ;
                    continue ;
                }
                lenJ = config.size( section,s ) ;
                keyFlag = false ;
                for( int j = 0 ; j < lenJ ; j ++ ) {
                    if( Table.COLUMN_KEY.equals( ( v = config.get( section,s,j ) ) ) ||
                        Table.COLUMN_INDEX.equals( v ) ) {
                        keyFlag = true ;
                        break ;
                    }
                }
                buf.append( keyFlag ) ;
            }
            buf.append( "];\n" ) ;
        }
    }
    
    /**
     * 詳細メンテナンス用:not_null表示名を設定.
     */
    private static final void detailNotNull( StringBuilder buf,Config config,String section,String javaTableName,boolean seqFlag )
        throws Exception {
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        String s ;
        if( len > 0 ) {
            buf.append( "\n" ).
                append( "// ** 詳細カラムnot_null定義. **" ).
                append( "\n" ).
                append( javaTableName ).append( "." ).
                append( DETAIL_MAP ).
                append( "." ).append( NOT_NULL_DEFINE ).append( "=[" ) ;
            int lenJ ;
            String v ;
            boolean notNullFlag ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                s = names.get( i ) ;
                if( s.equals( "id" ) ) {
                    buf.append( true ) ;
                    continue ;
                }
                lenJ = config.size( section,s ) ;
                notNullFlag = false ;
                for( int j = 0 ; j < lenJ ; j ++ ) {
                    if( Table.COLUMN_NOT_NULL.equals( ( v = config.get( section,s,j ) ) ) ||
                        Table.COLUMN_NOT_NULL2.equals( v ) ) {
                        notNullFlag = true ;
                        break ;
                    }
                }
                buf.append( notNullFlag ) ;
            }
            buf.append( "];\n" ) ;
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // ** 入力長とカラム定義の関連 **
    // Table.BOOLEAN   : 強制的チェックボックス.
    // Table.INTEGER   : 11(数値のみ入力可能にする)
    // Table.LONG      : 21(数値のみ入力可能にする)
    // Table.FLOAT     : 32(数値のみ入力可能にする)
    // Table.DOUBLE    : 32(数値のみ入力可能にする)
    // Table.BINARY    : 強制的ファイルアップロード.
    // Table.CHAR      : なし(サイズ指定された場合は、その値).
    // Table.TEXT      : 強制的にテキストエリア(平仮名モード).
    // Table.DATE      : 強制的日付入力タグ(selectymd).
    // Table.TIME      : 強制的時間入力タグ(selecttime).
    // Table.TIMESTAMP : 強制的日付入力タグと時間入力タグ(selectymd+selecttime).
    ////////////////////////////////////////////////////////////////////////////
    
    /**
     * 詳細メンテナンス用:入力長を設定.
     */
    private static final void detailInputSize( StringBuilder buf,Config config,String section,String javaTableName,boolean seqFlag )
        throws Exception {
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        String s ;
        if( len > 0 ) {
            buf.append( "\n" ).
                append( "// ** 詳細カラム入力長定義. **" ).
                append( "\n" ).
                append( javaTableName ).append( "." ).
                append( DETAIL_MAP ).
                append( "." ).append( NOT_INPUT_SIZE_DEFINE ).append( "=[" ) ;
            int lenJ ;
            String v ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                s = names.get( i ) ;
                if( s.equals( "id" ) ) {
                    buf.append( "\"" ).append( "$21" ).append( "\"" ) ;
                    continue ;
                }
                lenJ = config.size( section,s ) ;
                for( int j = 0 ; j < lenJ ; j ++ ) {
                    v = config.get( section,s,j ) ;
                    if( Table.BOOLEAN.equals( v ) ) {
                        buf.append( "\"" ).append( "#f" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.INTEGER.equals( v ) ) {
                        buf.append( "\"" ).append( "$11" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.LONG.equals( v ) ) {
                        buf.append( "\"" ).append( "$21" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.FLOAT.equals( v ) ) {
                        buf.append( "\"" ).append( "$32" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.DOUBLE.equals( v ) ) {
                        buf.append( "\"" ).append( "$32" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.BINARY.equals( v ) ) {
                        buf.append( "\"" ).append( "#b" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.CHAR.equals( v ) ) {
                        buf.append( "\"" ).append( "#c" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.TEXT.equals( v ) ) {
                        buf.append( "\"" ).append( "#t" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.DATE.equals( v ) ) {
                        buf.append( "\"" ).append( "#d" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.TIME.equals( v ) ) {
                        buf.append( "\"" ).append( "#i" ).append( "\"" ) ;
                        break ;
                    }
                    else if( Table.TIMESTAMP.equals( v ) ) {
                        buf.append( "\"" ).append( "#s" ).append( "\"" ) ;
                        break ;
                    }
                    else if( v.startsWith( Table.CHAR ) ) {
                        boolean addFlag = false ;
                        int p = v.indexOf( "(" ) ;
                        if( p != -1 ) {
                            int pp = v.indexOf( ")",p+1 ) ;
                            if( pp != -1 ) {
                                v = StringUtil.trim( v.substring( p+1,pp ) ) ;
                                buf.append( "\"" ).append( v ).append( "\"" ) ;
                                addFlag = true ;
                            }
                        }
                        if( !addFlag ) {
                            buf.append( "\"" ).append( "#c" ).append( "\"" ) ;
                        }
                        break ;
                    }
                }
            }
            buf.append( "];\n" ) ;
        }
    }
    
    /**
     * 詳細メンテナンス用:要素定義を設定.
     */
    private static final void detailElementDefine( StringBuilder buf,Config config,String section,String javaTableName,boolean seqFlag )
        throws Exception {
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        String s ;
        if( len > 0 ) {
            buf.append( "\n" ).
                append( "// ** 詳細要素定義. **" ).
                append( "\n" ).
                append( javaTableName ).append( "." ).
                append( DETAIL_MAP ).
                append( "." ).append( ELEMENT_DEFINE ).append( "=[\n    " ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( ",\n    " ) ;
                }
                s = names.get( i ) ;
                if( config.size( section,s+TableMaster.ELEMENT_DEFINE ) <= 0 ) {
                    buf.append( "{}" ) ;
                }
                else {
                    s = StringUtil.trim( config.get( section,s+TableMaster.ELEMENT_DEFINE,0 ) ) ;
                    if( !s.startsWith( "{" ) || !s.endsWith( "}" ) ) {
                        buf.append( "{" ).append( s ).append( "}" ) ;
                    }
                    else {
                        buf.append( s ) ;
                    }
                }
            }
            buf.append( "\n];\n" ) ;
        }
    }
}

