package org.maachang.dao.dbms.ctbl;

import java.io.IOException;
import java.io.Reader;
import java.sql.ResultSet;
import java.sql.Types;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.conf.Config;
import org.maachang.dao.DaoSessionFactory;
import org.maachang.dao.dbms.DbUtil;
import org.maachang.dao.dbms.Record;
import org.maachang.util.Base64;
import org.maachang.util.CsvReader;
import org.maachang.util.FileUtil;
import org.maachang.util.StringUtil;

/**
 * テーブルコンフィグに付随するCSVファイルをInsert.
 * 
 * @version 2009/11/09
 * @author masahito suzuki
 * @since MaachangDao 1.14
 */
public abstract class InsertTable {
    private InsertTable() {}
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( InsertTable.class ) ;
    
    /**
     * マスターデータ拡張子.
     */
    protected static final String MASTER_CSV = ".csv" ;
    
    /**
     * マスターヘッダ拡張子.
     */
    protected static final String MASTER_HEADER = ".header" ;
    
    /**
     * マスターデータキャラクタセット.
     */
    protected static final String MASTER_CSV_CHARSET = "Windows-31J" ;
    
    /**
     * シーケンス条件.
     */
    protected static final String SEQUENCE_MODE = TableMaster.SEQUENCE_MODE ;
    
    /**
     * マスター追加.
     * @param mode [true]の場合、ログファイルに内容を出力します.
     * @param table テーブルオブジェクトを設定します.
     * @param tableConfig 対象テーブルのコンフィグ情報を設定します.
     * @param record データベース接続(JDBC)オブジェクトを設定します.
     * @param base 基本フォルダを設定します.
     * @exception Exception 例外.
     */
    public static final void insert( boolean mode,Set<String> nonAccessList,Table table,Config config,Record record,String base )
        throws Exception {
        String dir = base ;
        if( dir.endsWith( "/" ) == false && dir.endsWith( "\\" ) == false ) {
            dir += FileUtil.FILE_SPACE ;
        }
        String[] list = FileUtil.getFileDirList( dir ) ;
        if( list != null && list.length > 0 ) {
            int len = list.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                String c = dir + list[ i ] ;
                if( FileUtil.isDirExists( c ) == true ) {
                    insert( mode,nonAccessList,table,config,record,c ) ;
                }
                else if( FileUtil.isRead( c ) && c.toLowerCase().endsWith( MASTER_CSV ) ) {
                    // nonAccessListに存在する内容は処理しない.
                    if( nonAccessList.contains( list[ i ].substring( 0,list[ i ].length() - 4 ) ) ) {
                        continue ;
                    }
                    if( mode == false ) {
                        System.out.println( "-- readMasterFile(start):[" + list[ i ] + "]" ) ;
                    }
                    else {
                        LOG.info( "-- readMastarFile(start):[" + list[ i ] + "]" ) ;
                    }
                    // １つのCSVファイルに対して処理.
                    int cnt = insertCSV( mode,c,table,record,config,null ) ;
                    if( mode == false ) {
                        System.out.println( "-- readMasterFile(end):[" + list[ i ] + "] success:" + cnt ) ;
                    }
                    else {
                        LOG.info( "-- readMastarFile(end):[" + list[ i ] + "] success:" + cnt ) ;
                    }
                }
                list[ i ] = null ;
            }
        }
    }
    
    /**
     * １つのCSVファイルに対してInsert処理.
     */
    public static final int insertCSV( boolean mode,String fileName,Table table,Record record,Config config,Reader reader )
        throws Exception {
        if( fileName.toLowerCase().endsWith( MASTER_CSV ) ) {
            String confName = fileName.substring( 0,fileName.length()-4 ) + ".conf" ;
            if( reader == null ) {
                if( !FileUtil.isFileExists( confName ) ) {
                    return 0 ;
                }
            }
            int ret = 0 ;
            Map<String,Integer> colMap = new HashMap<String,Integer>() ;
            List<Integer> colTypes = new ArrayList<Integer>() ;
            List<String> targetCols = new ArrayList<String>() ;
            Map<String,Object> def = new HashMap<String,Object>() ;
            CsvReader csv = null ;
            String sectionName ;
            fileName = FileUtil.getFullPath( fileName ) ;
            String name = fileName.substring( fileName.lastIndexOf( "/" )+1 ) ;
            try {
                sectionName = DbUtil.convertJavaNameByDBName( name.substring( 0,name.length() - 4 ) ) ;
                if( config == null || !config.isSection( sectionName ) ) {
                    config = Config.read( confName ) ;
                    CreateTable.removeAllEnhancingDefine( true,config ) ;
                }
                boolean seqFlag = CreateTable.isSequence( config,sectionName ) ;
                // insert対象テーブルを再生成.
                CreateTable.recreateTable( mode,sectionName,table,config,record ) ;
                // Javaシーケンスが有効な場合は、対象テーブルのJavaSequenceをクリアする.
                if( isJavaSequence() ) {
                    DaoSessionFactory.getInstance().getJavaSequence().clearSequence( sectionName ) ;
                }
                // テーブルコンフィグファイル一覧を取得し、CSVファイル内容をターゲットテーブルにInsert.
                columnsMap( colMap,config,sectionName,seqFlag ) ;
                columnTypeList( colTypes,config,sectionName,seqFlag ) ;
                defaultDefine( def,colMap,colTypes,config,sectionName,seqFlag ) ;
                // readerが設定されている場合は、ファイル名を無効にする.
                if( reader != null ) {
                    fileName = null ;
                }
                csv = getColumns( targetCols,colMap,fileName,reader ) ;
                // insert処理.
                ret = csvReaderToInsert( mode,csv,sectionName,config,record,table,targetCols,colMap,colTypes,def,seqFlag ) ;
                csv.close() ;
                csv = null ;
                return ret ;
            } finally {
                if( csv != null ) {
                    try {
                        csv.close() ;
                    } catch( Exception e ) {
                    }
                    csv = null ;
                }
            }
        }
        return 0 ;
    }
    
    /**
     * カラム名の順序を取得.
     */
    private static final void columnsMap( Map<String,Integer> out,Config config,String section,boolean seqFlag )
        throws Exception {
        out.clear() ;
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        if( len > 0 ) {
            out.put( "id",0 ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                out.put( DbUtil.convertJavaNameByDBName( names.get( i ) ),i ) ;
            }
        }
        else {
            throw new IOException( "section:" + section + " にカラム情報は存在しません" ) ;
        }
    }
    
    private static final String CHAR_CHECK = Table.CHAR + "(" ;
    
    /**
     * カラム順序に対するカラム型を取得.
     */
    private static final void columnTypeList( List<Integer> out,Config config,String section,boolean seqFlag )
        throws Exception {
        out.clear() ;
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        if( len > 0 ) {
            boolean addFlag ;
            int lenJ ;
            String name ;
            String colType ;
            if( seqFlag ) {
                out.add( Types.BIGINT ) ;
            }
            for( int i = 0 ; i < len ; i ++ ) {
                name = names.get( i ) ;
                lenJ = config.size( section,name ) ;
                addFlag = false ;
                for( int j = 0 ; j < lenJ ; j ++ ) {
                    colType = StringUtil.trim( config.get( section,name,j ).toLowerCase() ) ;
                    if( colType.startsWith( CHAR_CHECK ) && colType.endsWith( ")" ) ) {
                        colType = Table.CHAR ;
                    }
                    if( Table.TYPES.containsKey( colType ) ) {
                        out.add( Table.TYPES.get( colType ) ) ;
                        addFlag = true ;
                        break ;
                    }
                }
                if( !addFlag ) {
                    if( !"id".equals( name ) ) {
                        throw new IOException( "section:" + section + " のカラム条件は不正です:"+name ) ;
                    }
                }
            }
        }
        else {
            throw new IOException( "section:" + section + " にカラム情報は存在しません" ) ;
        }
    }
    
    /**
     * デフォルト定義を取得.
     */
    private static final void defaultDefine( Map<String,Object> out,Map<String,Integer> nos,List<Integer> types,
        Config config,String section,boolean seqFlag )
        throws Exception {
        out.clear() ;
        List<String> names = CreateTable.getColumns( config,section,seqFlag ) ;
        int len = names.size() ;
        if( len > 0 ) {
            String name ;
            String value ;
            int lenJ ;
            for( int i = 0 ; i < len ; i ++ ) {
                name = names.get( i ) ;
                lenJ = config.size( section,name ) ;
                for( int j = 0 ; j < lenJ ; j ++ ) {
                    value = config.get( section,name,j ) ;
                    if( value.toLowerCase().startsWith( "default" ) ) {
                        out.put( DbUtil.convertJavaNameByDBName( name ),
                            convertStringByTypes( types.get( nos.get( name ) ),
                                StringUtil.trim( value.substring( "default".length() ) ) ) ) ;
                        break ;
                    }
                }
            }
        }
        else {
            throw new IOException( "section:" + section + " にカラム情報は存在しません" ) ;
        }
    }
    
    /**
     * カラム定義を取得.
     */
    private static final CsvReader getColumns( List<String> out,Map<String,Integer> colsMap,String name,Reader reader ) throws Exception {
        out.clear() ;
        CsvReader csv = null ;
        String[] o = null ;
        // ファイル名指定でのCSV取得.
        if( name != null ) {
            String headerName = name.substring( 0,name.length()-4 ) + MASTER_HEADER ;
            try {
                if( FileUtil.isRead( headerName ) ) {
                    // ヘッダ専用定義ファイルが存在する.
                    csv = new CsvReader( headerName,MASTER_CSV_CHARSET,"," ) ;
                    if( csv.next() ) {
                        o = csv.getArray() ;
                        int len = o.length ;
                        for( int i = 0 ; i < len ; i ++ ) {
                            out.add( DbUtil.convertJavaNameByDBName( o[i] ) ) ;
                        }
                    }
                    csv.close() ;
                }
                if( csv == null ) {
                    csv = new CsvReader() ;
                }
                csv.open( name,MASTER_CSV_CHARSET,"," ) ;
                if( o == null ) {
                    // ヘッダ専用定義がしないため、CSVの最初の１行を取得.
                    if( csv.next() ) {
                        o = csv.getArray() ;
                        int len = o.length ;
                        for( int i = 0 ; i < len ; i ++ ) {
                            out.add( DbUtil.convertJavaNameByDBName( o[i] ) ) ;
                        }
                    }
                    else {
                        throw new IOException( "不正なMasterCSVファイル:" + name ) ;
                    }
                }
                else {
                    if( !csv.next() ) {
                        throw new IOException( "不正なMasterCSVファイル:" + name ) ;
                    }
                }
                // ヘッダ内容が正しい条件かチェック.
                int len = out.size() ;
                for( int i = 0 ; i < len ; i ++ ) {
                    if( !colsMap.containsKey( out.get( i ) ) ) {
                        throw new IOException( "不正なCSVヘッダ[" + out.get( i ) + "]を含んだMasterCSVファイル:" + name ) ;
                    }
                }
                return csv ;
            } catch( Exception e ) {
                if( csv != null ) {
                    try {
                        csv.close() ;
                    } catch( Exception ee ) {
                    }
                }
                throw e ;
            }
        }
        // reader定義のCSV取得.
        else if( reader != null ) {
            try {
                csv = new CsvReader() ;
                csv.open( reader,"," ) ;
                if( o == null ) {
                    // ヘッダ専用定義がしないため、CSVの最初の１行を取得.
                    if( csv.next() ) {
                        o = csv.getArray() ;
                        int len = o.length ;
                        for( int i = 0 ; i < len ; i ++ ) {
                            out.add( DbUtil.convertJavaNameByDBName( o[i] ) ) ;
                        }
                    }
                    else {
                        throw new IOException( "不正なMasterCSVファイル:" + name ) ;
                    }
                }
                else {
                    if( !csv.next() ) {
                        throw new IOException( "不正なMasterCSVファイル:" + name ) ;
                    }
                }
                // ヘッダ内容が正しい条件かチェック.
                int len = out.size() ;
                for( int i = 0 ; i < len ; i ++ ) {
                    if( !colsMap.containsKey( out.get( i ) ) ) {
                        throw new IOException( "不正なCSVヘッダ[" + out.get( i ) + "]を含んだMasterCSVファイル:" + name ) ;
                    }
                }
                return csv ;
            } catch( Exception e ) {
                if( csv != null ) {
                    try {
                        csv.close() ;
                    } catch( Exception ee ) {
                    }
                }
                throw e ;
            }
        }
        else {
            throw new IOException( "有効な呼び出しではありません" ) ;
        }
    }
    
    /**
     * 文字列に対して、タイプ定義から変換.
     */
    private static final Object convertStringByTypes( int type,String value )
        throws Exception {
        String low  ;
        int len ;
        if( value == null || ( len = value.length() ) <= 0 ) {
            return null ;
        }
        switch( type ) {
            case Types.BOOLEAN :
                low = value.toLowerCase() ;
                if( low.equals( "true" ) || low.equals( "on" ) || low.equals( "1" ) ) {
                    return Boolean.TRUE ;
                }
                else if( low.equals( "false" ) || low.equals( "off" ) || low.equals( "0" ) ) {
                    return Boolean.FALSE ;
                }
                throw new IOException( "Boolean型に対して一致しない内容です:" + value ) ;
            case Types.INTEGER :
                low = value.toLowerCase() ;
                if( low.startsWith( "0x" ) ) {
                    return new Integer( Integer.parseInt( low.substring( 2 ),16 ) ) ;
                }
                if( low.endsWith( "l" ) ) {
                    return new Integer( value.substring( 0,value.length()-1 ) ) ;
                }
                return new Integer( value ) ;
            case Types.BIGINT :
                low = value.toLowerCase() ;
                if( low.startsWith( "0x" ) ) {
                    return new Long( Long.parseLong( low.substring( 2 ),16 ) ) ;
                }
                if( low.endsWith( "l" ) ) {
                    return new Integer( value.substring( 0,value.length()-1 ) ) ;
                }
                return new Long( value ) ;
            case Types.FLOAT :
                low = value.toLowerCase() ;
                if( low.endsWith( "f" ) || low.endsWith( "d" ) ) {
                    return new Float( value.substring( 0,value.length()-1 ) ) ;
                }
                return new Float( value ) ;
            case Types.DOUBLE :
                low = value.toLowerCase() ;
                if( low.endsWith( "f" ) || low.endsWith( "d" ) ) {
                    return new Double( value.substring( 0,value.length()-1 ) ) ;
                }
                return new Double( value ) ;
            case Types.BINARY :
                return Base64.decode( value ) ;
            case Types.VARCHAR :
            case Types.LONGVARCHAR :
                return value ;
            case Types.DATE :
                if( len == 8 ) {
                    return new SimpleDateFormat( "yyyyMMdd" ).parse( value ) ;
                }
                else {
                    if( value.indexOf( "-" ) != -1 ) {
                        return new SimpleDateFormat( "yyyy-MM-dd" ).parse( value ) ;
                    }
                    else if( value.indexOf( "/" ) != -1 ) {
                        return new SimpleDateFormat( "yyyy/MM/dd" ).parse( value ) ;
                    }
                    throw new IOException( "Date型に対して一致しない内容です:" + value ) ;
                }
            case Types.TIME :
                if( len == 4 ) {
                    return new SimpleDateFormat( "HHmm" ).parse( value ) ;
                }
                else if( len == 5 ) {
                    if( value.indexOf( "-" ) != -1 ) {
                        return new SimpleDateFormat( "HH-mm" ).parse( value ) ;
                    }
                    else if( value.indexOf( ":" ) != -1 ) {
                        return new SimpleDateFormat( "HH:mm" ).parse( value ) ;
                    }
                }
                else if( len == 6 ) {
                    return new SimpleDateFormat( "HHmmss" ).parse( value ) ;
                }
                else {
                    if( value.indexOf( "-" ) != -1 ) {
                        return new SimpleDateFormat( "HH-mm-ss" ).parse( value ) ;
                    }
                    else if( value.indexOf( ":" ) != -1 ) {
                        return new SimpleDateFormat( "HH:mm:ss" ).parse( value ) ;
                    }
                }
                throw new IOException( "Time型に対して一致しない内容です:" + value ) ;
            case Types.TIMESTAMP :
                if( len == 8 ) {
                    return new SimpleDateFormat( "yyyyMMdd" ).parse( value ) ;
                }
                else if( len == 10 ) {
                    if( value.indexOf( "-" ) != -1 ) {
                        return new SimpleDateFormat( "yyyy-MM-dd" ).parse( value ) ;
                    }
                    else if( value.indexOf( "/" ) != -1 ) {
                        return new SimpleDateFormat( "yyyy/MM/dd" ).parse( value ) ;
                    }
                }
                else if( len == 12 ) {
                    return new SimpleDateFormat( "yyyyMMddHHmm" ).parse( value ) ;
                }
                else if( len == 13 ) {
                    if( value.indexOf( " " ) != -1 ) {
                        return new SimpleDateFormat( "yyyyMMdd HHmm" ).parse( value ) ;
                    }
                    else {
                        return new SimpleDateFormat( "yyyyMMdd-HHmm" ).parse( value ) ;
                    }
                }
                else if( len == 14 ) {
                    return new SimpleDateFormat( "yyyyMMddHHmmss" ).parse( value ) ;
                }
                else if( len == 15 ) {
                    if( value.indexOf( " " ) != -1 ) {
                        return new SimpleDateFormat( "yyyyMMdd HHmmss" ).parse( value ) ;
                    }
                    else {
                        return new SimpleDateFormat( "yyyyMMdd-HHmmss" ).parse( value ) ;
                    }
                }
                else if( len == 16 ) {
                    if( value.indexOf( " " ) != -1 ) {
                        if( value.indexOf( "/" ) != -1 ) {
                            if( value.indexOf( ":" ) != -1 ) {
                                return new SimpleDateFormat( "yyyy/MM/dd HH:mm" ).parse( value ) ;
                            }
                            else {
                                return new SimpleDateFormat( "yyyy/MM/dd HH-mm" ).parse( value ) ;
                            }
                        }
                        else if( value.indexOf( "-" ) != -1 ) {
                            if( value.indexOf( ":" ) != -1 ) {
                                return new SimpleDateFormat( "yyyy-MM-dd HH:mm" ).parse( value ) ;
                            }
                            else {
                                return new SimpleDateFormat( "yyyy-MM-dd HH-mm" ).parse( value ) ;
                            }
                        }
                    }
                    else {
                        if( value.indexOf( "/" ) != -1 ) {
                            if( value.indexOf( ":" ) != -1 ) {
                                return new SimpleDateFormat( "yyyy/MM/dd-HH:mm" ).parse( value ) ;
                            }
                            else {
                                return new SimpleDateFormat( "yyyy/MM/dd-HH-mm" ).parse( value ) ;
                            }
                        }
                        else if( value.indexOf( "-" ) != -1 ) {
                            if( value.indexOf( ":" ) != -1 ) {
                                return new SimpleDateFormat( "yyyy-MM-dd-HH:mm" ).parse( value ) ;
                            }
                            else {
                                return new SimpleDateFormat( "yyyy-MM-dd-HH-mm" ).parse( value ) ;
                            }
                        }
                    }
                }
                else {
                    if( value.indexOf( " " ) != -1 ) {
                        if( value.indexOf( "/" ) != -1 ) {
                            if( value.indexOf( ":" ) != -1 ) {
                                return new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss" ).parse( value ) ;
                            }
                            else {
                                return new SimpleDateFormat( "yyyy/MM/dd HH-mm-ss" ).parse( value ) ;
                            }
                        }
                        else if( value.indexOf( "-" ) != -1 ) {
                            if( value.indexOf( ":" ) != -1 ) {
                                return new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss" ).parse( value ) ;
                            }
                            else {
                                return new SimpleDateFormat( "yyyy-MM-dd HH-mm-ss" ).parse( value ) ;
                            }
                        }
                    }
                    else {
                        if( value.indexOf( "/" ) != -1 ) {
                            if( value.indexOf( ":" ) != -1 ) {
                                return new SimpleDateFormat( "yyyy/MM/dd-HH:mm:ss" ).parse( value ) ;
                            }
                            else {
                                return new SimpleDateFormat( "yyyy/MM/dd-HH-mm-ss" ).parse( value ) ;
                            }
                        }
                        else if( value.indexOf( "-" ) != -1 ) {
                            if( value.indexOf( ":" ) != -1 ) {
                                return new SimpleDateFormat( "yyyy-MM-dd-HH:mm:ss" ).parse( value ) ;
                            }
                            else {
                                return new SimpleDateFormat( "yyyy-MM-dd-HH-mm-ss" ).parse( value ) ;
                            }
                        }
                    }
                }
                throw new IOException( "Timestamp型に対して一致しない内容です:" + value ) ;
        }
        return null ;
    }
    
    /**
     * Javaシーケンスが有効かチェック.
     */
    private static final boolean isJavaSequence() {
        return DaoSessionFactory.getInstance().isJavaSequence() ;
    }
    
    /**
     * CSV-ReaderによるInsert処理.
     */
    private static final int csvReaderToInsert( boolean mode,CsvReader csv,String sectionName,Config config,Record record,Table table,
        List<String> targetCols,Map<String,Integer> colsMap,List<Integer> colsTypes,Map<String,Object> def,boolean seqFlag )
        throws Exception {
        String[] line ;
        int srcLen = targetCols.size() ;
        int lineCnt = 1 ;
        int ret = 0 ;
        boolean javaSeqFlag = isJavaSequence() ;
        List<String> names = CreateTable.getColumns( config,sectionName,seqFlag ) ;
        if( names == null || names.size() <= 0 ) {
            throw new IOException( "不正なセクション名:" + sectionName ) ;
        }
        String sql = insertString( sectionName,names,colsMap,table ) ;
        Object[] params = createParamsLength( names ) ;
        String[] thisCols = getTargetColumns( names ) ;
        names = null ;
        java.util.Date nowDate = new java.util.Date() ;
        while( csv.next() ) {
            clearParams( params ) ;
            try {
                line = csv.getArray() ;
                if( line.length != srcLen ) {
                    throw new IOException( "長さが不一致:header:" + srcLen + " target:" + line.length ) ;
                }
                // CSV内容とパラメータをセット.
                settingParams( params,thisCols,targetCols,colsMap,colsTypes,def,sectionName,nowDate,line ) ;
                // IDがシーケンス系で追加する必要がある場合の処理.
                addId( params,thisCols,record,sectionName,table,seqFlag,javaSeqFlag ) ;
                // booleanなど、DBに存在しない機能に対する値修正.
                convertUniqueTypes( params,colsTypes,table ) ;
                // insert処理.
                record.executeUpdate( sql,params,params.length ) ;
                // insert後処理.
                endInsert( params,colsMap,sectionName,seqFlag,javaSeqFlag ) ;
                // １件毎コミット.
                record.commit() ;
                ret ++ ;
            } catch( Exception e ) {
                try {
                    record.rollback() ;
                } catch( Exception ee ) {
                }
                if( mode == false ) {
                    System.out.println( sectionName + " csv - error:" + lineCnt + " [" + e + "]" ) ;
                    e.printStackTrace() ;
                }
                else {
                    LOG.info( sectionName + " csv - error:" + lineCnt + " [" + e + "]",e ) ;
                }
                throw e ;
            }
            lineCnt ++ ;
        }
        return ret ;
    }
    
    /**
     * 定型Insert文を生成.
     */
    private static final String insertString( String sectionName,List<String> names,Map<String,Integer> colsMap,Table table )
        throws Exception {
        int len = names.size() ;
        int valueLen = len ;
        StringBuilder buf = new StringBuilder() ;
        buf.append( "INSERT INTO " ).append( sectionName ).append( " (" ) ;
        // PreparedStatement用SQL文の作成.
        for( int i = 0 ; i < len ; i ++ ) {
            if( i != 0 ) {
                buf.append( "," ) ;
            }
            buf.append( names.get( i ).toUpperCase() ) ;
        }
        buf.append( ") VALUES (" ) ;
        for( int i = 0 ; i < valueLen ; i ++ ) {
            if( i != 0 ) {
                buf.append( "," ) ;
            }
            buf.append( "?" ) ;
        }
        return buf.append( ")" ).append( table.getSemicolon() ).toString() ;
    }
    
    /**
     * パラーメータを生成.
     */
    private static final Object[] createParamsLength( List<String> names )
        throws Exception {
        return new Object[ names.size() ] ;
    }
    
    /**
     * カラム名群を抽出.
     */
    private static final String[] getTargetColumns( List<String> names )
        throws Exception {
        int len = names.size() ;
        String[] ret = new String[ len ] ;
        for( int i = 0 ; i < len ; i ++ ) {
            ret[ i ] = names.get( i ) ;
        }
        return ret ;
    }
    
    /**
     * パラメータをクリア.
     */
    private static final void clearParams( Object[] o ) {
        int len = o.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            o[ i ] = null ;
        }
    }
    
    /**
     * パラメータを設定.
     */
    private static final void settingParams( Object[] params,String[] thisCols,List<String> targetCols,
        Map<String,Integer> colMap,List<Integer> colTypes,Map<String,Object> def,String sectionName,java.util.Date nowDate,
        String[] line )
        throws Exception {
        int len = targetCols.size() ;
        String name ;
        Integer no ;
        // def定義内容をセット.
        Iterator it = def.keySet().iterator() ;
        while( it.hasNext() ) {
            name = (String)it.next() ;
            no = colMap.get( name ) ;
            params[ no ] = def.get( name ) ;
        }
        // CSV定義内容をセット.
        for( int i = 0 ; i < len ; i ++ ) {
            name = targetCols.get( i ) ;
            no = colMap.get( name ) ;
            if( no == null ) {
                throw new IOException( sectionName + " のファイル構成は不正です:" + name ) ;
            }
            params[ no ] = convertStringByTypes( colTypes.get( no ),line[ i ] ) ;
        }
        // データ生成時間／データ更新時間が存在する場合、CSV側に存在しない場合は、現在の日付をセット.
        if( colMap.containsKey( "create_time" ) ) {
            if( params[ colMap.get( "create_time" ) ] == null ) {
                params[ colMap.get( "create_time" ) ] = nowDate ;
            }
        }
        if( colMap.containsKey( "update_time" ) ) {
            if( params[ colMap.get( "update_time" ) ] == null ) {
                params[ colMap.get( "update_time" ) ] = nowDate ;
            }
        }
    }
    
    /**
     * シーケンスIDを取得.
     */
    private static final long getSequenceId( Record record,String table ) throws Exception {
        long ret = -1 ;
        ResultSet rs = record.executeQuery( record.getSupportKind().getSequenceId( table ) ) ;
        try {
            while( true ) {
                if( !rs.next() ) {
                    break ;
                }
                ret = rs.getLong( 1 ) ;
                break ;
            }
            rs.close() ;
            rs = null ;
        } finally {
            if( rs != null ) {
                try {
                    rs.close() ;
                } catch( Exception e ) {
                }
            }
        }
        return ret ;
    }
    
    /**
     * JavaシーケンスでシーケンスIDを取得.
     */
    private static final long getSequenceIdByJava( String table )
        throws Exception {
        return DaoSessionFactory.getInstance().getJavaSequence().getId( table ) ;
    }
    
    /**
     * Javaシーケンスを更新.
     */
    private static final void updateJavaSeqence( String table,long value )
        throws Exception {
        DaoSessionFactory.getInstance().getJavaSequence().setId( table,value ) ;
    }
    
    /**
     * IDを外部追加する必要がある場合は、追加処理.
     */
    private static final void addId( Object[] params,String[] thisCols,Record record,
        String sectionName,Table table,boolean seqFlag,boolean javaSeqFlag )
        throws Exception {
        // シーケンスが利用可能.
        if( seqFlag ) {
            // Javaシーケンスが利用可能.
            if( javaSeqFlag ) {
                params[ 0 ] = getSequenceIdByJava( sectionName ) ;
            }
            // Javaシーケンスが利用できない場合.
            // IDを直接設定できるDBMSの場合.
            else if( table.isInsertId() ) {
                params[ 0 ] = getSequenceId( record,sectionName ) ;
            }
        }
    }
    
    /**
     * booleanなど、DBに存在しない機能に対する値修正.
     */
    private static final void convertUniqueTypes( Object[] params,List<Integer> colTypes,Table table )
        throws Exception {
        if( !"true".equals( table.getBooleanByTrue().toLowerCase() ) ) {
            int len = colTypes.size() ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( colTypes.get( i ) == Types.BOOLEAN ) {
                    if( ( ( Boolean )params[ i ] ) == Boolean.TRUE ) {
                        params[ i ] = table.getBooleanByTrue() ;
                    }
                    else {
                        params[ i ] = table.getBooleanByFalse() ;
                    }
                }
            }
        }
    }
    
    /**
     * insert後処理.
     */
    private static final void endInsert( Object[] params,Map<String,Integer> colsMap,
        String sectionName,boolean seqFlag,boolean javaSeqFlag )
        throws Exception {
        // IDが設定されていて、シーケンスが利用可能な場合.
        // Javaシーケンスが利用可能.
        if( params[ colsMap.get( "id" ) ] != null && seqFlag && javaSeqFlag ) {
            // Javaシーケンスをアップデート.
            updateJavaSeqence( sectionName,( Long )params[ colsMap.get( "id" ) ] ) ;
        }
    }
}

