package org.maachang.dao.dbms.ctbl;

/**
 * 生成テーブル管理.
 * 
 * @version 2007/10/18
 * @author masahito suzuki
 * @since MaachangDao 1.00
 */
public class TableMaster {
    
    /**
     * 特殊定義ヘッダ文字列.
     */
    protected static final String SPECIAL_DEFINE = "@" ;
    
    /**
     * 条件定義:結合元テーブルを示す条件.
     */
    protected static final String SRC_JOIN = "@src_join" ;
    
    /**
     * 条件定義:シーケンス条件.
     */
    protected static final String SEQUENCE_MODE = "@sequence" ;
    
    /**
     * 条件定義:FILLFACTOR(PostgreSQLのみ)を示す条件.
     */
    protected static final String FILLFACTOR_NAME = "@fill_factor" ;
    
    /**
     * 条件定義:デフォルトソート.
     */
    protected static final String DEFAULT_SORT_MODE = "@sort" ;
    
    /**
     * 条件定義:表示カラム名.
     */
    protected static final String VIEW_COLUMN_NAMES = "@views" ;
    
    /**
     * 条件定義:名前管理マスター条件.
     */
    protected static final String NAMES_MASTER_MODE = "@names_master" ;
    
    /**
     * 条件定義:マスターキャッシュ条件.
     */
    protected static final String MASTER_CACHE_MODE = "@cache" ;
    
    /**
     * 条件定義:マスター編集条件.
     */
    protected static final String MASTER_EDIT_MODE = "@edit" ;
    
    /**
     * 条件定義:マスター新規作成条件.
     */
    protected static final String MASTER_CREATE_MODE = "@create" ;
    
    /**
     * 条件定義:テーブル名を示す条件.
     */
    protected static final String TABLE_NAME = "@name" ;
    
    /**
     * 条件定義:テーブル区分.
     */
    protected static final String TABLE_DIVISION = "@division" ;
    
    /**
     * 条件定義:その他拡張条件.
     */
    protected static final String TABLE_ETC = "@etc" ;
    
    /**
     * コンフィグ定義 : テーブル結合条件.
     */
    protected static final String JOIN = ".join" ;
    
    
    /**
     * コンフィグ定義 : 要素表示テーブルサイズ.
     */
    public static final String ELEMENT_DEFINE = ".define" ;
    
    /**
     * データ削除確認フラグ.
     */
    protected static final String DELETE_FLAG_COLUMN = "delete_flag" ;
    
    /**
     * 検索表示フラグ.
     */
    protected static final String SEARCH_FLAG_COLUMN = "search_flag" ;
    
    /**
     * 変数定義:テーブル名定義.
     */
    public static final String TABLE_NAME_DEFINE = "tableName" ;
    
    /**
     * 変数定義:カラム名群定義.
     */
    public static final String TABLE_COLUMNS_DEFINE = "columns" ;
    
    /**
     * 変数定義:カラムタイプ群定義.
     */
    public static final String TABLE_TYPES_DEFINE = "types" ;
    
    /**
     * 変数定義:テーブル区分定義.
     */
    public static final String TABLE_DIVISION_DEFINE = "division" ;
    
    /**
     * 変数定義:シーケンス定義.
     */
    public static final String SEQUENCE_DEFINE = "sequence" ;
    
    /**
     * 変数定義:デフォルトソート定義.
     */
    public static final String DEFAULT_SORT_DEFINE = "default_sort" ;
    
    /**
     * 変数定義:表示カラム名定義.
     */
    public static final String VIEW_COLUMNS_DEFINE = "view_columns" ;
    
    /**
     * 変数定義:名前管理マスター定義.
     */
    public static final String NAMES_MASTER_DEFINE = "namesMaster" ;
    
    /**
     * 変数定義:テーブル元Join定義.
     */
    public static final String SRC_JOIN_DEFINE = "srcJoin" ;
    
    /**
     * 変数定義:デフォルト値定義.
     */
    public static final String DEFAULT_COLUMN_DEFINE = "defaultColumns" ;
    
    /**
     * 変数定義:マスターキャッシュ定義.
     */
    public static final String MASTER_CACHE_DEFINE = "isCacheMap" ;
    
    /**
     * 変数定義:マスター編集定義.
     */
    public static final String MASTER_EDIT_DEFINE = "isMasterEdit" ;
    
    /**
     * 変数定義:マスター新規作成定義.
     */
    public static final String MASTER_CREATE_DEFINE = "isMasterCreate" ;
    
    /**
     * 変数定義:削除フラグ定義.
     */
    public static final String IS_DELETE_DEFINE = "isDeleteFlag" ;
    
    /**
     * 変数定義:検索フラグ定義.
     */
    public static final String IS_SEARCH_DEFINE = "isSearchFlag" ;
    
    /**
     * 変数定義:その他拡張定義.
     */
    public static final String TABLE_ETC_DEFINE = "etc" ;
    
    /**
     * 管理マスター.
     */
    private static final Table[] TBLS = {
        new TableMySql(),
        new TablePostgre(),
        new TableOracle(),
        new TableDerby(),
        new TableFireBird(),
        new TableH2(),
        new TableHSql()
    } ;
    
    /**
     * アダプタ名から、マスター情報を取得.
     * <BR><BR>
     * アダプタ名から、マスター情報を取得します.
     * <BR>
     * @param adapter 対象のアダプタ名を設定します.
     * @return Table 管理されたマスターテーブル情報を設定します.
     */
    public static final Table get( String adapter ) {
        if( adapter == null || ( adapter = adapter.trim().toLowerCase() ).length() <= 0 ) {
            return null ;
        }
        int len = TBLS.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( adapter.equals( TBLS[ i ].getAdapter() ) ) {
                return TBLS[ i ] ;
            }
        }
        return null ;
    }
    
}
