package org.maachang.dao.dbms.kind;

import org.maachang.dao.dbms.DbUtil;

/**
 * Postgre用DBMSサポート定義.
 * 
 * @version 2007/10/18
 * @author masahito suzuki
 * @since MaachangDao 1.00
 */
public class KindByPostgre extends SupportKind {
    
    /**
     * リミットSQL文を取得.
     * @param mode join/srcJoinが存在する場合は、trueを設定します.
     * @param offset オフセット値を設定します.
     * @param limit リミット値を設定します.
     * @return String リミットSQL文が返されます.
     */
    public String getLimit( boolean mode,int offset,int limit ) {
        if( mode ) {
            return new StringBuilder().
                append( " OFFSET " ).append( offset ).toString() ;
        }
        return new StringBuilder().
            append( " LIMIT " ).append( limit ).
            append( " OFFSET " ).append( offset ).append( " " ).toString() ;
    }
    
    /**
     * Boolean非サポートフラグ.
     * <BR><BR>
     * Boolean非サポートフラグを取得します.
     * <BR>
     * @return boolean [true]の場合は、サポートしています.
     */
    public boolean isBoolean() {
        return true ;
    }
    
    /**
     * limitサポートフラグ.
     * @return boolean [true]の場合、limit処理はサポートしています.
     */
    public boolean isLimit() {
        return true ;
    }
    
    /**
     * アダプタ名を取得.
     * <BR><BR>
     * アダプタ名を取得します.
     * <BR>
     * @return String アダプタ名が返されます.
     */
    public String getAdapter() {
        return "postgresql" ;
    }
    
    /**
     * 新しいIDの取得SQL.
     * <BR><BR>
     * 指定モデルに対する新しいIDを取得するSQLが返されます.
     * <BR>
     * @param model 対象のモデル名を設定します.
     * @return String SQLが返されます.
     */
    public String getSequenceId( String model ) {
        if( model == null ) {
            return "support" ;
        }
        return new StringBuilder().append( "select nextval ('" ).
            append( DbUtil.convertJavaNameByDBName( model ) ).
            append( SEQ_NAME ).
            append( "')" ).toString() ;
    }
}
