package org.maachang.dao.dbms.pool;

import java.sql.Connection;

/**
 * DBCPコネクションプーリング.
 * 
 * @version 2007/10/18
 * @author masahito suzuki
 * @since MaachangDao 1.00
 */
public interface ConnectionPooling {
    
    /**
     * オブジェクト破棄.
     */
    public void destroy() ;
    
    /**
     * コネクションを取得.
     * @return Connection コネクションオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public Connection getConnection() throws Exception ;
    
    /**
     * 最大コネクション数を取得.
     * @return int 最大コネクション数が返されます.
     */
    public int max() ;
    
    /**
     * 使用中のコネクション数を取得.
     * @return int 使用中のコネクション数が返されます.
     */
    public int active() ;
    
    /**
     * 未使用コネクション数を取得.
     * @return int 未使用中のコネクション数が返されます.
     */
    public int idle() ;
}
