package org.maachang.dao.dbms.pool.atomic;

/**
 * RDBMS接続定義.
 * 
 * @version 2009/05/30
 * @author masahito suzuki
 * @since MaachangDao 1.13
 */
public class RdbmsDefine {
    
    private String url = null ;
    private String userName = null ;
    private String passwd = null ;
    private boolean readOnly = false ;
    private boolean autoCommit = false ;
    
    /**
     * コンストラクタ.
     */
    private RdbmsDefine() {
        
    }
    
    /**
     * コンストラクタ.
     * @param url コネクションURLを設定します.
     * @param userName コネクションユーザ名を設定します.
     * @param passwd コネクションパスワードを設定します.
     * @param readOnly [true]の場合、ReadOnlyで接続します.
     * @param autoCommit [true]の場合、AutoCommitで処理します.
     */
    public RdbmsDefine( String url,String userName,String passwd,
        boolean readOnly,boolean autoCommit ) {
        this.url = url.trim() ;
        this.userName = userName.trim() ;
        this.passwd = passwd.trim() ;
        this.readOnly = readOnly ;
        this.autoCommit = autoCommit ;
    }
    
    /**
     * autoCommit を取得.
     * <BR><BR>
     * @return autoCommit が返されます.
     */
    public boolean isAutoCommit() {
        return autoCommit;
    }
    
    /**
     * passwd を取得.
     * <BR><BR>
     * @return passwd が返されます.
     */
    public String getPasswd() {
        return passwd;
    }
    
    /**
     * readOnly を取得.
     * <BR><BR>
     * @return readOnly が返されます.
     */
    public boolean isReadOnly() {
        return readOnly;
    }
    
    /**
     * url を取得.
     * <BR><BR>
     * @return url が返されます.
     */
    public String getUrl() {
        return url;
    }
    
    /**
     * userName を取得.
     * <BR><BR>
     * @return userName が返されます.
     */
    public String getUserName() {
        return userName;
    }
    
    /**
     * ユーザ名が存在するかチェック.
     * @return boolean [true]の場合、存在します.
     */
    public boolean isUserName() {
        return this.userName != null && this.userName.length() > 0 ;
    }
    
    /**
     * パスワードが存在するかチェック.
     * @return boolean [true]の場合、存在します.
     */
    public boolean isPasswd() {
        return this.passwd != null && this.passwd.length() > 0 ;
    }
    
    /**
     * 文字列出力.
     * @return String 文字列が返されます.
     */
    public String toString() {
        return new StringBuilder().
            append( " url:" ).append( url ).
            append( " userName:" ).append( userName ).
            append( " passwd:" ).append( passwd ).
            append( " readOnly:" ).append( readOnly ).
            append( " autoCommit:" ).append( autoCommit ).
            toString() ;
    }
}
