package org.maachang.dao.dbms.pool.atomic;

import java.sql.Connection;
import java.sql.DriverManager;
import java.util.Properties;

/**
 * RDBMS接続ドライバーマネージャ.
 * 
 * @version 2009/05/30
 * @author masahito suzuki
 * @since MaachangDao 1.13
 */
abstract class RdbmsDriverManager {
    private RdbmsDriverManager(){}
    private static final String BLOCK_SIZE = "512" ;
    
    /**
     * ドライバマネージャ経由でコネクションを取得.
     * @param bean コネクション定義Beanを設定します.
     * @return Connection コネクションオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public static Connection driverConnection( RdbmsDefine bean )
        throws Exception {
        Connection ret;
        Properties p = new java.util.Properties();
        p.put("block size",BLOCK_SIZE);
        if( !bean.isUserName() ) {
            ret = DriverManager.getConnection( bean.getUrl(),p );
        }
        else {
            p.put("user",bean.getUserName());
            p.put("password",bean.getPasswd());
            ret = DriverManager.getConnection( bean.getUrl(),p );
        }
        ret.setReadOnly( bean.isReadOnly() );
        ret.setAutoCommit( bean.isAutoCommit() ) ;
        return ret;
    }
}

