package org.maachang.seabass;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.conf.MinStartupConfig;
import org.maachang.seabass.dbm.BdbService;
import org.maachang.seabass.down.ShutdownScript;
import org.maachang.seabass.httpd.HttpdVersionDef;
import org.maachang.seabass.httpd.engine.script.BaseModel;
import org.maachang.seabass.httpd.engine.script.BaseModelImpl;
import org.maachang.seabass.httpd.engine.script.EndScript;
import org.maachang.seabass.httpd.engine.script.ScriptDef;
import org.maachang.seabass.httpd.engine.script.cache.ApiManager;
import org.maachang.seabass.httpd.engine.script.cache.CacheScriptManager;
import org.maachang.seabass.httpd.engine.script.dao.DaoFactory;
import org.maachang.seabass.httpd.engine.script.js.InitJS;
import org.maachang.seabass.httpd.engine.script.scripts.ApplicationScriptFactory;
import org.maachang.seabass.startjava.StartupJavaManager;
import org.maachang.shutdown.ShutdownHook;
import org.maachang.util.FileUtil;
import org.maachang.util.StringUtil;

/**
 * SeaBassコンソールスクリプト.
 * 
 * @version 2007/08/26
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class ConsoleScript {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( ConsoleScript.class ) ;
    
    /**
     * コンストラクタ.
     */
    private ConsoleScript() {
        
    }
    
    /**
     * メインメソッド.
     * <BR><BR>
     * @param args Javaからのパラメータが設定されます.
     * @exception Exception 例外.
     */
    public static final void main( String[] args )
        throws Exception {
        new ConsoleScript().execution( args ) ;
    }
    
    /**
     * 内部変数.
     */
    private SeaBassContext ctx = null ;
    private ApplicationScriptFactory webApp = null ;
    private BaseModel baseModel = null ;
    private String path = null ;
    private SeaBassScriptEngine engine = ApiManager.getInstance().getScriptEngine() ;
    private final SimpleDateFormat fmt = new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss",Locale.US ) ;
    
    /**
     * 実行処理.
     */
    private void execution( String[] args )
        throws Exception {
        this.path = FileUtil.getFullPath( "." )+FileUtil.FILE_SPACE ;
        SeaBassMode.getInstance().complie() ;
        if( args != null && args.length > 0 ) {
            LOG.info( "## seabass - console .. " ) ;
            LOG.info( "## [ver."+HttpdVersionDef.getVersion()+" update."+HttpdVersionDef.getDate()+"]" ) ;
            LOG.info( "##" ) ;
            LOG.info( "## scriptVersion:" + SeaBassScriptEngine.getVersion() ) ;
            LOG.info( "##" ) ;
        }
        else {
            System.out.println( "seabass - console .. " ) ;
            System.out.println( "[ver."+HttpdVersionDef.getVersion()+" update."+HttpdVersionDef.getDate()+"]" ) ;
            System.out.println() ;
            System.out.println( "scriptVersion:" + SeaBassScriptEngine.getVersion() ) ;
            System.out.println() ;
        }
        
        // BdbServiceを初期化.
        BdbService.getInstance().init() ;
        
        // 最小コンフィグ読み込み.
        MinStartupConfig.init( path ) ;
        
        // スクリプトFactory情報を読み込み.
        this.webApp = new ApplicationScriptFactory( path ) ;
        GlobalManager.getInstance().put( ServiceDef.MANAGER_BY_WEB_APP_FACTORY,webApp ) ;
        
        // DaoFactoryを初期化.
        DaoFactory daoFactory = new DaoFactory() ;
        GlobalManager.getInstance().put( ServiceDef.DAO_FACTORY,daoFactory ) ;
        
        // 外部スタートアップ.
        StartupJavaManager.getInstance().init() ;
        StartupJavaManager.getInstance().startup() ;
        
        // シャットダウン処理を設定.
        ShutdownHook.registHook( new ShutdownScript() ) ;
        
        // スクリプトを読み込む.
        //readScript() ;
        initCtx() ;
        
        // バッチモードで起動する場合.
        if( args != null && args.length > 0 ) {
            StringBuilder buf = new StringBuilder() ;
            StringBuilder names = new StringBuilder() ;
            batch( buf ) ;
            buf.append( InitJS.getInstance().get() ) ;
            int len = args.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    names.append( "," ) ;
                }
                names.append( args[ i ] ) ;
                String str = FileUtil.getFileByString( args[ i ] ) ;
                if( str == null ) {
                    throw new IOException( "指定ファイル名[" + args[ i ] + "]は存在しません" ) ;
                }
                buf.append( str ) ;
                LOG.info( "## readBatch:" + args[ i ] ) ;
            }
            String nms = names.toString() ;
            names = null ;
            String script = buf.toString() ;
            buf = null ;
            // 処理実行.
            try {
                ctx.setName( "console["+nms+"]" ) ;
                engine.eval( CacheScriptManager.getInstance().insertPackage( script ),ctx ) ;
            } catch( Exception e ) {
                LOG.error( "## error",e ) ;
            }
        }
        // コンソール実行.
        else {
            startConsole() ;
        }
    }
    
    /**
     * バッチモードでのコンソール表示用内容をログ出力用に変換.
     */
    private void batch( StringBuilder buf ) {
        buf.append( "var print = function(info) { pkg(\"org.maachang.seabass.ConsoleScript\").logout(info); };\n" ).
            append( "var println = function(info) { pkg(\"org.maachang.seabass.ConsoleScript\").logout(info); };\n" ).
            append( "var p = function(info) { pkg(\"org.maachang.seabass.ConsoleScript\").logout(info); };\n" ).
            append( "var puts = function(info) { pkg(\"org.maachang.seabass.ConsoleScript\").logout(info); };\n" ).
            append( "var alert = function(info) { pkg(\"org.maachang.seabass.ConsoleScript\").logout(info); };\n" ) ;
    }
    
    /**
     * ログ出力用.
     */
    public static final void logout( String string ) {
        if( string != null && string.length() <= 0 ) {
            return ;
        }
        LOG.info( string ) ;
    }
    
    /**
     * スクリプトを読み込む.
     */
    private void readScript()
        throws Exception {
        System.out.println( "" ) ;
        
        // ローカルデータを破棄.
        CacheScriptManager.getInstance().exitScript( this.ctx ) ;
        
        // キャッシュリロード処理.
        CacheScriptManager.getInstance().reload() ;
        
        // Context初期化.
        initCtx() ;
    }
    
    /**
     * init-Context.
     */
    private void initCtx() throws Exception {
        // モデル再構築.
        this.baseModel = new BaseModelImpl() ;
        
        // キャッシュスクリプトマネージャを読み込む.
        this.ctx = CacheScriptManager.getInstance().script( "console" ) ;
        
        // デフォルトパラメータを読み込む.
        readDefaultParams() ;
        
        // デフォルト条件を設定.
        ScriptDef.setDirectoryByBindings( this.ctx ) ;
    }
    
    /**
     * デフォルトパラメータを読み込む.
     */
    private void readDefaultParams()
        throws Exception {
        ctx.put( ScriptDef.CURRENT_SCRIPT,ScriptDef.MODE_CONSOLE ) ;
        ctx.put( ScriptDef.SCRIPT_MODE,ScriptDef.MODE_CONSOLE ) ;
        ctx.put( ScriptDef.SCRIPT_DETAIL,ScriptDef.MODE_CONSOLE ) ;
        ctx.put( ScriptDef.SCRIPT_BY_MODEL,baseModel ) ;
        ctx.put( ScriptDef.SCRIPT_BY_LOG,LOG ) ;
        ctx.put( ScriptDef.SCRIPT_DATE,new java.util.Date() ) ;
    }
    
    /**
     * 基本コマンドライン.
     */
    private static final String RELOAD = "reload" ;
    private static final String EXIT = "exit" ;
    private static final String MEM = "mem" ;
    private static final String GC = "gc" ;
    private static final String LIST = "list" ;
    private static final String VIEW = "view" ;
    private static final String CLS = "cls" ;
    
    /**
     * コンソール実行.
     */
    private void startConsole()
        throws Exception {
        StringBuilder buf = null ;
        System.out.println() ;
        BufferedReader bo = new BufferedReader(
            new InputStreamReader( System.in ) ) ;
        while( true ) {
            System.out.print( "$>" ) ;
            String one = bo.readLine() ;
            if( one == null || ( one = one.trim() ).length() <= 0 ) {
                continue ;
            }
            one = one.trim() ;
            String ck = one.toLowerCase() ;
            if( EXIT.equals( ck ) ) {
                if( baseModel.isCreate() == true ) {
                    baseModel.commit() ;
                }
                System.out.println( " ... success exit" ) ;
                return ;
            }
            if( ck.startsWith( "!" ) ) {
                ck = ck.substring( "!".length(),ck.length() ) ;
                // リロード.
                if( RELOAD.equals( ck ) ) {
                    readScript() ;
                    System.out.println( " ... success reload" ) ;
                    continue ;
                }
                // 処理終了.
                else if( EXIT.equals( ck ) ) {
                    if( baseModel.isCreate() == true ) {
                        baseModel.commit() ;
                    }
                    System.out.println( " ... success exit" ) ;
                    return ;
                }
                // メモリー参照.
                else if( MEM.equals( ck ) ) {
                    System.out.println( "+-------------------------------------------" ) ;
                    String[] objs = CacheScriptManager.getInstance().getNames( ctx ) ;
                    for( int i = 0 ; i < objs.length ; i ++ ) {
                        System.out.println( objs[ i ] + " : " + ctx.get( objs[ i ] ) ) ;
                    }
                    System.out.println( "+-------------------------------------------" ) ;
                    System.out.println( "total:" + Runtime.getRuntime().totalMemory() ) ;
                    System.out.println( "max  :" + Runtime.getRuntime().maxMemory() ) ;
                    System.out.println( "free :" + Runtime.getRuntime().freeMemory() ) ;
                    continue ;
                }
                // 利用可能メソッド出力.
                else if( LIST.equals( ck ) ) {
                    String[] objs = CacheScriptManager.getInstance().getNames( ctx ) ;
                    if( objs != null ) {
                        ArrayList<String> lst = new ArrayList<String>() ;
                        for( int i = 0 ; i < objs.length ; i ++ ) {
                            Object o = ctx.get( objs[ i ] ) ;
                            if( o == null ) {
                                continue ;
                            }
                            if( o instanceof pnuts.lang.PnutsFunction ) {
                                lst.add( ( String )objs[ i ] ) ;
                            }
                        }
                        String[] strs = sort( lst ) ;
                        lst = null ;
                        outputView( strs ) ;
                    }
                    continue ;
                }
                // 利用可能View情報出力.
                else if( VIEW.equals( ck ) ) {
                    String[] objs = CacheScriptManager.getInstance().getNames( ctx ) ;
                    if( objs != null ) {
                        ArrayList<String> lst = new ArrayList<String>() ;
                        for( int i = 0 ; i < objs.length ; i ++ ) {
                            Object o = ctx.get( objs[ i ] ) ;
                            if( o == null ) {
                                continue ;
                            }
                            if( ( ( String )objs[ i ] ).indexOf( "_" ) == -1 &&
                                ( ( String )objs[ i ] ).indexOf( "$" ) == -1 &&
                                ( o instanceof pnuts.lang.PnutsFunction ) == false ) {
                                lst.add( ( String )objs[ i ] ) ;
                            }
                        }
                        String[] strs = sort( lst ) ;
                        lst = null ;
                        outputView( strs ) ;
                    }
                    continue ;
                }
                // GC起動.
                else if( GC.equals( ck ) ) {
                    Runtime.getRuntime().gc() ;
                    continue ;
                }
                // 画面クリア.
                else if( CLS.equals( ck ) ) {
                    for( int i = 0 ; i < 120 ; i ++ ) {
                        System.out.println() ;
                    }
                    continue ;
                }
                // help内容を表示.
                else if( one.length() == 1 ) {
                    viewHelp() ;
                    continue ;
                }
            }
            else if( ck.startsWith( "?" ) ) {
                viewHelp() ;
                continue ;
            }
            // スクリプト内容を実行.
            //one = ConvertScript.convert( one ) ;
            one = one.trim() ;
            if( one.endsWith( "\\" ) ) {
                if( buf == null ) {
                    buf = new StringBuilder() ;
                }
                buf.append( one.substring( 0,one.length()-1 ) ).append( "\n" ) ;
                continue ;
            }
            else {
                if( buf != null ) {
                    buf.append( one ).append( "\n" ) ;
                    one = buf.toString() ;
                }
                buf = null ;
            }
            System.out.println( ">>" + one ) ;
            try {
                ctx.setName( "console" ) ;
                Object obj = engine.eval( CacheScriptManager.getInstance().insertPackage( one ),ctx ) ;
                if( obj != null ) {
                    if( EndScript.isEndScript( obj ) == true ) {
                        if( baseModel.isCreate() == true ) {
                            baseModel.commit() ;
                        }
                        System.out.println( "... exit" ) ;
                        return ;
                    }
                    if( obj instanceof java.util.Date ) {
                        System.out.println( fmt.format( (java.util.Date)obj ) ) ;
                    }
                    else {
                        System.out.println( obj ) ;
                    }
                }
                else {
                    System.out.println() ;
                }
            } catch( Exception e ) {
                try {
                    baseModel.rollback() ;
                } catch( Exception ee ) {
                }
                e.printStackTrace() ;
            }
            
        }
    }
    
    /**
     * 対象情報をソート.
     */
    private static final String[] sort( ArrayList<String> lst ) {
        String[] strs = null ;
        int len = lst.size() ;
        if( len > 0 ) {
            strs = new String[ len ] ;
            for( int i = 0 ; i < len ; i ++ ) {
                strs[ i ] = lst.get( i ) ;
            }
            Arrays.sort( strs ) ;
        }
        return strs ;
    }
    
    /**
     * 内容表示.
     */
    private static final void outputView( String[] strs ) {
        int cnt = 0 ;
        if( strs != null ) {
            for( int i = 0 ; i < strs.length ; i ++ ) {
                String view = strs[ i ] ;
                int viewLen = view.length() ;
                if( viewLen <= 16 ) {
                    view = "["+view+"] "+StringUtil.createString( ' ',16-viewLen ) ;
                }
                else {
                    view = "["+view+"] "+StringUtil.createString( ' ',(19*2)-(viewLen+3) ) ;
                }
                System.out.print( view ) ;
                cnt += view.length() ;
                if( cnt >= 75 ) {
                    System.out.println() ;
                    cnt = 0 ;
                }
            }
            System.out.println() ;
        }
        System.out.println() ;
    }
    
    private static final void viewHelp() {
        System.out.println( "** help ************************************" ) ;
        System.out.println( "!reload : ライブラリを再読み込み." ) ;
        System.out.println( "!exit   : スクリプトを終了." ) ;
        System.out.println( "!mem    : メモリ情報を出力." ) ;
        System.out.println( "!gc     : ガページコレクターを実行." ) ;
        System.out.println( "!list   : 有効メソッド一覧." ) ;
        System.out.println( "!view   : 有効変数一覧." ) ;
        System.out.println( "!cls    : 画面クリア." ) ;
        System.out.println( "** help ************************************" ) ;
        System.out.println() ;
    }
}
