package org.maachang.seabass;

import org.maachang.dao.dbms.RecordFactory;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.httpd.engine.HttpdEngine;
import org.maachang.seabass.net.NetService;
import org.maachang.seabass.net.nio.NioPoolThread;

/**
 * SeaBassシステム情報.
 * 
 * @version 2007/08/26
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class MSystem {
    
    /** ネットサービスを取得. */
    private static final NetService netService() {
        return ( ( HttpdEngine )GlobalManager.getInstance().get( ServiceDef.MANAGER_BY_HTTPD ) ).getNetService() ;
    }
    
    /** レコードファクトリを取得. */
    private static final RecordFactory recordFactory() {
        return ( RecordFactory )GlobalManager.getInstance().get( ServiceDef.MANAGER_BY_DBMS_POOL ) ;
    }
    
    /**
     * ネットワーク有効プーリングスレッド数を取得.
     * @return int 有効プーリングスレッド数が返されます.
     */
    public static int getNetUsePooling() {
        return netService().getNioService().getUsePooling() ;
    }
    
    /**
     * ネットワークActiveプーリングスレッド数を取得.
     * @return int Activeプーリングスレッド数が返されます.
     */
    public static int getNetActivePooling() {
        return netService().getNioService().getActivePooling() ;
    }
    
    /**
     * 指定プーリングオブジェクトを取得.
     * @param no 対象の項番を設定します.
     * @return NioPoolThread スレッドが返されます.
     */
    public static NioPoolThread getPoolThread( int no ) {
        return netService().getNioService().getNioPoolManager().getThread( no ) ;
    }
    
    /**
     * DBコネクション最大プーリング数を取得. <BR>
     * <BR>
     * 最大プーリング数を取得します. <BR>
     * 
     * @return int 管理されている最大プーリング数が返されます.
     */
    public static int getDbMax() {
        if( recordFactory() != null ) {
            return recordFactory().getConnectionManager().max() ;
        }
        return 0 ;
    }

    /**
     * DBコネクション現在アクティブであるプーリングコネクション数を取得. <BR>
     * <BR>
     * 現在アクティブであるプーリングコネクション数を取得します. <BR>
     * 
     * @return int 現在アクティブなプーリングコネクション数が返されます.
     */
    public static int getDbActives() {
        if( recordFactory() != null ) {
            return recordFactory().getConnectionManager().active() ;
        }
        return 0 ;
    }
    
    /**
     * DBコネクション現在アイドルであるプーリングコネクション数を取得. <BR>
     * <BR>
     * 現在アイドルであるプーリングコネクション数を取得します. <BR>
     * 
     * @return int 現在アイドルなプーリングコネクション数が返されます.
     */
    public static int getDbIdle() {
        if( recordFactory() != null ) {
            return recordFactory().getConnectionManager().idle() ;
        }
        return 0 ;
    }
}
