package org.maachang.seabass;

import java.util.List;

import org.maachang.util.FileUtil;
import org.maachang.util.Os;
import org.maachang.util.StringUtil;

/**
 * SeaBassプロジェクト生成用.
 * 
 * @version 2007/09/20
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class Project {
    
    /**
     * 生成ディレクトリ一覧.
     */
    private static final String[] CREATE_DIR = {
        SeaBassDef.DIRECTORY_CONFIG,
        SeaBassDef.DIRECTORY_APPLICATION,
        SeaBassDef.DIRECTORY_MODEL,
        SeaBassDef.DIRECTORY_LIB,
        SeaBassDef.DIRECTORY_JAR,
        SeaBassDef.DIRECTORY_LOG,
        SeaBassDef.DIRECTORY_TABLE,
        SeaBassDef.DIRECTORY_ERROR_PAGE,
        SeaBassDef.DIRECTORY_TEMPLATE,
        SeaBassDef.DIRECTORY_PUBLIC,
        SeaBassDef.DIRECTORY_PUBLIC+SeaBassDef.DIRECTORY_IMAGE,
        SeaBassDef.DIRECTORY_PUBLIC+SeaBassDef.DIRECTORY_CSS,
        SeaBassDef.DIRECTORY_PUBLIC+SeaBassDef.DIRECTORY_JS,
        SeaBassDef.DIRECTORY_STARTUP,
        SeaBassDef.DIRECTORY_CRON,
        SeaBassDef.DIRECTORY_TAGLIB,
        SeaBassDef.DIRECTORY_REPORT
    } ;
    
    /**
     * コピーディレクトリ一覧.
     */
    private static final String[] COPY_DIR = {
        SeaBassDef.DIRECTORY_CONFIG,
        SeaBassDef.DIRECTORY_APPLICATION,
        SeaBassDef.DIRECTORY_STARTUP,
        SeaBassDef.DIRECTORY_PUBLIC
    } ;
    
    /**
     * Windowsサービス用ディレクトリ.
     */
    private static final String WINSERVICE_DIR = ".wsrv" ;
    private static final String SRC_WSRV = "sh/startup/windows/" ;
    
    /**
     * コンストラクタ.
     */
    private Project() {
        
    }
    
    /**
     * メインメソッド.
     * <BR><BR>
     * @param args Javaからのパラメータが設定されます.
     * @exception Exception 例外.
     */
    public static final void main( String[] args )
        throws Exception {
        new Project().execution( args ) ;
    }
    
    /**
     * 実行処理.
     */
    private void execution( String[] args )
        throws Exception {
        if( args == null || args.length <= 0 ) {
            throw new IllegalArgumentException( "SeaBassディレクトリが必要です" ) ;
        }
        // OSが、Windows系かチェック.
        boolean isWindowsOs = Os.getInstance().isOs( Os.OS_WINNT ) ;
        
        int projType = -1 ;
        if( args.length >= 2 ) {
            if( "derby".equals( args[ 1 ].trim().toLowerCase() ) ) {
                projType = 1 ;
            }
            else if( "postgre".equals( args[ 1 ].trim().toLowerCase() ) ) {
                projType = 2 ;
            }
            else if( "mysql".equals( args[ 1 ].trim().toLowerCase() ) ) {
                projType = 3 ;
            }
            else if( "help".equals( args[ 1 ].trim().toLowerCase() ) ||
                "?".equals( args[ 1 ].trim().toLowerCase() ) ) {
                System.out.println( " mproj [database]" ) ;
                System.out.println( " [database] プロジェクトに対するデータベースを設定します." ) ;
                System.out.println( "            SeaBassでは、以下のデータベースをサポートします." ) ;
                System.out.println( "            derby   : Apache Derby(JavaDB)用プロジェクトを作成します." ) ;
                System.out.println( "            postgre : Postgre用プロジェクトを作成します." ) ;
                System.out.println( "            mysql   : MySQL用プロジェクトを作成します." ) ;
                System.out.println() ;
                System.out.println( " またパラメータなしの場合、データベースに依存しない、プロジェクト雛型を" ) ;
                System.out.println( " 作成します." ) ;
                return ;
            }
        }
        
        System.out.println( "*** seabass - プロジェクト生成 - ***" ) ;
        System.out.println() ;
        
        String dir = FileUtil.getFullPath( "." ) ;
        if( dir.endsWith( "\\" ) || dir.endsWith( "/" ) ) {
            dir = dir.substring( 0,dir.length()-1 ) ;
        }
        dir += FileUtil.FILE_SPACE ;
        String baseDir = FileUtil.getFullPath( args[ 0 ] ) ;
        if( baseDir.endsWith( "\\" ) || baseDir.endsWith( "/" ) ) {
            baseDir = baseDir.substring( 0,baseDir.length()-1 ) ;
        }
        baseDir += FileUtil.FILE_SPACE ;
        
        // 指定カレントディレクトリ以下にディレクトリを生成.
        int len = CREATE_DIR.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            System.out.print( "...[" ) ;
            FileUtil.mkdirs( dir+CREATE_DIR[ i ] ) ;
            System.out.println( CREATE_DIR[ i ] + "] を生成" ) ;
        }
        // OSがWindowsの場合、Windowsサービス用ディレクトリを生成.
        if( isWindowsOs ) {
            System.out.print( "...[" ) ;
            FileUtil.mkdirs( dir+WINSERVICE_DIR ) ;
            System.out.println( WINSERVICE_DIR + "] を生成" ) ;
        }
        
        System.out.println() ;
        
        // 指定カレントディレクトリ以下にディレクトリ以下をコピー.
        len = COPY_DIR.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            System.out.println( "..." + COPY_DIR[ i ] + " 内容をコピー" ) ;
            copyDirBySrcToDest( baseDir+COPY_DIR[ i ],COPY_DIR[ i ] ) ;
        }
        
        // OSがWindowsの場合、Windowsサービス用ディレクトリにサービス情報をコピー.
        if( isWindowsOs ) {
            System.out.println( "** windowsサービス内容をコピー" ) ;
            copyDirBySrcToDest( baseDir+SRC_WSRV,WINSERVICE_DIR ) ;
        }
        
        System.out.println() ;
        
        // 指定されたJDBCドライバをコピー.
        String jdbcName = null ;
        switch( projType ) {
            case 1 : jdbcName = "derby" ; break ;
            case 2 : jdbcName = "postgre" ; break ;
            case 3 : jdbcName = "mysql" ; break ;
        }
        // DBプロジェクト名が設定されている場合.
        if( jdbcName != null ) {
            System.out.println( "## JDBC:"+jdbcName+"用コピー" ) ;
            boolean res ;
            res = copyDirBySrcToDest( baseDir+"database/"+jdbcName+"/lib/","./jar/" ) ;
            System.out.println( ".... JDBCドライバコピー[" + ( ( res ) ? "成功":"失敗" ) + "]" ) ;
            res = copyFile( baseDir+"database/"+jdbcName+"/conf/record.conf","./conf/record.conf" ) ;
            System.out.println( ".... コネクション定義ファイルコンフィグコピー[" + ( ( res ) ? "成功":"失敗" ) + "]" ) ;
        }
        // DBプロジェクト名が設定されていない場合は、デフォルトのコネクション定義ファイルをコピー.
        else {
            boolean res ;
            res = copyFile( baseDir+"database/default/record.conf","./conf/record.conf" ) ;
            System.out.println( ".... コネクション定義ファイルコンフィグコピー[" + ( ( res ) ? "成功":"失敗" ) + "]" ) ;
        }
        
        System.out.println() ;
        
        // OSにあわせて起動コマンドをコピー.
        if( Os.getInstance().getOS() == Os.OS_WINNT ) {
            // windows系の内容をコピー.
            copyFile( baseDir+"sh/sscript.cmd","sscript.cmd" ) ;
            copyFile( baseDir+"sh/seabass.cmd","seabass.cmd" ) ;
        }
        else {
            // Linux系の内容をコピー.
            copyFile( baseDir+"sh/sscript","sscript" ) ;
            copyFile( baseDir+"sh/seabass","seabass" ) ;
        }
    }
    
    /**
     * srcディレクトリ内のファイルを、destディレクトリにコピー.
     */
    private final boolean copyDirBySrcToDest( String src,String dest )
        throws Exception {
        src = FileUtil.getFullPath( src ) ;
        if( src.endsWith( "\\" ) || src.endsWith( "/" ) ) {
            src = src.substring( 0,src.length() - 1 ) ;
        }
        dest = FileUtil.getFullPath( dest ) ;
        if( dest.endsWith( "\\" ) || dest.endsWith( "/" ) ) {
            dest = dest.substring( 0,dest.length() - 1 ) ;
        }
        List<String> lst = FileUtil.getListAll( src ) ;
        if( lst != null ) {
            boolean ret = false ;
            int len = lst.size() ;
            for( int i = 0 ; i < len ; i ++ ) {
                String s = lst.get( i ) ;
                if( s == null ) {
                    continue ;
                }
                if( copyFile( src+s,dest+s ) ) {
                    ret = true ;
                }
            }
            return ret ;
        }
        return false ;
    }
    
    /**
     * ファイル内容をコピー.
     */
    private final boolean copyFile( String src,String dest )
        throws Exception {
        src = StringUtil.changeString( src,"\\","/" ) ;
        dest = StringUtil.changeString( dest,"\\","/" ) ;
        if( FileUtil.isFileExists( src ) ) {
            String destDir = getDir( dest ) ;
            if( destDir != null && FileUtil.isDirExists( destDir ) == false ) {
                FileUtil.mkdirs( destDir ) ;
            }
            byte[] bin = FileUtil.getFile( src ) ;
            if( bin != null ) {
                 FileUtil.setFile( dest,bin ) ;
                 return true ;
            }
        }
        return false ;
    }
    
    /**
     * 指定フルパス内のディレクトリ名を取得.
     */
    private final String getDir( String file )
        throws Exception {
        int p = file.lastIndexOf( "\\" ) ;
        if( p == -1 ) {
            p = file.lastIndexOf( "/" ) ;
            if( p == -1 ) {
                return null ;
            }
        }
        return file.substring( 0,p ) ;
    }
    
}
