package org.maachang.seabass;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.conf.IniFile;
import org.maachang.seabass.conf.StartupConfig;
import org.maachang.seabass.dbm.BdbService;
import org.maachang.seabass.down.ShutdownServer;
import org.maachang.seabass.httpd.HttpdVersionDef;
import org.maachang.seabass.httpd.engine.StartupHttpdEngine;
import org.maachang.seabass.httpd.engine.script.ExecutionStartup;
import org.maachang.seabass.startjava.StartupJavaManager;
import org.maachang.shutdown.ShutdownHook;
import org.maachang.shutdown.WaitShutdown;

/**
 * SeaBass開始処理.
 * 
 * @version 2007/08/26
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class Startup {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( Startup.class ) ;
    
    /**
     * コンストラクタ.
     */
    private Startup() {
        
    }
    
    /**
     * メインメソッド.
     * <BR><BR>
     * @param args Javaからのパラメータが設定されます.
     * @exception Exception 例外.
     */
    public static final void main( String[] args )
        throws Exception {
        new Startup().execution( args ) ;
    }
    
    /**
     * 実行処理.
     */
    private void execution( String[] args )
        throws Exception {
        String path = "." ;
        long tm = System.currentTimeMillis() ;
        try {
            State.getInstance().state( State.START_UP ) ;// 起動中に設定.
            // 開始処理.
            LOG.info( "*** seabass - webapps .. " ) ;
            LOG.info( "*** [ver."+HttpdVersionDef.getVersion()+" update."+HttpdVersionDef.getDate()+"]" ) ;
            LOG.info( "***" ) ;
            
            // 実行モード.
            if( args == null || args.length <= 0 ) {
                // 実行モード指定なしの場合は、コンパイルモード.
                LOG.info( "*** mode - compile" ) ;
                SeaBassMode.getInstance().complie() ;
            }
            else {
                // 実行モードがoptimize指定の場合は、最適化モード.
                String md = args[ 0 ] ;
                if( md != null && ( md.startsWith( "o" ) || md.startsWith( "O" ) ) ) {
                    LOG.info( "*** mode - optimize" ) ;
                    SeaBassMode.getInstance().optimize() ;
                }
                //else if( md.startsWith( "i" ) || md.startsWith( "I" ) ) {
                //    LOG.info( "*** mode - interpreter" ) ;
                //    SeaBassMode.getInstance().interpreter() ;
                //}
                // 指定条件が当てはまらない場合は、コンパイルモード.
                else {
                    LOG.info( "*** mode - compile" ) ;
                    SeaBassMode.getInstance().complie() ;
                }
            }
            
            LOG.info( "*** scriptVersion:" + SeaBassScriptEngine.getVersion() ) ;
            LOG.info( "***" ) ;
            
            // BdbServiceを初期化.
            LOG.info( "*** BdbService .... [START]" ) ;
            if( !BdbService.getInstance().isUse() ) {
                BdbService.getInstance().init() ;
            }
            LOG.info( "*** BdbService .... [END]" ) ;
            
            LOG.info( "*** startupConfig....[START]" ) ;
            StartupConfig.init( path ) ;
            LOG.info( "*** startupConfig....[END]" ) ;
            LOG.info( "*** startupHttpd.....[START]" ) ;
            StartupHttpdEngine.init( path ) ;
            LOG.info( "*** startupHttpd.....[END]" ) ;
            LOG.info( "*** startupScript....[START]" ) ;
            ExecutionStartup.init() ;
            LOG.info( "*** startupScript....[END]" ) ;
            
            // 外部スタートアップ登録処理.
            LOG.info( "*** startupJavaManager....[START]" ) ;
            StartupJavaManager.getInstance().init() ;
            StartupJavaManager.getInstance().startup() ;
            LOG.info( "*** startupJavaManager....[END]" ) ;
            
            LOG.info( "### startup....[END:" + ( System.currentTimeMillis() - tm ) + "ms]" ) ;
            State.getInstance().state( State.SUCCESS ) ;// 起動完了に設定.
            
            // シャットダウンフックを登録.
            ShutdownHook.registHook( new ShutdownServer() ) ;
            
            // シャットダウン待ちオブジェクトを生成.
            WaitShutdown waitDown = new WaitShutdown( shutdownPort() ) ;
            
            // 終了待ち.
            while( true ) {
                Thread.sleep( 500 ) ;
                if( waitDown.isShutdown() ) {
                    LOG.info( "*** seabass - start shutdown " ) ;
                    waitDown.exitShutdown() ;
                    LOG.info( "*** seabass - shutdown ..... end " ) ;
                    break ;
                }
            }
        } catch( Exception e ) {
            LOG.error( "### error",e ) ;
        }
    }
    
    /**
     * シャットダウン待ちポート番号を取得.
     */
    private static final int shutdownPort() {
        int ret = -1 ;
        GlobalManager nm = GlobalManager.getInstance() ;
        IniFile conf = ( IniFile )nm.get( ServiceDef.MANAGER_BY_CONFIG ) ;
        if( conf != null ) {
            Object o = conf.get( "server","shutdown-port",0 ) ;
            if( o != null ) {
                try {
                    ret = Integer.parseInt( ( String )o ) ;
                } catch( Exception e ) {
                    ret = -1 ;
                }
            }
        }
        return ret ;
    }
}
