package org.maachang.seabass.conf ;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.conf.Config;
import org.maachang.conf.ReadIni;
import org.maachang.seabass.SeaBassDef;
import org.maachang.seabass.SeabassDI;
import org.maachang.util.FileUtil;

/**
 * DIコンテナ読み込み用コンフィグ.
 *
 * @version 2009/03/28
 * @author  masahito suzuki
 * @since   SeaBass 1.00
 */
public class DIConfig {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( DIConfig.class ) ;
    
    /**
     * ターゲットセクション名.
     */
    private static final String BASE_SECTION = "target-di" ;
    
    /**
     * キャラクタセット.
     */
    private static final String CHARSET = "UTF8" ;
    
    /**
     * 読み込みファイル名.
     */
    private static final String READ_FILE = SeaBassDef.DIRECTORY_CONFIG+"di.conf" ;
    
    /**
     * デフォルト読み込みオブジェクト.
     */
    private static final String[] DEF = {
        "seabassIO","org.maachang.seabass.httpd.engine.script.SeabassFileIO" } ;
    
    /**
     * オープン.
     * <BR><BR>
     * オープンします.
     * <BR>
     * @exception Exception 例外.
     */
    public static final void open() throws Exception {
        open( READ_FILE,BASE_SECTION ) ;
    }
    
    /**
     * オープン.
     * <BR><BR>
     * オープンします.
     * <BR>
     * @param name 対象のファイル名を設定します.
     * @param baseSection 基本セクション名を設定します.
     * @exception Exception 例外.
     */
    public static final void open( String name ) throws Exception {
        open( name,BASE_SECTION ) ;
    }
    
    /**
     * オープン.
     * <BR><BR>
     * オープンします.
     * <BR>
     * @param name 対象のファイル名を設定します.
     * @param baseSection 基本セクション名を設定します.
     * @exception Exception 例外.
     */
    public static final void open( String name,String baseSection ) throws Exception {
        String fileName = FileUtil.getFullPath( name ) ;
        try {
            load( fileName,baseSection ) ;
        } catch( Exception e ) {
           throw e ;
        }
    }
    
    /**
     * DI条件読み込み.
     */
    private static final void load( String fileName,String baseSection )
        throws Exception {
        // コンフィグファイルが存在する場合.
        if( FileUtil.isFileExists( fileName ) == true ) {
            try {
                Config iniParams = getConfig( fileName ) ;
                String sectionName = iniParams.get( baseSection,"target",0 ) ;
                if( sectionName == null || ( sectionName = sectionName.trim() ).length() <= 0 ) {
                    sectionName = "di" ;
                }
                // ターゲットセクション名に対して、読み込みファイルが直接指定されている場合.
                int p ;
                if( ( p = sectionName.indexOf( "|" ) ) != -1 ) {
                    fileName = sectionName.substring( 0,p ) ;
                    sectionName = sectionName.substring( p+1 ) ;
                    iniParams = getConfig( fileName ) ;
                    if( iniParams == null ) {
                        throw new IOException( "## error ターゲットのDIコンテナ定義ファイルは存在しません:" + fileName ) ;
                    }
                }
                baseSection = sectionName ;
                String[] keys = iniParams.getKeys( baseSection ) ;
                int len ;
                Map<String,Object> diMap = SeabassDI.getInstance().getMap() ;
                if( keys != null && ( len = keys.length ) > 0 ) {
                    for( int i = 0 ; i < len ; i ++ ) {
                        String val = iniParams.get( baseSection,keys[ i ],0 ) ;
                        if( val == null || ( val = val.trim() ).length() <= 0 ) {
                            continue ;
                        }
                        try {
                            Object o = Class.forName( val ).newInstance() ;
                            LOG.info( "## DI success [ name:" + keys[ i ] + " object:" + val + "]" ) ;
                            diMap.put( keys[ i ],o ) ;
                        } catch( Throwable e ) {
                            LOG.error( "## DI error [ name:" + keys[ i ] + " object:" + val + "]",e ) ;
                            System.exit( -1 ) ;
                        }
                    }
                }
                len = DEF.length ;
                for( int i = 0 ; i < len ; i += 2 ) {
                    if( !iniParams.isKeys( baseSection,DEF[ i ] ) ) {
                        try {
                            Object o = Class.forName( DEF[ i+1 ] ).newInstance() ;
                            LOG.info( "## DI success [ name:" + DEF[ i ] + " object:" + DEF[ i+1 ] + "]" ) ;
                            diMap.put( DEF[ i ],o ) ;
                        } catch( Throwable e ) {
                            LOG.error( "## DI error [ name:" + DEF[ i ] + " object:" + DEF[ i+1 ] + "]",e ) ;
                            System.exit( -1 ) ;
                        }
                    }
                }
            } catch( Exception e ) {
                throw e ;
            }
        }
        // コンフィグファイルが存在しない場合.
        else {
            int len = DEF.length ;
            Map<String,Object> diMap = SeabassDI.getInstance().getMap() ;
            for( int i = 0 ; i < len ; i += 2 ) {
                try {
                    Object o = Class.forName( DEF[ i+1 ] ).newInstance() ;
                    LOG.info( "## DI success [ name:" + DEF[ i ] + " object:" + DEF[ i+1 ] + "]" ) ;
                    diMap.put( DEF[ i ],o ) ;
                } catch( Throwable e ) {
                    LOG.error( "## DI error [ name:" + DEF[ i ] + " object:" + DEF[ i+1 ] + "]",e ) ;
                    System.exit( -1 ) ;
                }
            }
        }
    }
    
    /**
     * 指定ファイルのコンフィグ情報を取得.
     */
    private static final Config getConfig( String name )
        throws Exception {
        // コンフィグファイルが存在する場合.
        if( FileUtil.isFileExists( name ) == true ) {
            BufferedReader buf = null ;
            try {
                buf = new BufferedReader(
                    new InputStreamReader(
                        new FileInputStream( name ),CHARSET ) ) ;
                Config conf = new Config() ;
                ReadIni.analisys( conf,buf ) ;
                buf.close() ;
                buf = null ;
                return conf ;
            } finally {
                if( buf != null ) {
                    try {
                        buf.close() ;
                    } catch( Exception e ) {
                    }
                }
            }
        }
        return null ;
    }
}
