package org.maachang.seabass.dbm;

import java.io.IOException;

import org.maachang.bdb.Bdb;
import org.maachang.bdb.BdbKeyValue;
import org.maachang.bdb.BdbManager;
import org.maachang.bdb.TransactionBdb;

/**
 * Bdbファクトリ.
 * 
 * @version 2010/01/07
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public abstract class BdbFactory {
    private BdbFactory(){}
    
    /**
     * BdbKeyValueオブジェクトを取得.
     * <p>※このオブジェクトは、ThreadLocalから取得されているので、Threadセーフです.</p>
     * @return BdbKeyValue BdbKeyValueオブジェクトが返されます.
     */
    public static final BdbKeyValue getBdbKeyValue() {
        return BdbManager.getBdbKeyValue() ;
    }
    
    /**
     * 指定されたBDBオブジェクトを取得.
     * @param name 対象のBDB名を設定します.
     * @return Bdb 取得されたBDBオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public static final Bdb get( String name )
        throws Exception {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        BdbService s = BdbService.getInstance() ;
        if( !s.isUse() ) {
            throw new IOException( "BDBサービスが起動していません" ) ;
        }
        return s.get().get( name ) ;
    }
    
    /**
     * トランザクション用BDBオブジェクトを取得.
     * @param keyValue 対象のKeyValueオブジェクトを設定します.
     * @param name 対象のBDB名を設定します.
     * @return TransactionBdb トランザクションBDBオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public static final TransactionBdb getTransaction( BdbKeyValue keyValue,String name )
        throws Exception {
        return new TransactionBdb( keyValue,get( name ) ) ;
    }
    
    /**
     * Bdbサービス状態を取得.
     * @return boolean [true]の場合、サービスは実行中です.
     */
    public static final boolean isUse() {
        return BdbService.getInstance().isUse() ;
    }
}
