package org.maachang.seabass.dbm;

import org.maachang.bdb.BdbManager;
import org.maachang.util.atomic.AtomicOBJECT;

/**
 * Bdbサービス.
 * 
 * @version 2010/01/04
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class BdbService {
    private BdbService(){}
    private static final String TARGET_DIR = "./bdb" ;
    private static final BdbService SNGL = new BdbService() ;
    
    /**
     * オブジェクトを取得.
     * @return BdbService オブジェクトが返されます.
     */
    public static final BdbService getInstance() {
        return SNGL ;
    }
    
    /**
     * BdbManager.
     */
    private final AtomicOBJECT<BdbManager> manager = new AtomicOBJECT<BdbManager>( null ) ;
    
    /**
     * BDB初期化処理.
     * @exception Exception 例外.
     */
    public void init() throws Exception {
        if( manager.get() != null ) {
            destroy() ;
        }
        BdbManager man = new BdbManager( false,TARGET_DIR ) ;
        manager.set( man ) ;
    }
    
    /**
     * BDB終了化処理.
     * @exception Exception 例外.
     */
    public void destroy() throws Exception {
        BdbManager m ;
        if( ( m = manager.get() ) != null ) {
            manager.set( null ) ;
            m.destroy() ;
        }
    }
    
    /**
     * BDBマネージャを取得.
     * @return BdbManager マネージャが返されます.
     */
    public BdbManager get() {
        return manager.get() ;
    }
    
    /**
     * オブジェクトが有効かチェック.
     * @return boolean [true]の場合、有効です.
     */
    public boolean isUse() {
        BdbManager m = manager.get() ;
        return ( m != null && !m.isDestroy() ) ;
    }
}

