package org.maachang.seabass.httpd;

/**
 * HTTPD実行管理インターフェイス.
 * 
 * @version 2007/08/30
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public interface HttpdExecutionManager {
    
    /**
     * 指定内容を取得.
     * <BR><BR>
     * 指定された内容を取得します.
     * <BR>
     * @param b 一時的に利用するバイナリを設定します.
     * @param request 対象のリクエストを設定します.
     * @param response 対象のレスポンスを設定します.
     * @exception Exception 例外.
     */
    public void get( byte[] b,HttpdRequest request,HttpdResponse response )
        throws Exception  ;
    
    /**
     * 実行可能かチェックを行い、実行可能な場合は、最終更新日を取得.
     * <BR><BR>
     * 実行可能かチェックを行い、実行可能な場合は、最終更新日を取得します.
     * <BR>
     * @param request 対象のリクエストを設定します.
     * @return long 最終更新日が返されます.
     * @exception Exception 例外.
     */
    public long checkToLastUpdateTime( HttpdRequest request ) throws Exception ;
    
    /**
     * 指定パスのファイル日付を取得.
     * <BR><BR>
     * 指定パスのファイル日付を取得します.
     * <BR>
     * @parma path 対象のパスを設定します.
     * @return long ファイル日付が返されます.
     */
    public long getFileTime( String path ) ;
    
    /**
     * 指定パスの内容が存在するかチェック.
     * <BR><BR>
     * 指定パスの内容が存在するかチェックします.
     * <BR>
     * @param path 対象のパスを設定します.
     * @return boolean [true]の場合、存在します.
     */
    public boolean isPath( String path ) ;
    
    /**
     * 指定パスのディレクトリが存在するかチェック.
     * <BR><BR>
     * 指定パスのディレクトリが存在するかチェックします.
     * <BR>
     * @param path 対象のパスを設定します.
     * @return boolean [true]の場合、ディレクトリとして存在します.
     */
    public boolean isDirectory( String path ) ;
    
    /**
     * 指定パスの内容が読み込み可能かチェック.
     * <BR><BR>
     * 指定パスの内容が読み込み可能かチェックします.
     * <BR>
     * @param path 対象のパスを設定します.
     * @return boolean [true]の場合、読み込み可能です.
     */
    public boolean isRead( String path ) ;
    
    
}
