package org.maachang.seabass.httpd.engine;

import org.maachang.manager.GlobalManager;
import org.maachang.seabass.ServiceDef;
import org.maachang.seabass.conf.IniFile;
import org.maachang.seabass.conf.Seabass32Key;
import org.maachang.seabass.httpd.HttpdRequest;
import org.maachang.seabass.httpd.engine.session.HttpdSession;
import org.maachang.util.StringUtil;

/**
 * Javascript(js)ファイルを送信するときの予約変数置き換え処理.
 * 
 * @version 2008/05/20
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class EnvJavascript {
    
    /**
     * 予約変数置き換え.
     * @param request 対象のリクエストを設定します.
     * @param string 対象の文字列を設定します.
     * @return String 変換された文字列が返されます.
     * @exception Exception 例外.
     */
    public static final String convert( HttpdRequest request,String string )
        throws Exception {
        if( string == null || string.length() <= 0 ) {
            return "" ;
        }
        String httpUrl = null ;
        String httpsUrl = null ;
        if( string.indexOf( "$@@URL@@" ) != -1 ) {
            httpUrl = getURL( request,false ) ;
            httpsUrl = getURL( request,true ) ;
            string = StringUtil.changeString( string,"$@@URL@@",( request.isSsl() ) ? httpsUrl : httpUrl ) ;
        }
        if( string.indexOf( "$@@HTTP_URL@@" ) != -1 ) {
            if( httpUrl == null ) {
                httpUrl = getURL( request,false ) ;
            }
            string = StringUtil.changeString( string,"$@@HTTP_URL@@",httpUrl ) ;
        }
        if( string.indexOf( "$@@HTTPS_URL@@" ) != -1 ) {
            if( httpsUrl == null ) {
                httpsUrl = getURL( request,true ) ;
            }
            string = StringUtil.changeString( string,"$@@HTTPS_URL@@",httpsUrl ) ;
        }
        if( string.indexOf( "$@@LOCATION_PARAMS@@" ) != -1 ) {
            string = StringUtil.changeString( string,"$@@LOCATION_PARAMS@@",request.getQuery().getUrlParams() ) ;
        }
        if( string.indexOf( "$@@SESSION_ID@@" ) != -1 ) {
            HttpdSession session = request.getSession() ;
            if( session != null && session.getSessionId() != null ) {
                string = StringUtil.changeString( string,"$@@SESSION_ID@@",session.getSessionId() ) ;
            }
        }
        if( string.indexOf( "$@@S32_KY@@" ) != -1 ) {
            string = StringUtil.changeString( string,"$@@S32_KY@@",
                ( ( Seabass32Key )GlobalManager.getInstance().get( ServiceDef.MANAGER_BY_SEABASS32 ) ).
                    getSeabass32() ) ;
        }
        return string ;
    }
    
    private static final String HTTP_PROTOCOL = "http://" ;
    private static final String HTTPS_PROTOCOL = "https://" ;
    private static final int HTTP_PORT ;
    private static final int HTTPS_PORT ;
    static {
        IniFile conf = ( IniFile )GlobalManager.getInstance().get( ServiceDef.MANAGER_BY_CONFIG ) ;
        int port ;
        port = conf.getNumber( "net","server-port",0 ).intValue() ;
        if( port == 80 ) {
            port = -1 ;
        }
        HTTP_PORT = port ;
        port = conf.getNumber( "ssl","port",0 ).intValue() ;
        if( port == 443 ) {
            port = -1 ;
        }
        HTTPS_PORT = port ;
    }
    
    /**
     * 指定内容のURLを取得.
     */
    private static final String getURL( HttpdRequest request,boolean mode ) {
        int port ;
        String protocol ;
        if( mode == true ) {
            port = HTTPS_PORT ;
            protocol = HTTPS_PROTOCOL ;
        }
        else {
            port = HTTP_PORT ;
            protocol = HTTP_PROTOCOL ;
        }
        String host = request.getHeader().getHeader( "Host",0 ) ;
        int p = host.indexOf( ":" ) ;
        if( p >= 0 ) {
            host = host.substring( 0,p ) ;
        }
        StringBuilder buf = new StringBuilder() ;
        buf.append( protocol ).append( host ) ;
        if( port != -1 ) {
            buf.append( ":" ).append( port ) ;
        }
        return buf.toString() ;
    }
}

