package org.maachang.seabass.httpd.engine;

import org.maachang.seabass.SeaBassDef;
import org.maachang.seabass.httpd.HttpdExecutionManager;
import org.maachang.seabass.httpd.HttpdRequest;
import org.maachang.seabass.httpd.HttpdResponse;
import org.maachang.seabass.httpd.engine.script.scripts.ScriptManager;
import org.maachang.seabass.httpd.engine.script.scripts.TemplateScript;
import org.maachang.seabass.net.HttpdRequestImpl;
import org.maachang.util.FileUtil;

/**
 * HTTPDスクリプト実行管理.
 * 
 * @version 2007/08/30
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class HttpdScriptManager implements HttpdExecutionManager {
    /**
     * コントローラ名.
     */
    private static final String CONTROLLER = ScriptManager.SCRIPT_BY_CONTROLLER+ScriptManager.SCRIPT_PLUS ;
    
    /**
     * コンストラクタ.
     */
    public HttpdScriptManager() {
        
    }
    
    /**
     * 指定内容を取得.
     * <BR><BR>
     * 指定された内容を取得します.
     * <BR>
     * @param b 一時的に利用するバイナリを設定します.
     * @param request 対象のリクエストを設定します.
     * @param response 対象のレスポンスを設定します.
     * @exception Exception 例外.
     */
    public void get( byte[] b,final HttpdRequest request,final HttpdResponse respons )
        throws Exception {
        ScriptManager.execution( request,respons,SeaBassDef.NOW_APPLICATION_PATH ) ;
    }
    
    /**
     * 実行可能かチェックを行い、実行可能な場合は、最終更新日を取得.
     * <BR><BR>
     * 実行可能かチェックを行い、実行可能な場合は、最終更新日を取得します.
     * <BR>
     * @param request 対象のリクエストを設定します.
     * @return long 最終更新日が返されます.
     * @exception Exception 例外.
     */
    public long checkToLastUpdateTime( HttpdRequest request ) throws Exception {
        String path = getUsePath( request,null ) ;
        if( path == null ) {
            return -1L ;
        }
        else if( path.endsWith( ScriptManager.TEMPLATE_PLUS ) ) {
            return TemplateScript.getTime( path ) ;
        }
        return FileUtil.getLastTime( path ) ;
    }
    
    /**
     * 指定パスのファイル日付を取得.
     * <BR><BR>
     * 指定パスのファイル日付を取得します.
     * <BR>
     * @parma path 対象のパスを設定します.
     * @return long ファイル日付が返されます.
     */
    public long getFileTime( String path ) {
        path = ScriptManager.trimScriptPath( path ) ;
        if( path == null ) {
            return -1L ;
        }
        long ret = -1L ;
        // MTTML(Template)名でチェック.
        if( path.endsWith( ScriptManager.TEMPLATE_PLUS ) == true ) {
            try {
                ret = TemplateScript.getTime( path ) ;
                if( ret != -1L ) {
                    return ret ;
                }
            } catch( Exception e ) {
            }
            return -1L ;
        }
        String fullPath = SeaBassDef.NOW_APPLICATION_PATH+path.substring( 1,path.length() ) ;
        // 通常名でチェック.
        if( ( ret = FileUtil.getLastTime( fullPath+ScriptManager.SCRIPT_PLUS ) ) != -1L ) {
            return ret ;
        }
        // コントローラ名でチェック.
        if( ( ret = FileUtil.getLastTime( fullPath+CONTROLLER ) ) != -1L ) {
            return ret ;
        }
        // MHTML名でチェック.
        if( ( ret = FileUtil.getLastTime( fullPath+ScriptManager.SCRIPT_HTML_PLUS ) ) != -1L ) {
            return ret ;
        }
        // SDoc名でチェック.
        if( ( ret = FileUtil.getLastTime( fullPath+ScriptManager.SDOC_PLUS ) ) != -1L ) {
            return ret ;
        }
        // XReport名でチェック.
        if( ( ret = FileUtil.getLastTime( fullPath+ScriptManager.XRPT_PLUS ) ) != -1L ) {
            return ret ;
        }
        return -1L ;
    }
    
    /**
     * 指定パスの内容が存在するかチェック.
     * <BR><BR>
     * 指定パスの内容が存在するかチェックします.
     * <BR>
     * @param path 対象のパスを設定します.
     * @return boolean [true]の場合、存在します.
     */
    public boolean isPath( String path ) {
        path = ScriptManager.trimScriptPath( path ) ;
        if( path == null ) {
            return false ;
        }
        // MTTML(Template)名でチェック.
        if( path.endsWith( ScriptManager.TEMPLATE_PLUS ) == true ) {
            try {
                if( TemplateScript.isTemplate( path ) == true ) {
                    return true ;
                }
            } catch( Exception e ) {
            }
            return false ;
        }
        String fullPath = SeaBassDef.NOW_APPLICATION_PATH+path.substring( 1,path.length() ) ;
        return ( FileUtil.isFileExists( fullPath+ScriptManager.SCRIPT_PLUS ) ||
            FileUtil.isFileExists( fullPath+CONTROLLER ) ||
            FileUtil.isFileExists( fullPath+ScriptManager.SCRIPT_HTML_PLUS ) ||
            FileUtil.isFileExists( fullPath+ScriptManager.SDOC_PLUS ) ||
            FileUtil.isFileExists( fullPath+ScriptManager.XRPT_PLUS ) ) ;
    }
    
    /**
     * 指定パスのディレクトリが存在するかチェック.
     * <BR><BR>
     * 指定パスのディレクトリが存在するかチェックします.
     * <BR>
     * @param path 対象のパスを設定します.
     * @return boolean [true]の場合、ディレクトリとして存在します.
     */
    public boolean isDirectory( String path ) {
        if( path.startsWith( "/" ) == true ) {
            path = path.substring( 1,path.length() ) ;
        }
        path = SeaBassDef.NOW_APPLICATION_PATH + path ;
        if( FileUtil.isDirExists( path ) == true ) {
            return true ;
        }
        return false ;
    }
    
    /**
     * 指定パスの内容が読み込み可能かチェック.
     * <BR><BR>
     * 指定パスの内容が読み込み可能かチェックします.
     * <BR>
     * @param path 対象のパスを設定します.
     * @return boolean [true]の場合、読み込み可能です.
     */
    public boolean isRead( String path ) {
        path = ScriptManager.trimScriptPath( path ) ;
        if( path == null ) {
            return false ;
        }
        // MTTML(Template)名でチェック.
        if( path.endsWith( ScriptManager.TEMPLATE_PLUS ) == true ) {
            try {
                if( TemplateScript.isTemplate( path ) == true ) {
                    return true ;
                }
            } catch( Exception e ) {
            }
            return false ;
        }
        String fullPath = SeaBassDef.NOW_APPLICATION_PATH+path.substring( 1,path.length() ) ;
        return ( FileUtil.isRead( fullPath+ScriptManager.SCRIPT_PLUS ) ||
            FileUtil.isRead( fullPath+CONTROLLER ) ||
            FileUtil.isRead( fullPath+ScriptManager.SCRIPT_HTML_PLUS ) ||
            FileUtil.isRead( fullPath+ScriptManager.SDOC_PLUS ) ||
            FileUtil.isRead( fullPath+ScriptManager.XRPT_PLUS ) ) ;
    }
    
    /**
     * 有効パスを取得.
     */
    public static final String getUsePath( HttpdRequest request,String path )
        throws Exception {
        if( path == null ) {
            path = request.getUrlPath() ;
        }
        path = ScriptManager.trimScriptPath( path ) ;
        String lower = path.toLowerCase() ;
        if( lower.endsWith( ScriptManager.TEMPLATE_PLUS ) ) {
            return path ;
        }
        String fullPath = SeaBassDef.NOW_APPLICATION_PATH+path.substring( 1,path.length() ) ;
        String p = fullPath+ScriptManager.SCRIPT_PLUS ;
        if( FileUtil.isFileExists( p ) ) {
            if( request != null ) {
                ((HttpdRequestImpl)request).setFullPath( p ) ;
            }
            return p ;
        }
        p = fullPath+CONTROLLER ;
        if( FileUtil.isFileExists( p ) ) {
            if( request != null ) {
                ((HttpdRequestImpl)request).setFullPath( p ) ;
            }
            return p ;
        }
        p = fullPath+ScriptManager.SCRIPT_HTML_PLUS ;
        if( FileUtil.isFileExists( p ) ) {
            if( request != null ) {
                ((HttpdRequestImpl)request).setFullPath( p ) ;
            }
            return p ;
        }
        p = fullPath+ScriptManager.SDOC_PLUS ;
        if( FileUtil.isFileExists( p ) ) {
            if( request != null ) {
                ((HttpdRequestImpl)request).setFullPath( p ) ;
            }
            return p ;
        }
        p = fullPath+ScriptManager.XRPT_PLUS ;
        if( FileUtil.isFileExists( p ) ) {
            if( request != null ) {
                ((HttpdRequestImpl)request).setFullPath( p ) ;
            }
            return p ;
        }
        return null ;
    }
}

