package org.maachang.seabass.httpd.engine.comet ;

import java.util.Iterator;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.maachang.util.atomic.AtomicOBJECT;

/**
 * Comet用コネクションキュー.
 * 
 * @version 2008/10/24
 * @author masahito suzuki
 * @since  SeaBass 1.0.0
 */
class CometConnectQueue<T> {
    
    /** キュー管理 */
    private final AtomicOBJECT<ConcurrentLinkedQueue<T>> queue = new AtomicOBJECT<ConcurrentLinkedQueue<T>>() ;
    
    /**
     * コンストラクタ.
     */
    public CometConnectQueue() {
        queue.set( new ConcurrentLinkedQueue<T>() ) ;
    }
    
    protected void finalize() throws Exception {
        this.clear() ;
    }
    
    /**
     * オブジェクトクリア.
     */
    public void clear() {
        queue.get().clear() ;
    }
    
    /**
     * データ追加.
     * @param value 追加対象のデータを設定します.
     */
    public void add( T value ) {
        if( value == null ) {
            return ;
        }
        queue.get().offer( value ) ;
    }
    
    /**
     * 先頭データを取得して削除.
     * @return T 格納されているデータが返されます.
     */
    public T get() {
        return queue.get().poll() ;
    }
    
    /**
     * Iteratorを取得.
     * @return Iterator Iteratorが返されます.
     */
    public Iterator iterator() {
        return queue.get().iterator() ;
    }
    
    /**
     * 格納数取得.
     * @return int 格納数が返されます.
     */
    public int size() {
        return queue.get().size() ;
    }
    
    /**
     * 現在のコネクション情報を取得.
     * @return ConcurrentLinkedQueue<T> 現在のコネクション情報が返されます.
     */
    public ConcurrentLinkedQueue<T> nowConnection() {
        return queue.set( new ConcurrentLinkedQueue<T>() ) ;
    }
}
