package org.maachang.seabass.httpd.engine.filter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.seabass.ServiceDef;
import org.maachang.seabass.httpd.HttpdIOFilter;
import org.maachang.conf.Config;
import org.maachang.manager.GlobalManager;
import org.maachang.util.FileUtil;

/**
 * IO-Filter初期化処理.
 * 
 * @version 2008/12/14
 * @author  masahito suzuki
 * @since   SeaBass 1.2B
 */
public abstract class InitIOFilter {
    private InitIOFilter() {}
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( InitIOFilter.class ) ;
    
    /**
     * コンフィグファイル.
     */
    private static final String CONF = "./conf/filter.conf" ;
    
    /**
     * セクション名.
     */
    private static final String SESSION = "filter" ;
    
    /**
     * URLマスクキー名.
     */
    private static final String KEY_URL = "url" ;
    
    /**
     * 実行クラスキー名.
     */
    private static final String KEY_CLASS = "class" ;
    
    /**
     * IO-Filter初期化処理.
     * @exception Exception 例外.
     */
    public static final void init() throws Exception {
        HttpdIOFilterManager m = new HttpdIOFilterManager() ;
        GlobalManager.getInstance().put( ServiceDef.IO_FILTER_MANAGER,m ) ;
        if( FileUtil.isFileExists( CONF ) == false ) {
            LOG.info( "not filter" ) ;
            return ;
        }
        int cnt = 0 ;
        Config conf = Config.read( CONF ) ;
        int len = conf.size( SESSION,"class" ) ;
        String url ;
        String clazz ;
        if( len > 0 ) {
            for( int i = 0 ; i < len ; i ++ ) {
                url = null ;
                clazz = null ;
                try {
                    url = conf.get( SESSION,KEY_URL,i ) ;
                    clazz = conf.get( SESSION,KEY_CLASS,i ) ;
                    if( clazz == null || ( clazz = clazz.trim() ).length() <= 0 ) {
                        continue ;
                    }
                    Object o = Class.forName( clazz ) ;
                    if( o == null || ( o instanceof HttpdIOFilter ) == false ) {
                        LOG.warn( "no defile filter(" + i + ")-(url:" + url + " class:" + clazz +")" ) ;
                        continue ;
                    }
                    m.add( url,( HttpdIOFilter )o ) ;
                    LOG.info( "success filter(" + i + ")-(url:" + url + " class:" + clazz +")" ) ;
                    cnt ++ ;
                } catch( Exception e ) {
                    LOG.error( "filterException(" + i + ")-(url:" + url + " class:" + clazz +")",e ) ;
                } catch( Error er ) {
                    LOG.error( "filterError(" + i + ")-(url:" + url + " class:" + clazz +")",er ) ;
                }
            }
        }
        if( cnt == 0 ) {
            LOG.info( "not filter" ) ;
        }
    }
}
