package org.maachang.seabass.httpd.engine.include;

import gnu.trove.THashMap;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.SeaBassContext;
import org.maachang.seabass.ServiceDef;
import org.maachang.seabass.httpd.HttpdRequest;
import org.maachang.seabass.httpd.engine.HttpdDef;
import org.maachang.seabass.httpd.engine.HttpdPublicManager;
import org.maachang.seabass.httpd.engine.HttpdScriptManager;
import org.maachang.seabass.httpd.engine.script.BaseModel;
import org.maachang.seabass.httpd.engine.script.BaseModelImpl;
import org.maachang.seabass.httpd.engine.script.EndScript;
import org.maachang.seabass.httpd.engine.script.Script;
import org.maachang.seabass.httpd.engine.script.ScriptDef;
import org.maachang.seabass.httpd.engine.script.cache.CacheScriptManager;
import org.maachang.seabass.httpd.engine.script.scripts.ApplicationScriptFactory;
import org.maachang.seabass.httpd.engine.script.scripts.PageCache;
import org.maachang.seabass.httpd.engine.script.scripts.ScriptManager;
import org.maachang.seabass.httpd.engine.script.taglib.AnalysisTagLib;
import org.maachang.seabass.httpd.engine.session.HttpdSession;
import org.maachang.util.FileUtil;
import org.maachang.util.StringUtil;

/**
 * SSI専用HTML処理.
 * 
 * @version 2009/02/12
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class IHtml {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( IHtml.class ) ;
    
    /**
     * タグライブラリ開始タグ.
     */
    private static final String START_IHTML = "<$" ;
    
    /**
     * タグライブラリ終了タグ.
     */
    private static final String END_IHTML = ">" ;
    
    /**
     * IHTML変換処理.
     * @param request 対象のリクエストを設定します.
     * @param string 対象のIHTML内容を設定します.
     * @return String 変換されたIHTML内容が返されます.
     * @exception Exception 例外.
     */
    public static final String convert( HttpdRequest request,String string )
        throws Exception {
        string = StringUtil.changeString( string,"<tab>","&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
        string = StringUtil.changeString( string,"<TAB>","&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
        string = ihtmlConvert( request,string ) ;
        return string ;
    }
    
    /**
     * ihtml内容変換.
     */
    private static final String ihtmlConvert( HttpdRequest request,String string )
        throws Exception {
        int p = 0 ;
        int b = 0 ;
        if( ( p = string.indexOf( START_IHTML ) ) <= -1 ) {
            return string ;
        }
        StringBuilder buf = new StringBuilder() ;
        int[] outTagPos = new int[ 1 ] ;
        for( ;; ) {
            int e = -1 ;
            String tagName = AnalysisTagLib.getTagLibName( outTagPos,string,p ) ;
            // タグ名が、SSI系の条件.
            if( tagName.equals( "include" ) || tagName.equals( "direct" ) ) {
                e = StringUtil.indexToNotCote( string,END_IHTML,p ) ;
                if( e <= -1 ) {
                    buf.append( string.substring( b ) ) ;
                    break ;
                }
                // 前の終端位置から、次の開始位置までの内容をコピー.
                buf.append( string.substring( b,p ) ) ;
                
                // SSI処理.
                buf.append( addSSI( request,tagName,string,outTagPos[0],e ) ) ;
            }
            else {
                // 前の終端位置から、次の開始位置までの内容をコピー.
                buf.append( string.substring( b,p ) ) ;
            }
            
            // 次の処理を行う準備.
            b = e+1 ;
            p = string.indexOf( START_IHTML,b ) ;
            if( p <= -1 ) {
                buf.append( string.substring( b ) ) ;
                break ;
            }
        }
        return buf.toString() ;
    }
    
    /**
     * Innerスクリプトファイル識別子.
     */
    private static final String INNER_FILE = ScriptManager.SCRIPT_BY_INNER + ScriptManager.SCRIPT_PLUS ;
    
    /**
     * 静的ファイルキャラクタセット.
     */
    private static final String CHARSET = "UTF8" ;
    
    /**
     * SSI処理.
     */
    private static final String addSSI( HttpdRequest request,String tagName,String string,int p,int e )
        throws Exception {
        String inner = AnalysisTagLib.cutTagAndTagName( false,string,p,e ) ;
        ArrayList<String> lst = new ArrayList<String>() ;
        AnalysisTagLib.cutElement( lst,inner ) ;
        int len = lst.size() ;
        if( len <= 0 ) {
            throw new IOException( "SSIタグ[" + tagName + "]に対してsrc条件が存在しません" ) ;
        }
        String path = null ;
        for( int i = 0 ; i < len ; i += 2 ) {
            String key = lst.get( i ) ;
            if( key != null && ( key = key.trim().toLowerCase() ).length() <= 0 ) {
                continue ;
            }
            if( key.equals( "src" ) ) {
                path = lst.get( i+1 ).trim() ;
                if( ( path.startsWith( "\"" ) && path.endsWith( "\"" ) ) ||
                    ( path.startsWith( "\'" ) && path.endsWith( "\'" ) ) ) {
                    path = path.substring( 1,path.length()-1 ) ;
                }
                break ;
            }
        }
        if( path == null ) {
            return "" ;
        }
        path = HttpdDef.toFullUrl( request,path ) ;
        String targetPath = null ;
        // 静的ファイルとして存在するかチェック.
        if( ( targetPath = HttpdPublicManager.getUsePath( null,path ) ) != null ) {
            return FileUtil.getFileByString( targetPath,CHARSET ) ;
        }
        // 動的ファイルとして存在するかチェック.
        // Inner属性のファイルとして存在するかチェック.
        else {
            if( path.endsWith( ScriptManager.SCRIPT_PLUS ) == false ) {
                path += ScriptManager.SCRIPT_PLUS ;
            }
            if( ( targetPath = HttpdScriptManager.getUsePath( null,path ) ) != null &&
                targetPath.endsWith( INNER_FILE ) ) {
                // キャッシュとして存在するかチェック.
                String buf = PageCache.getInstance().get( "@inner@"+path ) ;
                if( buf != null ) {
                    return buf ;
                }
                // キャッシュとして存在しない場合は、Inner実行を行い、内容を取得する.
                ApplicationScriptFactory factory = ( ApplicationScriptFactory )GlobalManager.getValue(
                    ServiceDef.MANAGER_BY_WEB_APP_FACTORY ) ;
                Script script = factory.getApplication( path ) ;
                if( script == null ) {
                    return "" ;
                }
                SeaBassContext context = null ;
                BaseModel baseModel = new BaseModelImpl() ;
                Map<String,Object> params = new THashMap<String,Object>() ;
                params.put( ScriptDef.SCRIPT_MODE,ScriptDef.MODE_NOT_COMET ) ;
                try {
                    context = CacheScriptManager.getInstance().script( "" ) ;
                    createContext( path,request,context,baseModel ) ;
                    ScriptDef.setDirectoryByBindings( context ) ;
                    Object ret = script.getScript().execution( context,params ) ;
                    if( baseModel.isCreate() == true ) {
                        baseModel.commit() ;
                    }
                    if( EndScript.isEndScript( ret ) ) {
                        return "" ;
                    }
                    else if( ret instanceof String ) {
                        return (String)ret ;
                    }
                    return "" ;
                } catch( Exception ee ) {
                    LOG.error( "script[" + path + "]-error",ee ) ;
                    if( baseModel.isCreate() == true ) {
                        try {
                            baseModel.rollback() ;
                        } catch( Exception eee ) {
                        }
                    }
                    return "" ;
                } finally {
                    if( baseModel.isCreate() == true ) {
                        try {
                            baseModel.getRecord().close() ;
                        } catch( Exception ee ) {
                        }
                    }
                    baseModel = null ;
                    CacheScriptManager.getInstance().exitScript( context ) ;
                }
            }
        }
        return "" ;
    }
    
    /**
     * コンテキストを生成.
     */
    private static void createContext( String path,HttpdRequest req,SeaBassContext ctx,BaseModel baseModel )
        throws Exception {
        // 基本パラメータを定義.
        ctx.put( ScriptDef.SCRIPT_BY_MODEL,baseModel ) ;
        ctx.put( ScriptDef.SCRIPT_BY_HEADER,req.getHeader() ) ;
        HttpdSession session = req.getSession() ;
        if( session != null ) {
            ctx.put( ScriptDef.SCRIPT_BY_SESSION,session ) ;
        }
        // リクエスト内容を設定.
        ctx.put( ScriptDef.SCRIPT_BY_QUERY,ScriptDef.getQuery( req.getQuery() ) ) ;
    }
    
}
