package org.maachang.seabass.httpd.engine.script;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.SeaBassDef;
import org.maachang.seabass.SeaBassException;
import org.maachang.seabass.ServiceDef;
import org.maachang.seabass.httpd.engine.script.cache.CacheScriptManager;
import org.maachang.seabass.httpd.engine.script.scripts.ApplicationScriptFactory;
import org.maachang.util.StringUtil;

/**
 * スクリプト実行例外.
 * 
 * @version 2007/09/23
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class BaseScriptException extends Exception {
    private static final long serialVersionUID = -8407798868124851487L;
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( BaseScriptException.class ) ;
    
    /**
     * 出力行数.
     */
    private static final int OUTPUT_LINE_SIZE = SeaBassDef.OUTPUT_ERROR_LIST ;
    
    /**
     * 例外メッセージ.
     */
    private String scriptException = null ;
    
    /**
     * スクリプトエラー情報.
     */
    private String errorScriptInfo = null ;
    
    /**
     * 行数.
     */
    private int[] line = null ;
    
    /**
     * エラーページ名.
     */
    private String[] pageName = null ;
    
    /**
     * カレントディレクトリ名.
     */
    private static final String CURRENT_DIR ;
    static {
        String s = null ;
        try {
            s = ScriptDef.trimCurrentDirectory( "." ) ;
        } catch( Exception e ) {
            s = null ;
        }
        CURRENT_DIR = s ;
    }
    
    /**
     * 関数予約語群.
     */
    private static final String[] REM_NAMES = {
        "break",
        "case",
        "catch",
        "default",
        "do",
        "else",
        "eval",
        "false",
        "for",
        "function",
        "if",
        "instanceof",
        "new",
        "null",
        "return",
        "switch",
        "this",
        "super",
        "throw",
        "true",
        "try",
        "typeof",
        "var",
        "void",
        "while",
        "_eval",
        "_exit",
        "exit",
        "quit",
        "isNull",
        "useString",
        "trim",
        "print",
        "println",
        "reload",
        "include",
        "httpPrint",
        "httpPrintln",
        "valueof",
        "bindings",
        "isNumeric",
        "isDouble",
        "gc",
        "pkg",
        "JSAdapter",
        "StrBuf",
        "deepCopy",
        "getLastError",
        "instanceOf",
        "load",
        "isEmpty",
        //"params",
        "parseInt",
        "parseFloat",
        "parseNumber"
    } ;
    
    /**
     * 行番号カラー.
     */
    private static final String LINE_COLOR = "#8c8c8c" ;
    
    /**
     * コーテーションカラー.
     */
    private static final String COTE_COLOR = "#ff00ff" ;
    
    /**
     * コメントカラー.
     */
    private static final String COMMENT_COLOR = "#00ff00" ;
    
    /**
     * 予約語カラー.
     */
    private static final String REM_COLOR = "#00ffff" ;
    
    /**
     * 枠カラー.
     */
    private static final String WAKU_COLOR = "#0000ff" ;
    
    /**
     * エラー原因出力カラー.
     */
    private static final String DV_ERR_COLOR = "#3399cc" ;
    
    /**
     * 行番号桁.
     */
    private static final String LINE_VIEW = "00000" ;
    
    /**
     * ライン情報.
     */
    private static final String LINE_INFO = "<table width=100% cellpadding=0 cellspacing=0><tr><td bgcolor="+WAKU_COLOR+">"+
        "<img alt=\"\" width=1 height=2></td></tr></table>" ;
    
    /**
     * スクロール終了タグ.
     */
    private static final String END_SCROLL = "</div>" ;
    
    /**
     * コンストラクタ.
     */
    private BaseScriptException() {
        
    }
    
    /**
     * コンストラクタ.
     * <BR><BR>
     * 情報を設定してオブジェクトを生成します.
     * <BR>
     * @param exception SeaBassExceptionを設定します.
     * @param script 対象のスクリプト情報を設定します.
     */
    public BaseScriptException( SeaBassException exception ) {
        this.errorScriptInfo = convertErrorInfo( exception ) ;
        scriptException = exception.getMessage() ;
    }
    
    /**
     * エラーメッセージを取得.
     * @return String エラーメッセージが返されます.
     */
    public String getMessage() {
        return scriptException ;
    }
    
    /**
     * エラー行数を取得.
     * @return int[] エラー行数が返されます.
     */
    public int[] getLine() {
        return line ;
    }
    
    /**
     * エラーページ名を取得.
     * @return String[] エラーページ名が返されます.
     */
    public String[] getSourceName() {
        return pageName ;
    }
    
    /**
     * スクリプトエラー情報を取得.
     * <BR><BR>
     * スクリプトエラー情報を取得します.
     * <BR>
     * @return String スクリプトエラー情報が返されます.
     */
    public String getErrorScriptInfo() {
        return errorScriptInfo ;
    }
    
    /**
     * スクリプトエラー行数を取得.
     */
    private String convertErrorInfo( SeaBassException exception ) {
        int no = 0 ;
        String name = null ;
        SrcScript src = null ;
        int len = 0 ;
        ErrorScriptLine[] err = getErrorLine( exception ) ;
        if( err == null || ( len =err.length ) <= 0 ) {
            return "" ;
        }
        StringBuilder buf = new StringBuilder() ;
        for( int x = 0 ; x < len ; x ++ ) {
            no ++ ;
            String directory = "" ;
            String scriptName = err[x].getName() ;
            int line = err[x].getLine() ;
            err[x] = null ;
            if( LOG.isDebugEnabled() ) {
                LOG.debug( "... errorScript:" + scriptName ) ;
            }
            ApplicationScriptFactory factory = getFactory() ;
            
            // アプリケーションスクリプト.
            try {
                Script script = factory.getApplication( scriptName ) ;
                if( script != null ) {
                    src = script.getSrcScript() ;
                    name = script.getScriptName() ;
                    if( CURRENT_DIR != null ) {
                        name = name.substring( CURRENT_DIR.length() ) ;
                    }
                }
                else {
                    src = null ;
                    name = null ;
                }
            } catch( Exception e ) {
                src = null ;
                name = null ;
            }
            if( src == null ) {
                // キャッシュデフォルトスクリプト.
                if( CacheScriptManager.getInstance().isCache( scriptName ) == true ) {
                    src = CacheScriptManager.getInstance().getSrcScript( scriptName ) ;
                    name = scriptName ;
                    directory = CacheScriptManager.getInstance().getDirectory( scriptName ) ;
                }
            }
            if( src == null ) {
                return "" ;
            }
            // SrcScriptListを取得.
            SrcScriptList srcLst = src.getLineBySrcScriptList( line ) ;
            // SrcScriptListが存在しない場合.
            if( srcLst == null ) {
                buf.append( "<code>src&nbsp;&nbsp;:</code>&nbsp;" ) ;
                buf.append( name ).append( "&nbsp;&nbsp;行：" ).append( line ) ;
                buf.append( "<BR>" ) ;
                buf.append( LINE_INFO ) ;
                buf.append( "<div id=\"err_scroll_" ).append( no ).append( "\" class=\"err_scroll\">" ) ;
                int start = line - ( OUTPUT_LINE_SIZE / 2 ) ;
                int state = 0 ;
                int cnt = 0 ;
                int movePos = 0 ;
                buf.append( "\n" ) ;
                for( int i = 0 ; i < OUTPUT_LINE_SIZE ; i ++ ) {
                    int pos = i+start ;
                    if( pos <= 0 || pos > src.lines() ) {
                        continue ;
                    }
                    String sc = src.get( pos ) ;
                    if( sc == null ) {
                        buf.append( "<span style=\"color: " ).append( LINE_COLOR ).append( ";\">" ) ;
                        buf.append( LINE_VIEW.substring( String.valueOf( pos ).length() ) ) ;
                        buf.append( pos ) ;
                        buf.append( "&nbsp;：&nbsp;" ).append( "</span>" ) ;
                        buf.append( "<BR>" ) ;
                        cnt ++ ;
                        continue ;
                    }
                    if( sc.length() > 0 ) {
                        sc = StringUtil.changeString( sc," ","&nbsp;" ) ;
                    }
                    if( pos == line ) {
                        movePos = cnt ;
                        buf.append( "<span class=\"err_line\"><font color=\"#ffffff\">" ) ;
                    }
                    buf.append( "<span style=\"color: " ).append( LINE_COLOR ).append( ";\">" ) ;
                    buf.append( LINE_VIEW.substring( String.valueOf( pos ).length() ) ) ;
                    buf.append( pos ) ;
                    buf.append( "&nbsp;：&nbsp;" ).append( "</span>" ) ;
                    StringBuilder cb = new StringBuilder() ;
                    state = putColor( cb,state,sc ) ;
                    sc = cb.toString() ;
                    cb = null ;
                    buf.append( remColor( sc ) ) ;
                    if( pos == line ) {
                        buf.append( "<font color=" ).append( DV_ERR_COLOR ).append( ">" ).
                            append( "・・・・" ).append( SeaBassException.trimErrorMessage( exception.getMessage() ) ) ;
                        buf.append( "</font></font></font></span>" ) ;
                    }
                    buf.append( "<BR>" ) ;
                    cnt ++ ;
                    buf.append( "\n" ) ;
                }
                if( state != 0 ) {
                    buf.append( "</span>" ) ;
                }
                buf.append( END_SCROLL ) ;
                buf.append( LINE_INFO ) ;
                
                // スクロールを行うJavascriptを付加.
                buf.append( scrollJavascript( movePos,no ) ) ;
            }
            // SrcScriptListが存在する場合.
            else {
                buf.append( "<code>src&nbsp;&nbsp;:</code>&nbsp;" ) ;
                buf.append( directory ).append( srcLst.getName() ).append( "&nbsp;&nbsp;行：" ).
                    append( ( line - srcLst.getStart() ) ) ;
                buf.append( "<BR>" ) ;
                buf.append( LINE_INFO ) ;
                buf.append( "<div id=\"err_scroll_" ).append( no ).append( "\" class=\"err_scroll\">" ) ;
                int start = line - ( OUTPUT_LINE_SIZE / 2 ) ;
                int state = 0 ;
                int cnt = 0 ;
                int movePos = 0 ;
                for( int i = 0 ; i < OUTPUT_LINE_SIZE ; i ++ ) {
                    int pos = i+start ;
                    int viewLine = pos - srcLst.getStart() ;
                    if( viewLine <= 0 || viewLine > srcLst.getLength() ) {
                        continue ;
                    }
                    String sc = src.get( pos ) ;
                    if( sc == null ) {
                        buf.append( "<span style=\"color: " ).append( LINE_COLOR ).append( ";\">" ) ;
                        buf.append( LINE_VIEW.substring( String.valueOf( viewLine ).length() ) ) ;
                        buf.append( viewLine ) ;
                        buf.append( "&nbsp;：&nbsp;" ).append( "</span>" ) ;
                        buf.append( "<BR>" ) ;
                        cnt ++ ;
                        continue ;
                    }
                    if( sc.length() > 0 ) {
                        sc = StringUtil.changeString( sc," ","&nbsp;" ) ;
                    }
                    if( pos == line ) {
                        movePos = cnt ;
                        buf.append( "<span class=\"err_line\"><font color=#cc0000>" ) ;
                    }
                    
                    buf.append( "<span style=\"color: " ).append( LINE_COLOR ).append( ";\">" ) ;
                    buf.append( LINE_VIEW.substring( String.valueOf( viewLine ).length() ) ) ;
                    buf.append( viewLine ) ;
                    buf.append( "&nbsp;：&nbsp;" ).append( "</span>" ) ;
                    if( pos == line ) {
                        buf.append( sc ) ;
                        buf.append( "</font>" ) ;
                        buf.append( "<font color=" ).append( DV_ERR_COLOR ).append( ">" ).
                            append( "・・・・" ).append( SeaBassException.trimErrorMessage( exception.getMessage() ) ) ;
                        buf.append( "</font></font></span>" ) ;
                    }
                    else {
                        StringBuilder cb = new StringBuilder() ;
                        state = putColor( cb,state,sc ) ;
                        sc = cb.toString() ;
                        cb = null ;
                        buf.append( remColor( sc ) ) ;
                    }
                    buf.append( "<BR>" ) ;
                    cnt ++ ;
                }
                if( state != 0 ) {
                    buf.append( "</span>" ) ;
                }
                buf.append( END_SCROLL ) ;
                buf.append( LINE_INFO ) ;
                
                // スクロールを行うJavascriptを付加.
                buf.append( scrollJavascript( movePos,no ) ) ;
            }
        }
        return buf.toString() ;
    }
    
    /**
     * エラー行数を取得.
     */
    private ErrorScriptLine[] getErrorLine( SeaBassException e ) {
        this.line = e.getLine() ;
        this.pageName = e.getScriptName() ;
        int len = this.line.length ;
        ErrorScriptLine[] ch = new ErrorScriptLine[ len ] ;
        int cnt = 0 ;
        for( int i = len-1 ; i >= 0 ; i -- ) {
            ch[ cnt ] = new ErrorScriptLine() ;
            ch[ cnt ].setLine( this.line[ i ] ) ;
            ch[ cnt ].setName( this.pageName[ i ] ) ;
            cnt ++ ;
        }
        return ch ;
    }
    
    /**
     * WebAppScriptFactoryを取得.
     */
    private static final ApplicationScriptFactory getFactory() {
        return ( ApplicationScriptFactory )GlobalManager.getValue(
            ServiceDef.MANAGER_BY_WEB_APP_FACTORY ) ;
    }
    
    /**
     * 対象スクリプトにカラーを付加する.
     */
    private static final int putColor( StringBuilder buf,int state,String line ) {
        int len = line.length() ;
        int c = -1 ;
        int back = -1 ;
        int cote = -1 ;
        boolean yenFlag = false ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( c != -1 ) {
                if( yenFlag ) {
                    yenFlag = false ;
                    back = -1 ;
                }
                else {
                    back = c ;
                }
            }
            c = line.charAt( i ) ;
            if( state == 1 ) {
                if( c == '*' && i+1 < line.length() ) {
                    char next = line.charAt( i+1 ) ;
                    if( next == '/' ) {
                        buf.append( "*/</span>" ) ;
                        i ++ ;
                        state = 0 ;
                        continue ;
                    }
                }
            }
            else {
                if( cote != -1 ) {
                    if( cote == c && back != '\\' ) {
                        cote = -1 ;
                        yenFlag = false ;
                        buf.append( ( char )c ).append( "</span>" ) ;
                        continue ;
                    }
                    else if( c == '\\' && back == '\\' ) {
                        yenFlag = true ;
                    }
                    else {
                        yenFlag = false ;
                    }
                }
                else {
                    if( ( c == '\'' || c == '\"' ) && back != '\\'  ) {
                        cote = c ;
                        buf.append( "<span style=\"color: " ).append( COTE_COLOR ).append( ";\">" ).append( ( char )c ) ;
                        continue ;
                    }
                    else if( c == '/' && i+1 < line.length() ) {
                        char next = line.charAt( i+1 ) ;
                        if( next == '/' ) {
                            buf.append( "<span style=\"color: " ).append( COMMENT_COLOR ).append( "\">" ).
                                append( line.substring( i ) ).append( "</span>" ) ;
                            break ;
                        }
                        else if( next == '*' ) {
                            state = 1 ;
                            i += 1 ;
                            buf.append( "<span style=\"color: " ).append( COMMENT_COLOR ).append( ";\">" ).append( "/*" ) ;
                            continue ;
                        }
                    }
                }
            }
            buf.append( ( char )c ) ;
        }
        if( cote != -1 ) {
            buf.append( "</span>" ) ;
        }
        return state ;
    }
    
    /**
     * 予約語のカラーを付加する.
     */
    private static final String remColor( String line ) {
        String base = line ;
        try {
            int len = REM_NAMES.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                String x = REM_NAMES[ i ] ;
                int b = 0 ;
                while( true ) {
                    int p = StringUtil.indexToNotCote( line,x,b ) ;
                    //int p = line.indexOf( x,b ) ;
                    if( p <= -1 ) {
                        break ;
                    }
                    else {
                        boolean chk = true ;
                        if( p-1 >= 0 ) {
                            char s = line.charAt( p-1 ) ;
                            if( ( s >= 'a' && s <= 'z' ) ||
                                ( s >= 'A' && s <= 'Z' ) ||
                                ( s >= '0' && s <= '9' ) ||
                                s == '_' ) {
                                chk = false ;
                            }
                        }
                        if( chk == true && p+x.length() < line.length() ) {
                            char s = line.charAt( p+x.length() ) ;
                            if( ( s >= 'a' && s <= 'z' ) ||
                                ( s >= 'A' && s <= 'Z' ) ||
                                ( s >= '0' && s <= '9' ) ||
                                s == '_' ) {
                                chk = false ;
                            }
                        }
                        if( chk ) {
                            String ss = new StringBuilder().append( line.substring( 0,p ) ).
                                append( "<span style=\"color: " ).append( REM_COLOR ).append( "\">" ).
                                append( x ).append( "</span>" ).
                                append( line.substring( p+x.length() ) ).toString() ;
                            b = p+x.length() + ( ss.length() - line.length() ) ;
                            line = ss ;
                            ss = null ;
                        }
                        else {
                            b = p+x.length() ;
                        }
                    }
                }
            }
        } catch( Exception e ) {
            line = base ;
        }
        return line ;
    }
    
    /**
     * 移動加速度.
     */
    private static final double ONE_MOVE = 13.45f ;
    
    /**
     * スクロールを行うJavascriptを作成.
     */
    private static final String scrollJavascript( int movePos,int idNum ) {
        return new StringBuilder().append( "\n\n<script>" ).
            append( "(function() {" ).
            append( "var t = document.getElementById( \"err_scroll_" ).append( idNum ).append( "\" ) ;" ).
            append( "t.scrollTop=" ).append( (int)((double)movePos*ONE_MOVE) ).append( " ;" ).
            append( "})() ;" ).append( "</script>\n" ).toString() ;
    }
}
