package org.maachang.seabass.httpd.engine.script;

import org.maachang.seabass.ExitSeaBass;
import org.maachang.seabass.SeaBassContext;
import org.maachang.seabass.httpd.HttpdRequest;
import org.maachang.seabass.httpd.HttpdResponse;
import org.maachang.seabass.httpd.engine.WriteError;

/**
 * スクリプトを強制的に途中で終了する処理.
 * 
 * @version 2007/08/24
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class EndScript {
    /**
     * 強制終了のチェック.
     * @param o 対象のスクリプト実行戻り値を設定します.
     */
    public static final boolean isEndScript( Object o ) {
        return ( o instanceof ExitSeaBass ) ;
    }
    
    /**
     * 処理結果内容を取得.
     * <BR><BR>
     * 途中終了での処理結果内容を取得します.
     * <BR>
     * @param context 対象のコンテキストを設定します.
     * @return String 処理結果内容が返されます.
     */
    public static final String getEndByResult( SeaBassContext context ) {
        Object o = context.get( "_$exit$to" ) ;
        if( o != null ) {
            return ((ExitSeaBass)o).getExitCode() ;
        }
        return null ;
    }
    
    /**
     * 処理結果内容がエラーの場合の処理.
     * @param context 対象のコンテキストを設定します.
     * @param req 対象のリクエストを設定します.
     * @param res 対象のレスポンスを設定します.
     * @return boolean [true]の場合、エラーです.
     * @exception Exception 例外.
     */
    public static final boolean executionExitByError( SeaBassContext context,
        HttpdRequest req,HttpdResponse res ) throws Exception {
        if( res.isOutputStream() ) {
            return false ;
        }
        boolean ret = false ;
        String code = getEndByResult( context ) ;
        if( code != null && ( code = code.trim().toLowerCase() ).length() > 0 ) {
            if( code.startsWith( "error" ) ) {
                code = code.substring( "error".length() ) ;
                int err = -1 ;
                try {
                    err = Integer.parseInt( code ) ;
                } catch( Exception e ) {
                    err = -1 ;
                }
                if( err != -1 ) {
                    try {
                        WriteError.output( req,res,null,err,"error" ) ;
                    } catch( Exception e ) {
                    }
                    ret = true ;
                }
            }
        }
        return ret ;
    }
}
