package org.maachang.seabass.httpd.engine.script;

import java.util.Map;

import org.maachang.seabass.SeaBassContext;

/**
 * スクリプト実行処理.
 * 
 * @version 2007/08/24
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public interface ExecutionScript {
    
    /**
     * コネクションクローズフラグを設定.
     * @param mode [true]の場合、処理後コネクションをクローズします.
     */
    public void setCloseConnect( boolean mode ) ;
    
    /**
     * コネクションクローズフラグを取得.
     * @return boolean [true]の場合、処理後コネクションをクローズします.
     */
    public boolean isCloseConnect() ;
    
    /**
     * スクリプト単体実行.
     * <BR><BR>
     * スクリプトを単体で実行します.<BR>
     * このメソッドは、主に[InnerScript]等、スクリプト内で実行する場合に
     * 呼び出されます.
     * <BR>
     * @exception Exception 例外.
     */
    public Object eval() throws Exception ;
    
    /**
     * スクリプト単体実行.
     * <BR><BR>
     * スクリプトを単体で実行します.<BR>
     * このメソッドは、主に[InnerScript]等、スクリプト内で実行する場合に
     * 呼び出されます.
     * <BR>
     * @param ctx 対象のスクリプトコンテキストを設定します.
     * @exception Exception 例外.
     */
    public Object eval( SeaBassContext ctx ) throws Exception ;
    
    /**
     * スクリプト実行.
     * <BR><BR>
     * 対象のスクリプトを実行します.
     * <BR>
     * @param params 対象のパラメータを設定します.
     * @return Object 処理結果が返されます.
     * @exception Exception 例外.
     */
    public Object execution( Object params ) throws Exception ;
    
    /**
     * スクリプト実行.
     * <BR><BR>
     * 対象のスクリプトを実行します.
     * <BR>
     * @param ctx 対象のスクリプトコンテキストを設定します.
     * @param params 対象のパラメータを設定します.
     * @return Object 処理結果が返されます.
     * @exception Exception 例外.
     */
    public Object execution( SeaBassContext ctx,Object params ) throws Exception ;
    
    /**
     * スクリプトデータタイプを取得.
     * <BR><BR>
     * スクリプトデータタイプを取得します.
     * <BR>
     * @return スクリプトデータタイプが返されます.
     */
    public int getScriptDataType() ;
    
    /**
     * スクリプト名を取得.
     * <BR><BR>
     * スクリプト名を取得します.
     * <BR>
     * @return String スクリプト名が返されます.
     */
    public String getScriptName() ;
    
    /**
     * デフォルトパラメータを設定.
     * <BR><BR>
     * デフォルトパラメータを設定します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param value 対象の要素を設定します.
     */
    public void putDefault( String key,Object value ) ;
    
}
