package org.maachang.seabass.httpd.engine.script;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import org.maachang.util.FileUtil;
import org.maachang.util.atomic.TargetLock;

/**
 * 指定ファイル内容を行指定で取得.
 * 
 * @version 2009/08/02
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public abstract class ReadLineToPoint {
    private ReadLineToPoint() {}
    
    /**
     * 指定ファイル内容を読み込み、writerに反映.
     * @param sync TargetLockオブジェクトを設定します.
     * @param writer 読み込みデータ書き込み先を設定します.
     * @param name 対象のファイル名を設定します.
     * @param charset 対象の文字コードを設定します.
     * @param start 開始行位置を設定します.
     * @param size 取得行サイズを取得します.
     * @return String 取得結果が返されます.
     * @exception Exception 例外.
     */
    public static final void read( TargetLock sync,PrintWriter writer,String name,String charset,int start,int size )
        throws Exception {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            return ;
        }
        if( size <= 0 ) {
            return ;
        }
        if( charset == null || ( charset = charset.trim() ).length() <= 0 ) {
            charset = FileUtil.OS_CHARSET ;
        }
        if( start < 0 ) {
            start = 0 ;
        }
        BufferedReader r = null ;
        name = SeabassFileIO.fullPath( name ) ;
        sync.lock( name ) ;
        try {
            r = new BufferedReader( new InputStreamReader( new FileInputStream( name ),charset ) ) ;
            for( int i = 0 ; i < start ; i ++ ) {
                if( r.readLine() == null ) {
                    return ;
                }
            }
            String s ;
            for( int i = 0 ; i < size ; i ++ ) {
                s = r.readLine() ;
                if( s == null ) {
                    break ;
                }
                writer.print( s ) ;
                writer.print( "<br>" ) ;
            }
            r.close() ;
            r = null ;
        } finally {
            if( r != null ) {
                r.close() ;
            }
            sync.unlock( name ) ;
        }
    }
}
