package org.maachang.seabass.httpd.engine.script.cron;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.JDate;
import org.maachang.util.atomic.AtomicLONG;
import org.maachang.util.atomic.AtomicOBJECT;

/**
 * クーロン(スケジューリング実行)Bean.
 * 
 * @version 2008/07/01
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class CronBean {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( CronBean.class ) ;
    
    private long configId = 0L ;
    private long cronId = 0L ;
    private int minute = -1 ;
    private int minuteTm = -1 ;
    private int hour = -1 ;
    private int hourTm = -1 ;
    private int date = -1 ;
    private int dateTm = -1 ;
    private int month = -1 ;
    private int week = -1 ;
    private String script = null ;
    private final AtomicLONG nextExecutionTime = new AtomicLONG( -1L ) ;
    private final AtomicOBJECT<ExecutionCronThread> thread = new AtomicOBJECT<ExecutionCronThread>( null ) ;
    
    private CronBean(){}
    
    /**
     * コンストラクタ.
     * @param configId コンフィグ世代IDを設定します.
     * @param cronId CronBean発行Idを設定します.
     */
    public CronBean( long configId,long cronId ) {
        this.configId = configId ;
        this.cronId = cronId ;
        nextExecutionTime.set( -1L ) ;
    }
    
    /**
     * date を取得.
     * <BR><BR>
     * @return date が返されます.
     */
    public int getDate() {
        return date;
    }
    /**
     * date を設定.
     * <BR><BR>
     * @param date date を設定します.
     */
    public void setDate(int date) {
        this.date = date;
    }
    /**
     * dateTm を取得.
     * <BR><BR>
     * @return dateTm が返されます.
     */
    public int getDateTm() {
        return dateTm;
    }
    /**
     * dateTm を設定.
     * <BR><BR>
     * @param dateTm dateTm を設定します.
     */
    public void setDateTm(int dateTm) {
        this.dateTm = dateTm;
    }
    /**
     * hour を取得.
     * <BR><BR>
     * @return hour が返されます.
     */
    public int getHour() {
        return hour;
    }
    /**
     * hour を設定.
     * <BR><BR>
     * @param hour hour を設定します.
     */
    public void setHour(int hour) {
        this.hour = hour;
    }
    /**
     * hourTm を取得.
     * <BR><BR>
     * @return hourTm が返されます.
     */
    public int getHourTm() {
        return hourTm;
    }
    /**
     * hourTm を設定.
     * <BR><BR>
     * @param hourTm hourTm を設定します.
     */
    public void setHourTm(int hourTm) {
        this.hourTm = hourTm;
    }
    /**
     * minute を取得.
     * <BR><BR>
     * @return minute が返されます.
     */
    public int getMinute() {
        return minute;
    }
    /**
     * minute を設定.
     * <BR><BR>
     * @param minute minute を設定します.
     */
    public void setMinute(int minute) {
        this.minute = minute;
    }
    /**
     * minuteTm を取得.
     * <BR><BR>
     * @return minuteTm が返されます.
     */
    public int getMinuteTm() {
        return minuteTm;
    }
    /**
     * minuteTm を設定.
     * <BR><BR>
     * @param minuteTm minuteTm を設定します.
     */
    public void setMinuteTm(int minuteTm) {
        this.minuteTm = minuteTm;
    }
    /**
     * month を取得.
     * <BR><BR>
     * @return month が返されます.
     */
    public int getMonth() {
        return month;
    }
    /**
     * month を設定.
     * <BR><BR>
     * @param month month を設定します.
     */
    public void setMonth(int month) {
        this.month = month;
    }
    /**
     * script を取得.
     * <BR><BR>
     * @return script が返されます.
     */
    public String getScript() {
        return script;
    }
    /**
     * script を設定.
     * <BR><BR>
     * @param script script を設定します.
     */
    public void setScript(String script) {
        this.script = script;
    }
    /**
     * week を取得.
     * <BR><BR>
     * @return week が返されます.
     */
    public int getWeek() {
        return week;
    }
    /**
     * week を設定.
     * <BR><BR>
     * @param week week を設定します.
     */
    public void setWeek(int week) {
        this.week = week;
    }
    
    /** 次に起動させる時間を設定. **/
    protected void newNextExecutionTime() {
        if( nextExecutionTime.get() > -1L ) {
            return ;
        }
        JDate d = new JDate( ( System.currentTimeMillis() / 1000L ) * 1000L ) ;
        // 秒が存在する場合は、１分追加.
        if( d.getSeconds() > 0 ) {
            d.setMinutes( d.getMinutes() + 1 ) ;
        }
        boolean befFlag = false ;
        d.setSeconds( 0 ) ;
        // 分設定.
        if( minuteTm > -1 ) {
            d.setMinutes( d.getMinutes() + minuteTm ) ;
            befFlag = true ;
        }
        else if( minute > -1 ) {
            if( d.getMinutes() >= minute ) {
                d.setHours( d.getHours() + 1 ) ;
                d.setMinutes( minute ) ;
            }
            else {
                d.setMinutes( minute ) ;
            }
            befFlag = true ;
        }
        else {
            d.setMinutes( 0 ) ;
        }
        // 時間を設定.
        if( hourTm > -1 ) {
            d.setHours( d.getHours() + hourTm ) ;
            befFlag = true ;
        }
        else if( hour > -1 ) {
            if( ( befFlag == true && d.getHours() > hour ) ||
                ( befFlag == false && d.getHours() >= hour ) ) {
                d.setDate( d.getDate() + 1 ) ;
                d.setHours( hour ) ;
            }
            else {
                d.setHours( hour ) ;
            }
            befFlag = true ;
        }
        else {
            d.setHours( 0 ) ;
        }
        // 曜日を設定 ※曜日指定が、月／日よりも優先度が高い.
        if( week > -1 ) {
            if( d.getDay() == week ) {
                if( befFlag == false ) {
                    while( d.getDay() != week ) {
                        d.setDate( d.getDate() + 1 ) ;
                    }
                }
            }
            else {
                while( d.getDay() != week ) {
                    d.setDate( d.getDate() + 1 ) ;
                }
            }
        }
        // 曜日指定でない場合のみ、月／日の設定が有効となる.
        else {
            boolean dateFlag = false ;
            // 日付を設定.
            if( dateTm > -1 ) {
                d.setDate( d.getDate() + dateTm ) ;
                befFlag = true ;
                dateFlag = true ;
            }
            else if( date > -1 ) {
                if( ( befFlag == true && d.getDate() > date ) ||
                    ( befFlag == false && d.getDate() >= date ) ) {
                    d.setMonth( d.getMonth() + 1 ) ;
                    d.setDate( date ) ;
                }
                else {
                    d.setDate( date ) ;
                }
                befFlag = true ;
                dateFlag = true ;
            }
            // 月を設定.
            if( month > -1 ) {
                // 日付設定がされていない場合、かならず日付は１日とする.
                if( dateFlag == false ) {
                    d.setDate( 1 ) ;
                }
                if( ( befFlag = true && d.getMonth()+1 > month ) ||
                    ( befFlag = true && d.getMonth()+1 >= month ) ) {
                    d.setYear( d.getYear() + 1 ) ;
                    d.setMonth( month-1 ) ;
                }
                else {
                    d.setMonth( month-1 ) ;
                }
            }
        }
        // 次回実行予定時刻を設定.
        long time = d.getTime() ;
        LOG.info( "## cron [" + script + "] 次回実行予定時間:" + CronBean.getNextExecutionTime( time ) ) ;
        // 次の実行時間を設定する.
        nextExecutionTime.set( time ) ;
    }
    
    /** 次回実行時間を文字列で取得. **/
    protected static final String getNextExecutionTime( long time ) {
        if( time <= -1L ) {
            return "未定" ;
        }
        JDate d = new JDate( time ) ;
        return new StringBuilder().
                append( d.getFullYear() ).
                append( "/" ).
                append( "00".substring( String.valueOf( d.getMonth()+1 ).length() ) ).
                append( (d.getMonth()+1) ).
                append( "/" ).
                append( "00".substring( String.valueOf( d.getDate() ).length() ) ).
                append( d.getDate() ).
                append( " " ).
                append( "00".substring( String.valueOf( d.getHours() ).length() ) ).
                append( d.getHours() ).
                append( ":" ).
                append( "00".substring( String.valueOf( d.getMinutes() ).length() ) ).
                append( d.getMinutes() ).
                append( ":" ).
                append( "00".substring( String.valueOf( d.getSeconds() ).length() ) ).
                append( d.getSeconds() ).
                toString() ;
    }
    
    /** Cronスクリプトが実行可能かチェック. **/
    public boolean isExecution() {
        newNextExecutionTime() ;
        if( nextExecutionTime.get() <= System.currentTimeMillis() ) {
            nextExecutionTime.set( -1L ) ;
            return true ;
        }
        return false ;
    }
    
    /**
     * 文字列に変換.
     * @return String 変換された文字列が返されます.
     */
    public String toString() {
        return new StringBuilder().
            append( "minute=" ).append( minute ).append( "," ).
            append( "minuteTm=" ).append( minuteTm ).append( "," ).
            append( "hour=" ).append( hour ).append( "," ).
            append( "hourTm=" ).append( hourTm ).append( "," ).
            append( "date=" ).append( date ).append( "," ).
            append( "dateTm=" ).append( dateTm ).append( "," ).
            append( "month=" ).append( month ).append( "," ).
            append( "week=" ).append( week ).append( "," ).
            append( "script=" ).append( script ).toString() ;
    }
    
    /**
     * Config世代IDを取得.
     * @return long Config世代IDが返されます.
     */
    public long getConfigId() {
        return configId ;
    }
    
    /**
     * CronIdを取得.
     * @return long CronIdが返されます.
     */
    public long getCronId() {
        return cronId ;
    }
    
    /**
     * 同じ内容かチェック.
     * @param o オブジェクトを設定します.
     * @return boolean [true]の場合、同じオブジェクトです.
     */
    public boolean equals( Object o ) {
        if( o == null || ( o instanceof CronBean ) == false ) {
            return false ;
        }
        CronBean b = ( CronBean )o ;
        return b.getCronId() == cronId ;
    }
    
    /**
     * 実行中のスレッドオブジェクトを設定.
     */
    protected void setThread( ExecutionCronThread t ) {
        thread.set( t ) ;
    }
    
    /**
     * 実行中のスレッドが存在するかチェック.
     */
    protected boolean isThread() {
        return thread.get() != null ;
    }
}
