package org.maachang.seabass.httpd.engine.script.cron;

import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.util.thread.LoopThread;

/**
 * クーロン(スケジューリング実行)タイマー.
 * 
 * @version 2008/07/01
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class CronTiemrThread extends LoopThread {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( CronTiemrThread.class ) ;
    
    /**
     * スケジュールコンフィグ.
     */
    private CronConfig config = null ;
    
    /**
     * コンストラクタ.
     */
    public CronTiemrThread() throws Exception {
        this.config = new CronConfig() ;
        this.config.open() ;
        this.startThread() ;
    }
    
    /**
     * デストラクタ.
     */
    protected void finalize() throws Exception {
        this.destroy() ;
    }
    
    /**
     * オブジェクト破棄.
     */
    public synchronized void destroy() {
        super.stopThread() ;
        boolean exitFlag ;
        while( true ) {
            exitFlag = true ;
            Iterator it = config.executions() ;
            if( it != null ) {
                while( it.hasNext() ) {
                    if( config.getExecution( ( Long )it.next() ).isThread() ) {
                        exitFlag = false ;
                        break ;
                    }
                }
            }
            if( exitFlag ) {
                break ;
            }
        }
    }
    
    /**
     * オブジェクトクリア処理.
     * <p>スレッドが停止したときに呼び出されます.</p>
     */
    protected void clear() {
        if( this.config != null ) {
            config.close() ;
        }
        config = null ;
    }
    
    /**
     * 実行処理.
     * @exception Exception 例外.
     */
    protected boolean execution() throws Exception {
        CronBean bean = null ;
        while( true ) {
            updateConfig() ;
            swait() ;
            try {
                if( ( bean = config.schedulePoll() ) == null ) {
                    break ;
                }
                if( bean.isExecution() ) {
                    ExecutionCronThread thread = new ExecutionCronThread( config,bean ) ;
                    config.addExecution( bean ) ;
                    bean.setThread( thread ) ;
                    thread.start() ;
                    bean = null ;
                }
                else {
                    config.scheduleOffer( bean ) ;
                    bean = null ;
                }
            } catch( Exception e ) {
            } finally {
                if( bean != null ) {
                    config.scheduleOffer( bean ) ;
                }
            }
        }
        return false ;
    }
    
    /**
     * 更新条件が存在する場合の処理.
     */
    private void updateConfig() {
        config.updateToReload() ;
    }
    
    /**
     * 一時停止.
     */
    private void swait() throws Exception {
        Thread.sleep( SWAIT ) ;
    }
    
    private static final int SWAIT = 50 ;
}
