package org.maachang.seabass.httpd.engine.script.cron;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.seabass.httpd.engine.script.scripts.CronScript;
import org.maachang.util.atomic.AtomicBOOL;

/**
 * クーロン(スケジューリング実行)タイマー.
 * 
 * @version 2008/07/01
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
class ExecutionCronThread extends Thread {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( ExecutionCronThread.class ) ;
    
    /**
     * 実行CronBean.
     */
    private CronBean bean = null ;
    
    /**
     * CronConfig.
     */
    private CronConfig config = null ;
    
    /**
     * スレッド停止フラグ.
     */
    private final AtomicBOOL stopFlag = new AtomicBOOL( false ) ;
    
    /**
     * コンストラクタ.
     */
    private ExecutionCronThread() {
        
    }
    
    /**
     * コンストラクタ.
     * @param executions スケジュール実行要素を設定します.
     * @param bean スケジュール実行Beanを設定します.
     * @exception Exception 例外.
     */
    public ExecutionCronThread( CronConfig config,CronBean bean )
        throws Exception {
        this.config = config ;
        this.bean = bean ;
        this.setDaemon( true ) ;
    }
    
    /**
     * 停止フラグを取得.
     * @return boolean [true]の場合、停止しています.
     */
    public boolean isStop() {
        return stopFlag.get() ;
    }
    
    /**
     * スレッド実行.
     */
    public void run() {
        System.out.println( "### run" ) ;
        String scriptName = bean.getScript() ;
        try {
            long time = System.currentTimeMillis() ;
            LOG.info( "##[start] Cron実行:[" + scriptName + "] - {"+bean.toString()+"}" ) ;
            CronScript.execution( scriptName ) ;
            time = System.currentTimeMillis() - time ;
            LOG.info( "##[end  ] Cron実行:[" + scriptName + "] - {"+bean.toString()+"} - " + time + "msec" ) ;
        } catch( Exception e ) {
            LOG.error( "##[error] Cron実行[" + scriptName + "] - {"+bean.toString()+"}",e ) ;
        } finally {
            // 終了処理.
            try {
                if( config.isExecution( bean ) ) {
                    config.scheduleOffer( bean ) ;
                    config.removeExecution( bean ) ;
                }
            } catch( Exception e ) {
            }
            stopFlag.set( true ) ;
            bean = null ;
            config = null ;
            try {
                bean.setThread( null ) ;
            } catch( Exception e ) {
            }
        }
    }
    
}
