package org.maachang.seabass.httpd.engine.script.dao;

import java.io.Reader;
import java.io.Writer;

import org.maachang.dao.dbms.DbUtil;
import org.maachang.dao.dbms.OperationTable;
import org.maachang.dao.dbms.Record;
import org.maachang.dao.dbms.RecordFactory;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.SeaBassDef;
import org.maachang.seabass.ServiceDef;

/**
 * モデル操作オブジェクト.
 * 
 * @version 2010/08/28
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public abstract class OperationModel {
    
    /**
     * 指定テーブル情報を再構築.
     * @param model 対象のモデル名を設定します.
     * @exception Exception 例外.
     */
    public static final void recreateTable( String model )
        throws Exception {
        if( model == null || model.length() <= 0 ) {
            throw new IllegalArgumentException( "モデル名が設定されていません" ) ;
        }
        String javaModel = DbUtil.convertDBNameByJavaName( true,model ) ;
        String dbModel = DbUtil.convertJavaNameByDBName( javaModel ) ;
        RecordFactory factory = ( RecordFactory )GlobalManager.getInstance().get( ServiceDef.MANAGER_BY_DBMS_POOL ) ;
        MasterCache cache = ( MasterCache )GlobalManager.getInstance().get( ServiceDef.MASTER_DB_CACHE_MANAGER ) ;
        cache.clearCache( javaModel ) ;
        OperationTable.recreateTable( factory,SeaBassDef.DIRECTORY_TABLE,dbModel ) ;
        cache.clearCache( javaModel ) ;
    }
    
    /**
     * 指定テーブル情報に対象CSVをImport.
     * @param model 対象のモデル名を設定します.
     * @param record 対象のレコードオブジェクトを設定します.
     * @param reader インポート対象のInputStreamReaderを設定します.
     * @return int インポート件数が返されます.
     * @exception Exception 例外.
     */
    public static final int importCsv( String model,Record record,Reader reader )
        throws Exception {
        if( model == null || model.length() <= 0 ) {
            throw new IllegalArgumentException( "モデル名が設定されていません" ) ;
        }
        if( reader == null ) {
            throw new IllegalArgumentException( "Readerが設定されていません" ) ;
        }
        String javaModel = DbUtil.convertDBNameByJavaName( true,model ) ;
        String dbModel = DbUtil.convertJavaNameByDBName( javaModel ) ;
        RecordFactory factory = ( RecordFactory )GlobalManager.getInstance().get( ServiceDef.MANAGER_BY_DBMS_POOL ) ;
        MasterCache cache = ( MasterCache )GlobalManager.getInstance().get( ServiceDef.MASTER_DB_CACHE_MANAGER ) ;
        cache.clearCache( javaModel ) ;
        int ret = OperationTable.importCsv( factory,record,SeaBassDef.DIRECTORY_TABLE,dbModel,reader ) ;
        cache.clearCache( javaModel ) ;
        return ret ;
    }
    
    /**
     * 指定テーブル情報をExport.
     * @param writer 出力先のオブジェクトを設定します.
     * @param record 対象のレコードオブジェクトを設定します.
     * @param model 対象のモデル名を設定します.
     * @return int 件数が返されます.
     * @exception Exception 例外.
     */
    public static final int exportCsv( Writer writer,Record record,String model )
        throws Exception {
        if( model == null || model.length() <= 0 ) {
            throw new IllegalArgumentException( "モデル名が設定されていません" ) ;
        }
        if( writer == null ) {
            throw new IllegalArgumentException( "Writerが設定されていません" ) ;
        }
        String javaModel = DbUtil.convertDBNameByJavaName( true,model ) ;
        String dbModel = DbUtil.convertJavaNameByDBName( javaModel ) ;
        return OperationTable.exportCsv( writer,record,dbModel ) ;
    }
}
