package org.maachang.seabass.httpd.engine.script.doc ;

import java.util.ArrayList;
import java.util.List;

import org.maachang.seabass.httpd.engine.script.scripts.ParentMhtml;
import org.maachang.util.FileUtil;

/**
 * SeaBassドキュメント[Top]Bean.
 * 
 * @version 2009/01/22
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class SeaBassDoc implements ParentMhtml {
    /** ファイル更新管理 **/
    private static class FileToTime {
        String fileName = null ;
        long lastUpdate = -1L ;
        private FileToTime(){}
        public FileToTime( String fileName,long lastUpdate ) {
            this.fileName = fileName ;
            this.lastUpdate = lastUpdate ;
        }
        public FileToTime( String fileName ) {
            this.fileName = fileName ;
            this.lastUpdate = FileUtil.getLastTime( fileName ) ;
        }
        public String getFileName() {
            return fileName ;
        }
        public long getLastUpdate() {
            return lastUpdate ;
        }
        public boolean isUpdateFile() {
            long t = FileUtil.getLastTime( fileName ) ;
            if( t != lastUpdate ) {
                lastUpdate = t ;
                return false ;
            }
            return true ;
        }
    }
    
    private List<SeaBassDocCategory> categorys = null ;
    private List<FileToTime> fileList = null ;
    private FileToTime mainFile = null ;
    private String title = null ;
    private String templateName = null ;
    private int mode = -1 ;
    
    /**
     * コンストラクタ.
     */
    public SeaBassDoc( String fileName,long lastUpdate ) {
        if( fileName == null || ( fileName = fileName.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "ファイル名を設定してください" ) ;
        }
        this.categorys = new ArrayList<SeaBassDocCategory>() ;
        this.fileList = new ArrayList<FileToTime>() ;
        this.mainFile = new FileToTime( fileName,lastUpdate ) ;
    }
    
    protected void finalize() throws Exception {
        destroy() ;
    }
    
    /**
     * オブジェクト破棄.
     */
    public void destroy() {
        categorys = null ;
        fileList = null ;
        mainFile = null ;
        templateName = null ;
        mode = -1 ;
    }
    
    /**
     * 条件を決定.
     */
    public void flush() {
        switch( mode ) {
            case 1 : removeTarget( categorys ) ; break ;
            case 2 : removeParams( categorys ) ; break ;
        }
    }
    
    /**
     * １つのカテゴリを追加.
     * @param category 対象のカテゴリを設定します.
     */
    public void add( SeaBassDocCategory category ) {
        if( category == null ) {
            return ;
        }
        categorys.add( category ) ;
    }
    
    /**
     * 1つのカテゴリを取得.
     * @param no 取得対象の項番を設定します.
     * @return SeaBassDocCategory カテゴリが返されます.
     */
    public SeaBassDocCategory get( int no ) {
        if( no >= 0 && no < categorys.size() ) {
            return categorys.get( no ) ;
        }
        return null ;
    }
    
    /**
     * カテゴリ数を取得.
     * @return int size カテゴリ数が返されます.
     */
    public int size() {
        return categorys.size() ;
    }
    
    /**
     * Chainファイル名をセット.
     * @param fileName 対象のChainファイル名を設定します.
     */
    public void addChain( String fileName ) {
        fileList.add( new FileToTime( fileName ) ) ;
    }
     
    /**
     * テンプレート名を設定.
     * @param templateName 対象のテンプレート名を設定します.
     * @exception Exception 例外.
     */
    public void setTemplateName( String templateName )
        throws Exception {
        if( templateName == null || ( templateName = templateName.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "テンプレート名が存在しません" ) ;
        }
        this.templateName = templateName ;
    }
    
    /**
     * テンプレート名を取得.
     * @return String テンプレート名が返されます.
     */
    public String getTemplateName() {
        return templateName ;
    }
    
    /**
     * ファイル名を取得.
     * @return String ファイル名を取得します.
     */
    public String getFileName() {
        return mainFile.getFileName() ;
    }
    
    /**
     * タイトルを設定.
     * @param title 対象のタイトルを設定します.
     */
    public void setTitle( String title ) {
        this.title = title ;
    }
    
    /**
     * タイトルを取得.
     * @return String タイトルが返されます.
     */
    public String getTitle() {
        return title ;
    }
    
    /**
     * モードを設定.
     * @param mode 対象のモードを設定します.
     */
    public void setMode( int mode ) {
        this.mode = mode ;
    }
    
    /**
     * モードを取得.
     * @return int モードを取得します.
     */
    public int getMode() {
        return this.mode ;
    }
    
    /**
     * 最終更新時間を取得.
     * @return long 最終更新時間が返されます.
     */
    public long getLastUpdate() {
        return mainFile.getLastUpdate() ;
    }
    
    /**
     * ファイル更新チェック.
     * @return boolean [true]の場合、再読み込みが必要です.
     */
    public boolean isUpdate() {
        boolean ret = false ;
        if( !mainFile.isUpdateFile() ) {
            ret = true ;
        }
        int len = fileList.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( !fileList.get( i ).isUpdateFile() ) {
                ret = true ;
            }
        }
        return ret ;
    }
    
    public String toString() {
        StringBuilder buf = new StringBuilder() ;
        buf.append( " [@tagLibDoc] {" ) ;
        buf.append( " fileName:" ).append( mainFile.getFileName() ).append( " lastUpdate:" ).append( mainFile.getLastUpdate() ) ;
        buf.append( " template:" ).append( templateName ).
            append( " title:" ).append( title ).
            append( " mode:" ).append( mode ) ;
        int len = fileList.size() ;
        if( len > 0 ) {
            buf.append( " chain [" ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                buf.append( fileList.get( i ).getFileName() ) ;
            }
            buf.append( "]" ) ;
        }
        len = categorys.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            buf.append( categorys.get( i ).toString() ) ;
        }
        return buf.append( "}" ).toString() ;
    }
    
    /** ターゲット群を削除. **/
    private static final void removeTarget( List<SeaBassDocCategory> categorys ) {
        int len = categorys.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            categorys.get( i ).removeAll() ;
        }
    }
    
    /** パラメータ群を削除 **/
    private static final void removeParams( List<SeaBassDocCategory> categorys ) {
        int len = categorys.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            SeaBassDocCategory category = categorys.get( i ) ;
            int lenJ = category.size() ;
            for( int j = 0 ; j < lenJ ; j ++ ) {
                category.get( j ).removeAll() ;
            }
        }
    }
}
