package org.maachang.seabass.httpd.engine.script.doc ;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.maachang.seabass.SeaBassDef;
import org.maachang.seabass.httpd.engine.script.js.JsDef;
import org.maachang.util.FileUtil;
import org.maachang.util.StringUtil;

/**
 * SeaBassドキュメント解析.
 * 
 * @version 2009/01/22
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public abstract class SeaBassDocAnalysis {
    private SeaBassDocAnalysis(){}
    
    /** SeaBassドキュメント解析用スタック **/
    private static class SeaBassDocStack {
        Object object = null ;
        SeaBassDocStack next = null ;
    }
    
    /**
     * データ解析.
     * @param name 対象のファイル名を設定します.
     * @param charset 対象のキャラクタセットを設定します.
     * @return SeaBassDoc タグライブラリドキュメント情報が返されます.
     * @exception Exception 例外.
     */
    public static final SeaBassDoc analysis( String name,String charset )
        throws Exception {
        return analysis( null,new HashSet<String>(),name,charset ) ;
    }
    
    /**
     * データ解析.
     */
    private static final SeaBassDoc analysis( SeaBassDoc doc,Set<String> overFlow,String name,String charset )
        throws Exception {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "ファイル名は不正です" ) ;
        }
        name = FileUtil.getFullPath( name ) ;
        if( overFlow.contains( name ) ) {
            throw new IOException( "@chain " + name + " は、既に呼び出されています" ) ;
        }
        overFlow.add( name ) ;
        if( charset == null || ( charset = charset.trim() ).length() <= 0 ) {
            charset = "UTF8" ;
        }
        boolean top = ( doc == null ) ;
        BufferedReader buf = null ;
        try {
            buf = new BufferedReader( new InputStreamReader( new FileInputStream( name ),charset ) ) ;
            if( doc == null ) {
                doc = new SeaBassDoc( name,FileUtil.getLastTime( name ) ) ;
            }
            else {
                doc.addChain( name ) ;
            }
            String fileName = getFileName( name ) ;
            SeaBassDocStack stack = new SeaBassDocStack() ;
            stack.object = doc ;
            List<Integer> atPos = new ArrayList<Integer>() ;
            StringBuilder work = new StringBuilder() ;
            int[] type = new int[ 1 ] ;
            type[ 0 ] = -1 ;
            int cnt = 1 ;
            boolean noEnterFlag = false ;
            boolean templateFlag = false ;
            while( true ) {
                String oneLine = buf.readLine() ;
                if( oneLine == null ) {
                    addBeforeStack( fileName,type[0],stack,work ) ;
                    stack = exitStack( 1,cnt,name,stack ) ;
                    if( top ) {
                        doc.flush() ;
                    }
                    break ;
                }
                // 対象テンプレート読み込み.
                if( templateFlag == false ) {
                    int p = oneLine.indexOf( "@template" ) ;
                    if( p != -1 ) {
                        if( atCodeCount( "@" ) != 1 ) {
                            throw new IOException( "テンプレート設定行に対して、複数の命令は設定できません[" +
                                oneLine + "](" + name + ")" ) ;
                        }
                        if( doc.getTemplateName() == null ) {
                            p += 9 ;
                            String templateName = oneLine.substring( p ).trim() ;
                            if( templateName.startsWith( "/" ) ) {
                                if( templateName.startsWith( "/" ) ) {
                                    templateName = templateName.substring( 1 ) ;
                                }
                                templateName = new StringBuilder().append( SeaBassDef.NOW_APPLICATION_PATH ).append( templateName ).toString() ;
                            }
                            else {
                                templateName = name.substring( 0,name.length()-FileUtil.getFileName( name ).length() )+templateName ;
                            }
                            doc.setTemplateName( templateName ) ;
                        }
                        templateFlag = true ;
                        continue ;
                    }
                }
                else {
                    // 他SeaBassドキュメントと連動する場合.
                    int p = oneLine.indexOf( "@chain" ) ;
                    if( p != -1 ) {
                        if( atCodeCount( "@" ) != 1 ) {
                            throw new IOException( "結合設定行に対して、複数の命令は設定できません[" +
                                oneLine + "](" + name + ")" ) ;
                        }
                        p += 6 ;
                        String chainName = oneLine.substring( p ).trim() ;
                        if( chainName.startsWith( "/" ) ) {
                            chainName = new StringBuilder().append( SeaBassDef.NOW_APPLICATION_PATH ).append( chainName ).toString() ;
                        }
                        else {
                            chainName = new StringBuilder().append( name.substring( 0,name.length()-FileUtil.getFileName( name ).length() )).
                                append( chainName ).toString() ;
                        }
                        if( !chainName.toLowerCase().endsWith( ".sdoc" ) ) {
                            chainName += ".sdoc" ;
                        }
                        doc.addChain( chainName ) ;
                        analysis( doc,overFlow,chainName,charset ) ;
                        continue ;
                    }
                    // タイトル情報.
                    p = oneLine.indexOf( "@title" ) ;
                    if( p != -1 ) {
                        if( atCodeCount( "@" ) != 1 ) {
                            throw new IOException( "タイトル設定行に対して、複数の命令は設定できません[" +
                                oneLine + "](" + name + ")" ) ;
                        }
                        if( doc.getTitle() == null ) {
                            p += 6 ;
                            String title = oneLine.substring( p ).trim() ;
                            doc.setTitle( title ) ;
                        }
                        continue ;
                    }
                    // データモード情報.
                    p = oneLine.indexOf( "@mode" ) ;
                    if( p != -1 ) {
                        if( atCodeCount( "@" ) != 1 ) {
                            throw new IOException( "モード設定行に対して、複数の命令は設定できません[" +
                                oneLine + "](" + name + ")" ) ;
                        }
                        if( doc.getMode() != -1 ) {
                            p += 5 ;
                            String modeS = oneLine.substring( p ).trim() ;
                            int mode = -1 ;
                            try {
                                mode = Integer.parseInt( modeS ) ;
                            } catch( Exception e ) {
                                throw new IOException( "モード設定は数値で指定してください[" +
                                    oneLine + "](" + name + ") - " + modeS ) ;
                            }
                            doc.setMode( mode ) ;
                        }
                        continue ;
                    }
                }
                // 通常処理.
                if( atCodeArray( atPos,oneLine ) ) {
                    stack = aLine( type,cnt,name,fileName,stack,work,oneLine,atPos ) ;
                }
                else {
                    int p = oneLine.toLowerCase().indexOf( "$noenter$" ) ;
                    if( p != -1 ) {
                        noEnterFlag = true ;
                        oneLine = oneLine.substring( 0,p ) + oneLine.substring( p+9 ) ;
                        oneLine = oneLine.trim() ;
                    }
                    p = oneLine.toLowerCase().indexOf( "$/noenter$" ) ;
                    if( p != -1 ) {
                        noEnterFlag = false ;
                        oneLine = oneLine.substring( 0,p ) + oneLine.substring( p+10 ) ;
                        oneLine = oneLine.trim() ;
                    }
                    if( noEnterFlag == false && work.length() > 0 && ( oneLine = oneLine.trim() ).length() > 0 ) {
                        work.append( "<br>" ) ;
                    }
                    work.append( oneLine ) ;
                }
                cnt ++ ;
            }
            
            buf.close() ;
            buf = null ;
            if( templateFlag == false ) {
                throw new IOException( "sdoc先頭に対して@template設定が行われていません(" + name + ")" ) ;
            }
            return doc ;
        } finally {
            if( buf != null ) {
                try {
                    buf.close() ;
                } catch( Exception e ) {
                }
                buf = null ;
            }
        }
    }
    
    /** 前回の条件に文字列を付加. **/
    private static final void addBeforeStack( String fileName,int type,SeaBassDocStack stack,StringBuilder work )
        throws Exception {
        Object obj = stack.object ;
        String s = work.toString() ;
        work.setLength(0) ;
        if( obj instanceof SeaBassDoc ) {
            return ;
        }
        s.trim() ;
        if( s.length() > 0 ) {
            s = StringUtil.changeString( s,"<tab>","&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
            s = StringUtil.changeString( s,"<TAB>","&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
            s = convertTagLibHTML( s ) ;
            s = convertHTML( s ) ;
        }
        switch( type ) {
            case 1 :// @category.
                ((SeaBassDocCategory)obj).setName( s ) ;
                ((SeaBassDocCategory)obj).setFileName( fileName ) ;
                break ;
            case 2 :// @target.
                ((SeaBassDocBase)obj).setName( s ) ;
                break ;
            case 3 :// @param.
                ((SeaBassDocBase)obj).setName( s ) ;
                break ;
            case 4 :// @info.
                ((SeaBassDocBase)obj).setInfo( s ) ;
                break ;
            case 5 :// @sample.
                ((SeaBassDocTarget)obj).setSample( s ) ;
                break ;
        }
    }
    
    /** 現在のスタック条件を終了させる. **/
    private static final SeaBassDocStack exitStack( int type,int cnt,String name,SeaBassDocStack stack )
        throws Exception {
        switch( type ) {
            case 1 :// @category.
                while( true ) {
                    Object o = stack.object ;
                    if( o instanceof SeaBassDocParam ) {
                        stack = stack.next ;
                        if( !( stack.object instanceof SeaBassDocTarget ) ) {
                            throw new IOException( "[line:" + cnt + " @paramsに対する親条件は不正です["+
                                stack.object.getClass().getName()+"](" + name + ")" ) ;
                        }
                        ((SeaBassDocTarget)stack.object).add( (SeaBassDocParam)o ) ;
                    }
                    else if( o instanceof SeaBassDocTarget ) {
                        stack = stack.next ;
                        if( !( stack.object instanceof SeaBassDocCategory ) ) {
                            throw new IOException( "[line:" + cnt + " @targetに対する親条件は不正です["+
                                stack.object.getClass().getName()+"](" + name + ")" ) ;
                        }
                        ((SeaBassDocCategory)stack.object).add( (SeaBassDocTarget)o ) ;
                    }
                    else if( o instanceof SeaBassDocCategory ) {
                        stack = stack.next ;
                        if( !( stack.object instanceof SeaBassDoc ) ) {
                            throw new IOException( "[line:" + cnt + " @categoryに対する親条件は不正です["+
                                stack.object.getClass().getName()+"](" + name + ")" ) ;
                        }
                        ((SeaBassDoc)stack.object).add( (SeaBassDocCategory)o ) ;
                    }
                    else if( o instanceof SeaBassDoc ) {
                        break ;
                    }
                }
                break ;
            case 2 :// @target.
                while( true ) {
                    Object o = stack.object ;
                    if( o instanceof SeaBassDocParam ) {
                        stack = stack.next ;
                        if( !( stack.object instanceof SeaBassDocTarget ) ) {
                            throw new IOException( "[line:" + cnt + " @paramsに対する親条件は不正です["+
                                stack.object.getClass().getName()+"](" + name + ")" ) ;
                        }
                        ((SeaBassDocTarget)stack.object).add( (SeaBassDocParam)o ) ;
                    }
                    else if( o instanceof SeaBassDocTarget ) {
                        stack = stack.next ;
                        if( !( stack.object instanceof SeaBassDocCategory ) ) {
                            throw new IOException( "[line:" + cnt + " @targetに対する親条件は不正です["+
                                stack.object.getClass().getName()+"](" + name + ")" ) ;
                        }
                        ((SeaBassDocCategory)stack.object).add( (SeaBassDocTarget)o ) ;
                    }
                    else if( o instanceof SeaBassDocCategory || o instanceof SeaBassDocCategory ||
                        o instanceof SeaBassDoc ) {
                        break ;
                    }
                }
                break ;
            case 3 :// @param.
                while( true ) {
                    Object o = stack.object ;
                    if( o instanceof SeaBassDocParam ) {
                        stack = stack.next ;
                        if( !( stack.object instanceof SeaBassDocTarget ) ) {
                            throw new IOException( "[line:" + cnt + " @paramsに対する親条件は不正です["+
                                stack.object.getClass().getName()+"](" + name + ")" ) ;
                        }
                        ((SeaBassDocTarget)stack.object).add( (SeaBassDocParam)o ) ;
                    }
                    else if( o instanceof SeaBassDocTarget || o instanceof SeaBassDocCategory ||
                        o instanceof SeaBassDocCategory || o instanceof SeaBassDoc ) {
                        break ;
                    }
                }
                break ;
            case 4 :// @info.
                Object o = stack.object ;
                if( o instanceof SeaBassDocParam ) {
                    if( ((SeaBassDocParam)o).getInfo() != null ) {
                        // パラメータの内容が既に内容が設定されているときは、１つ上のスタックに移動.
                        stack = stack.next ;
                        if( !( stack.object instanceof SeaBassDocTarget ) ) {
                            throw new IOException( "[line:" + cnt + " @paramsに対する親条件は不正です["+
                                stack.object.getClass().getName()+"](" + name + ")" ) ;
                        }
                        ((SeaBassDocTarget)stack.object).add( (SeaBassDocParam)o ) ;
                    }
                }
                break ;
        }
        return stack ;
    }
    
    /** @マーク以下の内容を解析. **/
    private static final SeaBassDocStack aLine( int[] type,int cnt,String name,String fileName,
        SeaBassDocStack stack,StringBuilder work,String oneLine,List<Integer> atPos )
        throws Exception {
        SeaBassDocStack now = stack ;
        int len = atPos.size() ;
        int b = 0 ;
        for( int i = 0 ; i < len ; i ++ ) {
            int p = atPos.get( i ) ;
            String target = cutToSpace( p,oneLine ) ;
            target = target.toLowerCase() ;
            if( "@category".equals( target ) ) {// type=1
                String info = oneLine.substring( b,p ).trim() ;
                if( work.length() > 0 && info.length() > 0 ) {
                    work.append( "<br>" ) ;
                }
                work.append( info ) ;
                addBeforeStack( fileName,type[0],now,work ) ;
                now = exitStack( 1,cnt,name,now ) ;
                SeaBassDocStack ns = new SeaBassDocStack() ;
                ns.object = new SeaBassDocCategory() ;
                ns.next = now ;
                now = ns ;
                // 現在のタイプはカテゴリ設定.
                type[0] = 1 ;
                b = p + 9 ;
            }
            else if( "@target".equals( target ) ) {// type=2
                String info = oneLine.substring( b,p ).trim() ;
                if( work.length() > 0 && info.length() > 0 ) {
                    work.append( "<br>" ) ;
                }
                work.append( info ) ;
                addBeforeStack( fileName,type[0],now,work ) ;
                now = exitStack( 2,cnt,name,now ) ;
                SeaBassDocStack ns = new SeaBassDocStack() ;
                ns.object = new SeaBassDocTarget() ;
                ns.next = now ;
                now = ns ;
                // 現在のタイプはタグライブラリ設定.
                type[0] = 2 ;
                b = p + 7 ;
            }
            else if( "@param".equals( target ) ) {// type=3
                String info = oneLine.substring( b,p ).trim() ;
                if( work.length() > 0 && info.length() > 0 ) {
                    work.append( "<br>" ) ;
                }
                work.append( info ) ;
                addBeforeStack( fileName,type[0],now,work ) ;
                now = exitStack( 3,cnt,name,now ) ;// 3固定(@target)とする.
                SeaBassDocStack ns = new SeaBassDocStack() ;
                ns.object = new SeaBassDocParam() ;
                ns.next = now ;
                now = ns ;
                // 現在のタイプはパラメータ設定.
                type[0] = 3 ;
                b = p + 6 ;
            }
            else if( "@info".equals( target ) ) {// type=4
                String info = oneLine.substring( b,p ).trim() ;
                if( work.length() > 0 && info.length() > 0 ) {
                    work.append( "<br>" ) ;
                }
                work.append( info ) ;
                addBeforeStack( fileName,type[0],now,work ) ;
                now = exitStack( 4,cnt,name,now ) ;// 4固定(@info)とする.
                // 現在のタイプは説明設定.
                type[0] = 4 ;
                b = p + 5 ;
            }
            else if( "@sample".equals( target ) ) {// type=5
                String info = oneLine.substring( b,p ).trim() ;
                if( work.length() > 0 && info.length() > 0 ) {
                    work.append( "<br>" ) ;
                }
                work.append( info ) ;
                addBeforeStack( fileName,type[0],now,work ) ;
                now = exitStack( 3,cnt,name,now ) ;// 3固定(@target)とする.
                // 現在のタイプはサンプル設定.
                type[0] = 5 ;
                b = p + 7 ;
            }
        }
        work.append( oneLine.substring( b ).trim() ) ;
        return now ;
    }
    
    /** 指定位置から空白までの文字列を取得. **/
    private static final String cutToSpace( int index,String oneLine )
        throws Exception {
        int len = oneLine.length() ;
        for( int i = index ; i < len ; i ++ ) {
            char c = oneLine.charAt( i ) ;
            if( c == ' ' || c == '\t' ) {
                return oneLine.substring( index,i ) ;
            }
        }
        return oneLine.substring( index ) ;
    }
    
    /** @マークポイントを取得. **/
    private static final int atCodeCount( String oneLine )
        throws Exception {
        int len = oneLine.length() ;
        int cnt = 0 ;
        char b = 0 ;
        for( int i = 0 ; i < len ; i ++ ) {
            char c = oneLine.charAt( i ) ;
            if( c == '@' && ( b == 0 || b == ' ' || b == '\t' ) ) {
                cnt ++ ;
            }
            b = c ;
        }
        return cnt ;
    }
    
    /** @マークポイントを取得. **/
    private static final boolean atCodeArray( List<Integer> out,String oneLine )
        throws Exception {
        out.clear() ;
        int len = oneLine.length() ;
        boolean ret = false ;
        char b = 0 ;
        for( int i = 0 ; i < len ; i ++ ) {
            char c = oneLine.charAt( i ) ;
            if( c == '@' && ( b == 0 || b == ' ' || b == '\t' ) ) {
                out.add( i ) ;
                ret = true ;
            }
            b = c ;
        }
        return ret ;
    }
    
    /** <<xx>>の範囲内のものを、HTML表示可能に変換する. **/
    private static final String convertHTML( String str ) throws Exception {
        StringBuilder buf = new StringBuilder((int)(str.length()*1.5f)) ;
        int p = 0 ;
        int b = 0 ;
        while( true ) {
            p = str.indexOf( "<<",b ) ;
            if( p <= -1 ) {
                buf.append( str.substring( b ) ) ;
                break ;
            }
            int x = str.indexOf( ">>",p ) ;
            if( x <= -1 ) {
                buf.append( str.substring( b ) ) ;
                break ;
            }
            buf.append( str.substring( b,p ) ) ;
            buf.append( JsDef.convertByViewHtml( false,str.substring( p+1,x+1 ) ) ) ;
            b = x + 2 ;
        }
        return buf.toString() ;
    }
    
    /** <$ .. >の範囲内のものをHTML表示可能に変換する. **/
    private static final String convertTagLibHTML( String str ) throws Exception {
        StringBuilder buf = new StringBuilder((int)(str.length()*1.5f)) ;
        int p = 0 ;
        int b = 0 ;
        while( true ) {
            p = str.indexOf( "<<$",b ) ;
            if( p <= -1 ) {
                buf.append( str.substring( b ) ) ;
                break ;
            }
            int x = str.indexOf( ">>",p ) ;
            if( x <= -1 ) {
                buf.append( str.substring( b ) ) ;
                break ;
            }
            buf.append( str.substring( b,p ) ) ;
            buf.append( "&lt;" ).append( str.substring( p+2,x ) ).append( "&gt;" ) ;
            b = x + 2 ;
        }
        return buf.toString() ;
    }
    
    /** ファイル名から、ファイルの名前のみを取得. **/
    private static final String getFileName( String name ) {
        name = FileUtil.getFileName( name ) ;
        if( name.endsWith( ".sdoc" ) ) {
            name = name.substring( 0,name.length()-5 ) ;
        }
        return name ;
    }
    
    /** ディレクトリ名を取得. **/
    
    /** test. **/
    public static final void main( String[] args ) throws Exception {
        SeaBassDoc doc = analysis( "c:\\control.sdoc","UTF8" ) ;
        System.out.println( doc ) ;
    }
}
