package org.maachang.seabass.httpd.engine.script.doc ;

import java.util.ArrayList;
import java.util.List;

/**
 * SeaBassドキュメント[カテゴリ]Bean.
 * 
 * @version 2009/01/22
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class SeaBassDocCategory implements SeaBassDocBase {
    private List<SeaBassDocTarget> taglibs = null ;
    private String name = null ;
    private String info = null ;
    private String fileName = null ;
    
    /**
     * コンストラクタ.
     */
    public SeaBassDocCategory() {
        this.taglibs = new ArrayList<SeaBassDocTarget>() ;
    }
    
    protected void finalize() throws Exception {
        destroy() ;
    }
    
    /**
     * オブジェクト破棄.
     */
    public void destroy() {
        taglibs = null ;
        name = null ;
        info = null ;
        fileName = null ;
    }
    
    /**
     * １つのターゲットオブジェクトを追加.
     * @param tagLib 対象のターゲットオブジェクトを設定します.
     */
    public void add( SeaBassDocTarget tagLib ) {
        if( tagLib == null ) {
            return ;
        }
        taglibs.add( tagLib ) ;
    }
    
    /**
     * 1つのターゲットオブジェクトを取得.
     * @param no 取得対象の項番を設定します.
     * @return SeaBassDocTarget ターゲットオブジェクトが返されます.
     */
    public SeaBassDocTarget get( int no ) {
        if( no >= 0 && no < taglibs.size() ) {
            return taglibs.get( no ) ;
        }
        return null ;
    }
    
    /**
     * 全てのターゲットオブジェクトを破棄.
     */
    public void removeAll() {
        taglibs.clear() ;
    }
    
    /**
     * ターゲットオブジェクト数を取得.
     * @return int size ターゲットオブジェクト数が返されます.
     */
    public int size() {
        return taglibs.size() ;
    }
    
    /**
     * 名前を設定.
     * @param name 名前を設定します.
     */
    public void setName( String name ) {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "カテゴリ名が設定されていません" ) ;
        }
        this.name = name ;
    }
    
    /**
     * 名前を取得.
     * @return String 名前が返されます.
     */
    public String getName() {
        return name ;
    }
    
    /**
     * 説明情報を設定.
     * @param info 説明情報を設定します.
     */
    public void setInfo( String info ) {
        if( info == null || ( info = info.trim() ).length() <= 0 ) {
            info = "" ;
        }
        this.info = info ;
    }
    
    /**
     * 説明情報を取得.
     * @return String 説明情報が返されます.
     */
    public String getInfo() {
        return info ;
    }
    
    /**
     * ファイル名を設定.
     * @param fileName 対象のファイル名を設定します.
     */
    public void setFileName( String fileName ) {
        this.fileName = fileName ;
    }
    
    /**
     * ファイル名を取得.
     * @return String ファイル名を取得します.
     */
    public String getFileName() {
        return fileName ;
    }
    
    public String toString() {
        StringBuilder buf = new StringBuilder() ;
        buf.append( " [@category] {" ) ;
        buf.append( " fileName:" ).append( fileName ).
            append( " name:" ).append( name ).append( " info:" + info ) ;
        int len = taglibs.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            buf.append( taglibs.get( i ).toString() ) ;
        }
        return buf.append( "}" ).toString() ;
    }
}
