package org.maachang.seabass.httpd.engine.script.doc ;

import java.util.ArrayList;
import java.util.List;

/**
 * SeaBassドキュメント[ターゲット]Bean.
 * 
 * @version 2009/01/22
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class SeaBassDocTarget implements SeaBassDocBase {
    private List<SeaBassDocParam> params = null ;
    private String name = null ;
    private String info = null ;
    private String sample = null ;
    
    /**
     * コンストラクタ.
     */
    public SeaBassDocTarget() {
        this.params = new ArrayList<SeaBassDocParam>() ;
    }
    
    protected void finalize() throws Exception {
        destroy() ;
    }
    
    /**
     * オブジェクト破棄.
     */
    public void destroy() {
        params = null ;
        name = null ;
        info = null ;
        sample = null ;
    }
    
    /**
     * １つのパラメータ情報を追加.
     * @param param 対象のパラメータ情報を設定します.
     */
    public void add( SeaBassDocParam param ) {
        if( param == null ) {
            return ;
        }
        params.add( param ) ;
    }
    
    /**
     * 1つのパラメータ情報を取得.
     * @param no 取得対象の項番を設定します.
     * @return SeaBassDocParam パラメータ情報が返されます.
     */
    public SeaBassDocParam get( int no ) {
        if( no >= 0 && no < params.size() ) {
            return params.get( no ) ;
        }
        return null ;
    }
    
    /**
     * 全てのパラメータオブジェクトを破棄.
     */
    public void removeAll() {
        params.clear() ;
    }
    
    /**
     * パラメータ情報数を取得.
     * @return int size パラメータ情報数が返されます.
     */
    public int size() {
        return params.size() ;
    }
    
    /**
     * 名前を設定.
     * @param name 名前を設定します.
     */
    public void setName( String name ) {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "カテゴリ名が設定されていません" ) ;
        }
        this.name = name ;
    }
    
    /**
     * 名前を取得.
     * @return String 名前が返されます.
     */
    public String getName() {
        return name ;
    }
    
    /**
     * 説明情報を設定.
     * @param info 説明情報を設定します.
     */
    public void setInfo( String info ) {
        if( info == null || ( info = info.trim() ).length() <= 0 ) {
            info = "" ;
        }
        this.info = info ;
    }
    
    /**
     * 説明情報を取得.
     * @return String 説明情報が返されます.
     */
    public String getInfo() {
        return info ;
    }
    
    /**
     * サンプル情報を設定.
     * @param sample サンプル情報を設定します.
     */
    public void setSample( String sample ) {
        if( sample == null || ( sample = sample.trim() ).length() <= 0 ) {
            sample = "" ;
        }
        this.sample = sample ;
    }
    
    /**
     * サンプル情報を取得.
     * @return String サンプル情報が返されます.
     */
    public String getSample() {
        return sample ;
    }
    
    public String toString() {
        StringBuilder buf = new StringBuilder() ;
        buf.append( " [@target] {" ) ;
        buf.append( " name:" ).append( name ).append( " info:" + info ).
            append( " sample:" ).append( sample ) ;
        int len = params.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            buf.append( params.get( i ).toString() ) ;
        }
        return buf.append( "}" ).toString() ;
    }
}
