package org.maachang.seabass.httpd.engine.script.image;

import java.io.ByteArrayInputStream;

import org.maachang.util.image.BaseImageConvertor;
import org.maachang.util.image.ImageConvertor;
import org.maachang.util.image.ImageInfo;

/**
 * イメージユーティリティ
 * 
 * @version 2008/01/27
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public abstract class ImageUtil {
    
    private ImageUtil() {
        
    }
    
    /**
     * 画像タイプを取得.
     * <BR><BR>
     * 指定バイナリを解析して、画像タイプを取得します.
     * <BR>
     * @param binary 対象のバイナリを設定します.
     * @return String 画像タイプが返されます.
     * @exception Exception 例外.
     */
    public static final String getImageCodec( byte[] binary )
        throws Exception {
        return BaseImageConvertor.getImageFormat( new ByteArrayInputStream( binary ) ) ;
    }
    
    /**
     * 画像解像度を取得.
     * <BR><BR>
     * 画像解像度が返されます.
     * <BR>
     * @param binary 対象のバイナリを設定します.
     * @return int[] 画像解像度が返されます.
     * @exception Exception 例外.
     */
    public static final int[] getImageWidthHeight( byte[] binary )
        throws Exception {
        ImageInfo img = ImageConvertor.loadImage( new ByteArrayInputStream( binary ) ) ;
        return new int[]{ img.getWidth(),img.getHeight() } ;
    }
}
