package org.maachang.seabass.httpd.engine.script.js;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.maachang.manager.GlobalManager;
import org.maachang.seabass.ServiceDef;
import org.maachang.seabass.conf.PackagesConfig;
import org.maachang.seabass.httpd.HttpdBinary;
import org.maachang.seabass.httpd.HttpdParams;
import org.maachang.seabass.httpd.HttpdResponse;
import org.maachang.seabass.httpd.engine.HttpdPublicManager;
import org.maachang.seabass.httpd.engine.HttpdUtil;
import org.maachang.seabass.net.HttpdParamsImpl;
import org.maachang.util.FileUtil;
import org.maachang.util.StringUtil;
import org.maachang.util.ZenToHan;

/**
 * 内部JS関連定義.
 * 
 * @version 2007/08/24
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class JsDef {
    private JsDef() {
    }
    
    /**
     * JS定義名 : リミットオプション.
     */
    public static final String[] JS_OPTION = { "$limit","$offset","$order" } ;
    
    /**
     * 指定InputStreamをHttpdResponseに出力.
     * <BR><BR>
     * 指定InputStreamをHttpdResponseに出力します.
     * <BR>
     * @param response 対象のレスポンスを設定します.
     * @parm data 対象のInputStreamを設定します.
     * @exception Exception 例外.
     */
    public static final void putResponseByData( HttpdResponse response,InputStream data )
        throws Exception {
        if( response == null || data == null ) {
            if( data != null ) {
                try {
                    data.close() ;
                } catch( Exception e ) {
                }
            }
            return ;
        }
        BufferedInputStream bi = null ;
        if( ( data instanceof BufferedInputStream ) == false ) {
            bi = new BufferedInputStream( data ) ;
        }
        else {
            bi = ( BufferedInputStream )data ;
        }
        OutputStream output = null ;
        try {
            output = response.getOutput() ;
            byte[] bin = new byte[ 512 ] ;
            for( ;; ) {
                int len = bi.read( bin ) ;
                if( len <= 0 ) {
                    if( len <= -1 ) {
                        break ;
                    }
                    continue ;
                }
                output.write( bin,0,len ) ;
            }
            bi.close() ;
            bi = null ;
        } catch( Exception e ) {
            throw e ;
        } finally {
            if( bi != null ) {
                try {
                    bi.close() ;
                } catch( Exception e ) {
                }
            }
        }
    }
    
    /**
     * バイナリ内容を表示.
     * <BR><BR>
     * バイナリ内容を表示します.
     * <BR>
     * @param binary 対象のバイナリ情報を設定します.
     * @return String 表示対象内容が返されます.
     */
    public static final String toBinaryString( byte[] binary ) {
        if( binary == null || binary.length <= 0 ) {
            return "" ;
        }
        StringBuilder buf = new StringBuilder() ;
        int len = binary.length ;
        buf.append( "[" ) ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( i != 0 ) {
                buf.append( "," ) ;
            }
            String c = Integer.toHexString( ( binary[ i ] & 0x000000ff ) ) ;
            if( c.length() == 1 ) {
                c = "0" + c ;
            }
            buf.append( "0x" ).append( c ) ;
        }
        buf.append( "]" ) ;
        return buf.toString() ;
    }
    
    /**
     * スクリプトデフォルトパッケージIDを取得.
     * <BR><BR>
     * スクリプトデフォルトパッケージIDを取得します.
     * <BR>
     * @return int スクリプトデフォルトパッケージIDが返されます.
     */
    public static final int getDefaultPackageId() {
        PackagesConfig pconf = ( PackagesConfig )GlobalManager.getValue(
            ServiceDef.MANAGER_BY_PACKAGES ) ;
        if( pconf == null ) {
            return -1 ;
        }
        return pconf.getUpdateId() ;
    }
    
    /**
     * パッケージIDを取得.
     * @return int パッケージIDが返されます.
     */
    public static final int getPackageId() {
        PackagesConfig pconf = ( PackagesConfig )GlobalManager.getValue(
            ServiceDef.MANAGER_BY_PACKAGES ) ;
        if( pconf == null ) {
            return -1 ;
        }
        return pconf.getUpdateId() ;
    }
    
    /**
     * スクリプトデフォルトパッケージを取得.
     * <BR><BR>
     * スクリプトデフォルトパッケージを取得します.
     * <BR>
     * @param buf 追加対象のStringBuilderを設定します.
     * @return int スクリプトデフォルトパッケージ更新IDが返されます.
     * @exception Exception 例外.
     */
    public static final int pushDefaultPackage( StringBuilder buf )
        throws Exception {
        PackagesConfig pconf = ( PackagesConfig )GlobalManager.getValue(
            ServiceDef.MANAGER_BY_PACKAGES ) ;
        if( pconf == null ) {
            return -1 ;
        }
        int updateId = pconf.getUpdateId() ;
        String[] pk = pconf.getPackages() ;
        if( pk != null && pk.length > 0 ) {
            int len = pk.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( pk[i] != null ) {
                    buf.append( pk[i] ) ;
                    if( pk[i].endsWith( ";" ) == false ) {
                        buf.append( " ;" ) ;
                    }
                    buf.append( "\n" ) ;
                }
            }
        }
        return updateId ;
    }
    
    /**
     * スクリプトデフォルトパッケージを取得.
     * <BR><BR>
     * スクリプトデフォルトパッケージを取得します.
     * <BR>
     * @param outLine 設定改行数が返されます.
     * @param buf 追加対象のStringBuilderを設定します.
     * @return int スクリプトデフォルトパッケージ更新IDが返されます.
     * @exception Exception 例外.
     */
    public static final int pushDefaultPackage( int[] outLine,StringBuilder buf )
        throws Exception {
        if( outLine == null || outLine.length <= 0 ) {
            return pushDefaultPackage( buf ) ;
        }
        outLine[ 0 ] = 0 ;
        PackagesConfig pconf = ( PackagesConfig )GlobalManager.getValue(
            ServiceDef.MANAGER_BY_PACKAGES ) ;
        if( pconf == null ) {
            return -1 ;
        }
        int updateId = pconf.getUpdateId() ;
        String[] pk = pconf.getPackages() ;
        if( pk != null && pk.length > 0 ) {
            int len = pk.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( pk[i] != null ) {
                    buf.append( pk[i] ).
                        append( "\n" ) ;
                    outLine[ 0 ] ++ ;
                }
            }
        }
        return updateId ;
    }
    
    /**
     * SimpleDateFormatで、日付文字列を解析.
     * <BR>
     * @param format 対象のフォーマットを設定します.
     * @param value 変換する文字列を設定します.
     * @return long 変換された数値情報が返されます.
     * @exception Exception 例外.
     */
    public static final long convertDateFormatByString( String format,String value )
        throws Exception {
        return new SimpleDateFormat( format,Locale.US ).parse( value ).getTime() ;
    }
    
    /**
     * SimpleDateFormatで、指定数値を文字列変換.
     * <BR>
     * @param format 対象のフォーマットを設定します.
     * @param value 変換する数値を設定します.
     * @return String 変換された文字列情報が返されます.
     * @exception Exception 例外.
     */
    public static final String convertDateFormatByLong( String format,long value )
        throws Exception {
        return new SimpleDateFormat( format,Locale.US ).format( new Date( value ) ) ;
    }
    
    /**
     * HTMLに表示可能な形式に変換.
     * <BR><BR>
     * @param mode [true]の場合、改行情報を&lt;br&gt;に変換します.
     * @param o 対象の情報を設定します.
     * @return String 変換された内容が返されます.
     */
    public static final String convertByViewHtml( boolean mode,String o ) {
        if( o == null || o.length() <= 0 ) {
            return "" ;
        }
        o = StringUtil.changeString(o, "&", "&amp;");
        o = StringUtil.changeString(o, "<", "&lt;");
        o = StringUtil.changeString(o, ">", "&gt;");
        o = StringUtil.changeString(o, "\'", "&#039;");
        o = StringUtil.changeString(o, "\"", "&#034;");
        if( mode == true ) {
            o = StringUtil.changeString(o, "\n", "<br>");
        }
        o = StringUtil.changeString(o, " ", "&nbsp;");
        return o ;
    }
    
    /**
     * HTMLに表示可能な形式に変換.
     * <BR><BR>
     * @param mode [true]の場合、改行情報を&lt;br&gt;に変換します.
     * @param o 対象の情報を設定します.
     * @return String 変換された内容が返されます.
     */
    public static final String convertHtmlByCode( boolean mode,String o ) {
        if( o == null || o.length() <= 0 ) {
            return "" ;
        }
        o = StringUtil.changeString(o, "&amp;", "&");
        o = StringUtil.changeString(o, "&lt;", "<");
        o = StringUtil.changeString(o, "&gt;", ">");
        o = StringUtil.changeString(o, "&#039;", "\'" );
        o = StringUtil.changeString(o, "&#034;", "\"");
        if( mode == true ) {
            o = StringUtil.changeString(o, "<br>", "\n");
        }
        o = StringUtil.changeString(o, "&nbsp;", " ");
        return o ;
    }
    
    /**
     * HTMLに表示可能な形式に変換(コーテーション範囲外の条件のみを変換).
     * <BR><BR>
     * @param mode [true]の場合、改行情報を&lt;br&gt;に変換します.
     * @param o 対象の情報を設定します.
     * @return String 変換された内容が返されます.
     * @exception Exception 例外.
     */
    public static final String convertByViewHtmlNotCode( boolean mode,String o )
        throws Exception {
        if( o == null || o.length() <= 0 ) {
            return "" ;
        }
        o = StringUtil.changeStringByNoCote(o, "&", "&amp;");
        o = StringUtil.changeStringByNoCote(o, "<", "&lt;");
        o = StringUtil.changeStringByNoCote(o, ">", "&gt;");
        o = StringUtil.changeStringByNoCote(o, "\'", "&#039;");
        o = StringUtil.changeStringByNoCote(o, "\"", "&#034;");
        if( mode == true ) {
            o = StringUtil.changeStringByNoCote(o, "\n", "<br>");
        }
        o = StringUtil.changeStringByNoCote(o, " ", "&nbsp;");
        return o ;
    }
    
    /**
     * HTMLに表示可能な形式に変換(コーテーション範囲外の条件のみを変換).
     * <BR><BR>
     * @param mode [true]の場合、改行情報を&lt;br&gt;に変換します.
     * @param o 対象の情報を設定します.
     * @return String 変換された内容が返されます.
     * @exception Exception 例外.
     */
    public static final String convertHtmlByCodeNotCode( boolean mode,String o )
        throws Exception {
        if( o == null || o.length() <= 0 ) {
            return "" ;
        }
        o = StringUtil.changeStringByNoCote(o, "&amp;", "&");
        o = StringUtil.changeStringByNoCote(o, "&lt;", "<");
        o = StringUtil.changeStringByNoCote(o, "&gt;", ">");
        o = StringUtil.changeStringByNoCote(o, "&#039;", "\'" );
        o = StringUtil.changeStringByNoCote(o, "&#034;", "\"");
        if( mode == true ) {
            o = StringUtil.changeStringByNoCote(o, "<br>", "\n");
        }
        o = StringUtil.changeStringByNoCote(o, "&nbsp;", " ");
        return o ;
    }
    
    /**
     * JSON形式変換.
     * <BR><BR>
     * @param o 対象の情報を設定します.
     * @return String 変換された内容が返されます.
     */
    public static final String convertByJson( String o ) {
        if( o == null || o.length() <= 0 ) {
            return "" ;
        }
        o = StringUtil.upIndentDoubleCote( o ) ;
        o = StringUtil.upIndentSingleCote( o ) ;
        o = StringUtil.changeString(o, "\r", "\\r");
        o = StringUtil.changeString(o, "\n", "\\n");
        o = StringUtil.changeString(o, "\t", "\\t");
        return o ;
    }
    
    /**
     * 文字列内コーテーション変換.
     * @param mode [true]の場合、ダブルコーテーションを変換します.
     * @param o 対象の文字列を設定します.
     * @return String 変換文字列が返されます.
     */
    public static final String convetByCote( boolean mode,String o ) {
        return StringUtil.indentCote( o,0,mode ) ;
        /*
        if( o == null || o.length() <= 0 ) {
            return "" ;
        }
        int bafCode = -1 ;
        char target = ( mode == false ) ? '\'' : '\"' ;
        int len = o.length() ;
        StringBuilder buf = new StringBuilder() ;
        for( int i = 0 ; i < len ; i ++ ) {
            char c = o.charAt( i ) ;
            if( target == c ) {
                if( bafCode == '\\' ) {
                    buf.append( "\\\\" ) ;
                }
                else {
                    buf.append( "\\" ) ;
                }
            }
            buf.append( c ) ;
            bafCode = c ;
        }
        return buf.toString() ;
        */
    }
    
    /**
     * 改行を文字列変換する.
     * @param o 対象の情報を設定します.
     * @return String 変換された内容が返されます.
     */
    public static final String convertByEnter( String o ) {
        if( o == null || o.length() <= 0 ) {
            return "" ;
        }
        o = StringUtil.changeString(o, "\r", "\\r");
        return StringUtil.changeString(o, "\n", "\\n");
    }
    
    /**
     * 指定内容をバイナリで取得.
     * <BR><BR>
     * 指定内容をバイナリで取得します.
     * <BR>
     * @param o 対象のオブジェクトを設定します.
     * @return byte[] 対象のバイナリ情報が返されます.
     */
    public static final byte[] getParamByBinary( Object o ) {
        if( o == null || ( o instanceof HttpdBinary ) == false ) {
            return null ;
        }
        return ( ( HttpdBinary )o ).getBinary() ;
    }
    
    /**
     * 指定内容をバイナリサイズで取得.
     * <BR><BR>
     * 指定内容をバイナリサイズで取得します.
     * <BR>
     * @param o 対象のオブジェクトを設定します.
     * @return int 対象のバイナリサイズが返されます.
     */
    public static final int getParamByBinaryLength( Object o ) {
        if( o == null || ( o instanceof HttpdBinary ) == false ) {
            return -1 ;
        }
        return ( ( HttpdBinary )o ).getLength() ;
    }
    
    /**
     * 指定内容のオリジナル名を取得.
     * <BR><BR>
     * 指定内容のオリジナル名を取得します.
     * <BR>
     * @param o 対象のオブジェクトを設定します.
     * @return String 対象のオリジナル名が返されます.
     */
    public static final String getParamByOriginalName( Object o ) {
        if( o == null || ( o instanceof HttpdBinary ) == false ) {
            return null ;
        }
        return ( ( HttpdBinary )o ).getOriginalName() ;
    }
    
    /**
     * 指定パラメータをエンコード/デコード.
     * @param mode [true]の場合、エンコードします.
     * @param body 対象のBody情報を設定します.
     * @param charset 対象のcharsetを設定します.
     * @return String 変換された内容が返されます.
     */
    public static final String convertURL( boolean mode,String body,String charset ) {
        if( body == null || ( body = body.trim() ).length() <= 0 ) {
            return null ;
        }
        if( charset == null || ( charset = charset.trim() ).length() <= 0 ) {
            charset = "UTF8" ;
        }
        try {
            HttpdParams params = new HttpdParamsImpl() ;
            HttpdUtil.convertQueryByParams( params,body,charset ) ;
            StringBuilder buf = new StringBuilder() ;
            String[] keys = params.getKeys() ;
            if( keys != null ) {
                int len = keys.length ;
                int cnt = 0 ;
                for( int i = 0 ; i < len ; i ++ ) {
                    int lenJ = params.size( keys[ i ] ) ;
                    for( int j = 0 ; j < lenJ ; j ++ ) {
                        if( cnt != 0 ) {
                            buf.append( "&" ) ;
                        }
                        String v = ( String )params.getParam( keys[ i ],j ) ;
                        if( v == null || ( v = v.trim() ).length() <= 0 ) {
                            buf.append( keys[ i ] ).append( "=" ) ;
                        }
                        else if( mode == true ) {
                            buf.append( keys[ i ] ).append( "=" ).
                                append( URLEncoder.encode( v,charset ) ) ;
                        }
                        else if( mode == true ) {
                            buf.append( keys[ i ] ).append( "=" ).
                                append( v ) ;
                        }
                        cnt ++ ;
                    }
                }
                return buf.toString() ;
            }
        } catch( Exception e ) {
        }
        return null ;
    }
    
    /**
     * 指定文字列内に有効な文字列が存在するかチェック.
     * @param str 対象の文字列を設定します.
     * @return boolean [true]の場合、有効な文字列は存在します.
     */
    public static final boolean isUseString( String str ) {
        if( str == null || str.length() <= 0 ) {
            return false ;
        }
        int len = str.length() ;
        for( int i = 0 ; i < len ; i ++ ) {
            char c = str.charAt( i ) ;
            //if( c != ' ' && c != '\t' && c != '\r' && c != '\n' ) {
            if( c != ' ' && c != '\t' && c != '　' ) {
                return true ;
            }
        }
        return false ;
    }
    
    /**
     * 静的ページ内容を取得.
     * @param path 対象のパス名を設定します.
     * @return String 静的ページ内容が返されます.
     * @exception Exception 例外.
     */
    public static final String readStaticHtml( String path )
        throws Exception {
        String p = HttpdPublicManager.getUsePath( null,path ) ;
        if( p == null ) {
            return "" ;
        }
        return FileUtil.getFileByString( p,"UTF8" ) ;
    }
    
    /**
     * 指定文字列のうち、条件に従い文字を切り分ける.
     * @param han [true]の場合は、特定文字を半角に変換します.
     * @param mode [true]の場合は、改行で折り返し、[false]の場合は、削除する.
     * @param p プロポーショナルフォントの処理では、[true]を設定します.
     * @param string 対象の文字列を設定します.
     * @param length 区切り対象の文字列を設定します.
     */
    public static final String viewString( boolean han,boolean mode,boolean p,String string,int length ) {
        if( length <= 0 ) {
            length = 2 ;
        }
        string = StringUtil.changeString( string,"\r","" ) ;
        string = StringUtil.changeString( string,"\t","    " ) ;
        string = convertHtmlByCode( true,string ) ;
        if( han ) {
            string = ZenToHan.convertHankaku( false,string ) ;
        }
        StringBuilder buf = new StringBuilder() ;
        int len = string.length() ;
        length = length * 10 ;
        int cnt = 0 ;
        for( int i = 0 ; i < len ; i ++ ) {
            char c = string.charAt( i ) ;
            if( mode && c == '\n' ) {
                cnt = 0 ;
                buf.append( "\n" ) ;
                continue ;
            }
            if( length <= cnt ) {
                if( mode ) {
                    buf.append( "\n" ) ;
                }
                else {
                    if( buf.length() > 0 ) {
                        String x = buf.toString() ;
                        buf = new StringBuilder() ;
                        buf.append( x.substring( 0,x.length()-1 ) ) ;
                        x = null ;
                    }
                    buf.append( "…" ) ;
                    break ;
                }
                cnt = 0 ;
            }
            buf.append( c ) ;
            if( c <= 255 ) {
                if( p ) {
                    if( ( c >= 'A' && c <= 'Z' ) ) {
                        cnt += 14 ;
                    }
                    else if( ".,:;[](){}".indexOf( c ) != -1 ) {
                        cnt += 8 ;
                    }
                    else {
                        cnt += 10 ;
                    }
                }
                else {
                    cnt += 10 ;
                }
            }
            else if( "ｧｱｨｲｩｳｪｴｫｵｶｷｸｹｺｻｼｽｾｿﾀﾁｯﾂﾃﾄﾅﾆﾇﾈﾉﾊﾋﾌﾍﾎﾏﾐﾑﾒﾓｬﾔｭﾕｮﾖﾗﾘﾙﾚﾛヮﾜｦﾝｰ".indexOf( c ) != -1 ) {
                if( p ) {
                    cnt += 14 ;
                }
                else {
                    cnt += 10 ;
                }
            }
            else if( "ﾞﾟ｡｢｣､･".indexOf( c ) != -1 ) {
                if( p ) {
                    cnt += 8 ;
                }
                else {
                    cnt += 10 ;
                }
            }
            else {
                cnt += 20 ;
            }
        }
        return convertByViewHtml( true,buf.toString() ) ;
    }
    
    /**
     * 全角条件の内容を全て半角変換.
     * @param string 半角変換対象の文字列を設定します.
     * @return String 変換された文字列が返されます.
     */
    public static final String convertHankaku( String string ) {
        return ZenToHan.convertHankaku( string ) ;
    }
    
    /**
     * 全角条件の内容を全て半角変換.
     * @param string 半角変換対象の文字列を設定します.
     * @param mode [html]型式に変換する場合は、trueを指定.
     * @return String 変換された文字列が返されます.
     */
    public static final String convertHankaku( String string,boolean mode ) {
        return ZenToHan.convertHankaku( mode,string ) ;
    }
    
    /**
     * カナ全角を半角変換.
     * @param string 対象の文字列を設定します.
     * @return String 変換された文字列が返されます.
     */
    public static final String kanaHankaku( String string ) {
        return ZenToHan.convertKanaHan( string ) ;
    }
    
    /**
     * カナ半角を全角変換.
     * @param string 対象の文字列を設定します.
     * @return String 変換された文字列が返されます.
     */
    public static final String kanaZenkaku( String string ) {
        return ZenToHan.convertKanaZen( string ) ;
    }
}

