// 帳票出力処理系.

////////////////////////////////////////////////////////////////////////////////
// Apache POIを利用したレポートフレームワーク.
////////////////////////////////////////////////////////////////////////////////
Report = {
    // 帳票出力用フレームワーク.
    prototype : {
        // 帳票オブジェクト.
        engine : null,
        // レスポンス処理終了フラグ.
        resultFlag: false,
        // コンストラクタ.
        _constructor : function( name ) {
            if( useString( name ) ) {
                this.create( name ) ;
            }
        },
        // 情報生成.
        create : function( name ) {
            if( !useString( name ) ) {
                return -1 ;
            }
            var conf = null ;
            name = changeString( name,"\\","/" ) ;
            if( !name.endsWith( ".conf" ) ) {
                name += ".conf" ;
            }
            if( Io.isFileExists( name ) ) {
                conf = pkg( "org.maachang.conf.Config" ).read( name ) ;
            }
            else {
                return -1 ;
            }
            var excelName = name.substring( 0,name.length() - ".conf".length() ) + ".xls" ;
            if( Io.isFileExists( name ) && Io.isFileExists( excelName ) ) {
                var binary = Io.readFileByBinary( excelName ) ;
                this.engine = new org.maachang.report.ReportEngine( name,binary,conf ) ;
                binary = null ;
                return 0 ;
            }
            return -1 ;
        },
        // 帳票エンジンを削除.
        destroy : function() {
            if( isNull( this.engine ) == false ) {
                this.engine.destroy() ;
            }
            this.engine = null ;
        },
        // 現在アクティブなシートを取得.
        activeSheetName : function() {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return null ;
            }
            return this.engine.getActiveSheetName() ;
        },
        // シートコピー.
        copySheet : function( src,dest ) {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return false ;
            }
            if( !useString( src ) || !useString( dest ) ) {
                return false ;
            }
            return this.engine.copySheet( src,dest ) ;
        },
        // シート削除.
        removeSheet : function( name ) {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return false ;
            }
            if( !useString( name ) ) {
                return false ;
            }
            return this.engine.removeSheet( name ) ;
        },
        // シートチェンジ.
        changeSheet : function( name ) {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return false ;
            }
            if( !useString( name ) ) {
                return false ;
            }
            return this.engine.changeSheet( name ) ;
        },
        // テンプレート削除.
        removeTemplate : function( name ) {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return false ;
            }
            this.engine.removeTemplate() ;
        },
        // リスト帳票改行処理.
        enter : function() {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return -1 ;
            }
            this.engine.nextList() ;
            return 0 ;
        },
        // 変数名を指定して情報追加.
        put : function( name,value ) {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return -1 ;
            }
            if( useString( name ) == false ) {
                return -1 ;
            }
            if( isNull( value ) ) {
                value = "" ;
            }
            var t = typeof( value ) ;
            if( t == "string" || t == "number" || t == "date" ) {
                this.engine.putValue( name,value ) ;
                return 0 ;
            }
            return -1 ;
        },
        // 指定変数名が存在するかチェック.
        containsKey : function( name ) {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return false ;
            }
            if( useString( name ) == false ) {
                return false ;
            }
            return this.engine.containsKey( name ) ;
        },
        // データ群をセット.
        push : function( value ) {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return -1 ;
            }
            if( isNull( value ) ) {
                return -1 ;
            }
            var t = typeof( value ) ;
            if( t == "map" ) {
                for( var k in value ) {
                    if( this.engine.containsKey( k ) ) {
                        this.put( k,value[k] ) ;
                    }
                }
                return 0 ;
            }
            else if( t == "array" ) {
                var len = value.length ;
                var cnt = 0 ;
                for( var i = 0 ; i < len ; i ++ ) {
                    var o = value[ i ] ;
                    if( typeof( o ) == "map" ) {
                        if( cnt != 0 ) {
                            this.enter() ;
                        }
                        for( var k in o ) {
                            if( this.engine.containsKey( k ) ) {
                                this.put( k,o[k] ) ;
                                cnt ++ ;
                            }
                        }
                    }
                }
                return 0 ;
            }
            return -1 ;
        },
        // レスポンスに帳票データを出力.
        flush : function( name ) {
            if( this.resultFlag ) {
                return 0 ;
            }
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return -1 ;
            }
            if( isResponse() == false ) {
                return -1 ;
            }
            if( getHttpdResponse().isOutputStream() == true ) {
                return -1 ;
            }
            if( useString( name ) == false ) {
                name = this.engine.getOutName() ;
            }
            if( endsWith( name,".xls" ) == false && endsWith( name,".xrpt" ) == false ) {
                name += ".xls" ;
            }
            setDownloadContent( name ) ;
            responseToBinary( this.engine.getBytes(),"application/vnd.ms-excel" ) ;
            this.resultFlag = true ;
            return 0 ;
        },
        // XReportの場合は、こちらで出力.
        flushXRPT : function() {
            if( this.resultFlag ) {
                return 0 ;
            }
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return -1 ;
            }
            if( isResponse() == false ) {
                return -1 ;
            }
            if( getHttpdResponse().isOutputStream() == true ) {
                return -1 ;
            }
            responseToBinary( this.engine.getBytes(),"application/rpt.ms-excel" ) ;
            this.resultFlag = true ;
            return 0 ;
        },
        // ファイル出力.
        writeFile : function( name ) {
            if( isNull( this.engine ) || this.engine.isUse() == false ) {
                return -1 ;
            }
            if( useString( name ) == false ) {
                return -1 ;
            }
            this.engine.output( name ) ;
            return 0 ;
        },
        // レスポンス出力状態を取得.
        isOutputResponseFlag : function() {
            return this.resultFlag ;
        },
        // ReportEngineを取得.
        getEngine : function() {
            return this.engine ;
        }
    }
} ;

////////////////////////////////////////////////////////////////////////////////
// Apache POI用処理.
////////////////////////////////////////////////////////////////////////////////
POI = {
    // excel読み込み用(File指定).
    readExcel : function( name,sheetName ) {
        if( useString( name ) == false ) {
            return null ;
        }
        name = changeString( name,"\\","/" ) ;
        if( Io.isFileExists( name ) ) {
            //name = toFullPath( name ) ;
            var binary = Io.readFileByBinary( name ) ;
            var ret = pkg( "org.maachang.report.ReadExcelSheet" ).readBinary( binary,sheetName ) ;
            binary = null ;
            return ret ;
        }
        return null ;
    },
    // excel読み込み用(Binary指定).
    readExcelBinary : function( binary,sheetName ) {
        if( isNull( binary ) ) {
            return null ;
        }
        var ret = pkg( "org.maachang.report.ReadExcelSheet" ).readBinary( binary,sheetName ) ;
        binary = null ;
        return ret ;
    },
    // excel読み書き用(File指定).
    writeExcel : function( name,sheetName ) {
        if( useString( name ) == false ) {
            return null ;
        }
        name = changeString( name,"\\","/" ) ;
        if( Io.isFileExists( name ) ) {
            //name = toFullPath( name ) ;
            var binary = Io.readFileByBinary( name ) ;
            var ret = pkg( "org.maachang.report.WriteExcelSheet" ).readBinary( binary,sheetName ) ;
            binary = null ;
            return ret ;
        }
        return null ;
    },
    // excel読み書き用(Binary指定).
    writeExcelBinary : function( binary,sheetName ) {
        if( isNull( binary ) ) {
            return null ;
        }
        var ret = pkg( "org.maachang.report.WriteExcelSheet" ).readBinary( binary,sheetName ) ;
        binary = null ;
        return ret ;
    }
}

