package org.maachang.seabass.httpd.engine.script.scripts;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.SeaBassContext;
import org.maachang.seabass.SeaBassDef;
import org.maachang.seabass.SeaBassScriptEngine;
import org.maachang.seabass.httpd.engine.script.BaseModel;
import org.maachang.seabass.httpd.engine.script.BaseModelImpl;
import org.maachang.seabass.httpd.engine.script.EndScript;
import org.maachang.seabass.httpd.engine.script.ScriptDef;
import org.maachang.seabass.httpd.engine.script.cache.ApiManager;
import org.maachang.seabass.httpd.engine.script.cache.CacheScriptManager;
import org.maachang.seabass.httpd.engine.script.scripts.IncludeScript;
import org.maachang.seabass.httpd.engine.script.scripts.ScriptManager;
import org.maachang.util.FileUtil;
import org.maachang.util.StringUtil;

/**
 * クーロン(スケジューリング実行)スクリプトを実行.
 * 
 * @version 2008/07/01
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class CronScript {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( CronScript.class ) ;
    
    /**
     * スクリプトエンジン名.
     */
    protected static final String ENGINE_NAME = "js" ;
    
    /**
     * スクリプト詳細名.
     */
    protected static final String DETAIL_NAME = "cron" ;
    
    /**
     * スタートアップ名.
     */
    protected static final String CRON_NAME = "Cron" ;
    
    /**
     * コンストラクタ.
     */
    private CronScript() {
        
    }
    
    /**
     * 指定Cronスクリプトを実行.
     * @param name 実行スクリプト名を設定します.
     * @exception Exception 例外.
     */
    public static final void execution( String name ) throws Exception {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        if( name.endsWith( ScriptManager.SCRIPT_PLUS ) ) {
            name = name.substring( 0,name.length() - ScriptManager.SCRIPT_PLUS.length() ) ;
        }
        if( name.endsWith( CRON_NAME ) ) {
            name = name.substring( 0,name.length() - CRON_NAME.length() ) ;
        }
        executionScript( ApiManager.getInstance().getScriptEngine(),name+CRON_NAME+ScriptManager.SCRIPT_PLUS ) ;
    }
    
    /**
     * １つの指定スクリプトを実行.
     */
    private static final boolean executionScript( SeaBassScriptEngine engine,String name )
        throws Exception {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            return false ;
        }
        if( name.startsWith( "/" ) || name.startsWith( "\\" ) ) {
            name = name.substring( 1 ) ;
        }
        name = StringUtil.changeString( name,"\\","/" ) ;
        name = "/" + SeaBassDef.DIRECTORY_CRON + name ;
        String file = FileUtil.getFullPath( "."+name ) ;
        if( FileUtil.isFileExists( file ) == false || FileUtil.isRead( file ) == false ) {
            return false ;
        }
        //String script = FileUtil.getFileByString( file,"UTF8" ) ;
        String script = IncludeScript.readScript( file,"UTF8" ) ;
        SeaBassContext context = null ;
        BaseModel baseModel = null ;
        boolean errorFlag = false ;
        try {
            context = context = CacheScriptManager.getInstance().script( name ) ;
            putBindings( context ) ;
            baseModel = new BaseModelImpl() ;
            context.put( ScriptDef.SCRIPT_BY_MODEL,baseModel ) ;
            context.put( ScriptDef.SCRIPT_BY_ENGINE,engine ) ;
            context.put( ScriptDef.SCRIPT_BY_ENGINE,engine ) ;
            context.put( ScriptDef.SCRIPT_DATE,new java.util.Date() ) ;
            Object ret = engine.eval( CacheScriptManager.getInstance().insertPackage( script ),context ) ;
            if( EndScript.isEndScript( ret ) ) {
                String code = ( String )EndScript.getEndByResult( context ) ;
                if( code != null && code.startsWith( "error" ) ) {
                    errorFlag = true ;
                }
            }
        } catch( Exception e ) {
            errorFlag = true ;
            throw e ;
        } finally {
            if( baseModel != null && baseModel.isCreate() == true ) {
                try {
                    if( errorFlag == true ) {
                        baseModel.rollback() ;
                    }
                    else {
                        baseModel.commit() ;
                    }
                } catch( Exception ee ) {
                }
            }
            CacheScriptManager.getInstance().exitScript( context ) ;
        }
        return true ;
    }
    
    /**
     * デフォルトのBindings条件を設定.
     */
    private static final void putBindings( SeaBassContext context )
        throws Exception {
        context.put( ScriptDef.CURRENT_SCRIPT,DETAIL_NAME ) ;
        context.put( ScriptDef.SCRIPT_MODE,ScriptDef.MODE_NOT_COMET ) ;
        context.put( ScriptDef.SCRIPT_DETAIL,DETAIL_NAME ) ;
        context.put( ScriptDef.MANAGER,GlobalManager.getInstance() ) ;
        context.put( ScriptDef.SCRIPT_BY_LOG,LOG ) ;
    }
    
}
