package org.maachang.seabass.httpd.engine.script.scripts;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.seabass.httpd.engine.script.Script;
import org.maachang.util.FileUtil;
import org.maachang.util.StringUtil;

/**
 * 指定ディレクトリをコンパイルする処理
 * 
 * @version 2009/08/30
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public abstract class DirCompile {
    private DirCompile(){}
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( DirCompile.class ) ;
    
    /**
     * 指定ディレクトリ以下をコンパイル.
     * @param factory ApplicationScriptFactoryを設定します.
     * @param dir 対象のディレクトリを設定します.
     * @return int コンパイル失敗件数が返されます.
     * @exception Exception 例外.
     */
    public static final int compile( ApplicationScriptFactory factory,String dir )
        throws Exception {
        if( dir == null || ( dir = dir.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "コンパイル対象のディレクトリは設定されていません" ) ;
        }
        if( !FileUtil.isDirExists( dir ) ) {
            throw new IllegalArgumentException( "コンパイル対象のディレクトリは存在しません:" + dir ) ;
        }
        int ret = 0 ;
        dir = FileUtil.getFullPath( dir ) ;
        if( dir.endsWith( "/" ) ) {
            dir = dir.substring( 0,dir.length() - 1 ) ;
        }
        List<String> list = FileUtil.getListAll( dir ) ;
        int len = list.size() ;
        String name = null ;
        String target = null ;
        Script script = null ;
        for( int i = 0 ; i < len ; i ++ ) {
            name = StringUtil.changeString( list.get( i ),"\\","/" ) ;
            target = getTarget( name ) ;
            if( (
                    target.endsWith( ".ms" ) == false &&
                    target.endsWith( ".mhtml" ) == false
                ) ||
                name.indexOf( "/~seabass/" ) != -1 ||
                target.startsWith( "@" ) == true ||
                target.startsWith( "~" ) == true ) {
                continue ;
            }
            try {
                if( ( script = factory.getApplication( name ) ) == null ) {
                    LOG.warn( "## ERROR:" + name ) ;
                    ret ++ ;
                }
                else {
                    script.getScript() ;
                    if( LOG.isDebugEnabled() ) {
                        LOG.debug( "## SUCCESS:" + name ) ;
                    }
                }
            } catch( Exception e ) {
                LOG.error( "ERROR:\'" + name + "\'",e ) ;
                ret ++ ;
            }
        }
        return ret ;
    }
    
    /** スラッシュで区切られた最後の文字列を取得. **/
    private static final String getTarget( String name ) {
        int p = name.lastIndexOf( "/" ) ;
        if( p != -1 ) {
            return name.substring( p+1 ) ;
        }
        return name ;
    }
}

