package org.maachang.seabass.httpd.engine.script.scripts;

import gnu.trove.THashMap;

import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.SeaBassCompile;
import org.maachang.seabass.SeaBassContext;
import org.maachang.seabass.SeaBassScriptEngine;
import org.maachang.seabass.httpd.HttpdResponse;
import org.maachang.seabass.httpd.engine.HttpdDef;
import org.maachang.seabass.httpd.engine.script.ExecutionScript;
import org.maachang.seabass.httpd.engine.script.ScriptDef;
import org.maachang.seabass.httpd.engine.script.cache.ApiManager;
import org.maachang.seabass.httpd.engine.script.cache.CacheScriptManager;

/**
 * スクリプト実行処理実装.
 * 
 * @version 2007/08/24
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
class ExecutionScriptImpl implements ExecutionScript {
    /**
     * LOG用.
     */
    private static final Log LOG = LogFactory.getLog( ExecutionScript.class ) ;
    
    /**
     * SeaBass管理テーブル.
     */
    private static final String MANAGER = ScriptDef.MANAGER ;
    
    /**
     * ログファイル出力用.
     */
    private static final String KEY_LOG = ScriptDef.SCRIPT_BY_LOG ;
    
    /**
     * エンジン処理用.
     */
    private static final String ENGINE = ScriptDef.SCRIPT_BY_ENGINE ;
    
    /**
     * スクリプト開始日付.
     */
    private static final String SCRIPT_DATE = ScriptDef.SCRIPT_DATE ;
    
    /**
     * コンパイル済みスクリプト.
     */
    private SeaBassCompile comp = null ;
    
    /**
     * スクリプトデータフラグ.
     */
    private int scriptDataType = -1 ;
    
    /**
     * スクリプト名.
     */
    private String scriptName = null ;
    
    /**
     * スクリプト内容.
     */
    private String scriptData = null ;
    
    /**
     * デフォルトパラメータ.
     */
    private Map<String,Object> defaultParams = null ;
    
    /**
     * 処理後コネクションクローズフラグ.
     */
    private boolean closeConnectFlag = false ;
    
    /**
     * コンストラクタ.
     */
    private ExecutionScriptImpl() {
        
    }
    
    /**
     * コンストラクタ.
     * <BR><BR>
     * 対象スクリプト文字列を渡して、オブジェクトを生成します.
     * <BR>
     * @param scriptDataType 対象のスクリプトデータタイプを設定します.
     * @param name 対象のスクリプト名を設定します.
     * @param script 対象のスクリプトを設定します.
     * @exception Exception 例外.
     */
    public ExecutionScriptImpl( int scriptDataType,String name,String script ) throws Exception {
        boolean closeContextFlag = false ;
        SeaBassContext ctx = AbstractScript.getSeaBassContext() ;
        if( ctx == null ) {
            closeContextFlag = true ;
            ctx = CacheScriptManager.getInstance().script( name ) ;
        }
        try {
            SeaBassScriptEngine engine = ApiManager.getInstance().getScriptEngine() ;
            ctx.setName( name ) ;
            this.comp = engine.compile( script,ctx ) ;
            this.scriptName = name ;
            this.scriptDataType = scriptDataType ;
            if( LOG.isDebugEnabled() ) {
                this.scriptData = script ;
                LOG.debug( ">> createScript:" + name ) ;
            }
        } catch( Exception e ) {
            //if( LOG.isDebugEnabled() ) {
            //    LOG.debug( "## COMPILE ERROR:" + name +
            //        "\n<<<START>>>\n" + script + "\n<<<END>>>",e ) ;
            //}
            LOG.error( "## COMPILE ERROR:" + name,e ) ;
            throw e ;
        } finally {
            if( closeContextFlag ) {
                CacheScriptManager.getInstance().exitScript( ctx ) ;
            }
        }
    }
    
    /**
     * デストラクタ.
     */
    protected void finalize() throws Exception {
    }
    
    /**
     * コネクションクローズフラグを設定.
     * @param mode [true]の場合、処理後コネクションをクローズします.
     */
    public void setCloseConnect( boolean mode ) {
        closeConnectFlag = mode ;
    }
    
    /**
     * コネクションクローズフラグを取得.
     * @return boolean [true]の場合、処理後コネクションをクローズします.
     */
    public boolean isCloseConnect() {
        return closeConnectFlag ;
    }
    
    /**
     * スクリプト単体実行.
     * <BR><BR>
     * スクリプトを単体で実行します.<BR>
     * このメソッドは、主に[InnerScript]等、スクリプト内で実行する場合に
     * 呼び出されます.
     * <BR>
     * @exception Exception 例外.
     */
    public Object eval() throws Exception {
        return eval( null ) ;
    }
    
    /**
     * スクリプト単体実行.
     * <BR><BR>
     * スクリプトを単体で実行します.<BR>
     * このメソッドは、主に[InnerScript]等、スクリプト内で実行する場合に
     * 呼び出されます.
     * <BR>
     * @param ctx 対象のスクリプトコンテキストを設定します.
     * @exception Exception 例外.
     */
    public Object eval( SeaBassContext ctx ) throws Exception {
        if( ctx == null ) {
            ctx = AbstractScript.getSeaBassContext() ;
            if( ctx == null ) {
                throw new IllegalArgumentException( "SeaBassContextが未設定です" ) ;
            }
        }
        if( LOG.isDebugEnabled() ) {
            LOG.debug( ">> eval[" + scriptName + "]" ) ;
        }
        Object ret = null ;
        long t = -1 ;
        Boolean befInner = Boolean.FALSE ;
        String befPath = null ;
        try {
            if( LOG.isDebugEnabled() ) {
                t = System.currentTimeMillis() ;
            }
            befInner = ( Boolean )ctx.get( "_$inner" ) ;
            befPath = ( String )ctx.get( "_$innerPath" ) ;
            if( scriptDataType == ScriptManager.SCRIPT_TYPE_BY_INNER ) {
                ctx.put( "_$inner",Boolean.TRUE ) ;
                ctx.put( "_$innerPath",scriptName ) ;
            }
            else {
                ctx.put( "_$inner",Boolean.FALSE ) ;
                ctx.put( "_$innerPath",null ) ;
            }
            ret = comp.eval( ctx ) ;
            if( LOG.isDebugEnabled() ) {
                t = System.currentTimeMillis() - t ;
                LOG.debug( "++++++ comp-exec(" + scriptName + ") - time[" + t + "msec]" ) ;
            }
        } catch( Exception e ) {
            //if( LOG.isDebugEnabled() ) {
            //    LOG.debug( "## SCRIPT ERROR:" + this.scriptName +
            //        "\n<<<START>>>\n" + this.scriptData + "\n<<<END>>>",e ) ;
            //}
            LOG.error( "## COMPILE ERROR:" + this.scriptName,e ) ;
            throw e ;
        } finally {
            ctx.put( "_$inner",befInner ) ;
            ctx.put( "_$innerPath",befPath ) ;
        }
        return ret ;
    }
    
    /**
     * スクリプト実行.
     * <BR><BR>
     * 対象のスクリプトを実行します.
     * <BR>
     * @param params 対象のパラメータを設定します.
     * @return Object 処理結果が返されます.
     * @exception Exception 例外.
     */
    public Object execution( Object params ) throws Exception {
        return execution( null,params ) ;
    }
    
    /**
     * スクリプト実行.
     * <BR><BR>
     * 対象のスクリプトを実行します.
     * <BR>
     * @param ctx 対象のスクリプトコンテキストを設定します.
     * @param params 対象のパラメータを設定します.
     * @return Object 処理結果が返されます.
     * @exception Exception 例外.
     */
    public Object execution( SeaBassContext ctx,Object params ) throws Exception {
        if( ctx == null ) {
            ctx = AbstractScript.getSeaBassContext() ;
            if( ctx == null ) {
                throw new IllegalArgumentException( "SeaBassContextが未設定です" ) ;
            }
        }
        putParams( params,ctx ) ;
        putObject( scriptDataType,scriptName,defaultParams,ctx ) ;
        Object ret = null ;
        long t = -1 ;
        try {
            if( LOG.isDebugEnabled() ) {
                LOG.debug( ">> execution[" + scriptName + "]" ) ;
                t = System.currentTimeMillis() ;
            }
            executionByClose( ctx ) ;
            ret = comp.eval( ctx ) ;
            if( LOG.isDebugEnabled() ) {
                t = System.currentTimeMillis() - t ;
                LOG.debug( "++++++ comp-exec(" + scriptName + ") - time[" + t + "msec]" ) ;
            }
        } catch( Exception e ) {
            //if( LOG.isDebugEnabled() ) {
            //    LOG.debug( "## SCRIPT ERROR:" + this.scriptName +
            //        "\n<<<START>>>\n" + this.scriptData + "\n<<<END>>>",e ) ;
            //}
            LOG.error( "## COMPILE ERROR:" + this.scriptName,e ) ;
            throw e ;
        }
        return ret ;
    }
    
    /**
     * スクリプトデータタイプを取得.
     * <BR><BR>
     * スクリプトデータタイプを取得します.
     * <BR>
     * @return スクリプトデータタイプが返されます.
     */
    public int getScriptDataType() {
        return scriptDataType ;
    }
    
    /**
     * スクリプト名を取得.
     * <BR><BR>
     * スクリプト名を取得します.
     * <BR>
     * @return String スクリプト名が返されます.
     */
    public String getScriptName() {
        return scriptName ;
    }
    
    /**
     * デフォルトパラメータを設定.
     * <BR><BR>
     * デフォルトパラメータを設定します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param value 対象の要素を設定します.
     */
    public void putDefault( String key,Object value ) {
        if( key == null || value == null ) {
            return ;
        }
        if( defaultParams == null ) {
            defaultParams = new THashMap<String,Object>() ;
        }
        defaultParams.put( key,value ) ;
    }
    
    /**
     * 実行前に追加するオブジェクト.
     */
    private static final void putObject( int scriptDataType,String name,
        Map<String,Object> defaultParams,SeaBassContext ctx )
        throws Exception {
        SeaBassScriptEngine engine = ApiManager.getInstance().getScriptEngine() ;
        ctx.put( ScriptDef.CURRENT_SCRIPT,name ) ;
        ctx.put( ScriptDef.SCRIPT_DETAIL,ScriptManager.getTypeByString( scriptDataType ) ) ;
        ctx.put( MANAGER,GlobalManager.getInstance() ) ;
        ctx.put( KEY_LOG,LOG ) ;
        ctx.put( ENGINE,engine ) ;
        // 実行開始時間をセット.
        ctx.put( SCRIPT_DATE,new java.util.Date() ) ;
        // defaultパラメータを設定.
        if( defaultParams != null ) {
            Iterator it = defaultParams.keySet().iterator() ;
            while( it.hasNext() ) {
                String key = ( String )it.next() ;
                ctx.put( key,defaultParams.get( key ) ) ;
            }
        }
    }
    
    /**
     * パラメータを追加.
     */
    private static final void putParams( Object params,SeaBassContext ctx ) {
        if( params != null ) {
            if( params instanceof Map ) {
                 Map pms = (Map)params ;
                if( pms != null && pms.size() > 0 ) {
                    Iterator it = pms.keySet().iterator() ;
                    while( it.hasNext() ) {
                        String key = ( String )it.next() ;
                        ctx.put( key,pms.get( key ) ) ;
                    }
                }
            }
            else {
                ctx.put( "args",params ) ;
            }
        }
    }
    
    /**
     * 実行時にcloseConnectFlag=trueの場合の処理.
     */
    private final void executionByClose( SeaBassContext ctx ) {
        if( closeConnectFlag ) {
            HttpdResponse response = ( HttpdResponse )ctx.get( ScriptDef.SCRIPT_BY_RESPONSE ) ;
            if( response != null && !response.isOutputStream() ) {
                response.setHttpClose( true ) ;
            }
        }
    }
}
