package org.maachang.seabass.httpd.engine.script.scripts;

import gnu.trove.THashMap;

import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.dao.OptimisticLockException;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.SeaBassContext;
import org.maachang.seabass.SeaBassException;
import org.maachang.seabass.ServiceDef;
import org.maachang.seabass.httpd.HttpdErrorDef;
import org.maachang.seabass.httpd.HttpdParams;
import org.maachang.seabass.httpd.HttpdRequest;
import org.maachang.seabass.httpd.HttpdResponse;
import org.maachang.seabass.httpd.HttpdStateException;
import org.maachang.seabass.httpd.engine.HttpdDef;
import org.maachang.seabass.httpd.engine.script.BaseModel;
import org.maachang.seabass.httpd.engine.script.BaseModelImpl;
import org.maachang.seabass.httpd.engine.script.BaseScriptException;
import org.maachang.seabass.httpd.engine.script.EndScript;
import org.maachang.seabass.httpd.engine.script.Script;
import org.maachang.seabass.httpd.engine.script.ScriptDef;
import org.maachang.seabass.httpd.engine.script.SrcScript;
import org.maachang.seabass.httpd.engine.script.cache.CacheScriptManager;
import org.maachang.seabass.httpd.engine.session.HttpdSession;
import org.maachang.seabass.net.HttpdResponseImpl;
import org.maachang.util.FileUtil;

import pnuts.lang.PnutsException;

/**
 * リクエストに対する実行用ターゲットスクリプト.
 * 
 * @version 2007/08/24
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class TargetScript {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( TargetScript.class ) ;
    
    /**
     * フォワード実行処理.
     */
    private static final String FORWARD = "@forward" ;
    
    /**
     * リダイレクト実行処理.
     */
    private static final String REDIRECT = "@redirect" ;
    
    /**
     * スクリプト実行処理.
     * @param path 対象のパス名を設定します.
     * @param request 対象のリクエスト情報を設定します.
     * @param params 対象のパラメータを設定します.
     * @return Object スクリプト処理結果が返されます.
     * @exception Exception 例外.
     */
    public static final Object executionScript( String path,HttpdRequest request,
        Map<String,Object> params ) throws Exception {
        if( request == null ) {
            throw new HttpdStateException( HttpdErrorDef.HTTP10_400,"不正なリクエストを受信しました" ) ;
        }
        long allTime = -1L ;
        if( LOG.isDebugEnabled() ) {
            allTime = System.currentTimeMillis() ;
        }
        ApplicationScriptFactory factory = getFactory() ;
        path = path.trim() ;
        String name = FileUtil.getFileName( path ) ;
        path = path.substring( 0,path.length()-name.length() ) ;
        // 読み込み対象の拡張子が[.ms] or 拡張子なし以外の場合.
        if( name.endsWith( ScriptManager.SCRIPT_PLUS ) == true ) {
            name = name.substring( 0,name.length()-ScriptManager.SCRIPT_PLUS.length() ) ;
        }
        String acName = path + name ;
        // ターゲットのスクリプトエンジンを取得.
        Script script = factory.getApplication( acName ) ;
        if( script == null ) {
            throw new HttpdStateException( HttpdErrorDef.HTTP11_500,
                "対象パス["+(acName)+"]スクリプトの読み込みに失敗しました" ) ;
        }
        long time = -1L ;
        // ターゲットURLが、ページキャッシュされている場合の処理.
        String cache = PageCache.getInstance().get( acName ) ;
        if( cache != null ) {
            HttpdResponse res = ( HttpdResponse )params.get( ScriptDef.SCRIPT_BY_RESPONSE ) ;
            // レスポンスヘッダ周りを設定する.
            res.getHeader().setHeader( HttpdDef.VALUE_CONTENT_TYPE,HttpdDef.MHTML_MIME_TYPE ) ;
            res.getPrint().print( cache ) ;
            return "exit-script" ;
        }
        // キャッシュされていない場合などは、通常処理.
        Object ret = null ;
        SeaBassContext context = null ;
        BaseModel baseModel = new BaseModelImpl() ;
        if( params == null ) {
            params = new THashMap<String,Object>() ;
        }
        params.put( ScriptDef.SCRIPT_MODE,ScriptDef.MODE_NOT_COMET ) ;
        boolean errorFlag = false ;
        try {
            if( LOG.isDebugEnabled() ) {
                /** スクリプト初期化処理用 デバッグログ */
                time = System.currentTimeMillis() ;
            }
            // ライブラリスクリプトを読み込む.
            context = CacheScriptManager.getInstance().script( "" ) ;
            // 各パラメータを取得.
            createContext( path,request,context,baseModel ) ;
            // デフォルトパラメータを取得.
            ScriptDef.setDirectoryByBindings( context ) ;
            
            if( LOG.isDebugEnabled() ) {
                /** スクリプト初期化処理用 デバッグログ */
                time = System.currentTimeMillis() - time ;
                LOG.debug( "## init(" + acName + ") = " + time + "msec" ) ;
                time = System.currentTimeMillis() ;
            }
            
            // *****************************************************************
            // * フォワード対応スクリプト実行時のループ.
            // *****************************************************************
            String redirectURL = null ;
            while( true ) {
                
                // ターゲットスクリプト実行
                ret = script.getScript().execution( context,params ) ;
                
                if( EndScript.isEndScript( ret ) ) {
                    // スクリプト終了シグナル受け取り時.
                    ret = EndScript.getEndByResult( context ) ;
                    if( LOG.isDebugEnabled() ) {
                        /** スクリプト実行処理用 デバッグログ */
                        time = System.currentTimeMillis() - time ;
                        LOG.debug( "## script(e)(" + acName + ") = " + time + "msec" ) ;
                        time = System.currentTimeMillis() ;
                    }
                    // スクリプト結果、文字列を返す場合.
                    if( ret instanceof String ) {
                        String result = ( String )ret ;
                        // スクリプトがForward実行の場合.
                        if( result.startsWith( FORWARD ) ) {
                            acName = result.substring( FORWARD.length() ) ;
                            script = factory.getApplication( acName ) ;
                            ret = FORWARD ;
                            continue ;
                        }
                        // スクリプトがRedirectを返した場合.
                        else if( result.startsWith( REDIRECT ) ) {
                            redirectURL = result.substring( REDIRECT.length() ) ;
                            ret = REDIRECT ;
                            break ;
                        }
                    }
                    // 文字列が存在しない場合.
                    else {
                        HttpdResponse res = ( HttpdResponse )params.get( ScriptDef.SCRIPT_BY_RESPONSE ) ;
                        // HTML出力が行われている場合は、強制出力を行う.
                        if( res.isOutputStream() ) {
                            res.flush() ;
                        }
                    }
                }
                else if( LOG.isDebugEnabled() ) {
                    // 通常スクリプト終了.
                    /** スクリプト実行処理用 デバッグログ */
                    time = System.currentTimeMillis() - time ;
                    LOG.debug( "## script(" + acName + ") = " + time + "msec" ) ;
                    time = System.currentTimeMillis() ;
                }
                break ;
            }
            
            // コミット処理.
            if( baseModel.isCreate() ) {
                baseModel.commit() ;
                baseModel.getRecord().close() ;
                baseModel = null ;
                if( LOG.isDebugEnabled() ) {
                    /** スクリプトコミット処理用 デバッグログ */
                    time = System.currentTimeMillis() - time ;
                    LOG.debug( "## commit(" + acName + ") = " + time + "msec" ) ;
                }
            }
            
            // リダイレクト処理の場合は、リダイレクト条件をレスポンスに設定.
            if( redirectURL != null ) {
                HttpdResponseImpl response = ( HttpdResponseImpl )context.get( ScriptDef.SCRIPT_BY_RESPONSE ) ;
                // リダイレクト用のレスポンス処理を実行.
                response.setState( 303 ) ;
                response.getHeader().setHeader( HttpdDef.VALUE_LOCATION,redirectURL ) ;
                response.getHeader().setHeader( HttpdDef.VALUE_CONTENT_LENGTH,"0" ) ;
                response.getOutput() ;
            }
            
        } catch( BaseScriptException be ) {
            errorFlag = true ;
            throw be ;
        } catch( SeaBassException sb ) {
            errorFlag = true ;
            Throwable t = sb.getException() ;
            if( t instanceof PnutsException ) {
                t = ((PnutsException)t).getThrowable() ;
            }
            if( t instanceof OptimisticLockException ) {
                // OptimisticLock例外を返す.
                throw (OptimisticLockException)t ;
            }
            String errName = sb.getTopScriptName() ;
            SrcScript currentSrcScript = CacheScriptManager.getInstance().getSrcScript( errName ) ;
            if( currentSrcScript == null ) {
                Script errScript = factory.getApplication( errName ) ;
                if( errScript != null ) {
                    currentSrcScript = errScript.getSrcScript() ;
                }
            }
            if( currentSrcScript != null ) {
                CacheScriptManager.viewParseException( LOG,sb,currentSrcScript ) ;
            }
            throw new BaseScriptException( sb ) ;
        } catch( Exception e ) {
            errorFlag = true ;
            throw e ;
        } finally {
            if( baseModel != null && baseModel.isCreate() ) {
                if( errorFlag ) {
                    try {
                        baseModel.rollback() ;
                        baseModel.getRecord().close() ;
                        baseModel = null ;
                    } catch( Exception ee ) {
                    }
                }
                if( baseModel != null ) {
                    try {
                        baseModel.getRecord().close() ;
                    } catch( Exception ee ) {
                    }
                }
            }
            baseModel = null ;
            CacheScriptManager.getInstance().exitScript( context ) ;
            if( LOG.isDebugEnabled() ) {
                LOG.debug( "... (end)readScript:" + acName +
                    " - ["+( System.currentTimeMillis()-allTime )+"ms]" ) ;
            }
        }
        return ret ;
    }
    
    /**
     * コンテキストを生成.
     */
    private static void createContext( String path,HttpdRequest req,SeaBassContext ctx,BaseModel baseModel )
        throws Exception {
        // 基本パラメータを定義.
        ctx.put( ScriptDef.SCRIPT_BY_MODEL,baseModel ) ;
        ctx.put( ScriptDef.SCRIPT_BY_HEADER,req.getHeader() ) ;
        HttpdSession session = req.getSession() ;
        if( session != null ) {
            ctx.put( ScriptDef.SCRIPT_BY_SESSION,session ) ;
        }
        // リクエスト内容を設定.
        pushRequestParams( ctx,req.getQuery() ) ;
    }
    
    /**
     * WebAppScriptFactoryを取得.
     */
    private static final ApplicationScriptFactory getFactory() {
        return ( ApplicationScriptFactory )GlobalManager.getValue(
            ServiceDef.MANAGER_BY_WEB_APP_FACTORY ) ;
    }
    
    /**
     * リクエストパラメータを設定.
     */
    private static final void pushRequestParams( SeaBassContext context,HttpdParams params ) {
        context.put( ScriptDef.SCRIPT_BY_QUERY,ScriptDef.getQuery( params ) ) ;
    }
}
