package org.maachang.seabass.httpd.engine.script.scripts;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.seabass.httpd.HttpdErrorDef;
import org.maachang.seabass.httpd.HttpdStateException;
import org.maachang.seabass.httpd.engine.script.ExecutionScript;
import org.maachang.seabass.httpd.engine.script.SrcScript;

/**
 * サーバValidateスクリプト.
 * 
 * @version 2009/01/19
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class ValidateScript {
    
    /**
     * LOG.
     */
    private static final Log LOG = LogFactory.getLog( ValidateScript.class ) ;
    
    /**
     * スクリプト名.
     */
    private String name = null ;
    
    /**
     * スクリプト内容.
     */
    private SrcScript srcScript = null ;
    
    /**
     * 実行スクリプトオブジェクト.
     */
    private ExecutionScript executionScript = null;
    
    /**
     * Javascript用Validate内容.
     */
    private byte[] toJavascript = null ;
    
    /**
     * 生成時間.
     */
    private long createTime = -1L ;
    
    /**
     * コンストラクタ.
     * @param name スクリプト名を設定します.
     * @param script 対象のスクリプトを設定します.
     * @exception Exception 例外.
     */
    public ValidateScript( String name,String script )
        throws Exception {
        if( name== null || ( name = name.trim() ).length() <= 0 ) {
            throw new HttpdStateException( HttpdErrorDef.HTTP11_500,
                "スクリプト名は不正です" ) ;
        }
        else if( script == null || script.length() <= 0 ) {
            script = null ;
        }
        this.name = name ;
        String serverScript = convertServerScript( script ) ;
        this.srcScript = new SrcScript( serverScript ) ;
        try {
            this.executionScript = new ExecutionScriptImpl(
                ScriptManager.SCRIPT_TYPE_BY_ETC,this.name,serverScript ) ;
        } catch( Exception e ) {
            LOG.error( "errorValidate[" + name + "]:script:\n" + serverScript ) ;
            throw e ;
        }
        convertClientScript( script ) ;
        this.createTime = System.currentTimeMillis() ;
    }
    
    /**
     * 情報クリア.
     */
    protected void clear() {
        this.name = null ;
        this.srcScript = null ;
        this.executionScript = null ;
        this.toJavascript = null ;
        this.createTime = -1L ;
    }
    
    /**
     * 基本スクリプトを取得.
     * <BR><BR>
     * 基本スクリプトを取得します.
     * <BR>
     * @return ExecutionScript 基本スクリプトが返されます.
     * @exception Exception 例外.
     */
    public ExecutionScript getScript() throws Exception {
        return executionScript ;
    }
    
    /**
     * スクリプト名を取得.
     * <BR><BR>
     * スクリプト名を取得します.
     * <BR>
     * @return String スクリプト名が返されます.
     */
    public String getName() {
        return this.name ;
    }
    
    /**
     * スクリプト内容を取得.
     * <BR><BR>
     * スクリプト内容を取得します.
     * <BR>
     * @return SrcScript スクリプト内容が返されます.
     */
    public SrcScript getSrcScript() {
        return srcScript ;
    }
    
    /**
     * スクリプトが利用可能かチェック.
     * <BR><BR>
     * スクリプトが利用可能かチェックします.
     * <BR>
     * @return boolean [true]の場合、利用可能です.
     */
    public boolean isScript() {
        return ( executionScript == null ) ? false : true ;
    }
    
    /**
     * Javascript用Validate情報を取得.
     * @return byte[] Javascript用Validate情報が返されます.
     */
    public byte[] toJavascript() {
        return toJavascript ;
    }
    
    /**
     * 生成時間を取得.
     * @return long 生成時間が返されます.
     */
    public long getCreateTime() {
        return createTime ;
    }
    
    /** サーバValidateスクリプト変換処理 **/
    private String convertServerScript( String script ) {
        return new StringBuilder().
            append( "if( params.size() <= 0 ) { validate_errorFlag.on(); }\n" ).
            append( "else if( useString( getRequestParam( \"_submit_form_name\",0 ) ) ) { \n" ).
            append( "switch( getRequestParam( \"_submit_form_name\",0 ) ) { \n" ).
            append( script ).
            append( "\n}}\n" ).toString() ;
    }
    
    /** クライアントValidateスクリプト変換処理 **/
    private void convertClientScript( String script )
        throws Exception {
        this.toJavascript = new StringBuilder().
            append( "function _validate_by_form( name ) {\n" ).
            append( "validate_clear_all_message() ;\n" ).
            append( "if( $base.useString( name ) ) { \n" ).
            append( "switch( name ) { \n" ).
            append( script ).
            append( "\n}}}\n" ).toString().getBytes( "UTF8" ) ;
        
    }
}
