package org.maachang.seabass.httpd.engine.script.validate;

import java.util.Iterator;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.seabass.SeaBassContext;
import org.maachang.seabass.httpd.engine.script.scripts.ValidateScript;

/**
 * サーバValidateスクリプトキャッシュ.
 * 
 * @version 2009/01/19
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class ServerValidateCache {
    
    /**  LOG. **/
    private static final Log LOG = LogFactory.getLog( ServerValidateCache.class ) ;
    
    /** Validateスクリプト管理 **/
    private ConcurrentHashMap<String,ValidateScript> cache = new ConcurrentHashMap<String,ValidateScript>() ;
    
    /**
     * コンストラクタ.
     */
    public ServerValidateCache() {
        
    }
    
    /**
     * 新しいValidateスクリプトを追加.
     * @param name Validate呼び出し名を設定します.
     * @param script Validateスクリプトを設定します.
     * @exception Exception 例外.
     */
    public void add( String name,String script )
        throws Exception {
        ValidateScript s = new ValidateScript( name,script ) ;
        cache.put( name,s ) ;
        if( LOG.isDebugEnabled() ) {
            LOG.debug( "## addValidate:" + name + " script:\n" + script ) ;
        }
    }
    
    /**
     * 指定Validateスクリプトを削除.
     * @param name Validate呼び出し名を設定します.
     */
    public void remove( String name ) {
        cache.remove( name ) ;
        if( LOG.isDebugEnabled() ) {
            LOG.debug( "## removeValidate:" + name ) ;
        }
    }
    
    /**
     * 指定Validate情報を取得.
     * @param name Validate呼び出し名を設定します.
     * @return ValidateScript Validateスクリプトが返されます.
     */
    public ValidateScript get( String name ) {
        if( LOG.isDebugEnabled() ) {
            if( !cache.containsKey( name ) ) {
                LOG.debug( "## no validate:" + name ) ;
            }
            else {
                LOG.debug( "## use validate:" + name ) ;
            }
        }
        return cache.get( name ) ;
    }
    
    /**
     * 指定Validate情報が存在するか取得.
     * @param name Validate呼び出し名を設定します.
     * @return boolean [true]の場合、存在します.
     */
    public boolean contains( String name ) {
        return cache.containsKey( name ) ;
    }
    
    /**
     * 登録Validate一覧を取得.
     * @return Iterator Iteratorが返されます.
     */
    public Iterator iterator() {
        return cache.keySet().iterator() ;
    }
    
    /**
     * 登録Validate数を取得.
     * @return int 登録Validate数が返されます.
     */
    public int size() {
        return cache.size() ;
    }
    
    /**
     * 指定Validate名を実行.
     * @param name 対象のValidate名を設定します.
     * @param context 対象のSeaBassContextを設定します.
     * @return boolean [false]の場合、Validate対象条件が存在しません.
     * @exception Exception 例外.
     */
    public boolean validate( String name,SeaBassContext context )
        throws Exception {
        ValidateScript s = cache.get( name ) ;
        if( s != null ) {
            s.getScript().eval( context ) ;
            return true ;
        }
        return false ;
    }
}
