package org.maachang.seabass.httpd.engine.session ;

import org.maachang.bdb.BdbKeyValue;
import org.maachang.seabass.session.client.PSessionManager;
import org.maachang.seabass.session.engine.PersistenceDefine;
import org.maachang.util.BaseRandom;

/**
 * 永続化セッション管理[PersistenceSession]を利用した、
 * セッション管理オブジェクト.
 *
 * @version 2008/05/26
 * @author  masahito suzuki
 * @since   SeaBass 1.19
 */
public class HttpdSessionManager {
    
    /**
     * セッション文字数.
     */
    public static final int SESSION_ID_LENGTH = PersistenceDefine.SESSION_KEY_LENGTH ; ;
    
    /**
     * 永続化セッション管理.
     */
    private PSessionManager manager = null ;
    
    /**
     * コンストラクタ.
     */
    private HttpdSessionManager() {
    }
    
    /**
     * コンストラクタ.
     * @param manager 対象のセッション削除時間を設定します.
     */
    public HttpdSessionManager( PSessionManager manager ) {
        this.manager = manager ;
    }
    
    /**
     * 新しいセッションを発行.
     * <BR><BR>
     * 新しいセッションを発行します.
     * <BR>
     * @param baseRandom 対象のランダムオブジェクトを設定します.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @return HttpdSession 新しいセッション情報が返されます.
     * @exception Exception 例外.
     */
    public HttpdSession getNewSession( BaseRandom baseRandom,BdbKeyValue keyValue ) throws Exception {
        return new PHttpdSessionImpl( manager.getSession( baseRandom,keyValue ) ) ;
    }
    
    /**
     * セッションIDを指定して、セッションを取得.
     * <BR><BR>
     * セッションIDを指定して、セッションを取得します.
     * <BR>
     * @param baseRandom 対象のランダムオブジェクトを設定します.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param id 対象のセッションIDを設定します.
     * @return HttpdSession 対象のセッション情報が返されます.
     * @exception Exception 例外.
     */
    public HttpdSession getSession( BaseRandom baseRandom,BdbKeyValue keyValue,String id ) throws Exception {
        return new PHttpdSessionImpl( manager.getSession( baseRandom,keyValue,id ) ) ;
    }
    
    /**
     * セッションIDが存在するかチェック.
     * <BR><BR>
     * セッションIDが存在するかチェックします.
     * <BR>
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param id 対象のセッションIDを設定します.
     * @return boolean [true]の場合、セッションIDが存在します.
     * @exception Exception 例外.
     */
    public boolean isSession( BdbKeyValue keyValue,String id ) throws Exception {
        return manager.isSession( keyValue,id ) ;
    }
    
    /**
     * 対象セッションを削除.
     * <BR><BR>
     * 対象のセッションを削除します.
     * <BR>
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param id 対象のセッションIDを設定します.
     * @exception Exception 例外.
     */
    public void removeSession( BdbKeyValue keyValue,String id ) throws Exception {
        manager.reomveSession( keyValue,id ) ;
    }
    
    /**
     * セッション削除時間を取得.
     * @return long セッション削除時間が返されます.
     */
    public long getDeleteTime() {
        long ret = -1L ;
        try {
            ret = manager.getDeleteSessionTime() ;
        } catch( Exception e ) {
            ret = System.currentTimeMillis() ;
        }
        return ret ;
    }
    
}
