package org.maachang.seabass.net ;

import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.maachang.ModUtil;
import org.maachang.seabass.httpd.HttpdBinary;
import org.maachang.seabass.httpd.HttpdParams;

/**
 * HTTPD-パラメータ群実装.
 *
 * @version 2007/08/20
 * @author  masahito suzuki
 * @since   SeaBass 1.00
 */
public class HttpdParamsImpl implements HttpdParams {
    
    /**
     * パラメータ管理.
     */
    private ParamsMap params = null ;
    
    /**
     * コンストラクタ.
     */
    public HttpdParamsImpl() {
    }
    
    /**
     * デストラクタ.
     */
    protected void finalize() throws Exception {
        this.params = null ;
    }
    
    /**
     * オブジェクトクリア.
     */
    public void clear() {
        this.params = null ;
    }
    
    /**
     * オブジェクトコピー.
     * @param obj コピー先のオブジェクトを設定します.
     */
    public void copy( HttpdParamsImpl obj ) {
        if( obj == null ) {
            return ;
        }
        obj.params = new ParamsMap() ;
        obj.params.putAll( this.params );
    }
    
    /**
     * パラメータオブジェクトをマージ.
     * <BR><BR>
     * 指定オブジェクトをマージします.
     * <BR>
     * @param param 対象のパラメータオブジェクトを設定します.
     */
    public void add( HttpdParams param ) {
        if( param == null || param.size() <= 0 ) {
            return ;
        }
        Iterator it = param.iterator() ;
        if( it != null ) {
            while( it.hasNext() ) {
                String key = ( String )it.next() ;
                int len = param.size( key ) ;
                for( int i = 0 ; i < len ; i ++ ) {
                    this.addParam( key,param.getParam( key,i ) ) ;
                }
            }
        }
    }
    
    /**
     * パラメータ追加.
     * <BR><BR>
     * パラメータを追加します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param value 対象の要素を設定します.
     */
    public void addParam( String key,Object value ) {
        pushParam( false,key,value ) ;
    }
    
    /**
     * パラメータ設定.
     * <BR><BR>
     * パラメータを設定します.<BR>
     * この条件で設定した場合、以前に登録したkeyに
     * 対する内容はクリアされます.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param value 対象の要素を設定します.
     */
    public void setParam( String key,Object value ) {
        pushParam( true,key,value ) ;
    }
    
    /**
     * パラメータ上書き.
     * <BR><BR>
     * 指定パラメータ内容を上書きします.
     * @param key 対象のキー名を設定します.
     * @param num 対象の項番を設定します.
     * @param value 対象の要素を設定します.
     */
    public void update( String key,int num,Object value ) {
        if( key == null ) {
            return ;
        }
        key = cnv( key ) ;
        Object v = params.getOriginal( key ) ;
        if( v instanceof java.util.List ) {
            List lst = (List)v ;
            if( num >= 0 && num < lst.size() ) {
                lst.set( num,value ) ;
            }
        }
        else {
            setParam( key,value ) ;
        }
    }
    
    /**
     * パラメータを追加／設定.
     * mode[true]の場合は、パラメータ設定.
     */
    private void pushParam( boolean mode,String key,Object value ) {
        if( key == null ) {
            return ;
        }
        key = cnv( key ) ;
        if( value == null ) {
            value = "" ;
        }
        if( !( value instanceof HttpdBinary ) ) {
            value = ModUtil.parseValue( value ) ;
        }
        if( params == null ) {
            params = new ParamsMap() ;
        }
        if( mode == true ) {
            params.put( key,value ) ;
        }
        else {
            if( params.containsKey( key ) == false ) {
                params.put( key,value ) ;
            }
            else {
                List<Object> lst = null ;
                Object v = params.getOriginal( key ) ;
                if( v instanceof List ) {
                    lst = ( List<Object> )v ;
                }
                else {
                    lst = new ArrayList<Object>() ;
                    lst.add( v ) ;
                    params.put( key,lst ) ;
                }
                lst.add( value ) ;
            }
        }
    }
    
    /**
     * パラメータを削除.
     * <BR><BR>
     * 対象のパラメータを削除します.
     * <BR>
     * @param key 対象のキー名を設定します.
     */
    public void removeParam( String key ) {
        if( key == null || params == null ) {
            return ;
        }
        key = cnv( key ) ;
        params.remove( key ) ;
    }
    
    /**
     * パラメータを削除.
     * <BR><BR>
     * 対象のパラメータを削除します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param no 対象の項番を設定します.
     */
    public void removeParam( String key,int no ) {
        if( key == null || params == null || no < 0 ) {
            return ;
        }
        key = cnv( key ) ;
        if( params.containsKey( key ) ) {
            int len = getKeyByLength( key ) ;
            if( len <= 0 || no >= len ) {
                return ;
            }
            if( len == 1 ) {
                params.remove( key ) ;
            }
            else {
                ArrayList v = ( ArrayList )params.getOriginal( key ) ;
                v.remove( no ) ;
            }
        }
    }
    
    /**
     * パラメータを取得.
     * <BR><BR>
     * 対象のパラメータを取得します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @return Object 対象の要素情報が返されます.
     */
    public String getParam( String key ) {
        return getParam( key,0 ) ;
    }
    
    /**
     * パラメータを取得.
     * <BR><BR>
     * 対象のパラメータを取得します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param no 対象の項番を設定します.
     * @return Object 対象の要素情報が返されます.
     */
    public String getParam( String key,int no ) {
        if( key == null || params == null || no < 0 ) {
            return null ;
        }
        key = cnv( key ) ;
        if( params.containsKey( key ) ) {
            int len = getKeyByLength( key ) ;
            if( len <= 0 || no >= len ) {
                return null ;
            }
            Object v = params.get( key ) ;
            if( v == null ) {
                return null ;
            }
            if( v instanceof ArrayList ) {
                v = ( ( ArrayList )v ).get( no ) ;
            }
            if( v instanceof String ) {
                return ( String )v ;
            }
            return v.toString() ;
        }
        return null ;
    }
    
    /**
     * パラメータをバイナリオブジェクトとして取得.
     * <BR><BR>
     * 対象のパラメータをバイナリオブジェクトとして取得します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @return HttpdBinary 対象の要素情報が返されます.
     */
    public HttpdBinary getParamByBinary( String key ) {
        return getParamByBinary( key,0 ) ;
    }
    
    /**
     * パラメータをバイナリオブジェクトとして取得.
     * <BR><BR>
     * 対象のパラメータをバイナリオブジェクトとして取得します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param no 対象の項番を設定します.
     * @return HttpdBinary 対象の要素情報が返されます.
     */
    public HttpdBinary getParamByBinary( String key,int no ) {
        if( key == null || params == null || no < 0 ) {
            return null ;
        }
        key = cnv( key ) ;
        if( params.containsKey( key ) ) {
            int len = getKeyByLength( key ) ;
            if( len <= 0 || no >= len ) {
                return null ;
            }
            Object v = params.getOriginal( key ) ;
            if( v == null ) {
                return null ;
            }
            if( v instanceof ArrayList ) {
                v = ( ( ArrayList )v ).get( no ) ;
            }
            if( v instanceof String ) {
                HttpdBinary b = null ;
                try {
                    b = new HttpdBinary( key,"application/octet-stream",( ( String )v ).getBytes( "UTF8" ) ) ;
                } catch( Exception e ) {
                    b = null ;
                }
                return b ;
            }
            return ( HttpdBinary )v ;
        }
        return null ;
    }
    
    /**
     * パラメータタイプを取得.
     * <BR><BR>
     * パラメータタイプを取得します.
     * <BR>
     * @param key 対象のパラメータキー名を設定します.
     * @return boolean 対象のパラメータタイプが返されます.<BR>
     *                 [true]の場合は、バイナリタイプです.<BR>
     *                 [false]の場合は、文字タイプです.
     */
    public boolean getParamType( String key ) {
        return getParamType( key,0 ) ;
    }
    
    /**
     * パラメータタイプを取得.
     * <BR><BR>
     * パラメータタイプを取得します.
     * <BR>
     * @param key 対象のパラメータキー名を設定します.
     * @param no 対象の項番を設定します.
     * @return boolean 対象のパラメータタイプが返されます.<BR>
     *                 [true]の場合は、バイナリタイプです.<BR>
     *                 [false]の場合は、文字タイプです.
     */
    public boolean getParamType( String key,int no ) {
        if( key == null || params == null || no < 0 ) {
            return false ;
        }
        key = cnv( key ) ;
        if( params.containsKey( key ) ) {
            int len = getKeyByLength( key ) ;
            if( len <= 0 || no >= len ) {
                return false ;
            }
            Object v = params.getOriginal( key ) ;
            if( v == null ) {
                return false ;
            }
            if( v instanceof ArrayList ) {
                v = ( ( ArrayList )v ).get( no ) ;
            }
            if( v instanceof String ) {
                return false ;
            }
            return true ;
        }
        return false ;
    }
    
    /**
     * パラメータ数を取得.
     * <BR><BR>
     * 対象のパラメータ数を取得します.
     * <BR>
     * @return int パラメータ数が返されます.
     */
    public int size() {
        return ( params == null ) ? 0 : params.size() ;
    }
    
    /**
     * パラメータ数を取得.
     * <BR><BR>
     * 対象のパラメータ数を取得します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @return int パラメータ数が返されます.
     */
    public int size( String key ) {
        if( key == null || params == null ) {
            return 0 ;
        }
        return getKeyByLength( key ) ;
    }
    
    /**
     * キー名群を取得.
     * <BR><BR>
     * 対象のキー名群を取得します.
     * <BR>
     * @return String[] キー名群が返されます.
     */
    public String[] getKeys() {
        if( params == null || params.size() <= 0 ) {
            return null ;
        }
        Object[] keys = params.keySet().toArray() ;
        if( keys != null ) {
            int len = keys.length ;
            String[] ret = new String[ len ] ;
            for( int i = 0 ; i < len ; i ++ ) {
                ret[ i ] = ( String )keys[ i ] ;
            }
            return ret ;
        }
        return null ;
    }
    
    /**
     * Iteratorを取得.
     * @return Iterator Iteratorが返されます.
     */
    public Iterator iterator() {
        if( params != null && params.size() > 0 ) {
            return params.keySet().iterator() ;
        }
        return null ;
    }
    
    /**
     * 指定キーが存在するかチェック.
     * <BR><BR>
     * 指定キーが存在するかチェックします.
     * <BR>
     * @param key 対象のキー名を取得します.
     * @return boolean [true]の場合、存在します.
     */
    public boolean isKey( String key ) {
        if( key == null || params == null ) {
            return false ;
        }
        key = cnv( key ) ;
        return params.containsKey( key ) ;
    }
    
    /**
     * 管理パラメータを取得.
     * <BR><BR>
     * このオブジェクトが管理しているパラメータを取得します.
     * <BR>
     * @return Map 管理パラメータが返されます.
     */
    public Map getPrivateMap() {
        if( params == null ) {
            params = new ParamsMap() ;
        }
        return params ;
    }
    
    /**
     * パラメータ内容をURL形式で取得.
     * @return String パラメータ内容が返されます.
     */
    public String getUrlParams() {
        String[] keys = getKeys() ;
        if( keys != null ) {
            StringBuilder buf = new StringBuilder() ;
            int len = keys.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "&" ) ;
                }
                Object v = params.getOriginal( keys[ i ] ) ;
                if( v == null ) {
                    buf.append( keys[ i ] ).
                        append( "=" ) ;
                }
                else if( v instanceof List ) {
                    List<Object> lst = ( List<Object> )v ;
                    if( lst != null ) {
                        int lenJ = lst.size() ;
                        for( int j = 0 ; j < lenJ ; j ++ ) {
                            if( j != 0 ) {
                                buf.append( "&" ) ;
                            }
                            Object objs = lst.get( j ) ;
                            buf.append( keys[ i ] ).
                                append( "=" ) ;
                            if( !(objs instanceof HttpdBinary) ) {
                                buf.append( objs ) ;
                            }
                        }
                    }
                }
                else {
                    buf.append( keys[ i ] ).
                        append( "=" ) ;
                    if( !(v instanceof HttpdBinary) ) {
                        buf.append( v ) ;
                    }
                }
            }
            return buf.toString() ;
        }
        return "" ;
    }
    
    /**
     * 文字列に変換.
     */
    public String toString() {
        String[] keys = getKeys() ;
        if( keys != null ) {
            StringBuilder buf = new StringBuilder() ;
            int len = keys.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                Object v = params.getOriginal( keys[ i ] ) ;
                if( v == null ) {
                    buf.append( " [" ).
                        append( keys[ i ] ).
                        append( ":" ).append( "null" ).append( "]" ) ;
                }
                else if( v instanceof List ) {
                    List<Object> lst = ( List<Object> )v ;
                    if( lst != null ) {
                        int lenJ = lst.size() ;
                        for( int j = 0 ; j < lenJ ; j ++ ) {
                            if( j != 0 ) {
                                buf.append( "," ) ;
                            }
                            Object objs = lst.get( j ) ;
                            buf.append( " [" ).
                                append( keys[ i ] ).
                                append( ":" ) ;
                            if( objs instanceof String ) {
                                buf.append( objs ).append( "]" ) ;
                            }
                            else if( objs instanceof HttpdBinary ) {
                                buf.append( " orginal/" ).
                                    append( ( ( HttpdBinary )objs ).getOriginalName() ).
                                    append( " length/" ).
                                    append( ( ( HttpdBinary )objs ).getLength() ).
                                    append( "]" ) ;
                            }
                        }
                    }
                }
                else {
                    buf.append( " [" ).
                        append( keys[ i ] ).
                        append( ":" ) ;
                    if( v instanceof String ) {
                        buf.append( v ).append( "]" ) ;
                    }
                    else if( v instanceof HttpdBinary ) {
                        buf.append( " orginal/" ).
                            append( ( ( HttpdBinary )v ).getOriginalName() ).
                            append( " length/" ).
                            append( ( ( HttpdBinary )v ).getLength() ).
                            append( "]" ) ;
                    }
                }
            }
            return buf.toString() ;
        }
        else {
            return "null" ;
        }
    }
    
    /**
     * 指定名のサイズを取得.
     */
    private int getKeyByLength( String key ) {
        if( key == null || key.length() <= 0 ) {
            return 0 ;
        }
        key = cnv( key ) ;
        if( params.containsKey( key ) == false ) {
            return 0 ;
        }
        Object v = params.getOriginal( key ) ;
        if( v instanceof ArrayList ) {
            return ( ( ArrayList )v ).size() ;
        }
        return 1 ;
    }
    
    /**
     * キー変換.
     */
    private static final String cnv( String key ) {
        try {
            key = URLDecoder.decode( key,"UTF8" ) ;
        } catch( Exception e ) {
        }
        return key ;
    }
}

