package org.maachang.seabass.net;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import org.maachang.conf.ConvIniParam;
import org.maachang.manager.GlobalManager;
import org.maachang.seabass.ServiceDef;
import org.maachang.seabass.conf.MimeConfig;
import org.maachang.seabass.httpd.HttpdErrorDef;
import org.maachang.seabass.httpd.HttpdHeaders;
import org.maachang.seabass.httpd.HttpdMode;
import org.maachang.seabass.httpd.HttpdRequest;
import org.maachang.seabass.httpd.HttpdResponse;
import org.maachang.seabass.httpd.engine.HttpdCookie;
import org.maachang.seabass.httpd.engine.HttpdDef;
import org.maachang.seabass.httpd.engine.session.HttpdSession;
import org.maachang.seabass.httpd.engine.session.HttpdSessionManager;
import org.maachang.seabass.net.nio.ConnectionInfo;
import org.maachang.seabass.net.nio.NetWriteHttpd;
import org.maachang.util.StringUtil;
import org.maachang.util.java.GZIPOutputStreamPls;
import org.maachang.util.java.PrintWriterPls;

/**
 * HTTPD基本レスポンス実装.
 * 
 * @version 2007/08/26
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class HttpdResponseImpl implements HttpdResponse {
    
    /**
     * Mimeタイプ.
     */
    public static final MimeConfig mimeType = ( MimeConfig )GlobalManager.getValue( ServiceDef.MANAGER_BY_MIME_TYPE ) ;
    
    /**
     * HTTPコード.
     */
    protected int state = HttpdErrorDef.HTTP11_200 ;
    
    /**
     * 戻りHTTPバージョン.
     */
    protected String version = HttpdDef.VERSION_11 ;
    
    /**
     * HTTPバージョン(double).
     */
    protected double versionDouble = 0.0d ;
    
    /**
     * コネクションオブジェクト.
     */
    private ConnectionInfo connectionInfo = null ;
    
    /**
     * ヘッダ.
     */
    private final HttpdHeadersImpl header = new HttpdHeadersImpl() ;
    
    /**
     * 出力用オブジェクト.
     */
    private final NetWriteHttpd output = new NetWriteHttpd() ;
    
    /**
     * チャンク用Stream.
     */
    private final ChunkedOutputStream chOutput = new ChunkedOutputStream() ;
    
    /**
     * HTTPキャッシュ.
     */
    protected boolean cacheMode = false ;
    
    /**
     * gzip圧縮フラグ.
     */
    protected boolean gzipFlag = true ;
    
    /**
     * リクエストGZIP名.
     */
    protected String gzipName = null ;
    
    /**
     * リクエストURL.
     */
    private String requestURL = null ;
    
    /**
     * OutputStream.
     */
    private OutputStream outputStream = null ;
    
    /**
     * printWriter.
     */
    private PrintWriterPls printWriter = null ;
    
    /**
     * クローズフラグ.
     */
    private boolean closeFlag = true ;
    
    /**
     * リクエストKeepAlive.
     */
    private boolean requestKeepAlive = false ;
    
    /**
     * コネクションヘッダなしフラグ.
     * ※デフォルトはコネクションヘッダなし.
     */
    private boolean noConnection = false ;
    
    /**
     * リクエストオブジェクト.
     */
    private HttpdRequest request = null ;
    
    /**
     * SSLアクセス.
     */
    private boolean sslFlag = false ;
    
    /**
     * ローカルホストアクセス.
     */
    private boolean localhostFlag = false ;
    
    /**
     * Seabass32暗号化フラグ.
     */
    private boolean seabass32 = false ;
    
    /**
     * Seabass32暗号化オプション.
     */
    private String[] seabass32Opt = null ;
    
    /**
     * コンストラクタ.
     */
    public HttpdResponseImpl() {
        
    }
    
    /**
     * オブジェクト生成.
     * <BR><BR>
     * ステータスを設定してオブジェクトを生成します.
     * <BR>
     * @param request 対象のリクエストを設定します.
     * @param url 対象のリクエストURLを設定します.
     * @param timeout 対象のタイムアウト値を設定します.
     * @param count 対象のカウント値を設定します.
     * @return HttpdResponse レスポンスオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public HttpdResponse create( HttpdRequest request,String url,long timeout,int count )
        throws Exception {
        return create( request,url,HttpdErrorDef.HTTP10_200,timeout,count ) ;
    }
    
    /**
     * オブジェクト生成.
     * <BR><BR>
     * ステータスを設定してオブジェクトを生成します.
     * <BR>
     * @param request 対象のリクエストを設定します.
     * @param url 対象のリクエストURLを設定します.
     * @param state 対象のHTTPコードを設定します.
     * @param timeout 対象のタイムアウト値を設定します.
     * @param count 対象のカウント値を設定します.
     * @return HttpdResponse レスポンスオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public HttpdResponse create( HttpdRequest request,String url,int state,long timeout,int count )
        throws Exception {
        ConnectionInfo conn = null ;
        if( request == null || ( conn = request.getConnectionInfo() ) == null || conn.isUse() == false ) {
            if( request == null ) {
                throw new NioException( "リクエスト情報がnullで渡されています" ) ;
            }
            else if( conn == null ) {
                throw new NioException( "リクエスト情報内のコネクション情報が存在しません" ) ;
            }
            else if( conn.isUseElement() == false ) {
                throw new NioException( "リクエスト情報内の通信要素が存在しません" ) ;
            }
            throw new NioException( "オブジェクトは既に破棄されています" ) ;
        }
        this.clear() ;
        this.requestURL = url ;
        this.state = state ;
        this.connectionInfo = conn ;
        this.seabass32 = request.isSeabass32() ;
        this.seabass32Opt = request.getSeabass32Opt() ;
        // リクエストのHTTPバージョンを取得.
        this.version = request.getVersion() ;
        double versionDouble ;
        // リクエストのHTTPバージョンが数値の場合.
        if( StringUtil.isNumber( this.version ) ) {
            versionDouble = new Double( this.version ) ;
        }
        else {
            throw new NioException( "HTTPバージョンは不正です:"+this.version ) ;
        }
        this.versionDouble = versionDouble ;
        // HTTPバージョンが1.1の場合.
        if( versionDouble >= 1.1d ) {
            HttpdHeaders rh = request.getHeader() ;
            // GZIP圧縮可能かチェック.
            if( rh.getHeader( HttpdDef.VALUE_ENCODE ) != null ) {
                // IE6でアクセスされている場合は、GZIPを全面的にOFF.
                if( rh.getHeader( HttpdDef.VALUE_CLIENT_NAME ).indexOf( "MSIE 6." ) == -1 ) {
                    int len = rh.size( HttpdDef.VALUE_ENCODE ) ;
                    boolean chkFlag = false ;
                    String encode ;
                    if( request.isGzip() ) {
                        for( int i = 0 ; i < len ; i ++ ) {
                            encode = rh.getHeader( HttpdDef.VALUE_ENCODE,i ) ;
                            if( encode != null && encode.indexOf( "gzip" ) != -1 ) {
                                if( ( encode = encode.trim() ).length() <= 0 ) {
                                    this.gzipName = null ;
                                    chkFlag = false ;
                                }
                                else {
                                    this.gzipName = encode ;
                                    chkFlag = true ;
                                }
                                break ;
                            }
                        }
                    }
                    if( chkFlag == false ) {
                        this.gzipName = null ;
                    }
                    // 指定拡張子が、非圧縮対象の場合かチェック.
                    else {
                        int p = url.lastIndexOf( "." ) ;
                        String plus = null ;
                        if( p != -1 ) {
                            plus = url.substring( p+1 ) ;
                        }
                        if( plus == null || ( plus = plus.trim() ).length() <= 0 ) {
                            plus = "" ;
                        }
                        String type = mimeType.get( MimeConfig.OPTION_GZIP+plus ) ;
                        if( ConvIniParam.getBoolean( type ) == false ) {
                            this.gzipName = null ;
                        }
                    }
                }
                else {
                    this.gzipName = null ;
                }
            }
            else {
                this.gzipName = null ;
            }
            // keepaliveが有効な場合.
            if( rh.isKeepAlive() == true ) {
                if( timeout > 0L && count > 0 ) {
                    this.request = request ;
                    requestKeepAlive = true ;
                }
                else {
                    this.request = null ;
                    requestKeepAlive = false ;
                }
            }
            else {
                this.request = null ;
                requestKeepAlive = false ;
            }
        }
        // HTTPバージョンが1.1以下の場合.
        else {
            this.request = null ;
            requestKeepAlive = false ;
            gzipFlag = false ;
        }
        this.localhostFlag = conn.isLocalAddress() ;
        this.sslFlag = conn.getElement().isSSL() ;
        setHttpClose( true ) ;
        return this ;
    }
    
    /**
     * レスポンスクリア.
     */
    public void clear() {
        try {
            header.clear() ;
        } catch( Exception e ) {
        }
        try {
            output.close() ;
        } catch( Exception e ) {
        }
        chOutput.clear() ;
        state = HttpdErrorDef.HTTP11_200 ;
        version = HttpdDef.VERSION_11 ;
        versionDouble = 0.0d ;
        connectionInfo = null ;
        cacheMode = false ;
        gzipFlag = true ;
        gzipName = null ;
        requestURL = null ;
        outputStream = null ;
        printWriter = null ;
        closeFlag = true ;
        requestKeepAlive = false ;
        noConnection = false ;
        request = null ;
        localhostFlag = false ;
        sslFlag = false ;
        seabass32 = false ;
        seabass32Opt = null ;
        
    }
    
    /**
     * レスポンスを更新/クローズ.
     * @return boolean [true]の場合、成功しました.
     */
    public boolean flushAndClose() {
        boolean ret = true ;
        if( !destroyByNoOutputStream() ) {
            this.flush() ;
            if( printWriter != null ) {
                try {
                    printWriter.close() ;
                } catch( Exception e ) {
                    ret = false ;
                }
            }
            else if( outputStream != null ) {
                try {
                    outputStream.close() ;
                } catch( Exception e ) {
                    ret = false ;
                }
            }
        }
        clear() ;
        return ret ;
    }
    
    /**
     * 出力完了処理.
     * <BR><BR>
     * 出力を完了する処理を実施します.
     */
    public void flush() {
        if( output.useObject() ) {
            if( printWriter != null ) {
                try {
                    printWriter.flush() ;
                } catch( Exception e ) {
                }
            }
            if( outputStream != null ) {
                try {
                    outputStream.flush() ;
                } catch( Exception e ) {
                }
            }
        }
    }
    
    /**
     * フラッシュ処理が行われたかチェック.
     * @return boolean [true]の場合、フラッシュ情報が必要です.
     */
    public boolean isFlush() {
        return output.isFlush() ;
    }
    
    /**
     * HTTPコードを設定.
     * <BR><BR>
     * HTTPコードを設定します.
     * <BR>
     * @param state 対象のHTTPコードを設定します.
     */
    public void setState( int state ) {
        this.state = state ;
    }
    
    /**
     * HTTPコードを取得.
     * <BR><BR>
     * HTTPコードを取得します.
     * <BR>
     * @return int HTTPコードが返されます.
     */
    public int getState() {
        return state ;
    }
    
    /**
     * HTTPバージョン情報を設定.
     * <BR><BR>
     * HTTPバージョン情報を設定します.
     * <BR>
     * @param version 対象のHTTPバージョンを設定します.
     */
    public void setVersion( String version ) {
        double versionDouble ;
        // リクエストのHTTPバージョンが数値の場合.
        if( StringUtil.isNumber( this.version ) ) {
            versionDouble = new Double( this.version ) ;
        }
        else {
            throw new NioException( "HTTPバージョンは不正です:"+this.version ) ;
        }
        this.version = version ;
        this.versionDouble = versionDouble ;
    }
    
    /**
     * HTTPバージョン情報を取得.
     * <BR><BR>
     * HTTPバージョン情報を取得します.
     * <BR>
     * @return String 対象のHTTPバージョンが返されます.
     */
    public String getVersion() {
        return version ;
    }
    
    /**
     * GZIP圧縮条件を設定.
     * <BR><BR>
     * GZIP圧縮条件を設定します.
     * <BR>
     * @param flag GZIP圧縮条件を設定します.
     */
    public void setGzip( boolean flag ) {
        gzipFlag = flag ;
    }
    
    /**
     * GZIP圧縮条件を取得.
     * <BR><BR>
     * GZIP圧縮条件を取得します.
     * <BR>
     * @return boolean GZIP圧縮条件が返されます.
     */
    public boolean getGzip() {
        if( gzipName == null ) {
            return false ;
        }
        return gzipFlag ;
    }
    
    /**
     * HTTPキャッシュ条件を設定.
     * <BR><BR>
     * HTTPキャッシュ条件を設定します.
     * <BR>
     * @param cache HTTPキャッシュ条件を設定します.
     */
    public void setHttpCache( boolean cache ) {
        this.cacheMode = cache ;
    }
    
    /**
     * HTTPキャッシュ条件を取得.
     * <BR><BR>
     * HTTPキャッシュ条件を取得します.
     * <BR>
     * @return boolean HTTPキャッシュ条件が返されます.
     */
    public boolean getHttpCache() {
        return cacheMode ;
    }
    
    /**
     * リクエストに設定されているSessionCookie条件を設定.
     * <BR><BR>
     * リクエストに設定されているSessionCookie条件を設定します.
     * <BR>
     * @param request 対象のリクエストを設定します.
     * @exception Exception 例外.
     */
    public void setCookieSession( HttpdRequest request )
        throws Exception {
        if( request == null ) {
            return ;
        }
        if( request.isCookie() == true || ((HttpdRequestImpl)request).isSessionId() == true ) {
            HttpdSession session = request.getSession() ;
            if( session != null ) {
                int len = 0 ;
                if( ( len = header.size( HttpdCookie.SET_COOKIE ) ) > 0 ) {
                    for( int i = len-1 ; i >= 0 ; i -- ) {
                        String value = header.getHeader( HttpdCookie.SET_COOKIE,i ) ;
                        if( value != null ) {
                            if( value.indexOf( HttpdSession.SESSION_NAME+"=" ) != -1 ) {
                                header.removeHeader( HttpdCookie.SET_COOKIE,i ) ;
                            }
                        }
                    }
                }
                HttpdSessionManager sessionManager = ( HttpdSessionManager )GlobalManager.getValue(
                    ServiceDef.MANAGER_BY_SESSION ) ;
                long time = System.currentTimeMillis() + sessionManager.getDeleteTime() ;
                String value = HttpdCookie.createCookie(
                    HttpdSession.SESSION_NAME,
                    session.getSessionId(),
                    "/",time ) ;
                header.addHeader( HttpdCookie.SET_COOKIE,value ) ;
            }
        }
    }
    
    /**
     * ヘッダオブジェクトを取得.
     * <BR><BR>
     * ヘッダオブジェクトを取得します.
     * <BR>
     * @return HttpdHeaders ヘッダオブジェクトが返されます.
     */
    public HttpdHeaders getHeader() {
        return header ;
    }
    
    /**
     * ソケットクローズフラグを設定.
     * <BR><BR>
     * 送信結果と同時に、ソケットをクローズする場合は、[true]を設定します.
     * <BR>
     * @param mode [true]の場合、このオブジェクトが破棄されると同時に、コネクションが切断されます.
     */
    public void setDestroyByClose( boolean mode ) {
        try {
            this.connectionInfo.setCloseFlag( mode ) ;
        } catch( Exception e ) {
        }
    }
    
    /**
     * リクエストKeepAliveモードを取得.
     * <BR><BR>
     * リクエストKeepAliveモードを取得します.
     * <BR>
     * @return boolean [true]の場合リクエストのKeepAliveは有効です.
     */
    public boolean isRequestKeepAlive() {
        return requestKeepAlive ;
    }
    
    /**
     * HTTPクローズ条件を設定.
     * <BR><BR>
     * HTTPクローズ条件を設定します.
     * <BR>
     * @param closeFlag HTTPクローズ条件を設定します.
     */
    public void setHttpClose( boolean closeFlag ) {
        this.closeFlag = closeFlag ;
        closeOrNoConnect() ;
    }
    
    /**
     * HTTPクローズ条件を取得.
     * <BR><BR>
     * HTTPクローズ条件を取得します.
     * <BR>
     * @return boolean HTTPクローズ条件が返されます.
     */
    public boolean getHttpClose() {
        closeOrNoConnect() ;
        return closeFlag ;
    }
    
    /**
     * Conectionヘッダ付加設定.
     * @param mode [true]の場合は、Connectionヘッダを付加しません.
     */
    public void setConnectionHeader( boolean mode ) {
        this.noConnection = mode ;
        closeOrNoConnect() ;
    }
    
    /**
     * Connectionヘッダ付加チェック.
     * @return boolean Connectionヘッダを付加しない場合は、trueが返されます.
     */
    public boolean isConnectionHeader() {
        closeOrNoConnect() ;
        return noConnection ;
    }
    
    /** closeおよび、Connectionヘッダ付加条件処理. **/
    private void closeOrNoConnect() {
        //if( this.requestKeepAlive == false || localhostFlag == true || sslFlag == true ) {
        //if( HttpdMode.closeMode == true || this.requestKeepAlive == false || localhostFlag == true ) {
        //if( this.requestKeepAlive == false ) {
        
        if( HttpdMode.closeMode == true || this.requestKeepAlive == false ) {
            this.closeFlag = true ;
            this.noConnection = false ;
        }
        else if( closeFlag ) {
            this.noConnection = false ;
        }
        
        /*
        if( closeFlag ) {
            this.noConnection = false ;
        }
        */
    }
    
    /**
     * 戻り情報を設定するオブジェクトを取得.
     * <BR><BR>
     * 戻り情報を設定するオブジェクトを取得します.
     * <BR>
     * @return OutputStream 戻り情報を設定するオブジェクトが返されます.
     * @exception Exception 例外
     */
    public OutputStream getOutput()
        throws Exception {
        if( printWriter != null ) {
            return null ;
        }
        if( outputStream == null ) {
            boolean chkFlag = false ;
            boolean lenFlag = false ;
            boolean gzipMd = false ;
            boolean closeMd = false ;
            if( output.useObject() == false ) {
                output.create( connectionInfo,requestURL,state,version,versionDouble,isConnectionHeader() ) ;
                // コネクションなしフラグがOFFの場合.
                if( !noConnection ) {
                    // クローズ条件を設定.
                    if( closeFlag ) {
                        // close.
                        output.closeOn() ;
                        this.setDestroyByClose( true ) ;
                    }
                    else if( requestKeepAlive ) {
                        // keepAlive.
                        output.closeOff() ;
                        output.setKeepAlive( "timeout="+request.getKeepAliveTimeout(),
                            "max="+request.getKeepAliveCount() ) ;
                        this.setDestroyByClose( false ) ;
                    }
                    else {
                        closeFlag = true ;
                        output.closeOn() ;
                        this.setDestroyByClose( true ) ;
                    }
                }
                else {
                    this.setDestroyByClose( false ) ;
                }
                // 基本ヘッダ群を設定.
                output.baseHeader() ;
                if( header != null ) {
                    output.pushHeaders( header ) ;
                }
                // 非キャッシュ状態の場合.
                if( cacheMode ) {
                    output.cacheOff() ;
                }
                // GZIP圧縮が有効の場合.
                // ただし、seabass32が有効な場合は、GZIP圧縮をOFF.
                if( !seabass32 && ( versionDouble >= 1.1d && getGzip() ) ) {
                    output.setGZIPName( gzipName ) ;
                }
            }
            
            // 出力用OutputStreamを生成.
            OutputStream os = output.getOutputStream( seabass32 ) ;
            // チャングフラグを取得.
            chkFlag = output.isChunked() ;
            // コンテンツ長設定フラグを取得.
            lenFlag = output.isContentLengthHeader() ;
            // GZIPモードを取得.
            gzipMd = output.isGZIP() ;
            // クローズフラグを取得.
            closeMd = output.isClose() ;
            
            // チャング／コンテンツ長が設定されている場合は、
            // 通信終了後に、コネクションクローズしない.
            if( chkFlag || lenFlag ) {
                this.setDestroyByClose( false ) ;
            }
            this.closeFlag = closeMd ;
            if( !closeMd ) {
                this.setDestroyByClose( false ) ;
            }
            
            // GZIP処理の場合.
            // (GZIP圧縮の場合は、確実にチャング転送になっている).
            if( gzipMd ) {
                chOutput.clear() ;
                outputStream = new GZIPOutputStreamPls( chOutput.create( os ) ) ;
            }
            // GZIP以外の場合.
            else {
                // チャング転送の場合.
                if( chkFlag ) {
                    chOutput.clear() ;
                    outputStream = chOutput.create( os ) ;
                }
                // 通常転送の場合.
                else {
                    //outputStream = new BufferedOutputStream( os ) ;
                    outputStream = os ;
                }
                // seabass32暗号が有効な場合は、Seabass32OutputStreamで処理する.
                if( seabass32 ) {
                    outputStream = new Seabass32OutputStream( outputStream,seabass32Opt[ 0 ],seabass32Opt[ 1 ] ) ;
                }
            }
        }
        return outputStream ;
    }
    
    /**
     * 戻り情報を設定するPrintWriterを取得.
     * <BR><BR>
     * 戻り情報を設定するPrintWriterを取得します.
     * <BR>
     * @return PrintWriter 戻り情報を設定するオブジェクトが返されます.
     * @exception Exception 例外
     */
    public PrintWriter getPrint()
        throws Exception {
        if( printWriter == null ) {
            OutputStream os = getOutput() ;
            if( seabass32 ) {
                ((Seabass32OutputStream)os).printWriter() ;
            }
            PrintWriterPls p = new PrintWriterPls(
                new OutputStreamWriter( os,HttpdDef.DEF_CHARSET ) ) ;
            printWriter = p ; 
        }
        return printWriter ;
    }
    
    /**
     * SSLアクセスか取得.
     * @return boolean [true]の場合、SSLアクセスされています.
     */
    public boolean isSSL() {
        return sslFlag ;
    }
    
    /**
     * ローカルホストアクセスか取得.
     * <BR><BR>
     * ローカルホストから、アクセスされているか取得します.
     * <BR>
     * @return boolean [true]の場合、ローカルホストアクセスです.
     */
    public boolean isLocalHost() {
        return localhostFlag ;
    }
    
    /**
     * OutputStreamが展開されているかチェック.
     * <BR><BR>
     * OutputStreamが展開されているかチェックします.
     * <BR>
     * @return boolean [true]の場合、展開されています.
     */
    public boolean isOutputStream() {
        return output.useObject() ;
    }
    
    /**
     * destroyメソッドで、OutputStreamをオープンしていない場合.
     */
    private boolean destroyByNoOutputStream() {
        if( !isOutputStream() ) {
            try {
                output.create( connectionInfo,requestURL,state,version,versionDouble,isConnectionHeader() ) ;
                // コネクションなしフラグがOFFの場合.
                if( !noConnection ) {
                    // クローズ条件を設定.
                    if( closeFlag ) {
                        // close.
                        output.closeOn() ;
                        this.setDestroyByClose( true ) ;
                    }
                    else {
                        closeFlag = true ;
                        output.closeOn() ;
                        this.setDestroyByClose( true ) ;
                    }
                }
                // コンテンツ長を0バイトに設定.
                header.setHeader( HttpdDef.VALUE_CONTENT_LENGTH,"0" ) ;
                
                // 基本ヘッダ群を設定.
                output.baseHeader() ;
                if( header != null ) {
                    output.pushHeaders( header ) ;
                }
                // 非キャッシュ状態の場合.
                if( cacheMode == true ) {
                    output.cacheOff() ;
                }
                OutputStream out = output.getOutputStream( seabass32 ) ;
                output.close() ;
            } catch( Exception e ) {
                try {
                    output.close() ;
                } catch( Exception ee ) {
                }
            }
            return true ;
        }
        return false ;
    }
}
