package org.maachang.seabass.net ;

import java.nio.ByteBuffer;

/**
 * ネットワークユーティリティ.
 * 
 * @version 2009/01/31
 * @author  masahito suzuki
 * @since   SeaBass 1.00
 */
public abstract class NetUtil {
    private NetUtil(){}
    
    /** ByteBuffer[Direct/非Direct]フラグ. **/
    private static boolean byteBufferByDirectFlag = true ;
    
    /**
     * ByteBuffer生成時にDirectBufferで生成するか設定.
     * @param mode [true]の場合、DirectBufferで生成します.
     */
    public static final void setDirectBufferMode( boolean mode ) {
        byteBufferByDirectFlag = mode ;
    }
    
    /**
     * ByteBuffer生成時にDirectBufferで生成するかチェック.
     * @return boolean [true]の場合、DirectBufferで生成します.
     */
    public static final boolean isDirectBufferMode() {
        return byteBufferByDirectFlag ;
    }
    
    /**
     * ByteBufferを生成.
     * @param size 対象のByteBufferサイズを設定します.
     * @return ByteBuffer 対象のByteBufferが返されます.
     */
    public static final ByteBuffer byteBuffer( int size ) {
        if( byteBufferByDirectFlag ) {
            return ByteBuffer.allocateDirect( size ) ;
        }
        return ByteBuffer.allocate( size ) ;
    }
}
