package org.maachang.seabass.net ;

import java.io.IOException;
import java.io.OutputStream;

import org.maachang.seabass.httpd.engine.Seabass32;
import org.maachang.util.BinaryBuffer;

/**
 * Seabass32暗号化処理用OutputStream.
 * 
 * @version 2009/03/11
 * @author  masahito suzuki
 * @since   SeaBass 1.0.0
 */
class Seabass32OutputStream extends OutputStream {
    private OutputStream base = null ;
    private String host = null ;
    private String sessionId = null ;
    private boolean isFinishFlag = false ;
    private BinaryBuffer buf = null ;
    private boolean printWriterMode = false ;
    private Seabass32OutputStream() {
        
    }
    public Seabass32OutputStream( OutputStream base,String host,String sessionId )
        throws Exception {
        if( base == null ||
            host == null || host.length() <= 0 ||
            sessionId == null || sessionId.length() <= 0 ) {
            throw new IllegalArgumentException( "引数が足りません" ) ;
        }
        this.base = base ;
        this.host = host ;
        this.sessionId = sessionId ;
        this.buf = new BinaryBuffer() ;
        this.isFinishFlag = false ;
    }
    protected void finalize() throws Exception {
        close() ;
    }
    public void close() throws IOException {
        if( !isFinishFlag ) {
            finish() ;
        }
        base.close() ;
    }
    public void flush() throws IOException {
        // 何もしない.
    }
    public void finish() throws IOException {
        if( isFinishFlag ) {
            throw new IOException( "不正なfinish処理を呼び出しました" ) ;
        }
        String s = null ;
        if( printWriterMode ) {
            s = Seabass32.encToString( new String( buf.getBinary(),"UTF8" ),host,sessionId ) ;
        }
        else {
            s = Seabass32.enc( buf.getBinary(),host,sessionId ) ;
        }
        buf = null ;
        base.write( s.getBytes( "UTF8" ) ) ;
        base.flush() ;
        isFinishFlag = true ;
    }
    public void write(int b) throws IOException {
        buf.write( b ) ;
    }
    public boolean isFinishd() {
        return isFinishFlag ;
    }
    public void printWriter() {
        this.printWriterMode = true ;
    }
}
