package org.maachang.seabass.net.nio ;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import org.maachang.util.BinaryUtil;

/**
 * コネクションテスト用.
 * 
 * @version 2009/02/07
 * @author  masahito suzuki
 * @since   SeaBass 1.0.0
 */
public class ConnectView {
    
    private static final String userAgent = "ConnectView" ;
    
    private static final String getParam( String word,String[] args ) {
        if( args == null || args.length <= 0 ) {
            return null ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String s = args[ i ].toLowerCase() ;
            if( s.startsWith( word ) ) {
                return s.substring( word.length() ) ;
            }
        }
        return null ;
    }
    public static final void main( String[] args ) throws Exception {
        System.setProperty( "java.net.preferIPv4Stack","true" ) ;
        String ip = getParam( "-ip",args ) ;
        String port = getParam( "-port",args ) ;
        String url = getParam( "-url",args ) ;
        if( ip == null ) {
            System.out.println( "-ip -port -url" ) ;
            return ;
        }
        int portNum = -1 ;
        try {
            portNum = Integer.parseInt( port ) ;
        } catch( Exception e ) {
            portNum = -1 ;
        }
        if( portNum == -1 ) {
            portNum = 3333 ;
        }
        if( url == null ) {
            url = "/" ;
        }
        
        Socket socket = new Socket( ip,portNum ) ;
        socket.setSendBufferSize( 43800 ) ;
        socket.setReceiveBufferSize( 25188 ) ;
        socket.setKeepAlive( false ) ;
        //socket.setTcpNoDelay( true ) ;
        socket.setTcpNoDelay( false ) ;
        socket.setReuseAddress( true ) ;
        //socket.setSoLinger( true,25 ) ;
        socket.setSoLinger( false,0 ) ;
        //socket.setOOBInline( true ) ;
        socket.setOOBInline( false ) ;
        socket.setPerformancePreferences( 1,2,0 ) ;
        socket.setTrafficClass( 0x04 | 0x08 | 0x010 ) ;
        socket.setSoTimeout( 1000 ) ;
        
        System.out.println( "## コネクション成功:ip[" + ip + "] port:[" + portNum + "] url:[" + url + "]" ) ;
        
        byte[] bin = new byte[ 65535 ] ;
        InputStream in = new BufferedInputStream( socket.getInputStream() ) ;
        OutputStream out = new BufferedOutputStream( socket.getOutputStream() ) ;
        
        StringBuilder sendHeader = new StringBuilder() ;
        sendHeader.
            append( "GET " ).append( url ).append( " HTTP/1.1" ).append( "\r\n" ) ;
        if( portNum == 80 ) {
            sendHeader.append( "Host: " ).append( ip ).append( "\r\n" ) ;
        }
        else {
            sendHeader.append( "Host: " ).append( ip ).append( ":" ).append( portNum ).append( "\r\n" ) ;
        }
        sendHeader.
            append( "Accept-Encoding: gzip" ).append( "\r\n" ).
            append( "Accept-Encoding: deflate" ).append( "\r\n" ).
            append( "Connection: keep-alive" ).append( "\r\n" ).
            append( "User-Agent: " ).append( userAgent ).append( "\r\n" ).
            append( "\r\n" ) ;
        
        try {
            out.write( sendHeader.toString().getBytes( "ISO-8859-1" ) ) ;
            out.flush() ;
            int len = 0 ;
            while( true ) {
                try {
                    len = in.read( bin ) ;
                } catch( Exception e ) {
                    break ;
                }
                if( len <= -1 ) {
                    break ;
                }
                if( len <= 0 ) {
                    continue ;
                }
                System.out.print( BinaryUtil.toString( bin,0,len ) ) ;
            }
            socket.close() ;
            socket = null ;
        } finally {
            if( socket != null ) {
                try {
                    socket.close() ;
                } catch( Exception e ) {
                }
            }
        }
    }
}
