package org.maachang.seabass.net.nio ;

import java.nio.channels.SocketChannel;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * NioElementPoolingFactory
 * 
 * @version 2009/02/09
 * @author  masahito suzuki
 * @since   SeaBass 1.0.0
 */
class NioElementPoolingFactory {
    private static final ConcurrentLinkedQueue<NioElement> stack = new ConcurrentLinkedQueue<NioElement>() ;
    protected static int MAX_STACK_INFO = 50 ;
    
    /**
     * スタックから、NioElementを取得.
     * @return NioElement NioElementが返されます.
     */
    protected static final NioElement get() {
        NioElement ret = stack.poll() ;
        if( ret == null ) {
            ret = new NioElement() ;
        }
        else {
            ret.restart() ;
        }
        return ret ;
    }
    
    /**
     * スタックから、NioElementを取得.
     * @param ch 対象のチャネル情報を設定します.
     * @return NioElement NioElementが返されます.
     * @exception Exception 例外.
     */
    protected static final NioElement get( SocketChannel ch )
        throws Exception {
        NioElement ret = stack.poll() ;
        if( ret == null ) {
            ret = new NioElement( ch ) ;
        }
        else {
            ret.create( ch ) ;
        }
        return ret ;
    }
    
    /**
     * スタックから、NioElementを取得.
     * @param ch 対象のチャネル情報を設定します.
     * @param queue 書き込み通知キューを設定します.
     * @return NioElement NioElementが返されます.
     * @exception Exception 例外.
     */
    protected static final NioElement get( SocketChannel ch,IsWriteQueue queue )
        throws Exception {
        NioElement ret = stack.poll() ;
        if( ret == null ) {
            ret = new NioElement( ch,queue ) ;
        }
        else {
            ret.create( ch,queue ) ;
        }
        return ret ;
    }
    
    /**
     * 利用終了のNioElementをスタックセット.
     * @param element 対象のNioElementを設定します.
     */
    protected static final void push( NioElement element ) {
        if( element == null || element.stackFlag.get() ) {
            return ;
        }
        element.stackFlag.set( true ) ;
        stack.offer( element ) ;
    }
    
    /**
     * 格納可能かチェック.
     * @return boolean [true]の場合、格納可能です.
     */
    protected static final boolean isPush() {
        return ( stack.size() <= MAX_STACK_INFO ) ;
    }
}
