package org.maachang.seabass.net.nio ;

import java.io.IOException;
import java.io.InputStream;

/**
 * Nio用InputStream.
 * 
 * @version 2008/05/28
 * @author  masahito suzuki
 * @since   SeaBass 1.0.0
 */
final class NioInputStream extends InputStream {
    
    protected static final int BUFFER_LENGTH = 4096 ;
    private static final long TIMEOUT = 5000L ;
    private static final long MIN_TIMEOUT = 500L ;
    
    private ReceiveBuffer recvBuffer = null ;
    private long timeout = -1L ;
    private byte[] currentBinary = null ;
    private int position = 0 ;
    
    public NioInputStream() {
        
    }
    
    public InputStream create( NioElement element )
        throws Exception {
        return create( element,TIMEOUT ) ;
    }
    
    public InputStream create( NioElement element,long timeout )
        throws Exception {
        if( element == null || element.isUse() == false ) {
            throw new IOException( "コネクションは既に破棄されています" ) ;
        }
        this.close() ;
        this.recvBuffer = element.getReceiveBuffer() ;
        this.timeout = ( timeout <= MIN_TIMEOUT ) ? MIN_TIMEOUT : timeout ;
        return this ;
    }
    
    protected void finalize() throws Exception {
        try {
            close() ;
        } catch( Exception e ) {
        }
    }
    
    public void close() {
        if( recvBuffer != null ) {
            recvBuffer.reset() ;
        }
        recvBuffer = null ;
        currentBinary = null ;
        position = 0 ;
    }
    
    private boolean readCurrent() throws IOException {
        if( currentBinary == null || ( currentBinary != null && currentBinary.length <= position ) ) {
            byte[] b = recvBuffer.get( timeout ) ;
            if( b != null ) {
                currentBinary = b ;
                position = 0 ;
                return true ;
            }
        }
        return false ;
    }
    
    public int available() throws IOException {
        if( isUse() == false ) {
            throw new IOException( "オブジェクトは既にクローズしています" ) ;
        }
        return ( recvBuffer.isData() ||
            ( currentBinary == null ||
                ( currentBinary != null && currentBinary.length <= position ) ) == false ) ?
                1 : 0 ;
    }
    
    public int read() throws IOException {
        if( isUse() == false ) {
            throw new IOException( "オブジェクトは既にクローズしています" ) ;
        }
        readCurrent() ;
        int ret = ( int )( currentBinary[ position ] & 0x000000ff ) ;
        position ++ ;
        return ret ;
    }
    
    public int read(byte b[], int off, int len) throws IOException {
        if( isUse() == false ) {
            throw new IOException( "オブジェクトは既にクローズしています" ) ;
        } else if (b == null) {
            throw new NullPointerException( "引数は不正です" );
        } else if (off < 0 || len < 0 || len > b.length - off) {
            throw new IndexOutOfBoundsException( "範囲指定が不正です" );
        } else if (len == 0) {
            return 0;
        }
        readCurrent() ;
        int cLen = currentBinary.length - position ;
        if( cLen <= 0 ) {
            return 0 ;
        }
        if( cLen > len ) {
            System.arraycopy( currentBinary,position,b,off,len ) ;
            position += len ;
            return len ;
        }
        int ret = 0 ;
        int nLen = len ;
        System.arraycopy( currentBinary,position,b,off,cLen ) ;
        ret += cLen ;
        position = cLen ;
        while( recvBuffer.isData() ) {
            currentBinary = recvBuffer.get( timeout ) ;
            position = 0 ;
            cLen = currentBinary.length ;
            nLen = len - ret ;
            if( cLen > nLen ) {
                System.arraycopy( currentBinary,position,b,off+ret,nLen ) ;
                position += nLen ;
                return len ;
            }
            System.arraycopy( currentBinary,position,b,off+ret,cLen ) ;
            ret += cLen ;
            position = cLen ;
        }
        return ret ;
    }
    
    public boolean markSupported() {
        return false ;
    }
    
    private boolean isUse() {
        return ( recvBuffer != null && recvBuffer.isUse() ) ;
    }
}
