package org.maachang.seabass.net.nio ;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;

import org.maachang.seabass.net.NetUtil;
import org.maachang.seabass.net.ssl.SslElement;

/**
 * Nio用OutputStream.
 * 
 * @version 2008/05/28
 * @author  masahito suzuki
 * @since   SeaBass 1.0.0
 */
class NioOutputStream extends OutputStream {
    protected static final int BUFFER_LENGTH = 1024 ;
    protected static final long BUFFER_TIMEOUT = 10000L ;
    private final ByteBuffer buf = NetUtil.byteBuffer( BUFFER_LENGTH ) ;
    private NioElement element = null ;
    private SendBuffer wBuf = null ;
    
    public NioOutputStream() {
        
    }
    
    public NioOutputStream create( NioElement element ) throws Exception {
        if( element == null || element.isUse() == false ) {
            throw new IOException( "コネクションは既に破棄されています" ) ;
        }
        this.element = element ;
        this.wBuf = element.getWriteBuffer() ;
        this.element.update() ;
        this.buf.clear() ;
        return this ;
    }
    
    protected void finalize() throws Exception {
        clear() ;
    }
    
    protected void clear() {
        try {
            close() ;
        } catch( Exception e ) {
        }
    }
    
    protected void cometClear() {
        element = null ;
        wBuf = null ;
    }
    
    protected NioElement getNioElement() {
        return element ;
    }
    
    public void close() throws IOException {
        if( element != null && element.isUse() ) {
            try {
                if( buf.position() > 0 ) {
                    // データが存在する場合.
                    SslElement sem = element.getSslElement() ;
                    if( sem != null ) {
                        buf.flip() ;
                        sem.write( buf ) ;
                    }
                    else {
                        wBuf.write( buf ) ;
                    }
                    buf.clear() ;
                    // 更新.
                    wBuf.last() ;// lastデータとしてセット.
                    element.update() ;
                    element.restartWrite() ;
                }
                else if( !wBuf.lastFlag() ) {
                    // データが存在しない場合.
                    wBuf.last() ;// lastデータとしてセット.
                    element.update() ;
                    element.restartWrite() ;
                }
            } catch( IOException e ) {
                throw e ;
            } finally {
                element = null ;
                wBuf = null ;
            }
        }
        else {
            element = null ;
            wBuf = null ;
        }
    }
    
    public void flush() throws IOException {
        flushTo() ;
    }
    
    private boolean flushTo() throws IOException {
        if( element == null || !element.isUse() ) {
            throw new IOException( "オブジェクトは既にクローズしています" ) ;
        }
        if( buf.position() > 0 ) {
            SslElement sem = element.getSslElement() ;
            if( sem != null ) {
                buf.flip() ;
                sem.write( buf ) ;
            }
            else {
                wBuf.write( buf ) ;
            }
            buf.clear() ;
            element.update() ;
            element.restartWrite() ;
            return true ;
        }
        return false ;
    }
    
    public void write(int b) throws IOException {
        if( buf.position() >= BUFFER_LENGTH ) {
            this.flushTo() ;
        }
        buf.put( ( byte )b ) ;
    }
    
    public void write( byte[] b ) throws IOException {
        write( b,0,b.length ) ;
    }
    
    public void write( byte[] b,int off,int len ) throws IOException {
        element.update() ;
        int wLen ;
        int cnt ;
        while( true ) {
            cnt = buf.position() ;
            if( cnt + len >= BUFFER_LENGTH ) {
                wLen = BUFFER_LENGTH - cnt ;
                buf.put( b,off,wLen ) ;
                flushTo() ;
                len -= wLen ;
                off += wLen ;
            }
            else {
                buf.put( b,off,len ) ;
                break ;
            }
        }
    }
    
    public boolean isData() {
        return buf.position() > 0 ;
    }
}
