package org.maachang.seabass.net.nio ;

import java.util.HashSet;
import java.util.Set;

/**
 * ヘッダ出力バッファ.
 * 
 * @version 2007/08/19
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class OutputHeader {
    private static final int _LENGTH = 4096 ;
    private class _$NetHeader_StackList {
        public char[] c ;
        public _$NetHeader_StackList next = null ;
    }
    private _$NetHeader_StackList first = null ;
    private _$NetHeader_StackList list = null ;
    private char[] last = null ;
    private int length ;
    private int stackLength ;
    private Set<String> map = null ;
    
    /** コンストラクタ. **/
    protected OutputHeader() {
        list = new _$NetHeader_StackList() ;
        last = new char[ _LENGTH ] ;
        list.c = last ;
        first = list ;
        length = 0 ;
        stackLength = 0 ;
        map = new HashSet<String>() ;
    }
    
    /**
     * デストラクタ.
     */
    protected void finalize() throws Exception {
        OutputHeaderFactory.release( this ) ;
    }
    
    /**
     * クリア処理.
     */
    protected void clear() {
        first.next = null ;
        list = first ;
        last = first.c ;
        length = 0 ;
        stackLength = 0 ;
        map = new HashSet<String>() ;
    }
    
    /**
     * ヘッダ追加.
     * @param key 対象のキーを設定します.
     * @param value 対象の要素を設定します.
     */
    public void add( String key,String value ) {
        if( key == null || ( key = key.trim() ).length() <= 0 ||
            value == null || value.length() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        map.add( key.toLowerCase() ) ;
        put( key ).put( ": " ).put( value ).put( "\r\n" ) ;
    }
    
    /** 情報追加. **/
    public OutputHeader put( String s ) {
        int len = s.length() ;
        int off = 0 ;
        int pls ;
        _$NetHeader_StackList n ;
        while( true ) {
            if( _LENGTH+off < len+stackLength) {
                pls = (_LENGTH-stackLength) ;
                s.getChars( off,off+pls,last,stackLength ) ;
                n = new _$NetHeader_StackList() ;
                last = new char[ _LENGTH ] ;
                n.c = last ;
                list = list.next = n ;
                stackLength = 0 ;
                off += pls ;
                if( off >= len ) {
                    length += len ;
                    break ;
                }
            }
            else {
                s.getChars( off,len,last,stackLength ) ;
                stackLength += len - off ;
                length += len ;
                break ;
            }
        }
        return this ;
    }
    
    /**
     * ヘッダ内容が存在するかチェック.
     * @param key チェック対象のキーを設定します.
     * @return boolean [true]の場合、指定名の内容は存在します.
     */
    public boolean isHeader( String key ) {
        if( key == null || ( key = key.trim() ).length() <= 0 ) {
            return false ;
        }
        return map.contains( key.toLowerCase() ) ;
    }
    
    /**
     * 現在の文字列長を取得.
     * @return int 現在の文字列長が返されます.
     */
    public int length() {
        return length ;
    }
    
    /**
     * クリアされているかチェック.
     * @return boolean [true]の場合、クリアされています.
     */
    public boolean isClear() {
        return length == 0 ;
    }
    
    /**
     * 文字列を結合して出力.
     * @return String 結合された文字列が返されます.
     * @exception Exception 例外.
     */
    public String get() throws Exception {
        if( length > 0 ) {
            put( "\r\n" ) ;// ヘッダ終了.
            char[] c = new char[ length ] ;
            int off = 0 ;
            _$NetHeader_StackList n = first ;
            while( true ) {
                if( n.next == null ) {
                    System.arraycopy( n.c,0,c,off,stackLength ) ;
                    break ;
                }
                else {
                    System.arraycopy( n.c,0,c,off,_LENGTH ) ;
                    off += _LENGTH ;
                }
                n = n.next ;
            }
            clear() ;// データクリア.
            String ret = new String( c ) ;
            c = null ;
            return ret ;
        }
        return "\r\n" ;
    }
}

