package org.maachang.seabass.service.client;

import gnu.trove.THashMap;

import java.util.ArrayList;
import java.util.Map;

/**
 * HTTPパラメータ.
 *
 * @version 2007/10/18
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class HttpParam {
    
    /**
     * ヘッダ管理.
     */
    private Map<String,ArrayList<Object>> params = null ;
    
    /**
     * コンストラクタ.
     */
    public HttpParam() {
        
    }
    
    /**
     * デストラクタ.
     */
    protected void finalize() throws Exception {
        this.params = null ;
    }
    
    /**
     * パラメータオブジェクトをマージ.
     * <BR><BR>
     * 指定オブジェクトをマージします.
     * <BR>
     * @param param 対象のパラメータオブジェクトを設定します.
     */
    public void add( HttpParam param ) {
        if( param == null || param.size() <= 0 ) {
            return ;
        }
        String[] keys = param.getKeys() ;
        if( keys != null ) {
            int len = keys.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                int lenJ = param.size( keys[ i ] ) ;
                for( int j = 0 ; j < lenJ ; j ++ ) {
                    this.addParam( keys[ i ],param.getParam( keys[ i ],j ) ) ;
                }
            }
        }
    }
    
    /**
     * パラメータ追加.
     * <BR><BR>
     * パラメータを追加します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param value 対象の要素を設定します.
     */
    public void addParam( String key,Object value ) {
        pushParam( false,key,value ) ;
    }
    
    /**
     * パラメータ設定.
     * <BR><BR>
     * パラメータを設定します.<BR>
     * この条件で設定した場合、以前に登録したkeyに
     * 対する内容はクリアされます.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param value 対象の要素を設定します.
     */
    public void setParam( String key,Object value ) {
        pushParam( true,key,value ) ;
    }
    
    /**
     * パラメータを追加／設定.
     */
    private void pushParam( boolean mode,String key,Object value ) {
        Object val = null ;
        if( key == null ) {
            return ;
        }
        if( params == null ) {
            params = new THashMap<String,ArrayList<Object>>() ;
        }
        if( value == null ) {
            val = new NullObjectParam() ;
        }
        else {
            val = value ;
        }
        if( mode == true ) {
            ArrayList<Object> lst = new ArrayList<Object>() ;
            lst.add( val ) ;
            params.put( key,lst ) ;
        }
        else {
            ArrayList<Object> lst = params.get( key ) ;
            if( lst == null ) {
                lst = new ArrayList<Object>() ;
                params.put( key,lst ) ;
            }
            lst.add( val ) ;
        }
    }
    
    /**
     * パラメータを削除.
     * <BR><BR>
     * 対象のパラメータを削除します.
     * <BR>
     * @param key 対象のキー名を設定します.
     */
    public void removeParam( String key ) {
        if( key == null || params == null ) {
            return ;
        }
        params.remove( key ) ;
    }
    
    /**
     * パラメータを削除.
     * <BR><BR>
     * 対象のパラメータを削除します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param no 対象の項番を設定します.
     */
    public void removeParam( String key,int no ) {
        if( key == null || params == null || no < 0 ) {
            return ;
        }
        ArrayList<Object> lst = params.get( key ) ;
        if( lst == null || lst.size() <= no ) {
            return ;
        }
        lst.remove( no ) ;
        if( lst.size() <= 0 ) {
            params.remove( key ) ;
        }
    }
    
    /**
     * パラメータを取得.
     * <BR><BR>
     * 対象のパラメータを取得します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @return Object 対象の要素情報が返されます.
     */
    public String getParam( String key ) {
        return getParam( key,0 ) ;
    }
    
    /**
     * パラメータを取得.
     * <BR><BR>
     * 対象のパラメータを取得します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @param no 対象の項番を設定します.
     * @return Object 対象の要素情報が返されます.
     */
    public String getParam( String key,int no ) {
        if( key == null || params == null || no < 0 ) {
            return null ;
        }
        ArrayList<Object> lst = params.get( key ) ;
        if( lst == null || lst.size() <= no ) {
            return null ;
        }
        Object ret = lst.get( no ) ;
        if( ret == null || ( ret instanceof NullObjectParam ) ) {
            return null ;
        }
        return ( String )ret ;
    }
    
    /**
     * パラメータ数を取得.
     * <BR><BR>
     * 対象のパラメータ数を取得します.
     * <BR>
     * @return int パラメータ数が返されます.
     */
    public int size() {
        return ( params == null ) ? 0 : params.size() ;
    }
    
    /**
     * パラメータ数を取得.
     * <BR><BR>
     * 対象のパラメータ数を取得します.
     * <BR>
     * @param key 対象のキー名を設定します.
     * @return int パラメータ数が返されます.
     */
    public int size( String key ) {
        if( key == null || params == null ) {
            return 0 ;
        }
        ArrayList<Object> lst = params.get( key ) ;
        if( lst == null ) {
            return 0 ;
        }
        return lst.size() ;
    }
    
    /**
     * キー名群を取得.
     * <BR><BR>
     * 対象のキー名群を取得します.
     * <BR>
     * @return String[] キー名群が返されます.
     */
    public String[] getKeys() {
        if( params == null || params.size() <= 0 ) {
            return null ;
        }
        Object[] keys = params.keySet().toArray() ;
        if( keys != null ) {
            int len = keys.length ;
            String[] ret = new String[ len ] ;
            for( int i = 0 ; i < len ; i ++ ) {
                ret[ i ] = ( String )keys[ i ] ;
            }
            return ret ;
        }
        return null ;
    }
    
    /**
     * 指定キーが存在するかチェック.
     * <BR><BR>
     * 指定キーが存在するかチェックします.
     * <BR>
     * @param key 対象のキー名を取得します.
     * @return boolean [true]の場合、存在します.
     */
    public boolean isKey( String key ) {
        if( params == null ) {
            return false ;
        }
        return ( params.get( key ) == null ) ? false : true ;
    }
    
    /**
     * 文字列に変換.
     */
    public String toString() {
        String[] keys = getKeys() ;
        if( keys != null ) {
            StringBuilder buf = new StringBuilder() ;
            int len = keys.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                if( i != 0 ) {
                    buf.append( "," ) ;
                }
                ArrayList<Object> lst = params.get( keys[ i ] ) ;
                if( lst != null ) {
                    int lenJ = lst.size() ;
                    for( int j = 0 ; j < lenJ ; j ++ ) {
                        if( j != 0 ) {
                            buf.append( "," ) ;
                        }
                        Object objs = lst.get( j ) ;
                        buf.append( " [" ).
                            append( keys[ i ] ).
                            append( ":" ) ;
                        if( objs == null || objs instanceof NullObjectParam ) {
                            buf.append( "]" ) ;
                        }
                        else {
                            buf.append( objs ).append( "]" ) ;
                        }
                    }
                }
            }
            return buf.toString() ;
        }
        else {
            return "null" ;
        }
    }
}

/**
 * NULL用オブジェクト.
 */
class NullObjectParam {
}

