package org.maachang.seabass.session.engine ;

import org.maachang.bdb.Bdb;
import org.maachang.bdb.BdbKeyValue;
import org.maachang.util.BaseRandom;

/**
 * 永続化セッション管理インターフェイス.
 * 
 * @version 2008/05/25
 * @author masahito suzuki
 * @since PersistenceSession 1.00
 */
public interface PersistenceEngine {
    
    /**
     * セッションIDサイズ.
     */
    public static final int SESSION_KEY_LENGTH = PersistenceDefine.SESSION_KEY_LENGTH ;
    
    /**
     * 新しいセッションキーを生成.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param baseRandom ランダムオブジェクトを設定します.
     * @return String 新しいSessionKeyが返されます.
     * @exception Exception 例外.
     */
    public String createSessionId( BdbKeyValue keyValue,BaseRandom baseRandom )
        throws Exception ;
    
    /**
     * セッションに１つの要素を設定.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param sessionId 対象のセッションIDを設定します.
     * @param key 対象のKeyを設定します.
     * @param value 対象の要素を設定します.
     * @exception Exception 例外.
     */
    public void put( BdbKeyValue keyValue,String sessionId,String key,byte[] value )
        throws Exception ;
    
    /**
     * セッションを削除.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param sessionId 対象のセッションIDを設定します.
     * @exception Exception 例外.
     */
    public void remove( BdbKeyValue keyValue,String sessionId ) throws Exception ;
    
    /**
     * セッションから１つの要素を削除.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param sessionId 対象のセッションIDを設定します.
     * @param key 対象のKeyを設定します.
     * @exception Exception 例外.
     */
    public void remove( BdbKeyValue keyValue,String sessionId,String key )
        throws Exception ;
    
    /**
     * セッションから１つの要素を取得.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param sessionId 対象のセッションIDを設定します.
     * @param key 対象のKeyを設定します.
     * @return byte[] １つの要素が返されます.
     * @exception Exception 例外.
     */
    public byte[] get( BdbKeyValue keyValue,String sessionId,String key )
        throws Exception ;
    
    /**
     * 指定セッション更新時間を取得.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param sessionId 対象のセッションIDを設定します.
     * @return long 対象の更新時間が返されます.
     * @exception Exception 例外.
     */
    public long getLastUpdateTime( BdbKeyValue keyValue,String sessionId ) throws Exception ;
    
    /**
     * 指定セッション情報が存在するかチェック.
     * @param keyValue スレッドで管理しているKeyValueオブジェクトを設定します.
     * @param sessionId 対象のセッションIDを設定します.
     * @return boolean [true]の場合、セッションIDは存在します.
     */
    public boolean containsKey( BdbKeyValue keyValue,String sessionId ) ;
    
    /**
     * Bdbオブジェクトを取得.
     * @return Bdb Bdbオブジェクトが返されます.
     */
    public Bdb getBdb() ;
    
    /**
     * セッション削除時間を取得.
     * @return long セッション削除時間が返されます.
     */
    public long getDeleteSessionTime() ;
    
    /**
     * このオブジェクトが有効かチェック.
     * @return boolean [true]が返された場合、オブジェクトは有効です.
     */
    public boolean isUse() ;
}

