package org.maachang.seabass.session.engine ;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.bdb.BdbManager;
import org.maachang.conf.Config;
import org.maachang.conf.ReadConfigUtil;

/**
 * 永続化サービス.
 * 
 * @version 2008/05/25
 * @author masahito suzuki
 * @since PersistenceSession 1.00
 */
public class PersistenceService {
    
    /**
     * ログ出力.
     */
    private static final Log LOG = LogFactory.getLog( PersistenceService.class ) ;
    
    /**
     * セッションコンフィグファイル名.
     */
    private static final String SESSION_CONF = "./conf/session.conf" ;
    
    /**
     * セッション定義セクション名.
     */
    private static final String SESSION_SECTION = "session-def" ;
    
    /**
     * シングルトン.
     */
    private static final PersistenceService SNGL = new PersistenceService() ;
    
    /**
     * 永続化オブジェクト.
     */
    private PersistenceEngineImpl engine = null ;
    
    /**
     * コンストラクタ.
     */
    private PersistenceService() {
    }
    
    /**
     * オブジェクトを取得.
     * @return PersistenceService オブジェクトが返されます.
     */
    public static final PersistenceService getInstance() {
        return SNGL ;
    }
    
    /**
     * オブジェクトを生成.
     * @param io 対象のBdbManagerを設定します.
     * @exception Exception 例外.
     */
    public synchronized void init( BdbManager io ) throws Exception {
        if( this.engine == null ) {
            LOG.info( "## PersistenceService - 生成." ) ;
            if( io == null || io.isDestroy() ) {
                throw new IllegalArgumentException( "指定BDB管理オブジェクトは不正です" ) ;
            }
            try {
                Config conf = new Config() ;
                if( ReadConfigUtil.readConfig( conf,SESSION_CONF ) == false ||
                    conf.isSection( SESSION_SECTION ) == false ) {
                    this.engine = new PersistenceEngineImpl( io,-1,-1 ) ;
                }
                else {
                    this.engine = new PersistenceEngineImpl( io,
                        conf.getInt( SESSION_SECTION,"cacheTime",0 ),
                        conf.getInt( SESSION_SECTION,"sessionTime",0 ) ) ;
                }
            } catch( Exception e ) {
                this.engine = null ;
                throw e ;
            }
        }
    }
    
    /**
     * オブジェクトを破棄.
     */
    public synchronized void destroy() {
        if( this.engine != null ) {
            LOG.info( "## PersistenceService - 破棄." ) ;
            try {
                this.engine.destroy() ;
            } catch( Exception e ) {
            }
        }
        this.engine = null ;
    }
    
    /**
     * PersistenceEngineオブジェクトを取得.
     * @return PersistenceEngine PersistenceEngineオブジェクトが返されます.
     */
    public synchronized PersistenceEngine getEngine() {
        return this.engine ;
    }
}
