package org.maachang.seabass.startjava ;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.conf.Config;
import org.maachang.util.FileUtil;

/**
 * Seabass開始／終了時のJavaプログラム管理.
 * 
 * @version 2009/12/17
 * @author masahito suzuki
 * @since SeaBass 1.0.0
 */
public class StartupJavaManager {
    private static final StartupJavaManager SNGL = new StartupJavaManager() ;
    private static final Log LOG = LogFactory.getLog( StartupJavaManager.class ) ;
    private static final String CONF_NAME = "conf/startup-java.conf" ;
    private static final String SECTION = "startup-java" ;
    private static final String KEY = "object" ;
    
    /** オブジェクト配列管理 **/
    private List<StartupJava> list = new ArrayList<StartupJava>() ;
    
    /** コンストラクタ. **/
    private StartupJavaManager() {
    }
    
    /**
     * オブジェクトを取得.
     * return StartupJavaManager オブジェクトが返されます.
     */
    public static final StartupJavaManager getInstance() {
        return SNGL ;
    }
    
    /**
     * このプログラムの初期化処理.
     * @exception Exception 例外.
     */
    public void init() throws Exception {
        if( !FileUtil.isFileExists( CONF_NAME ) ) {
            return ;
        }
        Config conf = Config.read( CONF_NAME ) ;
        int len = conf.size( SECTION,KEY ) ;
        if( len > 0 ) {
            int cnt = 0 ;
            LOG.info( "## StartupJavaManager開始" ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                String s = conf.get( SECTION,KEY,i ) ;
                if( s == null || ( s = s.trim() ).length() <= 0 ) {
                    continue ;
                }
                LOG.info( "## 起動プログラム:" + s + "を読み込み中" ) ;
                try {
                    Object o = Class.forName( s ).newInstance() ;
                    if( o == null || !( o instanceof StartupJava ) ) {
                        LOG.error( "## 起動プログラム:" + s + "を読み込み失敗" ) ;
                    }
                    else {
                        list.add( (StartupJava)o ) ;
                        cnt ++ ;
                    }
                } catch( Exception e ) {
                    LOG.error( "## 起動プログラム:" + s + "を読み込み失敗",e ) ;
                }
            }
            LOG.info( "## StartupJavaManager終了(" + cnt + "/" + len + ")" ) ;
        }
    }
    
    /**
     * 開始処理.
     * @exception Exception 例外.
     */
    public void startup() throws Exception {
        int len = list.size() ;
        StartupJava j = null ;
        for( int i = 0 ; i < len ; i ++ ) {
            j = list.get( i ) ;
            LOG.info( "## [init]外部Javaプログラム:" + j.getName() + " ...開始" ) ;
            try {
                j.startup() ;
                LOG.info( "## [init]外部Javaプログラム:" + j.getName() + " ...起動成功" ) ;
            } catch( Exception e ) {
                LOG.error( "## [init]外部Javaプログラム:" + j.getName() + " ...起動失敗",e ) ;
                throw e ;
            }
        }
    }
    
    /**
     * 終了処理.
     */
    public void destroy() {
        int len = list.size() ;
        StartupJava j = null ;
        for( int i = len-1 ; i >= 0 ; i -- ) {
            j = list.get( i ) ;
            LOG.info( "## [destroy]外部Javaプログラム:" + j.getName() + " ...開始" ) ;
            try {
                j.destroy() ;
                LOG.info( "## [destroy]外部Javaプログラム:" + j.getName() + " ...正常終了" ) ;
            } catch( Exception e ) {
                LOG.error( "## [destroy]外部Javaプログラム:" + j.getName() + " ...異常終了",e ) ;
            }
        }
    }
}
