package org.maachang ;

import java.util.* ;
import java.math.* ;
import java.lang.reflect.* ;
import pnuts.lang.* ;
import org.maachang.seabass.* ;

/**
 * Json変換処理.
 * 
 * @version 2008/12/25
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public abstract class Json {
    private Json(){}
    
    /**
     * JSON形式から、オブジェクト変換.
     * @param context 対象のコンテキストを設定します.
     * @param json 対象のJSON情報を設定します.
     * @return Object 変換されたJSON情報が返されます.
     */
    public static final Object decodeJSON( Context context,String json ) {
        return ModUtil.eval( context,false,"json",json ) ;
    }
    
    /**
     * JSON変換.
     * @param mode [true]の場合、byte配列条件に対して、Base64変換処理を行います.
     * @param target 対象のターゲットオブジェクトを設定します.
     * @return String 変換されたJSON情報が返されます.
     */
    public static final String encodeJSON( boolean mode,Object target ) {
        StringBuilder buf = new StringBuilder( 4096 ) ;
        switchJson( mode,buf,target,target ) ;
        return buf.toString() ;
    }
    
    private static final void switchJson( boolean mode,StringBuilder buf,Object base,Object target ) {
        if( target instanceof SeaBassMemory ) {
            // このオブジェクトを参照している場合、処理しない.
            return ;
        }
        if( target instanceof Map ) {
            jsonMap( mode,buf,base,(Map)target ) ;
        }
        else if( target instanceof List ) {
            jsonList( mode,buf,base,(List)target ) ;
        }
        else if( target instanceof Short ||
            target instanceof Integer ||
            target instanceof Long ||
            target instanceof Float ||
            target instanceof Double ||
            target instanceof BigInteger ||
            target instanceof BigDecimal ) {
            buf.append( target ) ;
        }
        else if( target instanceof Character ||
            target instanceof String ) {
            buf.append( "\"" ).append( target ).append( "\"" ) ;
        }
        else if( target instanceof byte[] ) {
            if( mode ) {
                buf.append( "decodeBase64(" ).append( "\"" ).
                    append( ModUtil.encodeBase64( (byte[])target ) ).append( "\"" ).append( ")" ) ;
            }
            else {
                buf.append( "null" ) ;
            }
        }
        else if( target instanceof char[] ) {
            buf.append( "\"" ).append( new String( (char[])target ) ).append( "\"" ) ;
        }
        else if( target instanceof java.util.Date ) {
            buf.append( "new Date(" ).append( ((java.util.Date)target).getTime() ).append( ")" ) ;
        }
        else if( target instanceof Boolean ) {
            buf.append( target ) ;
        }
        else if( target == null ) {
            buf.append( "null" ) ;
        }
        else if( target.getClass().isArray() ) {
            jsonArray( mode,buf,base,target ) ;
        }
        else {
            buf.append( "null" ) ;
        }
    }
    
    private static final void jsonMap( boolean mode,StringBuilder buf,Object base,Map map ) {
        boolean flg = false ;
        Map mp = ( Map )map ;
        Iterator it = mp.keySet().iterator() ;
        buf.append( "{" ) ;
        while( it.hasNext() ) {
            String key = ( String )it.next() ;
            Object value = mp.get( key ) ;
            if( base == value ) {
                continue ;
            }
            if( flg ) {
                buf.append( "," ) ;
            }
            flg = true ;
            buf.append( "\"" ).append( key ).append( "\":" ) ;
            switchJson( mode,buf,base,value ) ;
        }
        buf.append( "}" ) ;
    }
    
    private static final void jsonList( boolean mode,StringBuilder buf,Object base,List list ) {
        boolean flg = false ;
        List lst = ( List )list ;
        buf.append( "[" ) ;
        int len = lst.size() ;
        for( int i = 0 ; i < len ; i ++ ) {
            Object value = lst.get( i ) ;
            if( base == value ) {
                continue ;
            }
            if( flg ) {
                buf.append( "," ) ;
            }
            flg = true ;
            switchJson( mode,buf,base,value ) ;
        }
        buf.append( "]" ) ;
    }
    
    private static final void jsonArray( boolean mode,StringBuilder buf,Object base,Object list ) {
        boolean flg = false ;
        int len = Array.getLength( list ) ;
        buf.append( "[" ) ;
        for( int i = 0 ; i < len ; i ++ ) {
            Object value = Array.get( list,i ) ;
            if( base == value ) {
                continue ;
            }
            if( flg ) {
                buf.append( "," ) ;
            }
            flg = true ;
            switchJson( mode,buf,base,value ) ;
        }
        buf.append( "]" ) ;
    }
}
