package org.maachang ;

import java.io.*;
import java.util.*;
import java.math.*;
import java.text.*;
import java.lang.reflect.*;
import pnuts.lang.*;
import org.pnuts.util.*;
import org.maachang.*;
import org.maachang.atomic.*;
import org.maachang.seabass.* ;
import org.maachang.seabass.mod.object.* ;
import org.apache.oro.text.perl.*;
import gnu.trove.*;

/**
 * SeaBassモジュールユーティリティ.
 * 
 * @version 2008/12/25
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public abstract class ModUtil {
    private ModUtil(){}
    
    /** seabass version. **/
    public static final String VERSION = "1.0.0" ;
    
    /** フォーマット系スレッドローカル. **/
    private static final ThreadLocal<TempMap> formatLocal = new ThreadLocal<TempMap>() ;
    
    /** ランダムスレッドローカル. **/
    private static final ThreadLocal<XorRandom> randomLocal = new ThreadLocal<XorRandom>() ;
    
    /** スクリプトバージョン情報. **/
    private static final AtomicObject<String> scriptVersion = new AtomicObject<String>() ;
    
    /**
     * スクリプト実行.
     * @param context 実行コンテキストを設定します.
     * @param mode [true]の場合、SeaBass用スクリプト変換処理を行います.
     * @param name 実行名を設定します.
     * @param script 実行スクリプトを設定します.
     * @return Object 実行結果が返されます.
     */
    public static final Object eval( Context context,boolean mode,String name,String script ) {
        int befLine = context.getLine() ;
        context.setLine( -1 ) ;
        try {
            context.pushFileName( name ) ;
            PnutsException.clearPnutsErrorLineInfo( context ) ; // エラー情報をクリア.
            Object ret = null ;
            if( mode ) {
                ret = Pnuts.load( new StringReader( SeaBassScriptEngine.convertScript( script ) ),context ) ;
            }
            else {
                ret = Pnuts.load( new StringReader( script ),context ) ;
            }
            return ret ;
        } catch( Exception e ) {
            if( context.getLine() == -1 && befLine != -1 ) {
                context.setLine( befLine ) ;
            }
            SeaBassContext seabass = null ;
            pnuts.lang.Package pkg = context.getCurrentPackage() ;
            if( pkg instanceof SeaBassBindings ) {
                seabass = (SeaBassContext)( ( SeaBassBindings )pkg ).getMemory().get( "context" ) ;
            }
            if( SeaBassScriptEngine.toException( e,name,seabass,context ) != null ) {
                throw (PnutsException)e ;
            }
            if( e instanceof PnutsException ) {
                throw (PnutsException)e ;
            }
            throw new PnutsException( e,context ) ;
        } finally {
            pnuts.lang.Package pkg = context.getCurrentPackage() ;
            if( pkg instanceof SeaBassBindings ) {
                ( ( SeaBassBindings )pkg ).getMemory().remove( "_$lastError" ) ;
            }
            context.setLine( befLine ) ;
            context.popFileName();
        }
    }
    
    private static final SeaBassScriptEngine engine = new SeaBassScriptEngine() ;
    
    /**
     * コンパイル処理.
     * @param ctx 実行コンテキストを設定します.
     * @param name 実行名を設定します.
     * @param script 実行スクリプトを設定します.
     * @return SeabassCompile コンパイルオブジェクトが返されます.
     */
    public static final SeaBassCompile compile( Context context,String name,String script ) {
        SeaBassContext seabassContext = null ;
        pnuts.lang.Package pkg = context.getCurrentPackage() ;
        if( pkg instanceof SeaBassBindings ) {
            seabassContext = (SeaBassContext)( ( SeaBassBindings )pkg ).getMemory().get( "context" ) ;
        }
        else {
            throw new PnutsException( "contextにSeabassContext情報が存在しません",context ) ;
        }
        SeaBassCompile comp = null ;
        seabassContext.setName( name ) ;
        try {
            return engine.compile( script,seabassContext ) ;
        } catch( Exception e ) {
            if( e instanceof PnutsException ) {
                throw (PnutsException)e ;
            }
            throw new PnutsException( e,context ) ;
        }
    }
    
    /**
     * NumberFormatを取得.
     * @param format 対象のフォーマット値を設定します.
     * @return NumberFormat NumberFormatオブジェクトが返されます.
     */
    public static final NumberFormat getNumberFormat( String format ) {
        if( format == null || ( format = trimString( format ) ).length() <= 0 ) {
            return null ;
        }
        TempMap map = formatLocal.get() ;
        if( map == null ) {
            map = TempMap.getInstance() ;
            formatLocal.set( map ) ;
        }
        Object o = map.get( format ) ;
        if( o == null || ( o instanceof NumberFormat ) == false ) {
            o = new DecimalFormat( format ) ;
            map.put( format,o ) ;
        }
        return (NumberFormat)o ;
    }
    
    /**
     * 日付フォーマットを取得.
     * @param format 対象のフォーマット値を設定します.
     * @return SimpleDateFormat SimpleDateFormatオブジェクトが返されます.
     */
    public static final SimpleDateFormat getDateFormat( String format ) {
        if( format == null || ( format = trimString( format ) ).length() <= 0 ) {
            return null ;
        }
        TempMap map = formatLocal.get() ;
        if( map == null ) {
            map = TempMap.getInstance() ;
            formatLocal.set( map ) ;
        }
        Object o = map.get( format ) ;
        if( o == null || ( o instanceof SimpleDateFormat ) == false ) {
            o = new SimpleDateFormat( format,Locale.US ) ;
            map.put( format,o ) ;
        }
        return (SimpleDateFormat)o ;
    }
    
    /**
     * ランダムオブジェクトを取得.
     * @return XroRandom ランダムオブジェクトが返されます.
     */
    public static final XorRandom getRandom() {
        XorRandom ret = randomLocal.get() ;
        if( ret == null ) {
            ret = new XorRandom() ;
            randomLocal.set( ret ) ;
        }
        return ret ;
    }
    
    /**
     * スクリプトバージョンを取得.
     * @return String スクリプトバージョンが返されます.
     */
    public static final String version() {
        if( scriptVersion.get() == null ) {
            try {
                Class.forName("java.lang.Package");
                java.lang.Package pkg = Pnuts.class.getPackage();
                String s = new StringBuilder().append( pkg.getSpecificationVersion() ).append( " (" ).
                    append( pkg.getImplementationVersion() ).append( ")" ).toString() ;
                scriptVersion.set( s ) ;
            } catch (ClassNotFoundException e) {
                return Pnuts.pnuts_version;
            }
        }
        return scriptVersion.get() ;
    }
    
    /**
     * オブジェクトタイプを取得.
     * @param o 対象のオブジェクトを設定します.
     * @return String オブジェクトタイプが返されます.
     */
    public static final String typeof( Object o ) {
        if( o == null ) {
            return "null" ;
        }
        if( o instanceof Boolean ) {
            return "boolean" ;
        }
        else if( o instanceof Short ||
            o instanceof Character ||
            o instanceof Integer ||
            o instanceof Long ||
            o instanceof Float ||
            o instanceof Double ||
            o instanceof BigInteger ||
            o instanceof BigDecimal ) {
            return "number" ;
        }
        else if( o instanceof String ) {
            return "string" ;
        }
        else if( o instanceof char[] ) {
            return "char[]" ;
        }
        else if( o instanceof byte[] ) {
            return "binary" ;
        }
        else if( o instanceof JSAdapter ) {
            return "jsadapter" ;
        }
        else if( o instanceof Map ) {
            return "map" ;
        }
        else if( o instanceof List || o instanceof Object[] ) {
            return "array" ;
        }
        else if( o instanceof java.util.Date ) {
            return "date" ;
        }
        else if( o instanceof PnutsFunction ) {
            return "function" ;
        }
        return "object" ;
    }
    
    /**
     * オブジェクトタイプを詳しく取得.
     * <p>typeofより、詳しいオブジェクトタイプが返されます.</p>
     * @param o 対象のオブジェクトを設定します.
     * @return String オブジェクトタイプが返されます.
     */
    public static final String valueof( Object o ) {
        if( o == null ) {
            return "null" ;
        }
        if( o instanceof Boolean ) {
            return "boolean" ;
        }
        else if( o instanceof Short ||
            o instanceof Character ||
            o instanceof BigInteger ||
            o instanceof Integer ||
            o instanceof Long ) {
            return "number" ;
        }
        else if( o instanceof Float ||
            o instanceof Double ||
            o instanceof BigDecimal ) {
            return "float" ;
        }
        else if( o instanceof String ) {
            return "string" ;
        }
        else if( o instanceof char[] ) {
            return "char[]" ;
        }
        else if( o instanceof byte[] ) {
            return "binary" ;
        }
        else if( o instanceof JSAdapter ) {
            return "jsadapter" ;
        }
        else if( o instanceof Map ) {
            return "map" ;
        }
        else if( o instanceof List ) {
            return "array" ;
        }
        else if( o instanceof java.util.Date ) {
            return "date" ;
        }
        else if( o instanceof PnutsFunction ) {
            return "function" ;
        }
        else if( o.getClass().isArray() ) {
            return "object[]" ;
        }
        return "object" ;
    }
    
    /**
     * 有効な文字列が存在するかチェック.
     * @param string 対象の文字列を設定します.
     * @return boolean [true]の場合、有効な文字列が存在します.
     */
    public static final boolean useString( String string ) {
        if( string == null || string.length() <= 0 ) {
            return false ;
        }
        int len = string.length() ;
        for( int i = 0 ; i < len ; i ++ ) {
            char c = string.charAt( i ) ;
            if( c != ' ' && c != '\t' && c != '　' ) {
                return true ;
            }
        }
        return false ;
    }
    
    /**
     * 文字列をtrim.
     * @param string 対象の文字列を設定します.
     * @return String trimされた文字列が返されます.
     */
    public static final String trimString( String string ) {
        int s = -1 ;
        int e = -1 ;
        int len = string.length() ;
        s = 0 ;
        boolean sFlg = false ;
        for( int i = 0 ; i < len ; i ++ ) {
            char c = string.charAt( i ) ;
            if( c != ' '  && c != '　' && c != '\t' ) {
                s = i ;
                break ;
            }
            sFlg = true ;
        }
        e = len-1 ;
        boolean eFlg = false ;
        for( int i = len-1 ; i >= 0 ; i -- ) {
            char c = string.charAt( i ) ;
            if( c != ' '  && c != '　' && c != '\t' ) {
                e = i ;
                break ;
            }
            eFlg = true ;
        }
        if( sFlg == true && eFlg == true ) {
            return string.substring( s,e+1 ) ;
        }
        else if( sFlg == true ) {
            return string.substring( s ) ;
        }
        else if( eFlg == true ) {
            return string.substring( 0,e+1 ) ;
        }
        return string ;
    }
    
    /**
     * 文字列を指定条件でカット.
     * @param out カットされた内容が返されます.
     * @param mode [true]の場合、連続した区切りを示す条件となります.
     * @param str カット対象の文字列を設定します.
     * @param check カット対象の文字列群を設定します.
     */
    public static final void cutString(List<String> out, boolean mode,
        String str, String check) throws IllegalArgumentException {
        int i, j;
        int len;
        int lenJ;
        int s = -1;
        char strCode;
        char[] checkCode = null;
        String tmp = null;
        if (out == null || str == null || (len = str.length()) <= 0
                || check == null || check.length() <= 0) {
            throw new IllegalArgumentException("引数が不正です");
        }
        out.clear();
        lenJ = check.length();
        checkCode = new char[lenJ];
        check.getChars(0, lenJ, checkCode, 0);
        if (lenJ == 1) {
            for (i = 0, s = -1; i < len; i++) {
                strCode = str.charAt(i);
                s = (s == -1) ? i : s;
                if (strCode == checkCode[0]) {
                    if (s < i) {
                        tmp = str.substring(s, i);
                        out.add(tmp);
                        tmp = null;
                        s = -1;
                    } else if (mode == true) {
                        out.add("");
                        s = -1;
                    } else {
                        s = -1;
                    }
                }
            }
        } else {
            for (i = 0, s = -1; i < len; i++) {
                strCode = str.charAt(i);
                s = (s == -1) ? i : s;
                for (j = 0; j < lenJ; j++) {
                    if (strCode == checkCode[j]) {
                        if (s < i) {
                            tmp = str.substring(s, i);
                            out.add(tmp);
                            tmp = null;
                            s = -1;
                        } else if (mode == true) {
                            out.add("");
                            s = -1;
                        } else {
                            s = -1;
                        }
                        break;
                    }
                }
            }
        }
        if (s != -1) {
            tmp = str.substring(s, len);
            out.add(tmp);
            tmp = null;
        }
        checkCode = null;
        tmp = null;
    }
    
    /**
     * 文字列内の指定内容を置き換える.
     * @param str 対象の文字列を設定します.
     * @param src 変換元の文字列を設定します.
     * @param dest 変換先の文字列を設定します.
     * @return String 変換された内容が返されます.
     */
    public static final String changeString(String str, String src, String dest)
        throws IllegalArgumentException {
        int pnt;
        int end;
        int srcLen;
        StringBuilder buf = null;
        if (str == null || str.length() <= 0 || src == null
                || (srcLen = src.length()) <= 0) {
            if( str == null ) {
                return "" ;
            }
            return str ;
        }
        if (dest == null || dest.length() <= 0) {
            dest = "";
        }
        buf = new StringBuilder();
        pnt = str.indexOf(src);
        if (pnt == -1) {
            return str;
        } else if (pnt == 0) {
            buf.append(dest);
            end = srcLen;
        } else {
            buf.append(str.substring(0, pnt));
            buf.append(dest);
            end = pnt + srcLen;
        }
        for (pnt += srcLen;;) {
            if ((pnt = str.indexOf(src, pnt)) == -1) {
                buf.append(str.substring(end));
                break;
            }
            if (pnt == end) {
                buf.append(dest);
            } else {
                buf.append(str.substring(end, pnt));
                buf.append(dest);
            }
            end = pnt + srcLen;
            pnt += srcLen;
        }
        return buf.toString();
    }
    
    /**
     * Perl5用正規表現処理.
     * @param match マッチ対象の正規表現を設定します.
     * @param src 対象の文字列を設定します.
     * @return boolean [true]の場合、マッチします.
     */
    public static boolean isPerl5Util_Match( String match,String src ) {
        try {
            if( match == null || match.length() <= 0 ) {
                return false ;
            }
            if( src == null || ( src = src.trim() ).length() <= 0 ) {
                return false ;
            }
            return !( new Perl5Util().match( match,src ) ) ;
        } catch( Exception e ) {
            return false ;
        }
    }
    
    /**
     * 文字列に変換.
     * @param o 変換対象のオブジェクトを設定します.
     * @return String 変換された文字列が返されます.
     */
    public static final String parseString(Object o) {
        if( o == null ) {
            return null ;
        }
        if( o instanceof String ) {
            return (String)o ;
        }
        o = ModUtil.parseValue( o ) ;
        if( o instanceof java.util.Date ) {
            return ModUtil.getDateFormat( "yyyy/MM/dd HH:mm:ss" ).format( (java.util.Date)o ) ;
        }
        else if( o instanceof Map || o instanceof List || o.getClass().isArray() ) {
            return Json.encodeJSON( true,o ) ;
        }
        return o.toString() ;
    }
    
    /**
     * 最適な型に変換.
     * @param o 変換対象のオブジェクトを設定します.
     * @return Object 変換された内容が返されます.
     */
    public static final Object parseValue(Object o) {
        int[] dot = new int[ 1 ] ;
        if( isNumber( dot,o ) ) {
            if( dot[ 0 ] == 0x00000000 ) {
                return integerOrLong( o ) ;
            }
            else if( dot[ 0 ] == 0x00000001 ) {
                return convertNumber( ( Double )o ) ;
            }
            else if( dot[ 0 ] == 0x00000002 ) {
                return convertNumber( ( Float )o ) ;
            }
            else if( dot[ 0 ] == 0x00000003 ) {
                String s = o.toString() ;
                if( s.length() > 1 && s.charAt( 0 ) == '0' ) {
                    return s ;
                }
                return integerOrLong( o ) ;
            }
            else if( dot[ 0 ] == 0x00000004 ) {
                return convertNumber( o.toString() ) ;
            }
        }
        dot = null ;
        String value = null ;
        if( o instanceof String ) {
            value = trimString(( String )o) ;
            String x = value.toLowerCase() ;
            if( "true".equals( x ) ) {
                return Boolean.TRUE ;
            }
            else if( "false".equals( x ) ) {
                return Boolean.FALSE ;
            }
            else if( value.startsWith( "0x" ) ) {
                return integerOrLong16( value ) ;
            }
            if( value.startsWith( "__$formatter@date@=" ) ) {
                x = value.substring( "__$formatter@date@=".length() ) ;
                try {
                    java.util.Date ret = new java.util.Date( Long.parseLong( x ) ) ;
                    return ret ;
                } catch( Exception e ) {
                }
            }
            if( value.indexOf( "new" ) != -1 ) {
                try {
                    List<String> lst = new ArrayList<String>() ;
                    cutString( lst,false,value,"_=\r\n 　\t()," ) ;
                    int len = lst.size() ;
                    if( "new".equals( lst.get( 0 ) ) ) {
                        String code = lst.get( 1 ) ;
                        String args = null ;
                        for( int i = 2 ; i < len ; i ++ ) {
                            if( lst.get( i ).length() > 0 ) {
                                args = lst.get( i ) ;
                                break ;
                            }
                        }
                        if( "Date".equals( code ) ) {
                            return new java.util.Date( Long.parseLong( args ) ) ;
                        }
                        else if( "Time".equals( code ) ) {
                            return new java.sql.Time( Long.parseLong( args ) ) ;
                        }
                        else if( "Timestamp".equals( code ) ) {
                            return new java.sql.Timestamp( Long.parseLong( args ) ) ;
                        }
                    }
                } catch( Exception e ) {
                }
            }
        }
        return o ;
    }
    
    /**
     * Int型に変換.
     * @param o 対象のオブジェクトを設定します.
     */
    public static final Object parseInt(Object o)
        throws Exception {
        if( o != null ) {
            String s = null ;
            if( o instanceof Integer || o instanceof Long || o instanceof Short ) {
                return o ;
            }
            if( o instanceof Float ) {
                return ( int )((Float)o).floatValue() ;
            }
            else if( o instanceof Double ) {
                return ( int )((Double)o).doubleValue() ;
            }
            else if( o instanceof String ) {
                s = (String)o ;
            }
            else {
                s = o.toString() ;
            }
            int p = s.indexOf( "." ) ;
            if( p >= 0 ) {
                s = s.substring( 0,p ) ;
            }
            return integerOrLong( s ) ;
        }
        else {
            throw new IOException( "指定条件が不正です:null" ) ;
        }
    }
    
    /**
     * 数値変換.
     * @param o 変換対象のオブジェクトを設定します.
     * @return Object 変換された情報が返されます.
     */
    public static final Object parseNumber(Object o)
        throws Exception {
        int[] dot = new int[ 1 ] ;
        if( isNumber( dot,o ) ) {
            if( dot[ 0 ] == 0x00000000 ) {
                return o ;
            }
            else if( dot[ 0 ] == 0x00000001 ) {
                return ( Double )o ;
            }
            else if( dot[ 0 ] == 0x00000002 ) {
                return ( Float )o ;
            }
            else if( dot[ 0 ] == 0x00000003 ) {
                String s = o.toString() ;
                while(true) {
                    if( s.length() > 1 && s.charAt( 0 ) == '0' ) {
                        s = s.substring( 1 ) ;
                        if( s.length() <= 0 ) {
                            return new Integer( 0 ) ;
                        }
                        continue ;
                    }
                    return new Integer( o.toString() ) ;
                }
            }
            else if( dot[ 0 ] == 0x00000004 ) {
                return new Double( o.toString() ) ;
            }
        }
        throw new PnutsException( "数値変換に失敗しました:" + o + ":" + o.getClass().getName() ) ;
    }
    
    /**
     * 指定内容が数値かチェック.
     * @param dot 数値条件が返されます.
     * @param o 対象のオブジェクトを設定します.
     * @return boolean [true]の場合、数値です.
     */
    public static final boolean isNumber( int[] dot,Object o ) {
        if( dot != null && dot.length > 0 ) {
            dot[ 0 ] = 0x00000000 ;
        }
        if( o instanceof Double || o instanceof Float ||
            o instanceof Integer || o instanceof Long || o instanceof Short ) {
            if( dot != null && dot.length > 0 ) {
                if( o instanceof Double ) {
                    dot[ 0 ] = 0x00000001 ;
                }
                else if( o instanceof Float ) {
                    dot[ 0 ] = 0x00000002 ;
                }
            }
            return true ;
        }
        if( dot != null && dot.length > 0 ) {
            dot[ 0 ] = 0x00000003 ;
        }
        return isNumberByString( dot,o ) ;
    }

    /**
     * [Base64]余りデコード値.
     */
    private static final char REMAINDER_ENC = '=';

    /**
     * [Base64]エンコード表.
     */
    private static final char[] ENC_CD = { 'A', 'B', 'C', 'D', 'E', 'F', 'G',
            'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
            'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
            'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
            'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6',
            '7', '8', '9', '+', '/' };

    /**
     * [Base64]デコード表.
     */
    private static final int[] DEC_CD = { 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x0000003e, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x0000003f,
            0x00000034, 0x00000035, 0x00000036, 0x00000037, 0x00000038,
            0x00000039, 0x0000003a, 0x0000003b, 0x0000003c, 0x0000003d,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x00000040, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x00000000, 0x00000001, 0x00000002, 0x00000003,
            0x00000004, 0x00000005, 0x00000006, 0x00000007, 0x00000008,
            0x00000009, 0x0000000a, 0x0000000b, 0x0000000c, 0x0000000d,
            0x0000000e,
            0x0000000f, 0x00000010, 0x00000011, 0x00000012, 0x00000013,
            0x00000014, 0x00000015, 0x00000016, 0x00000017, 0x00000018,
            0x00000019, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x0000001a, 0x0000001b, 0x0000001c, 0x0000001d,
            0x0000001e, 0x0000001f, 0x00000020, 0x00000021, 0x00000022,
            0x00000023, 0x00000024, 0x00000025, 0x00000026, 0x00000027,
            0x00000028,
            0x00000029, 0x0000002a, 0x0000002b, 0x0000002c, 0x0000002d,
            0x0000002e, 0x0000002f, 0x00000030, 0x00000031, 0x00000032,
            0x00000033, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff,
            0x7fffffff };
    
    /**
     * エンコード処理.
     * @param binary エンコード対象のバイナリ情報を設定します.
     * @return String Base64にエンコードされた文字情報が返されます.
     */
    public static final String encodeBase64(byte[] binary) {
        int i, j, k;
        int len;
        int allLen;
        int etc;
        String ret = null;
        char[] ary = null;
        if (binary == null || (allLen = binary.length) <= 0) {
            throw new IllegalArgumentException("引数は不正です");
        }
        etc = allLen % 3;
        len = allLen / 3;
        ary = new char[(len * 4) + ((etc != 0) ? 4 : 0)];
        for (i = 0, j = 0, k = 0; i < len; i++, j += 3, k += 4) {
            ary[k] = ENC_CD[(int) ((binary[j] & 0x000000fc) >> 2)];
            ary[k + 1] = ENC_CD[(int) (((binary[j] & 0x00000003) << 4) | ((binary[j + 1] & 0x000000f0) >> 4))];
            ary[k + 2] = ENC_CD[(int) (((binary[j + 1] & 0x0000000f) << 2) | ((binary[j + 2] & 0x000000c0) >> 6))];
            ary[k + 3] = ENC_CD[(int) (binary[j + 2] & 0x0000003f)];
        }
        switch (etc) {
        case 1:
            j = len * 3;
            k = len * 4;
            ary[k] = ENC_CD[(int) ((binary[j] & 0x000000fc) >> 2)];
            ary[k + 1] = ENC_CD[(int) ((binary[j] & 0x00000003) << 4)];
            ary[k + 2] = '=';
            ary[k + 3] = '=';
            break;
        case 2:
            j = len * 3;
            k = len * 4;
            ary[k] = ENC_CD[(int) ((binary[j] & 0x000000fc) >> 2)];
            ary[k + 1] = ENC_CD[(int) (((binary[j] & 0x00000003) << 4) | ((binary[j + 1] & 0x000000f0) >> 4))];
            ary[k + 2] = ENC_CD[(int) (((binary[j + 1] & 0x0000000f) << 2))];
            ary[k + 3] = '=';
            break;
        }
        ret = new String(ary, 0, ary.length);
        ary = null;
        return ret;
    }
    
    /**
     * デコード処理.
     * @param base64 対象のBase64データを設定します.
     * @return byte[] 変換されたバイナリ情報が返されます.
     * @exception IllegalArgumentExceptio 入力例外.
     */
    public static final byte[] decodeBase64(String base64) {
        int i, j, k;
        int len;
        int allLen;
        int etc;
        byte[] ret = null;
        if (base64 == null || (allLen = base64.length()) <= 0) {
            throw new IllegalArgumentException("引数は不正です");
        }
        for (i = allLen - 1, etc = 0; i >= 0; i--) {
            if (base64.charAt(i) == '=') {
                etc++;
            } else {
                break;
            }
        }
        len = allLen / 4;
        ret = new byte[(len * 3) - etc];
        len -= 1;
        for (i = 0, j = 0, k = 0; i < len; i++, j += 4, k += 3) {
            ret[k] = (byte) (((DEC_CD[base64.charAt(j)] & 0x0000003f) << 2) | ((DEC_CD[base64
                    .charAt(j + 1)] & 0x00000030) >> 4));
            ret[k + 1] = (byte) (((DEC_CD[base64.charAt(j + 1)] & 0x0000000f) << 4) | ((DEC_CD[base64
                    .charAt(j + 2)] & 0x0000003c) >> 2));
            ret[k + 2] = (byte) (((DEC_CD[base64.charAt(j + 2)] & 0x00000003) << 6) | (DEC_CD[base64
                    .charAt(j + 3)] & 0x0000003f));
        }
        switch (etc) {
        case 0:
            j = len * 4;
            k = len * 3;
            ret[k] = (byte) (((DEC_CD[base64.charAt(j)] & 0x0000003f) << 2) | ((DEC_CD[base64
                    .charAt(j + 1)] & 0x00000030) >> 4));
            ret[k + 1] = (byte) (((DEC_CD[base64.charAt(j + 1)] & 0x0000000f) << 4) | ((DEC_CD[base64
                    .charAt(j + 2)] & 0x0000003c) >> 2));
            ret[k + 2] = (byte) (((DEC_CD[base64.charAt(j + 2)] & 0x00000003) << 6) | (DEC_CD[base64
                    .charAt(j + 3)] & 0x0000003f));
            break;
        case 1:
            j = len * 4;
            k = len * 3;
            ret[k] = (byte) (((DEC_CD[base64.charAt(j)] & 0x0000003f) << 2) | ((DEC_CD[base64
                    .charAt(j + 1)] & 0x00000030) >> 4));
            ret[k + 1] = (byte) (((DEC_CD[base64.charAt(j + 1)] & 0x0000000f) << 4) | ((DEC_CD[base64
                    .charAt(j + 2)] & 0x0000003c) >> 2));
            break;
        case 2:
            j = len * 4;
            k = len * 3;
            ret[k] = (byte) (((DEC_CD[base64.charAt(j)] & 0x0000003f) << 2) | ((DEC_CD[base64
                    .charAt(j + 1)] & 0x00000030) >> 4));
            break;
        }
        return ret;
    }
    
    /**
     * オブジェクトディープコピー.
     * @param obj コピー元のオブジェクトを設定します.
     * @return Object コピーされたオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public static Object deepCopy(Object obj) throws Exception {
        return deepCopy( false,obj ) ;
    }
    
    /**
     * オブジェクトディープコピー.
     * @param mode [true]で、第二引数がjava.util.Map系の場合、put/removeを無効にするMapとして処理します.
     * @param obj コピー元のオブジェクトを設定します.
     * @return Object コピーされたオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public static Object deepCopy(boolean mode,Object obj) throws Exception {
        // null.
        if( obj == null ) {
            return null ;
        }
        // コピーできないオブジェクト.
        else if( obj instanceof SeaBassMemory || obj instanceof SeaBassBindings ||
            obj instanceof SeaBassContext || obj instanceof pnuts.lang.Context ) {
            return null ;
        }
        // 文字列.
        else if( obj instanceof String ) {
            return new String( (( String )obj).toCharArray() ) ;
        }
        // プリミティブ系.
        else if( obj instanceof Boolean ) {
            return new Boolean( ((Boolean)obj).booleanValue() ) ;
        }
        else if( obj instanceof Byte ) {
            return new Byte( ((Byte)obj).byteValue() ) ;
        }
        else if( obj instanceof Short ) {
            return new Short( ((Short)obj).shortValue() ) ;
        }
        else if( obj instanceof Character ) {
            return new Character( ((Character)obj).charValue() ) ;
        }
        else if( obj instanceof Integer ) {
            return new Integer( ((Integer)obj).intValue() ) ;
        }
        else if( obj instanceof Long ) {
            return new Long( ((Long)obj).longValue() ) ;
        }
        else if( obj instanceof Float ) {
            return new Float( ((Float)obj).floatValue() ) ;
        }
        else if( obj instanceof Double ) {
            return new Double( ((Double)obj).doubleValue() ) ;
        }
        // math系.
        else if( obj instanceof BigInteger ) {
            return new BigInteger( obj.toString() ) ;
        }
        else if( obj instanceof BigDecimal ) {
            return new BigDecimal( obj.toString() ) ;
        }
        // 日付関連.
        else if( obj instanceof java.util.Date ) {
            java.util.Date src = ( java.util.Date )obj ;
            return obj.getClass().getConstructor( Long.TYPE ).newInstance( src.getTime() ) ;
        }
        // バイナリ.
        else if( obj instanceof byte[] ) {
            byte[] src = (byte[])obj ;
            byte[] dest = new byte[ src.length ] ;
            System.arraycopy( src,0,dest,0,src.length ) ;
            return dest ;
        }
        // キャラクタ群.
        else if( obj instanceof char[] ) {
            char[] src = (char[])obj ;
            char[] dest = new char[ src.length ] ;
            System.arraycopy( src,0,dest,0,src.length ) ;
            return dest ;
        }
        // SeaBass用オブジェクト[StrBuf].
        else if( obj instanceof StrBuf ) {
            return new StrBuf( ((StrBuf)obj).ts() ) ;
        }
        // SeaBass用オブジェクト[JSAdapter].
        else if( obj instanceof JSAdapter ) {
            Map map = ((JSAdapter)obj).getMap() ;
            Map ret = new StringMap() ;
            Iterator it = map.keySet().iterator() ;
            while( it.hasNext() ) {
                Object key = it.next() ;
                ret.put( key,deepCopy( map.get( key ) ) ) ;
            }
            return new JSAdapter( ret ) ;
        }
        // リスト.
        else if( obj instanceof java.util.List ) {
            List list = (List)obj ;
            List ret = new SkipList() ;
            for( int i = 0 ; i < list.size() ; i ++ ) {
                ret.add( deepCopy( list.get( i ) ) ) ;
            }
            return ret ;
        }
        // マップ.
        else if( obj instanceof java.util.Map ) {
            boolean wMode = false ;
            Map map = (Map)obj ;
            Map ret = null ;
            if( mode && map instanceof Map ) {
                ret = new WriteContralMap() ;
                wMode = true ;
            }
            else {
                ret = new StringMap() ;
            }
            Iterator it = map.keySet().iterator() ;
            while( it.hasNext() ) {
                Object key = it.next() ;
                ret.put( key,deepCopy( map.get( key ) ) ) ;
            }
            if( wMode ) {
                // 書き込み不可にセット.
                ((WriteContralMap)ret).setWriteMode( false ) ;
            }
            return ret ;
        }
        // 配列.
        else if( obj.getClass().isArray() ) {
            int len = Array.getLength( obj ) ;
            Object ret = Array.newInstance( obj.getClass(),len ) ;
            for( int i = 0 ; i < len ; i ++ ) {
                Array.set( ret,i,deepCopy( Array.get( obj,i ) ) ) ;
            }
            return ret ;
        }
        // メソッド.
        else if( obj instanceof PnutsFunction ) {
            // メソッドの場合は、そのまま返す.
            return obj ;
        }
        // シリアライズ可能オブジェクト.
        else if( obj instanceof Serializable ) {
            // シリアライズ可能な内容はシリアライズしてコピー.
            ByteArrayOutputStream baos = null;
            ObjectOutputStream oos = null;
            ObjectInputStream ois = null;
            Object ret = null;
            try {
                baos = new ByteArrayOutputStream();
                oos = new ObjectOutputStream(baos);
                oos.writeObject(obj);
                oos.flush() ;
                ois = new ObjectInputStream(new ByteArrayInputStream(baos.toByteArray()));
                ret = ois.readObject();
                oos.close();
                ois.close();
                oos = null;
                ois = null;
            } catch( Exception e ) {
                throw new IOException( "オブジェクト[" + obj.getClass().getName() + "]の初期化に失敗しました" ) ;
            } finally {
                if( oos != null ) {
                    try {
                        oos.close();
                    } catch (Exception e) {
                    }
                }
                if( ois != null ) {
                    try {
                        ois.close();
                    } catch (Exception e) {
                    }
                }
            }
            return ret;
        }
        else {
            throw new IOException( "オブジェクト[" + obj.getClass().getName() + "]の初期化に失敗しました" ) ;
        }
    }
    
    /** ファイル出力. **/
    public static final void setFile(String name, byte[] binary)
            throws Exception {
        OutputStream buf = null;
        if (name == null || binary == null) {
            throw new IllegalArgumentException("引数は不正です");
        }
        try {
            buf = new BufferedOutputStream( new FileOutputStream( name ) ) ;
            buf.write(binary);
            buf.flush() ;
            buf.close() ;
            buf = null ;
        } catch (Exception e) {
            throw e;
        } finally {
            if( buf != null ) {
                try {
                    buf.close();
                } catch (Exception t) {
                }
            }
            buf = null;
        }
    }
    
    /** 指定引数が文字列の時の数値チェック処理. **/
    private static final boolean isNumberByString( int[] dot,Object o ) {
        String num = null ;
        if( o instanceof String ) {
            num = ( String )o ;
        }
        else {
            return false ;
        }
        if( num == null || ( num = ModUtil.trimString( num ) ).length() <= 0 ) {
            return false ;
        }
        int start = 0 ;
        if( num.startsWith( "-" ) ) {
            start = 1 ;
        }
        boolean dt = false ;
        int len = num.length() ;
        if( start < len ) {
            for( int i = start ; i < len ; i ++ ) {
                char c = num.charAt( i ) ;
                if( c == '.' ) {
                    if( dt ) {
                        return false ;
                    }
                    if( dot != null && dot.length > 0 ) {
                        dot[ 0 ] = 0x00000004 ;
                    }
                    dt = true ;
                }
                else if( ( c >= '0' && c <= '9' ) == false ) {
                    return false ;
                }
            }
        }
        else {
            return false ;
        }
        return true ;
    }
    
    /** 数値変換 **/
    private static final Object convertNumber( Object o ) {
        if( o == null ) {
            return null ;
        }
        if( o instanceof String ) {
            String s = ( String )o ;
            if( ".".equals( s ) || s.endsWith( "." ) ) {
                return s ;
            }
        }
        if( o instanceof Double ) {
            Double d = ( java.lang.Double )o ;
            long i = d.longValue() ;
            if( d.doubleValue() == i ) {
                return integerOrLong( String.valueOf( i ) ) ;
            }
        }
        else if( o instanceof Float ) {
            Float d = ( java.lang.Float )o ;
            long i = d.longValue() ;
            if( d.floatValue() == i ) {
                return integerOrLong( String.valueOf( i ) ) ;
            }
            else {
                o = new Double( d.doubleValue() ) ;
            }
        }
        else if( isNumberByString( null,o ) ) {
            String s = o.toString() ;
            int p = s.indexOf( "." ) ;
            if( p != -1 ) {
                int len = s.length() ;
                for( int i = p+1 ; i < len ; i ++ ) {
                    char c = s.charAt( i ) ;
                    if( c != '0' ) {
                        return new java.lang.Double( s ) ;
                    }
                }
                s = s.substring( 0,p ) ;
            }
            return integerOrLong( s ) ;
        }
        return o ;
    }
    
    private static final Long MAX_INTEGER = new Long( Integer.MAX_VALUE ) ;
    private static final Long MIN_INTEGER = new Long( Integer.MIN_VALUE ) ;
    
    /** 条件が文字列の時、数値内容によって、生成オブジェクトをLong or Integer選択 **/
    private static final Object integerOrLong( Object o ) {
        try {
            if( o instanceof String ) {
                String x = (String)o ;
                Long lng = new Long( x ) ;
                if( x.startsWith( "-" ) ) {
                    if( MIN_INTEGER > lng ) {
                        o = lng ;
                    }
                    else {
                        o = new Integer( x ) ;
                    }
                }
                else {
                    if( MAX_INTEGER < lng ) {
                        o = lng ;
                    }
                    else {
                        o = new Integer( x ) ;
                    }
                }
            }
            return o ;
        } catch( Exception e ) {
        }
        return o ;
    }
    
    /** (16進数版)条件が文字列の時、数値内容によって、生成オブジェクトをLong or Integer選択 **/
    private static final Object integerOrLong16( Object o ) {
        try {
            if( ( o instanceof Long ) == false && ( o instanceof Integer ) == false ) {
                String x = o.toString() ;
                if( x.startsWith( "0x" ) ) {
                    x = x.substring( "0x".length() ) ;
                    if( x.length() > 8 ) {
                        o = Long.parseLong( x,16 ) ;
                    }
                    else {
                        o = Integer.parseInt( x,16 ) ;
                    }
                }
                else {
                    o = integerOrLong( o ) ;
                }
            }
        } catch( Exception e ) {
        }
        return o ;
    }
}
