package org.maachang ;

import org.maachang.seabass.mod.base.* ;

/**
 * Parseユーティリティ.
 * 
 * @version 2008/12/21
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public abstract class ParseUtil {
    private ParseUtil() {}
    
    /**
     * コメント行を省いた情報を取得.
     * @param script 対象のスクリプトを設定します.
     * @return String コメント行を除外した内容が返されます.
     */
    public static final String cutComment( String script ) throws Exception {
        if( script == null || script.length() <= 0 ) {
            return "" ;
        }
        StringBuffer buf = new StringBuffer() ;
        int len = script.length() ;
        int cote = -1 ;
        int commentType = -1 ;
        int bef = -1 ;
        boolean yenFlag = false ;
        for( int i = 0 ; i < len ; i ++ ) {
            if( i != 0 ) {
                if( yenFlag ) {
                    bef = -1 ;
                    yenFlag = false ;
                }
                else {
                    bef = script.charAt( i-1 ) ;
                }
            }
            char c = script.charAt(i) ;
            // シングル／ダブルコーテーション内の処理.
            if( cote != -1 ) {
                if( c == cote && ( char )bef != '\\' ) {
                    cote = -1 ;
                    yenFlag = false ;
                }
                else if( c == '\\' && ( char )bef == '\\' ) {
                    yenFlag = true ;
                }
                else {
                    yenFlag = false ;
                }
                buf.append( c ) ;
                continue ;
            }
            // コメント内の処理.
            if( commentType != -1 ) {
                if( commentType == 1 ) {
                    if( c == '\n' || c == '\r' ) {
                        if( c == '\r' ) {
                            if( len > i + 1 && script.charAt( i+1 ) == '\n' ) {
                                i ++ ;
                                buf.append( "\r\n" ) ;
                            }
                            else {
                                buf.append( c ) ;
                            }
                        }
                        else {
                            buf.append( c ) ;
                        }
                        commentType = -1 ;
                    }
                }
                else if( commentType == 2 ) {
                    if( c == '*' ) {
                        if( len > i + 1 && script.charAt( i+1 ) == '/' ) {
                            i ++ ;
                            commentType = -1 ;
                        }
                    }
                }
                continue ;
            }
            // それ以外の処理.
            if( c == '/' ) {
                if( len <= i + 1 ) {
                    buf.append( c ) ;
                    continue ;
                }
                char c2 = script.charAt( i+1 ) ;
                if( c2 == '*' ) {
                    commentType = 2 ;
                    continue ;
                }
                else if( c2 == '/' ) {
                    commentType = 1 ;
                    continue ;
                }
            }
            else if( c == '\'' || c == '\"' ) {
                cote = ( int )( c & 0x0000ffff ) ;
            }
            buf.append( c ) ;
        }
        return buf.toString() ;
    }
    
    /**
     * 指定位置からの、有効文字列を取得.
     * @param outPos 検知情報の開始位置が返されます.
     * @param script 対象のスクリプトを設定します.
     * @param index 対象のインデックス値を設定します.
     */
    public static final String parseToSingle( int[] outPos,String script,int index ) {
        int len = script.length() ;
        if( index <= 0 ) {
            index = 0 ;
        }
        int cote = -1 ;
        char bef = 0 ;
        int pos = -1 ;
        boolean yenFlag = false ;
        for( int i = index ; i < len ; i ++ ) {
            char c = script.charAt( i ) ;
            if( cote != -1 ) {
                if( bef != '\\' && cote == c ) {
                    cote = -1 ;
                    yenFlag = false ;
                    if( pos >= 0 ) {
                        if( outPos != null && outPos.length >= 1 ) {
                            outPos[ 0 ] = pos ;
                        }
                        return script.substring( pos,i+1 ) ;
                    }
                }
                else if( c == '\\' && bef == '\\' ) {
                    yenFlag = true ;
                }
                else {
                    yenFlag = false ;
                }
            }
            else if( bef != '\\' && ( c == '\'' || c == '\"' ) ) {
                if( pos >= 0 ) {
                    if( outPos != null && outPos.length >= 1 ) {
                        outPos[ 0 ] = pos ;
                    }
                    return script.substring( pos,i ) ;
                }
                cote = c ;
                pos = i ;
            }
            else if( c == ' ' || c == '　' ||
                c == '\r' || c == '\n' || c == '\t' ) {
                if( pos >= 0 ) {
                    if( outPos != null && outPos.length >= 1 ) {
                        outPos[ 0 ] = pos ;
                    }
                    return script.substring( pos,i ) ;
                }
            }
            else if( c == '(' || c == '{' || c == '[' ||
                c == ')' || c == '}' || c == ']' ||
                c == ':' || c == ';' ) {
                if( pos >= 0 ) {
                    if( outPos != null && outPos.length >= 1 ) {
                        outPos[ 0 ] = pos ;
                    }
                    return script.substring( pos,i ) ;
                }
                else {
                    if( outPos != null && outPos.length >= 1 ) {
                        outPos[ 0 ] = i ;
                    }
                    return script.substring( i,i+1 ) ;
                }
            }
            else if( pos <= -1 ) {
                pos = i ;
            }
            if( yenFlag ) {
                bef = 0 ;
                yenFlag = false ;
            }
            else {
                bef = c ;
            }
        }
        if( outPos != null && outPos.length >= 1 ) {
            outPos[ 0 ] = -1 ;
        }
        return null ;
    }
    
    /**
     * 指定位置から前の、有効文字列を取得.
     * @param outPos 検知情報の開始位置が返されます.
     * @param script 対象のスクリプトを設定します.
     * @param index 対象のインデックス値を設定します.
     */
    public static final String parseToLastOfSingle( int[] outPos,String script,int index ) {
        if( index <= 0 ) {
            index = 0 ;
        }
        int cote = -1 ;
        char bef = 0 ;
        int pos = -1 ;
        boolean yenFlag = false ;
        for( int i = index-1 ; i >= 0 ; i -- ) {
            char c = script.charAt( i ) ;
            if( cote != -1 ) {
                if( bef != '\\' && cote == c ) {
                    cote = -1 ;
                    yenFlag = false ;
                    if( pos >= 0 ) {
                        if( outPos != null && outPos.length >= 1 ) {
                            outPos[ 0 ] = i ;
                        }
                        return script.substring( i,pos ) ;
                    }
                }
                else if( c == '\\' && bef == '\\' ) {
                    yenFlag = true ;
                }
                else {
                    yenFlag = false ;
                }
            }
            else if( bef != '\\' && ( c == '\'' || c == '\"' ) ) {
                if( pos >= 0 ) {
                    if( outPos != null && outPos.length >= 1 ) {
                        outPos[ 0 ] = i+1 ;
                    }
                    return script.substring( i+1,pos ) ;
                }
                cote = c ;
                pos = i+1 ;
            }
            else if( c == ' ' || c == '　' ||
                c == '\r' || c == '\n' || c == '\t' ) {
                if( pos >= 0 ) {
                    if( outPos != null && outPos.length >= 1 ) {
                        outPos[ 0 ] = i+1 ;
                    }
                    return script.substring( i+1,pos ) ;
                }
            }
            else if( c == '(' || c == '{' || c == '[' ||
                c == ')' || c == '}' || c == ']' ||
                c == ':' || c == ';' ) {
                if( pos >= 0 ) {
                    if( outPos != null && outPos.length >= 1 ) {
                        outPos[ 0 ] = i+1 ;
                    }
                    return script.substring( i+1,pos ) ;
                }
                else {
                    if( outPos != null && outPos.length >= 1 ) {
                        outPos[ 0 ] = i ;
                    }
                    return script.substring( i,i+1 ) ;
                }
            }
            else if( pos <= -1 ) {
                pos = i+1 ;
            }
            if( yenFlag ) {
                bef = 0 ;
                yenFlag = false ;
            }
            else {
                bef = c ;
            }
        }
        if( outPos != null && outPos.length >= 1 ) {
            outPos[ 0 ] = -1 ;
        }
        return null ;
    }
    
    /**
     * 文法検知処理.
     * <p>コーテーションや、スペース等の区切り文字を除外した、文法チェック.</p>
     * @param script 対象のスクリプトを設定します.
     * @param chk 対象の区切り文字を設定します.
     * @param index 検索開始位置を設定します.
     * @return int -1の場合、存在しません.
     */
    public static final int indexByParse( String script,String chk,int index ) {
        int len = script.length() ;
        int chkLen = chk.length() ;
        if( len <= 0 || chkLen <= 0 ) {
            return -1 ;
        }
        if( index <= 0 ) {
            index = 0 ;
        }
        int cote = -1 ;
        char bef = 0 ;
        int pos = 0 ;
        boolean yenFlag = false ;
        for( int i = index ; i < len ; i ++ ) {
            char c = script.charAt( i ) ;
            if( cote != -1 ) {
                if( bef != '\\' && cote == c ) {
                    cote = -1 ;
                    yenFlag = false ;
                }
                else if( c == '\\' && ( char )bef == '\\' ) {
                    yenFlag = true ;
                }
                else {
                    yenFlag = false ;
                }
            }
            else if( bef != '\\' && ( c == '\'' || c == '\"' ) ) {
                cote = c ;
                pos = -1 ;
            }
            else if( c == ' ' || c == '　' ||
                c == '\r' || c == '\n' || c == '\t' ||
                c == ';' || c == ':' ||
                c == '(' || c == '{' || c == '[' ||
                c == ')' || c == '}' || c == ']' ) {
                if( chkLen <= pos ) {
                    return i-chkLen ;
                }
                if( c == ';' || c == ':' ||
                    c == '(' || c == '{' || c == '[' ||
                    c == ')' || c == '}' || c == ']' ) {
                    if( pos >= 0 && c == chk.charAt( pos ) ) {
                        pos ++ ;
                    }
                    else {
                        pos = 0 ;
                    }
                }
                else {
                    pos = 0 ;
                }
            }
            else {
                if( chkLen <= pos ) {
                    pos = -1 ;
                }
                if( pos >= 0 && c == chk.charAt( pos ) ) {
                    pos ++ ;
                }
            }
            if( yenFlag ) {
                bef = 0 ;
                yenFlag = false ;
            }
            else {
                bef = c ;
            }
        }
        return -1 ;
    }
    
    /**
     * 文法の内、コーテーションを除く連続文字列を検知.
     * @param script 対象のスクリプトを設定します.
     * @param chk 対象の区切り文字を設定します.
     * @param index 検索開始位置を設定します.
     * @return int -1の場合、存在しません.
     */
    public static final int indexByEquals( String script,String chk,int index ) {
        int len = script.length() ;
        int chkLen = chk.length() ;
        if( len <= 0 || chkLen <= 0 ) {
            return -1 ;
        }
        if( index <= 0 ) {
            index = 0 ;
        }
        int cote = -1 ;
        char bef = 0 ;
        int pos = 0 ;
        boolean yenFlag = false ;
        for( int i = index ; i < len ; i ++ ) {
            char c = script.charAt( i ) ;
            if( cote != -1 ) {
                if( bef != '\\' && cote == c ) {
                    cote = -1 ;
                    yenFlag = false ;
                }
                else if( c == '\\' && ( char )bef == '\\' ) {
                    yenFlag = true ;
                }
                else {
                    yenFlag = false ;
                }
            }
            else if( bef != '\\' && ( c == '\'' || c == '\"' ) ) {
                cote = c ;
                pos = -1 ;
            }
            else if( pos >= 0 && c == chk.charAt( pos ) ) {
                if( pos <= 0 ) {
                    pos ++ ;
                }
                else if( chk.charAt( pos-1 ) == script.charAt( i-1 ) ) {
                    pos ++ ;
                    if( chkLen <= pos ) {
                        return (i-chkLen)+1 ;
                    }
                }
                else {
                    pos = 0 ;
                }
            }
            else {
                pos = 0 ;
            }
            if( yenFlag ) {
                bef = 0 ;
                yenFlag = false ;
            }
            else {
                bef = c ;
            }
        }
        return -1 ;
    }
}
