package org.maachang ;

import gnu.trove.*;
import java.util.*;

/**
 * 書き込み制御Map.
 * 
 * @version 2008/12/01
 * @author masahito suzuki
 * @since Maachang 1.00
 */
public class WriteContralMap extends StringMap implements Map {
    /**
     * 書き込み許可フラグ.
     */
    private boolean writeMode = true ;
    
    /**
     * コンストラクタ.
     */
    public WriteContralMap() {
        super() ;
    }
    
    /**
     * 書き込み可能フラグを設定.
     * @param mode [true]の場合、書き込み可能です.
     */
    public void setWriteMode( boolean mode ) {
        this.writeMode = mode ;
    }
    
    /**
     * 書き込み可能フラグを取得.
     * @return boolean [true]の場合、書き込み可能です.
     */
    public boolean isWriteMode() {
        return writeMode ;
    }
    
    public void clear() {
        if( writeMode ) {
            super.clear() ;
        }
    }
    
    public Object put(Object name, Object value) {
        if( writeMode ) {
            return super.put( name,value ) ;
        }
        return null ;
    }
    
    public Object remove(Object key) {
        if( writeMode ) {
            return super.remove( key ) ;
        }
        return null ;
    }
    
    public void putAll(Map toMerge) {
        if( writeMode ) {
            super.putAll( toMerge ) ;
        }
    }
}
