package org.maachang.atomic ;

import java.util.concurrent.atomic.AtomicReference;

/**
 * 単純AtomicStack.
 * 
 * @version 2009/09/01
 * @author masahito suzuki
 * @since MaachangAtomic 1.00
 */
public class AtomicSimpleStack<E> {
    private static class _asstackChild<E> {
        public volatile E object ;
        public volatile _asstackChild<E> next = null ;
    }
    
    private final AtomicReference<_asstackChild<E>> head = new AtomicReference<_asstackChild<E>>( null ) ;
    
    protected void finalize() throws Exception {
        clear() ;
    }
    
    /**
     * 情報をクリア.
     */
    public void clear() {
        head.set( null ) ;
    }
    
    /**
     * 情報追加.
     * @param e 追加対象の情報を設定します.
     */
    public void add( E e ) {
        if( e == null ) {
            return ;
        }
        _asstackChild<E> ee = new _asstackChild<E>() ;
        ee.object = e ;
        _asstackChild<E> c ;
        while( true ) {
            if( head.compareAndSet( ( c = head.get() ),ee ) ) {
                ee.next = c ;
                break ;
            }
        }
    }
    
    /**
     * 先頭情報を削除して取得.
     * @return E 対象の情報が返されます.
     */
    public E poll() {
        _asstackChild<E> c ;
        while( true ) {
            if( head.compareAndSet( ( c = head.get() ),( ( c != null && c.next != null ) ? c.next : null ) ) ) {
                if( c != null ) {
                    return c.object; 
                }
                return null ;
            }
        }
    }
    
    /**
     * 先頭情報を削除せず取得.
     * @return E 対象の情報が返されます.
     */
    public E peek() {
        _asstackChild<E> c = head.get() ;
        if( c != null ) {
            return c.object ;
        }
        return null ;
    }
    
}

