package org.maachang.atomic ;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

/**
 * AtomicStack.
 * 
 * @version 2009/09/01
 * @author masahito suzuki
 * @since MaachangAtomic 1.00
 */
public class AtomicStack<E> {
    private static class _astackChild<E> {
        public volatile E object ;
        public volatile _astackChild<E> next = null ;
    }
    
    private final AtomicReference<_astackChild<E>> head = new AtomicReference<_astackChild<E>>( null ) ;
    private final AtomicInt size = new AtomicInt( 0 ) ;
    
    protected void finalize() throws Exception {
        clear() ;
    }
    
    /**
     * 情報をクリア.
     */
    public void clear() {
        head.set( null ) ;
        size.set( 0 ) ;
    }
    
    /**
     * 情報追加.
     * @param e 追加対象の情報を設定します.
     */
    public void add( E e ) {
        if( e == null ) {
            return ;
        }
        _astackChild<E> ee = new _astackChild<E>() ;
        ee.object = e ;
        _astackChild<E> c ;
        while( true ) {
            if( head.compareAndSet( ( c = head.get() ),ee ) ) {
                ee.next = c ;
                size.inc() ;
                break ;
            }
        }
    }
    
    /**
     * 先頭情報を削除して取得.
     * @return E 対象の情報が返されます.
     */
    public E poll() {
        _astackChild<E> c ;
        while( true ) {
            if( head.compareAndSet( ( c = head.get() ),( ( c != null && c.next != null ) ? c.next : null ) ) ) {
                if( c != null ) {
                    size.dec() ;
                    return c.object; 
                }
                return null ;
            }
        }
    }
    
    /**
     * 先頭情報を削除せず取得.
     * @return E 対象の情報が返されます.
     */
    public E peek() {
        _astackChild<E> c = head.get() ;
        if( c != null ) {
            return c.object ;
        }
        return null ;
    }
    
    /**
     * サイズを取得.
     * @return int サイズが返されます.
     */
    public int size() {
        return size.get() ;
    }
    
    /**
     * 全内容を取得.
     * @return List<E> 全内容が返されます.
     */
    public List<E> getAll() {
        List<E> ret = new ArrayList<E>() ;
        _astackChild<E> n ;
        for( n = head.get() ; n != null ; n = n.next ) {
            ret.add( n.object ) ;
        }
        return ret ;
    }
}

