package org.maachang.atomic;

import java.util.ArrayList;
import java.util.List;

/**
 * スタック処理.
 * 
 * @version 2009/09/01
 * @author masahito suzuki
 * @since MaachangAtomic 1.00
 */
public class StackToNormal<E> {
    private static class _stackToNormalChild<E> {
        public E object ;
        public _stackToNormalChild<E> next = null ;
    }
    
    private _stackToNormalChild<E> c = null ;
    private int size = 0 ;
    
    protected void finalize() throws Exception {
        clear() ;
    }
    
    /**
     * 情報をクリア.
     */
    public void clear() {
        c = null ;
        size = 0 ;
    }
    
    /**
     * 情報追加.
     * @param e 追加対象の情報を設定します.
     */
    public void add( E e ) {
        if( e == null ) {
            return ;
        }
        _stackToNormalChild<E> a = new _stackToNormalChild<E>() ;
        a.object = e ;
        if( c == null ) {
            c = a ;
        }
        else {
            a.next = c ;
            c = a ;
        }
        size ++ ;
    }
    
    /**
     * 先頭情報を削除して取得.
     * @return E 対象の情報が返されます.
     */
    public E poll() {
        E ret = null ;
        if( c != null ) {
            ret = c.object ;
            c = c.next ;
            size -- ;
        }
        return ret ;
    }
    
    /**
     * 先頭情報を削除せず取得.
     * @return E 対象の情報が返されます.
     */
    public E peek() {
        E ret = null ;
        if( c != null ) {
            ret = c.object ;
        }
        return ret ;
    }
    
    /**
     * サイズを取得.
     * @return int サイズが返されます.
     */
    public int size() {
        return size ;
    }
    
    /**
     * 全内容を取得.
     * @return List<E> 全内容が返されます.
     */
    public List<E> getAll() {
        List<E> ret = new ArrayList<E>() ;
        _stackToNormalChild<E> n ;
        for( n = c ; n != null ; n = n.next ) {
            ret.add( n.object ) ;
        }
        return ret ;
    }
}
