package org.maachang.seabass ;

import java.util.*;
import pnuts.lang.*;
import org.maachang.* ;

/**
 * SeaBass用Bindings.
 * 
 * @version 2008/12/21
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public class SeaBassBindings extends pnuts.lang.Package {
    private static final Set<String> spKeys ;
    static {
        spKeys = new HashSet<String>() ;
        spKeys.add( "this" ) ;
        spKeys.add( "super" ) ;
    }
    
    private int id = 0 ;
    private SeaBassMemory memory = null ;
    
    private SeaBassBindings() {
    }
    protected SeaBassBindings(SeaBassMemory memory){
        super(null, null);
        this.memory = memory;
    }
    
    protected void clearBindings() {
        this.memory.clearMemory() ;
    }
    
    public SeaBassMemory getMemory() {
        return memory ;
    }
    
    public Object get(String symbol){
        return memory.get(symbol);
    }
    
    public Object get(String symbol, Context context){
        return memory.get(symbol);
    }
    
    public void set(String symbol, Object value, Context context) {
        if( spKeys.contains( symbol ) ) {
            return ;
        }
        memory.put(symbol, value);
    }
    
    public void set(String symbol, Object value){
        if( spKeys.contains( symbol ) ) {
            return ;
        }
        memory.put(symbol, value);
    }
    
    public boolean defined(String name, Context context){
        return memory.containsKey(name);
    }
    
    public Enumeration keys(){
        return Collections.enumeration(memory.keySet());
    }
    
    public Enumeration values(){
        return Collections.enumeration(memory.values());
    }
    
    public int size(){
        return memory.size();
    }
    
    public NamedValue lookup(final String symbol){
        if( memory.currentContainsKey(symbol) ) {
            return ((SeaBassMap)memory.getMap()).lookup( symbol ) ;
        }
        else if (memory.noCurrentContainsKey(symbol)){
            return new SeaBassNamedValue( memory,symbol,memory.get(symbol) ) ;
        }
        return null;
    }
    
    public NamedValue lookup(final String symbol, Context context){
        NamedValue ret = ((SeaBassMap)memory.getMap()).lookup( symbol ) ;
        if( ret == null && memory.noCurrentContainsKey(symbol) ) {
            ret = new SeaBassNamedValue( memory,symbol,memory.get(symbol) ) ;
        }
        return ret;
    }
    
    public void setId( int id ) {
        this.id = id ;
    }
    
    public int getId() {
        return id ;
    }
    
    public String toString() {
        return "id:" + id ;
    }
    
    private static class SeaBassNamedValue implements NamedValue {
        SeaBassMemory m ;
        String k ;
        Object v ;
        SeaBassNamedValue( SeaBassMemory mm,String ky,Object val ) {
            this.m = mm ;
            this.k = ky ;
            this.v = val ;
        }
        public String getName(){
            return k;
        }
        public Object get(){
            return v;
        }
        public void set(Object val){
            m.put(k, v);
        }
    }
}
