package org.maachang.seabass ;

import java.io.* ;
import pnuts.lang.*;

/**
 * ScriptCompile.
 * 
 * @version 2008/12/21
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public class SeaBassCompile {
    
    /**
     * 実行Executable.
     */
    private Executable executable = null ;
    
    /**
     * コンテンツ名.
     */
    private String name = null ;
    
    /**
     * コンストラクタ.
     */
    private SeaBassCompile() {
        
    }
    
    /**
     * コンストラクタ.
     * @param name 対象のコンテンツ名を設定します.
     * @param executable コンパイル済みオブジェクトを設定します.
     */
    protected SeaBassCompile( String name,Executable executable ) {
        this.name = name ;
        this.executable = executable ;
    }
    
    /**
     * スクリプト実行.
     * @return Object 実行結果が返されます.
     * @exception Exception 例外.
     */
    public Object eval() throws Exception {
        pnuts.lang.Context context = pnuts.lang.Context.getCurrentContext() ;
        if( context == null ) {
            throw new IOException( "カレントコンテキスト取得に失敗" ) ;
        }
        SeaBassContext seabassContext = null ;
        pnuts.lang.Package pkg = context.getCurrentPackage() ;
        if( pkg instanceof SeaBassBindings ) {
            seabassContext = (SeaBassContext)( ( SeaBassBindings )pkg ).getMemory().get( "context" ) ;
        }
        else {
            throw new IOException( "contextにSeabassContext情報が存在しません" ) ;
        }
        return eval( seabassContext ) ;
    }
    
    /**
     * スクリプト実行.
     * @param script 実行対象のスクリプトを設定します.
     * @param context SeaBassContext情報を設定します.
     * @return Object 実行結果が返されます.
     * @exception Exception 例外.
     */
    public Object eval(SeaBassContext context)
        throws Exception {
        if( context == null ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        Context ctx = null ;
        boolean destroyFlag = true ;
        int befLine = -1 ;
        String befName = null ;
        boolean exceptionFlag = false ;
        String oldName = null ;
        try {
            destroyFlag = context.isInitContext() ;
            ctx = context.getContext() ;
            if( this.name != null ) {
                ctx.pushFileName(this.name);
                oldName = ctx.getName() ;
                ctx.setName( this.name ) ;
            }
            // コンテキストが今回初期化されている場合.
            if( destroyFlag ) {
                ctx.appendCurrent() ;
                context.put( "ctx",ctx ) ;
                context.put( "context",context ) ;
            }
            befLine = ctx.getLine() ;
            ctx.setLine( 0 ) ;
            PnutsException.clearPnutsErrorLineInfo( ctx ) ; // エラー情報をクリア.
            Object ret = executable.run(ctx) ;
            context.remove( "_$lastError" ) ;
            return ret ;
        } catch( Exception e ) {
            exceptionFlag = true ;
            ctx = context.getContext() ;
            if( befLine != -1 && ctx.getLine() == -1 ) {
                ctx.setLine( befLine ) ;
                befLine = -1 ;
            }
            if( this.name != null ) {
                ctx.popFileName();
            }
            return SeaBassScriptEngine.toException( e,this.name,context,ctx ) ;
        } finally {
            ctx = context.getContext() ;
            if( destroyFlag ) {
                context.clearCurrent() ;
            }
            if( befLine != -1 ) {
                ctx.setLine( befLine ) ;
            }
            if( this.name != null ) {
                ctx.popFileName();
                if( oldName != null ) {
                    ctx.setName( "" ) ;
                }
                else {
                    ctx.setName( oldName ) ;
                }
            }
        }
    }
    
}
