package org.maachang.seabass ;

import pnuts.lang.*;
import org.maachang.seabass.mod.object.*;

/**
 * SeaBass用Context.
 * 
 * @version 2008/12/21
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public class SeaBassContext {
    private pnuts.lang.Context context = null ;
    private SeaBassBindings bindings = null ;
    private String name = null ;
    
    /**
     * コンストラクタ.
     */
    protected SeaBassContext() {
        bindings = new SeaBassBindings( new SeaBassMemory() ) ;
    }
    
    /**
     * コンテキストクリア.
     */
    protected void clearContext() {
        bindings.clearBindings() ;
        context = null ;
        name = null ;
    }
    
    /**
     * Pnutsコンテキストを取得.
     * @return pnuts.lang.Context Pnutsコンテキストが返されます.
     */
    public pnuts.lang.Context getContext() {
        if( context == null ) {
            context = new pnuts.lang.Context( bindings ) ;
        }
        return context ;
    }
    
    /**
     * Pnutsコンテキスト初期化チェック.
     * @return boolean [true]の場合、初期化されています.
     */
    public boolean isInitContext() {
        return ( pnuts.lang.Context.getCurrentContext() == null ) ;
    }
    
    /**
     * Bindingsオブジェクトを取得.
     * @return SeaBassBindings Bindingsオブジェクトが返されます.
     */
    public SeaBassBindings getBindings() {
        return bindings ;
    }
    
    /**
     * メモリオブジェクトを取得.
     * @return SeaBassMemory メモリオブジェクトが返されます.
     */
    public SeaBassMemory getMemory() {
        return bindings.getMemory() ;
    }
    
    /**
     * メモリ内容に追加.
     * @param key 対象のキー名を設定します.
     * @param value 対象の要素を設定します.
     */
    public void put( String key,Object value ) {
        bindings.getMemory().put( key,value ) ;
    }
    
    /**
     * メモリ内容から削除.
     * @param key 対象のキー名を設定します.
     * @return Object 削除された内容が返されます.
     */
    public Object remove( String key ) {
        return bindings.getMemory().remove( key ) ;
    }
    
    /**
     * メモリ内容から取得.
     * @param key 対象のキー名を設定します.
     * @return Object 取得された内容が返されます.
     */
    public Object get( String key ) {
        return bindings.getMemory().get( key ) ;
    }
    
    /**
     * メモリ内容に存在するかチェック.
     * @param key チェック対象の条件を設定します.
     * @return boolean [true]の場合、存在します.
     */
    public boolean containsKey( String key ) {
        return bindings.getMemory().containsKey( key ) ;
    }
    
    /**
     * スクリプト名を設定.
     * @param name 対象のスクリプト名を設定します.
     */
    public void setName( String name ) {
        this.name = name ;
    }
    
    /**
     * スクリプト名を取得.
     * @return String 対象のスクリプト名が返されます.
     */
    public String getName() {
        return name ;
    }
    
    /**
     * カレントコンテキストクリア.
     */
    protected void clearCurrent() {
        if( context != null ) {
            context.clearCurrent() ;
        }
    }
    
}
