package org.maachang.seabass ;

import pnuts.lang.*;
import org.pnuts.util.* ;

/**
 * SeaBass例外.
 * 
 * @version 2007/08/26
 * @author masahito suzuki
 * @since MaachangComet 1.00
 */
public class SeaBassException extends RuntimeException {
    private static final long serialVersionUID = -6855465702119811163L;
    
    private String[] scriptName = null ;
    private int[] line = null ;
    private String errorMessage = null ;
    private Throwable e = null ;
    
    protected SeaBassException() {
        super() ;
    }
    
    protected SeaBassException( Throwable e,Context context,String nowName,int nowLine ) {
        super( e ) ;
        this.e = e ;
        if( e instanceof PnutsException ) {
            initByPnutsException( ( PnutsException )e,context,nowName,nowLine ) ;
        }
        else if( e instanceof ParseException ) {
            scriptName = new String[]{ context.getErrorScriptName() } ;
            line = new int[]{ ((ParseException)e).getErrorLine() } ;
        }
        else {
            scriptName = null ;
            line = null ;
        }
        while( true ) {
            if( e instanceof PnutsException ) {
                if( ((PnutsException)e).getThrowable() != null ) {
                    e = ((PnutsException)e).getThrowable() ;
                    continue ;
                }
            }
            break ;
        }
        String msg = e.getMessage() ;
        if( msg == null ) {
            msg = e.getClass().getName() ;
        }
        else if( msg.trim().length() <= 0 ) {
            msg = e.getClass().getName() ;
        }
        errorMessage = createMessage( scriptName,line,msg ) ;
    }
    
    protected void initByPnutsException( PnutsException pe,Context context,String nowName,int nowLine ) {
        PnutsErrorLineInfo e = PnutsException.getPnutsErrorLineInfo( context ) ;
        if( e != null ) {
            int len = e.size() ;
            scriptName = new String[ len ] ;
            line = new int[ len ] ;
            for( int i = 0 ; i < len ; i ++ ) {
                PnutsErrorLineChild ch = e.get( i ) ;
                if( ch.contextName != null ) {
                    scriptName[ i ] = ch.contextName ;
                }
                else if( ch.errorScriptName != null ) {
                    scriptName[ i ] = ch.errorScriptName ;
                }
                else {
                    scriptName[ i ] = "unknown" ;
                }
                line[ i ] = ch.line ;
            }
            Cell sc = context.getFunctionToScriptName() ;
            if( sc != null ) {
                while( (sc=sc.next) != null ){}
            }
            Cell ln = context.getFunctionToScriptLine() ;
            if( ln != null ) {
                while( (ln=ln.next) != null ){}
            }
        }
        else {
            Cell sc = context.getFunctionToScriptName() ;
            Cell ln = context.getFunctionToScriptLine() ;
            if( sc == null || ln == null ) {
                if( nowName == null || nowName.length() <= 0 || nowLine <= -1 ) {
                    scriptName = new String[]{ pe.getErrorScriptName() } ;
                    line = new int[]{ pe.getLine() } ;
                }
                else {
                    scriptName = new String[]{ nowName } ;
                    line = new int[]{ nowLine } ;
                }
            }
            else {
                int len1 = 0 ;
                Cell x = sc ;
                while( x != null ) {
                    x = x.next ;
                    len1 ++ ;
                }
                int len2 = 0 ;
                x = ln ;
                while( x != null ) {
                    x = x.next ;
                    len2 ++ ;
                }
                if( len1 != len2 ) {
                    scriptName = new String[]{ nowName } ;
                    line = new int[]{ nowLine } ;
                }
                else {
                    if( nowName == null || nowName.length() <= 0 ||
                        nowLine <= -1 ) {
                        scriptName = new String[ len1 ] ;
                        line = new int[ len1 ] ;
                        len1 = 0 ;
                        x = sc ;
                        while( x != null ) {
                            scriptName[ len1 ] = (String)x.object ;
                            x = x.next ;
                            len1 ++ ;
                        }
                        len2 = 0 ;
                        x = ln ;
                        while( x != null ) {
                            line[ len2 ] = ((Integer)x.object).intValue() ;
                            x = x.next ;
                            len2 ++ ;
                        }
                    }
                    else {
                        scriptName = new String[ len1+1 ] ;
                        line = new int[ len1+1 ] ;
                        len1 = 1 ;
                        x = sc ;
                        while( x != null ) {
                            scriptName[ len1 ] = (String)x.object ;
                            x = x.next ;
                            len1 ++ ;
                        }
                        len2 = 1 ;
                        x = ln ;
                        while( x != null ) {
                            line[ len2 ] = ((Integer)x.object).intValue() ;
                            x = x.next ;
                            len2 ++ ;
                        }
                        if( nowLine <= -1 ) {
                            scriptName[ 0 ] = pe.getErrorScriptName() ;
                            line[ 0 ] = pe.getLine() ;
                        }
                        else {
                            scriptName[ 0 ] = nowName ;
                            line[ 0 ] = nowLine ;
                        }
                    }
                }
            }
        }
    }
    
    public String[] getScriptName() {
        return scriptName ;
    }
    
    public int[] getLine() {
        return line ;
    }
    
    public String getTopScriptName() {
        if( scriptName == null || scriptName.length <= 0 ) {
            return null ;
        }
        return scriptName[ scriptName.length-1 ] ;
    }
    
    public int getTopLine() {
        if( line == null || line.length <= 0 ) {
            return -1 ;
        }
        return line[ line.length-1 ] ;
    }
    
    public String getMessage() {
        return errorMessage ;
    }
    
    public Throwable getException() {
        return e ;
    }
    
    public static final String trimErrorMessage( String message ) {
        String srcMsg = message ;
        if( message == null || message.length() <= 0 ) {
            return "unknown message" ;
        }
        try {
            int p = message.lastIndexOf( "Exception" ) ;
            int p1 = message.lastIndexOf( "Error" ) ;
            if( p == -1 && p1 == -1 ) {
                return message ;
            }
            if( p != -1 ) {
                p += "Exception".length() ;
                if( p1 != -1 ) {
                    p1 = "Error".length() ;
                    if( p1 > p ) {
                        p = p1 ;
                    }
                }
            }
            else {
                p = p1 + "Error".length() ;
            }
            message = message.substring( p ).trim() ;
            if( message.length() <= 0 ) {
                message = srcMsg ;
            }
            else if( message.startsWith( ":" ) ) {
                message = message.substring( 1 ).trim() ;
            }
        } catch( Exception e ) {
        }
        return message ;
    }
    
    public static final String createMessage( String[] scriptName,int[] line,String message ) {
        if( scriptName == null || scriptName.length <= 0 || line == null || line.length <= 0 ) {
            return trimErrorMessage( message ) ;
        }
        int len = scriptName.length ;
        StringBuilder  buf = new StringBuilder() ;
        for( int i = len-1 ; i >= 0 ; i -- ) {
            buf.append( "[" ).append( scriptName[i] ).append( " line:(" ).append( line[i] ).append( ")] - " ) ;
        }
        return buf.append( trimErrorMessage( message ) ).toString() ;
    }
}
