package org.maachang.seabass ;

import java.util.*;
import pnuts.lang.*;
import org.pnuts.lang.*;


/**
 * SeaBassMode.
 * 
 * @version 2008/12/21
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public class SeaBassMode {
    private SeaBassMode() {}
    private static final SeaBassMode SNGL = new SeaBassMode() ;
    
    /**
     * シングルトンオブジェクトを取得.
     * @return SeaBassMode シングルトンオブジェクトが返されます.
     */
    public static final SeaBassMode getInstance() {
        return SNGL ;
    }
    
    /**
     * カレントスレッドをPnutsClassLoaderに変更.
     */
    protected static final void changeClassLoader( Context context ) {
        ClassLoader clz = Thread.currentThread().getContextClassLoader() ;
        if( clz instanceof PnutsClassLoader ) {
            context.setClassLoader( clz ) ;
        }
        else {
            clz = Pnuts.createClassLoader( context ) ;
            context.setClassLoader( clz ) ;
            Thread.currentThread().setContextClassLoader( clz ) ;
        }
    }
    
    /**
     * On-the-fly コンパイラモード+optimizeで実行.
     */
    public void optimize() {
        Properties defaultSettings = new Properties();
        defaultSettings.setProperty("pnuts.lang.defaultPnutsImpl","pnuts.compiler.CompilerPnutsImpl") ;
        defaultSettings.setProperty("pnuts.lang.defaultConfiguration","pnuts.lang.Java2Configuration") ;
        defaultSettings.setProperty("pnuts.compiler.optimize", "true") ;
        defaultSettings.setProperty("pnuts.compiler.traceMode", "false") ;
        defaultSettings.setProperty("pnuts.compiler.useDynamicProxy", "true") ;
        defaultSettings.setProperty("pnuts.verbose", "false") ;
        Pnuts.setDefaults(defaultSettings) ;
        SeaBassScriptEngine.COMP.setTraceMode( false ) ;
        SeaBassScriptEngine.COMP.includeLineNo( false ) ;
        SeaBassScriptEngine.COMP.includeColumnNo( false ) ;
        SeaBassScriptEngine.COMP.setConstantFolding( false ) ;
        SeaBassScriptEngine.COMP.includeMainMethod( false ) ;
    }
    
    /**
     * On-the-fly コンパイラモードで実行.
     */
    public void complie() {
        Properties defaultSettings = new Properties();
        defaultSettings.setProperty("pnuts.lang.defaultPnutsImpl","pnuts.compiler.CompilerPnutsImpl") ;
        defaultSettings.setProperty("pnuts.lang.defaultConfiguration","pnuts.lang.Java2Configuration") ;
        defaultSettings.setProperty("pnuts.compiler.optimize", "false") ;
        defaultSettings.setProperty("pnuts.compiler.traceMode", "true") ;
        defaultSettings.setProperty("pnuts.compiler.useDynamicProxy", "true") ;
        defaultSettings.setProperty("pnuts.verbose", "false") ;
        Pnuts.setDefaults(defaultSettings) ;
        SeaBassScriptEngine.COMP.setTraceMode( true ) ;
        SeaBassScriptEngine.COMP.includeLineNo( true ) ;
        SeaBassScriptEngine.COMP.includeColumnNo( false ) ;
        SeaBassScriptEngine.COMP.setConstantFolding( false ) ;
        SeaBassScriptEngine.COMP.includeMainMethod( false ) ;
    }
    
    /**
     * ASTインタプリタモードで実行.
     */
    public void interpreter() {
        Properties defaultSettings = new Properties();
        defaultSettings.setProperty("pnuts.lang.defaultPnutsImpl","pnuts.lang.PnutsImpl") ;
        defaultSettings.setProperty("pnuts.lang.defaultConfiguration","pnuts.lang.Java2Configuration") ;
        defaultSettings.setProperty("pnuts.compiler.optimize", "false") ;
        defaultSettings.setProperty("pnuts.compiler.traceMode", "true") ;
        defaultSettings.setProperty("pnuts.compiler.useDynamicProxy", "true") ;
        defaultSettings.setProperty("pnuts.verbose", "false") ;
        Pnuts.setDefaults(defaultSettings) ;
    }
    
    /**
     * Mixedモードで実行.
     */
    public void mixed() {
        Properties defaultSettings = new Properties();
        defaultSettings.setProperty("pnuts.lang.defaultPnutsImpl","pnuts.ext.CachedPnutsImpl") ;
        defaultSettings.setProperty("pnuts.lang.defaultConfiguration","pnuts.lang.Java2Configuration") ;
        defaultSettings.setProperty("pnuts.compiler.optimize", "false") ;
        defaultSettings.setProperty("pnuts.compiler.traceMode", "true") ;
        defaultSettings.setProperty("pnuts.compiler.useDynamicProxy", "true") ;
        defaultSettings.setProperty("pnuts.verbose", "false") ;
        Pnuts.setDefaults(defaultSettings) ;
    }
}
