package org.maachang.seabass ;

import java.io.* ;
import java.util.* ;
import pnuts.compiler.*;
import pnuts.lang.*;
import org.pnuts.lang.*;
import org.pnuts.util.*;

import org.maachang.*;
import org.maachang.atomic.*;

/**
 * ScriptEngine.
 * 
 * @version 2008/12/21
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public class SeaBassScriptEngine {
    
    /**
     * コンパイラ.
     */
    protected static final pnuts.compiler.Compiler COMP = new pnuts.compiler.Compiler("_$seabass$cmp$_") ;
    
    /**
     * コンストラクタ.
     */
    public SeaBassScriptEngine() {
        
    }
    
    /**
     * スクリプト実行.
     * @param script 実行対象のスクリプトを設定します.
     * @param context SeaBassContext情報を設定します.
     * @exception Exception 例外.
     */
    public Object eval(String script, SeaBassContext context)
        throws Exception {
        if( script == null || script.length() <= 0 || context == null ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        String name = null ;
        Context ctx = null ;
        boolean destroyFlag = true ;
        int befLine = -1 ;
        boolean exceptionFlag = false ;
        String oldName = null ;
        try {
            destroyFlag = context.isInitContext() ;
            ctx = context.getContext() ;
            if( ( name = context.getName() ) != null ) {
                ctx.pushFileName(name);
                oldName = ctx.getName() ;
                ctx.setName( name ) ;
            }
            // コンテキストが今回初期化されている場合.
            if( destroyFlag ) {
                ctx.appendCurrent() ;
                context.put( "ctx",ctx ) ;
                context.put( "context",context ) ;
            }
            befLine = ctx.getLine() ;
            ctx.setLine( 0 ) ;
            script = convertScript( script ) ;
            PnutsException.clearPnutsErrorLineInfo( ctx ) ; // エラー情報をクリア.
            Object ret = Pnuts.load( new StringReader( script ),ctx ) ;
            context.remove( "_$lastError" ) ;
            return ret ;
        } catch( Exception e ) {
            exceptionFlag = true ;
            ctx = context.getContext() ;
            if( befLine != -1 && ctx.getLine() == -1 ) {
                ctx.setLine( befLine ) ;
                befLine = -1 ;
            }
            return toException( e,name,context,ctx ) ;
        } finally {
            ctx = context.getContext() ;
            if( destroyFlag ) {
                context.clearCurrent() ;
                context.remove( "ctx" ) ;
                context.remove( "context" ) ;
                context.remove( "_$exit$to" ) ;
                context.remove( "arguments" ) ;
            }
            if( befLine != -1 ) {
                ctx.setLine( befLine ) ;
            }
            if( name != null ) {
                ctx.popFileName();
                if( oldName == null ) {
                    ctx.setName( "" ) ;
                }
                else {
                    ctx.setName( oldName ) ;
                }
            }
        }
    }
    
    /**
     * コンパイル済みスクリプトを生成.
     * @param script 実行対象のスクリプトを設定します.
     * @param context SeaBassContext情報を設定します.
     * @return SeaBassCompile コンパイル済みスクリプトが返されます.
     * @exception Exception 例外.
     */
    public SeaBassCompile compile(String script, SeaBassContext context)
        throws Exception {
        if( script == null || script.length() <= 0 || context == null ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        String name = null ;
        Context ctx = null ;
        boolean destroyFlag = true ;
        int befLine = -1 ;
        boolean exceptionFlag = false ;
        String oldName = null ;
        try {
            destroyFlag = context.isInitContext() ;
            ctx = context.getContext() ;
            if( ( name = context.getName() ) != null ) {
                ctx.pushFileName(name);
                oldName = ctx.getName() ;
                ctx.setName( name ) ;
            }
            // コンテキストが今回初期化されている場合.
            if( destroyFlag ) {
                ctx.appendCurrent() ;
                context.put( "ctx",ctx ) ;
                context.put( "context",context ) ;
            }
            // 行保存.
            befLine = ctx.getLine() ;
            ctx.setLine( 0 ) ;
            script = convertScript( script ) ;
            Executable ret = COMP.compile( Pnuts.parse( script ), ctx);
            context.remove( "_$lastError" ) ;
            return new SeaBassCompile( name,ret ) ;
        } catch( Exception e ) {
            exceptionFlag = true ;
            ctx = context.getContext() ;
            if( befLine != -1 && ctx.getLine() == -1 ) {
                ctx.setLine( befLine ) ;
                befLine = -1 ;
            }
            seabassException( e,name,context,ctx ) ;
            return null ;
        } finally {
            ctx = context.getContext() ;
            if( destroyFlag ) {
                context.clearCurrent() ;
                context.remove( "ctx" ) ;
                context.remove( "context" ) ;
                context.remove( "_$exit$to" ) ;
                context.remove( "arguments" ) ;
            }
            if( befLine != -1 ) {
                ctx.setLine( befLine ) ;
            }
            if( name != null ) {
                ctx.popFileName();
                if( oldName == null ) {
                    ctx.setName( "" ) ;
                }
                else {
                    ctx.setName( oldName ) ;
                }
            }
        }
    }
    
    /**
     * スクリプトバージョン情報を取得.
     * @return String スクリプトバージョン情報が返されます.
     */
    public static final String getVersion() {
        return ModUtil.version() ;
    }
    
    /**
     * 実行構文変換.
     * @param script 対象のスクリプトを設定します.
     * @return String 変換されたスクリプトが返されます.
     * @exception Exception 例外.
     */
    public static final String convertScript( String script ) throws Exception {
        if( script == null || script.length() <= 0 ) {
            return "" ;
        }
        script = ParseUtil.cutComment( script ) ;
        script = convertNotVAR( script ) ;
        script = convertCote( script ) ;
        script = convertForIn( script ) ;
        return script ;
    }
    
    /**
     * Exception処理.
     * @param e 対象の例外処理を設定します.
     * @param name 実行スクリプト名を設定します.
     * @param context SeaBassContextを設定します.
     * @param ctx PnutsContextを設定します.
     * @return ExitSeaBass スクリプト終了条件の場合返されます.
     */
    public static final ExitSeaBass toException( Exception e,String name,SeaBassContext context,pnuts.lang.Context ctx ) {
        SeaBassException le = null ;
        if( e instanceof PnutsException ) {
            Throwable t = ( ( PnutsException )e ).getThrowable() ;
            if( t != null ) {
                if( t instanceof ExitSeaBass ) {
                    // SeaBass終了条件.
                    if( context != null ) {
                        context.remove( "_$lastError" ) ;
                    }
                    ctx.getLastError() ;
                    return (ExitSeaBass)t ;
                }
            }
        }
        seabassException( e,name,context,ctx ) ;
        return null ;
    }
    
    /**
     * SeabassException処理.
     * @param e 対象の例外処理を設定します.
     * @param name 実行スクリプト名を設定します.
     * @param context SeaBassContextを設定します.
     * @param ctx PnutsContextを設定します.
     */
    public static final void seabassException( Exception e,String name,SeaBassContext context,pnuts.lang.Context ctx ) {
        SeaBassException le = null ;
        if( e instanceof PnutsException ) {
            Throwable t = ( ( PnutsException )e ).getThrowable() ;
            if( t != null ) {
                if( t instanceof PnutsException ) {
                    le = new SeaBassException( (PnutsException)t,ctx,name,( ( PnutsException )t ).getLine() ) ;
                    if( context != null ) {
                        context.put( "_$lastError",le ) ;
                    }
                    ctx.setLastError( le ) ;
                    throw le ;
                }
                else if( t instanceof SeaBassException ) {
                    if( context != null ) {
                        context.put( "_$lastError",t ) ;
                    }
                    ctx.setLastError( (SeaBassException)t ) ;
                    throw (SeaBassException)t ;
                }
                else {
                    le = ctx.getLastError() ;
                    if( le != null ) {
                        if( context != null ) {
                            context.put( "_$lastError",le ) ;
                        }
                        ctx.setLastError( le ) ;
                        throw le ;
                    }
                }
            }
            le = new SeaBassException( e,ctx,name,ctx.getLine() ) ;
            if( context != null ) {
                context.put( "_$lastError",le ) ;
            }
            ctx.setLastError( le ) ;
            throw le ;
        }
        else if( e instanceof ParseException ) {
            le = new SeaBassException( e,ctx,name,( ( ParseException )e ).getErrorLine() ) ;
            if( context != null ) {
                context.put( "_$lastError",e ) ;
            }
            ctx.setLastError( le ) ;
            throw le ;
        }
        else if( e instanceof SeaBassException ) {
            le = ( SeaBassException )e ;
            if( context != null ) {
                context.put( "_$lastError",le ) ;
            }
            ctx.setLastError( le ) ;
            throw le ;
        }
        if( context != null ) {
            context.put( "_$lastError",e ) ;
        }
        throw new SeaBassException( e,ctx,null,-1 ) ;
    }
    
    /**
     * var定義除外処理.
     */
    private static final String convertNotVAR( String script ) {
        StringBuilder buf = new StringBuilder( script.length() ) ;
        int b = 0 ;
        String chk = "var" ;
        int len = chk.length() ;
        while( true ) {
            int p = ParseUtil.indexByEquals( script,chk,b ) ;
            if( p == -1 ) {
                buf.append( script.substring( b ) ) ;
                break ;
            }
            buf.append( script.substring( b,p ) ) ;
            b = p+len ;
        }
        return buf.toString() ;
    }
    
    /**
     * for( x in y )を for( x : y )に変更処理.
     */
    private static final String convertForIn( String script ) {
        int[] opos = new int[ 1 ] ;
        List lst = new ArrayList() ;
        StringBuilder buf = new StringBuilder( script.length() ) ;
        int b = 0 ;
        while( true ) {
            int p = ParseUtil.indexByParse( script,"for",b ) ;
            if( p == -1 ) {
                buf.append( script.substring( b ) ) ;
                break ;
            }
            int next = p+3 ;
            int parCount = 0 ;
            int par2Count = 0 ;
            int par3Count = 0 ;
            lst.clear() ;
            for( int i = 0 ;; i ++ ) {
                String s = ParseUtil.parseToSingle( opos,script,next ) ;
                if( s == null ) {
                    buf.append( script.substring( b ) ) ;
                    b = -1 ;
                    break ;
                }
                if( i == 0 && "for".equals( s ) ) {
                    buf.append( script.substring( b,opos[0] ) ) ;
                    b = opos[0] ;
                    next = -1 ;
                    break ;
                }
                if( "(".equals( s ) ) {
                    parCount ++ ;
                    next = opos[0]+1 ;
                }
                else if( ")".equals( s ) ) {
                    parCount -- ;
                    next = opos[0]+1 ;
                    if( parCount <= 0 ) {
                        break ;
                    }
                    if( parCount == 1 ) {
                        lst.add( null ) ;
                    }
                }
                else if( "[".equals( s ) ) {
                    par2Count ++ ;
                    next = opos[0]+1 ;
                }
                else if( "]".equals( s ) ) {
                    par2Count -- ;
                    next = opos[0]+1 ;
                    if( par2Count == 0 ) {
                        lst.add( null ) ;
                    }
                }
                else if( "{".equals( s ) ) {
                    par3Count ++ ;
                    next = opos[0]+1 ;
                }
                else if( "}".equals( s ) ) {
                    par3Count -- ;
                    next = opos[0]+1 ;
                    if( par3Count == 0 ) {
                        lst.add( null ) ;
                    }
                }
                else if( parCount <= 1 && par2Count <= 0 && par3Count <= 0 ) {
                    if( ";".equals( s ) ) {
                        buf.append( script.substring( b,p+3 ) ) ;
                        b = p+3 ;
                        next = -1 ;
                        break ;
                    }
                    lst.add( new Object[]{ s,new Integer( opos[0] ) } ) ;
                    next = opos[0]+s.length() ;
                }
                else {
                    next = opos[0]+s.length() ;
                }
            }
            if( b == -1 ) {
                break ;
            }
            else if( next != -1 ) {
                Object in = null ;
                if( lst.size() == 3 && ( in = lst.get( 1 ) ) != null &&
                    in instanceof Object[] && ((Object[])in).length == 2 &&
                    "in".equals(((Object[])in)[0]) ) {
                    int si = ((Integer)((Object[])in)[1]).intValue() ;
                    buf.append( script.substring( b,si ) ).
                        append( ":" ).
                        append( script.substring( si+2,next ) ) ;
                    b = next ;
                }
                else {
                    buf.append( script.substring( b,next ) ) ;
                    b = next ;
                }
            }
        }
        return buf.toString() ;
    }
    
    /**
     * シングルコーテーションをダブルコーテーションに変換.
     */
    private static final String convertCote( String script ) {
        int len ;
        if( script == null || ( len =script.length() ) <= 0 ) {
            return "" ;
        }
        char b = 0 ;
        char cote = 0 ;
        CharBuffer buf = new CharBuffer( len ) ;
        for( int i = 0 ; i < len ; i ++ ) {
            char c = script.charAt( i ) ;
            if( cote != 0 ) {
                if( b != '\\' && c == cote ) {
                    if( c == '\'' ) {
                        buf.append( '\"' ) ;
                    }
                    else {
                        buf.append( c ) ;
                    }
                    cote = 0 ;
                }
                else {
                    buf.append( c ) ;
                }
            }
            else if( c == '\'' || c == '\"' ) {
                cote = c ;
                if( c == '\'' ) {
                    buf.append( '\"' ) ;
                }
                else {
                    buf.append( c ) ;
                }
            }
            else {
                buf.append( c ) ;
            }
            b = c ;
        }
        return buf.toString() ;
    }
    
}
