package org.maachang.seabass ;

import java.util.*;
import org.maachang.atomic.*;
import org.pnuts.lang.*;
import pnuts.lang.*;

/**
 * Contextをスタック.
 * 
 * @version 2008/12/21
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public class SeaBassStackContext {
    private SeaBassStackContext(){}
    private static final SeaBassStackContext SNGL = new SeaBassStackContext() ;
    //private final StackToLock<SeaBassContext> stack = new StackToLock<SeaBassContext>() ;
    private final AtomicSimpleStack<SeaBassContext> stack = new AtomicSimpleStack<SeaBassContext>() ;
    
    /**
     * シングルトンオブジェクトを取得.
     * @return SeaBassStackContext シングルトンオブジェクトが返されます.
     */
    public static SeaBassStackContext getInstance() {
        return SNGL ;
    }
    
    /**
     * コンテキストを取得.
     * @param topMemory TopLeveメモリを設定します.
     * @return SeaBassContext SeaBassContextが返されます.
     */
    public SeaBassContext getContext( Map topMemory ) {
        SeaBassContext ret = ( SeaBassContext )stack.poll() ;
        if( ret == null ) {
            ret = new SeaBassContext() ;
        }
        SeaBassMemory mem = ret.getMemory() ;
        mem.create( topMemory ) ;
        mem.put( "_$global",mem ) ;
        return ret ;
    }
    
    /**
     * メモリリリース.
     * @param context リリース対象のSeaBassContextを設定します.
     */
    public void releaseContext( SeaBassContext context ) {
        if( context != null ) {
            context.clearContext() ;
            stack.add( context ) ;
        }
    }
}
