package org.maachang.seabass ;

import java.util.*;
import gnu.trove.*;
import pnuts.lang.*;

/**
 * SeaBass基本モジュール管理.
 * 
 * @version 2008/12/21
 * @author masahito suzuki
 * @since SeaBassScriptEngine 1.00
 */
public class ModSeaBass {
    private ModSeaBass(){}
    
    private static final String[] MOD_PATH = {
        "org.maachang.seabass.mod.create.createJSAdapter",
        "org.maachang.seabass.mod.create.createStrBuf",
        "org.maachang.seabass.mod.base.newInstance",
        "org.maachang.seabass.mod.base.deepCopy",
        "org.maachang.seabass.mod.base.lockCopy",
        "org.maachang.seabass.mod.base.copy",
        "org.maachang.seabass.mod.base.eval",
        "org.maachang.seabass.mod.base.compile",
        "org.maachang.seabass.mod.base.load",
        "org.maachang.seabass.mod.base.exit",
        "org.maachang.seabass.mod.base.quit",
        "org.maachang.seabass.mod.base.gc",
        "org.maachang.seabass.mod.base.currentTime",
        "org.maachang.seabass.mod.base.changeString",
        "org.maachang.seabass.mod.base.convertString",
        "org.maachang.seabass.mod.base.cutString",
        "org.maachang.seabass.mod.base.decodeBase64",
        "org.maachang.seabass.mod.base.encodeBase64",
        "org.maachang.seabass.mod.base.decodeJSON",
        "org.maachang.seabass.mod.base.encodeJSON",
        "org.maachang.seabass.mod.base.getLastError",
        "org.maachang.seabass.mod.base.isEmpty",
        "org.maachang.seabass.mod.base.isNull",
        "org.maachang.seabass.mod.base.isNumeric",
        "org.maachang.seabass.mod.base.instanceOf",
        "org.maachang.seabass.mod.base.numberFormat",
        "org.maachang.seabass.mod.base.dateFormat",
        "org.maachang.seabass.mod.base.objectCode",
        "org.maachang.seabass.mod.base.parseNumberFormat",
        "org.maachang.seabass.mod.base.parseDateFormat",
        "org.maachang.seabass.mod.base.parseInt",
        "org.maachang.seabass.mod.base.parseBoolean",
        "org.maachang.seabass.mod.base.parseNumber",
        "org.maachang.seabass.mod.base.parseValue",
        "org.maachang.seabass.mod.base.pkg",
        "org.maachang.seabass.mod.base.print",
        "org.maachang.seabass.mod.base.println",
        "org.maachang.seabass.mod.base.regexp",
        "org.maachang.seabass.mod.base.random",
        "org.maachang.seabass.mod.base.randomSeed",
        "org.maachang.seabass.mod.base.sleep",
        "org.maachang.seabass.mod.base.typeof",
        "org.maachang.seabass.mod.base.trim",
        "org.maachang.seabass.mod.base.valueof",
        "org.maachang.seabass.mod.base.useString",
        "org.maachang.seabass.mod.base.scriptVersion",
        "org.maachang.seabass.mod.base.throwTo",
        "org.maachang.seabass.mod.base.exception" } ;
    
    private static final ModSeaBass SNGL = new ModSeaBass() ;
    protected final Map modList = new THashMap() ;
    
    static{
        init() ;
    }
    
    /** 基本モジュール追加処理 **/
    private static final void init() {
        int len = MOD_PATH.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            try {
                Object o = Class.forName( MOD_PATH[ i ] ).newInstance() ;
                if( o instanceof PnutsFunction ) {
                    PnutsFunction p = ( PnutsFunction )o ;
                    SNGL.modList.put( p.getName(),p ) ;
                }
            } catch( Exception e ) {
                e.printStackTrace() ;
            }
        }
    }
    
    /**
     * シングルトン取得.
     * @return ModSeaBass シングルトンオブジェクトが返されます.
     */
    public static final ModSeaBass getInstance() {
        return SNGL ;
    }
    
    /**
     * モジュール追加.
     * @param name 対象のモジュール名を設定します.
     * @param func 対象のモジュールを設定します.
     * @exception Exception 例外.
     */
    public static final void addMod( String name,PnutsFunction func )
        throws Exception {
        if( name == null || ( name = name.trim() ).length() <= 0 ||
            func == null ) {
            throw new IllegalArgumentException( "追加モジュール条件が不正です" ) ;
        }
        SNGL.modList.put( name,func ) ;
    }
    
    /**
     * モジュール名が存在するか確認.
     * @param key 対象のキー名を設定します.
     */
    public boolean containsKey( Object key ) {
        return modList.containsKey( key ) ;
    }
    
    /**
     * モジュール情報を取得.
     * @param key 対象のキー名を設定します.
     * @return Object 対象のモジュール情報が返されます.
     */
    public Object get( Object key ) {
        return modList.get( key ) ;
    }
    
    /**
     * 全体Map情報を取得.
     * @return Map 全体Map情報が返されます.
     */
    public Map get() {
        return modList ;
    }
}
