package org.maachang.seabass.mod.object ;

import gnu.trove.*;
import pnuts.lang.*;
import java.util.*;
import org.maachang.*;
import org.pnuts.util.*;

/*
 * JSAdapter.
 */
public class JSAdapter extends BaseMap {
    public static final String _GET_ = "__get__" ;
    public static final String _PUT_ = "__put__" ;
    public static final String _HAS_ = "__has__" ;
    public static final String _DELETE_ = "__delete__" ;
    public static final String _GET_IDS_ = "__getIds__" ;
    public static final String _LENGTH_ = "__length__" ;
    
    private Map map = null ;
    
    private JSAdapter() {
    }
    public JSAdapter( Map map ) {
        if( map == null ) {
            throw new IllegalArgumentException( "Map引数を設定する必要があります" ) ;
        }
        if( ( map.containsKey( JSAdapter._GET_ ) && !( map.get( JSAdapter._GET_ ) instanceof PnutsFunction ) ) || 
            ( map.containsKey( JSAdapter._PUT_ ) && !( map.get( JSAdapter._PUT_ ) instanceof PnutsFunction ) ) || 
            ( map.containsKey( JSAdapter._HAS_ ) && !( map.get( JSAdapter._HAS_ ) instanceof PnutsFunction ) ) || 
            ( map.containsKey( JSAdapter._DELETE_ ) && !( map.get( JSAdapter._DELETE_ ) instanceof PnutsFunction ) ) || 
            ( map.containsKey( JSAdapter._GET_IDS_ ) && !( map.get( JSAdapter._GET_IDS_ ) instanceof PnutsFunction ) ) ||
            ( map.containsKey( JSAdapter._LENGTH_ ) && !( map.get( JSAdapter._LENGTH_ ) instanceof PnutsFunction ) ) ) {
            throw new IllegalArgumentException( "JSAdapter設定はFunctionでなくてはなりません" ) ;
        }
        this.map = map ;
    }
    
    private static final boolean checkName( String name ) {
        name = name.trim() ;
        if( name.startsWith( "__" ) && name.endsWith( "__" ) ) {
            return ( name.equals( _GET_ ) || 
                name.equals( _PUT_ ) ||
                name.equals( _HAS_ ) ||
                name.equals( _DELETE_ ) ||
                name.equals( _GET_IDS_ ) ||
                name.equals( _LENGTH_ ) ) ;
        }
        return false ;
    }
    
    public Object put(Object name, Object value) {
        if( name instanceof String ) {
            if( checkName( (String)name ) ) {
                if( !(value instanceof PnutsFunction ) ) {
                    throw new IllegalArgumentException( "JSAdapter設定はFunctionでなくてはなりません" ) ;
                }
                return map.put( name,value ) ;
            }
            if( map.containsKey( _PUT_ ) ) {
                Object o = map.get( _PUT_ ) ;
                if( o instanceof PnutsFunction ) {
                    pnuts.lang.Context context = pnuts.lang.Context.getCurrentContext() ;
                    StackList stack = context.getThisStack() ;
                    boolean pushFlag = stack.push( map ) ;
                    try {
                        PnutsFunction func = ( PnutsFunction )o ;
                        return func.call(new Object[]{ name,value }, context);
                        //return func.execution(new Object[]{ name,value }, context);
                    } finally {
                        if( pushFlag ) {
                            stack.pop() ;
                        }
                    }
                }
            }
            if( map.containsKey( name ) ) {
                return map.put( name,value ) ;
            }
        }
        return null ;
    }
    
    public boolean containsKey(Object key) {
        if( key instanceof String ) {
            if( checkName( (String)key ) ) {
                return false ;
            }
            if( map.containsKey( _HAS_ ) ) {
                Object o = map.get( _HAS_ ) ;
                if( o instanceof PnutsFunction ) {
                    boolean ret = true ;
                    pnuts.lang.Context context = pnuts.lang.Context.getCurrentContext() ;
                    StackList stack = context.getThisStack() ;
                    boolean pushFlag = stack.push( map ) ;
                    try {
                        PnutsFunction func = ( PnutsFunction )o ;
                        Object result = func.call(new Object[]{ key }, context);
                        //Object result = func.execution(new Object[]{ key }, context);
                        if( result instanceof Boolean ) {
                            ret = (( Boolean )result).booleanValue() ;
                        }
                    } finally {
                        if( pushFlag ) {
                            stack.pop() ;
                        }
                    }
                    return ret ;
                }
            }
            if( map.containsKey( key ) ) {
                return true ;
            }
        }
        return false ;
    }
    
    public Object get(Object key) {
        if( key instanceof String ) {
            if( checkName( (String)key ) ) {
                return null ;
            }
            if( map.containsKey( _GET_ ) ) {
                Object o = map.get( _GET_ ) ;
                if( o instanceof PnutsFunction ) {
                    pnuts.lang.Context context = pnuts.lang.Context.getCurrentContext() ;
                    StackList stack = context.getThisStack() ;
                    boolean pushFlag = stack.push( map ) ;
                    try {
                        PnutsFunction func = ( PnutsFunction )o ;
                        return func.call(new Object[]{ key }, context);
                        //return func.execution(new Object[]{ key }, context);
                    } finally {
                        if( pushFlag ) {
                            stack.pop() ;
                        }
                    }
                }
            }
            if( map.containsKey( key ) ) {
                return map.get( key ) ;
            }
        }
        return null ;
    }
    
    public Object remove(Object key) {
        if( key instanceof String ) {
            if( checkName( (String)key ) ) {
                return null ;
            }
            if( map.containsKey( _DELETE_ ) ) {
                Object o = map.get( _DELETE_ ) ;
                if( o instanceof PnutsFunction ) {
                    pnuts.lang.Context context = pnuts.lang.Context.getCurrentContext() ;
                    StackList stack = context.getThisStack() ;
                    boolean pushFlag = stack.push( map ) ;
                    try {
                        PnutsFunction func = ( PnutsFunction )o ;
                        return func.call(new Object[]{ key }, context);
                        //return func.execution(new Object[]{ key }, context);
                    } finally {
                        if( pushFlag ) {
                            stack.pop() ;
                        }
                    }
                }
            }
            if( map.containsKey( key ) ) {
                return map.remove( key ) ;
            }
        }
        return null ;
    }
    
    public Set keySet() {
        if( map.containsKey( _GET_IDS_ ) ) {
            Object o = map.get( _GET_IDS_ ) ;
            if( o instanceof PnutsFunction ) {
                pnuts.lang.Context context = pnuts.lang.Context.getCurrentContext() ;
                StackList stack = context.getThisStack() ;
                boolean pushFlag = stack.push( map ) ;
                try {
                    PnutsFunction func = ( PnutsFunction )o ;
                    Set ret = new THashSet() ;
                    func.call(new Object[]{ ret }, context);
                    //func.execution(new Object[]{ ret }, context);
                    return ret ;
                } finally {
                    if( pushFlag ) {
                        stack.pop() ;
                    }
                }
            }
        }
        return map.keySet() ;
    }
    
    public int size() {
        if( map.containsKey( _LENGTH_ ) ) {
            Object o = map.get( _LENGTH_ ) ;
            if( o instanceof PnutsFunction ) {
                pnuts.lang.Context context = pnuts.lang.Context.getCurrentContext() ;
                StackList stack = context.getThisStack() ;
                boolean pushFlag = stack.push( map ) ;
                try {
                    PnutsFunction func = ( PnutsFunction )o ;
                    Object ret = func.call(NO_ARGS, context);
                    //Object ret = func.execution(NO_ARGS, context);
                    if( ret instanceof Integer ) {
                        return ((Integer)ret).intValue() ;
                    }
                } finally {
                    if( pushFlag ) {
                        stack.pop() ;
                    }
                }
                return -1 ;
            }
        }
        return map.size() ;
    }
    
    public Map getMap() {
        return map ;
    }
    
    public String toString() {
        return "JSAdapter" ;
    }
    
    private static final Object[] NO_ARGS = new Object[ 0 ] ;
}
