package org.pnuts.util;

import java.util.*;
import java.util.concurrent.locks.*;
import org.pnuts.util.*;

public class AtomicSkipListSet<E> extends SkipListSet<E> {
    private final PnutsLock sync = new PnutsLock() ;
    
    @Override
    public int size() {
        int ret ;
        sync.lock() ; try {
            ret = super.size();
        } finally { sync.unlock() ;}
        return ret ;
    }

    @Override
    public void clear() {
        sync.lock() ; try {
            super.clear() ;
        } finally { sync.unlock() ;}
        
    }

    @Override
    public boolean add(E element) {
        sync.lock() ; try {
            super.add( element ) ;
        } finally { sync.unlock() ;}
        return true;
    }

    @Override
    public void add(int index, E element) {
        sync.lock() ; try {
            super.add( index,element ) ;
        } finally { sync.unlock() ;}
    }

    @Override
    public E remove(int index) {
        E ret = null ;
        sync.lock() ; try {
            ret = super.remove( index ) ;
        } finally { sync.unlock() ;}
        return ret ;
    }

    @Override
    public boolean remove(Object o) {
        boolean ret = false ;
        sync.lock() ; try {
            ret = super.remove( o ) ;
        } finally { sync.unlock() ;}
        return ret ;
    }
    
    @Override
    public boolean contains(Object o) {
        boolean ret = false ;
        sync.lock() ; try {
            ret = super.contains( o ) ;
        } finally { sync.unlock() ;}
        return ret ;
    }

    @Override
    public E get(int index) {
        E ret = null ;
        sync.lock() ; try {
            ret = super.get( index ) ;
        } finally { sync.unlock() ;}
        return ret ;
    }

    @Override
    public E set(int index, E value) {
        E ret = null ;
        sync.lock() ; try {
            ret = super.set( index,value ) ;
        } finally { sync.unlock() ;}
        return ret ;
    }

    @Override
    public int indexOf(Object o) {
        int ret = -1 ;
        sync.lock() ; try {
            ret = super.indexOf( o ) ;
        } finally { sync.unlock() ;}
        return ret ;
    }

    @Override
    public int lastIndexOf(Object o) {
        int ret = -1 ;
        sync.lock() ; try {
            ret = super.lastIndexOf( o ) ;
        } finally { sync.unlock() ;}
        return ret ;
    }
    
    @Override
    public boolean removeAll(Collection<?> c) {
        boolean ret = false ;
        sync.lock() ; try {
            ret = super.removeAll( c ) ;
        } finally { sync.unlock() ;}
        return ret ;
    }

    @Override
    public Iterator<E> iterator() {
        return new AtomicSkipListSetIterator<E>( this,this.toArray() ) ;
    }
    
    @Override
    public Object[] toArray() {
        Object[] ret = null ;
        sync.lock() ; try {
            int len = super.size();
            if( len <= 0 ) {
                ret = new Object[ 0 ] ;
            }
            else {
                ret = new Object[ len ] ;
                for( int i = 0 ; i < len ; i ++ ) {
                    ret[ i ] = super.get( i ) ;
                }
            }
        } finally { sync.unlock() ;}
        return ret ;
    }
    
    @Override
    public Object clone() {
        AtomicSkipListSet<E> ret = null ;
        sync.lock() ; try {
            ret = new AtomicSkipListSet<E>() ;
            int len = super.size() ;
            for( int i = 0 ; i < len ; i ++ ) {
                ret.add( super.get( i ) ) ;
            }
        } finally { sync.unlock() ;}
        return ret ;
    }
    
    public void toTop( E o ) {
        sync.lock() ; try {
            super.remove( o ) ;
            super.add( o ) ;
        } finally { sync.unlock() ;}
    }
    
    private static class AtomicSkipListSetIterator<E> implements Iterator {
        private AtomicSkipListSet lst = null ;
        private E[] arrays = null ;
        private int count = 0 ;
        private boolean nextFlag = false ;
        private AtomicSkipListSetIterator() {
        }
        public AtomicSkipListSetIterator( AtomicSkipListSet lst,Object[] arys ) {
            this.lst = lst ;
            this.arrays = (E[])arys ;
            this.count = 0 ;
        }
        public boolean hasNext() {
            return ( count < arrays.length ) ;
        }
        public E next() {
            if( count < arrays.length ) {
                nextFlag = true ;
                int c = count ;
                count ++ ;
                return arrays[ c ] ;
            }
            throw new NoSuchElementException() ;
        }
        public void remove() {
            if( count < arrays.length ) {
                if( nextFlag == false ) {
                    throw new IllegalStateException() ;
                }
                nextFlag = false ;
                lst.remove( arrays[ count ] ) ;
            }
        }
    }

}
