/*
 * @(#)MemoryCache.java 1.2 04/12/06
 *
 * Copyright (c) 1997-2003 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "LICENSE.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package org.pnuts.util;

import pnuts.lang.Runtime;
import java.lang.ref.SoftReference;
import java.util.*;

import java.util.concurrent.locks.*;

/**
 * A Cache implementation that uses SoftReference.
 */
public class MemoryCache implements Cache {
    private final ReentrantReadWriteLock sync = new ReentrantReadWriteLock() ;
    private final Lock rsync = sync.readLock() ;
    private final Lock wsync = sync.writeLock() ;

    private Map map;

    public MemoryCache() {
        this(Runtime.createWeakMap());
    }

    public MemoryCache(Map map) {
        this.map = map;
    }
    
    public void clear() {
        wsync.lock() ; try {
            map.clear() ;
        } finally { wsync.unlock() ; }
    }

    public Object get(Object key) {
        Object ret = null ;
        rsync.lock() ; try {
            SoftReference ref = (SoftReference) map.get(key);
            if (ref == null) {
                ret = null;
            } else {
                ret = ref.get();
            }
        } finally { rsync.unlock() ; }
        return ret ;
    }

    public Object put(Object key, Object value) {
        wsync.lock() ; try {
            map.put(key, new SoftReference(value));
        } finally { wsync.unlock() ; }
        return null;
    }

    public void reset() {
        wsync.lock() ; try {
            this.map = new WeakHashMap();
        } finally { wsync.unlock() ; }
    }
}
