package org.pnuts.util;

import java.util.*;
import java.util.concurrent.locks.*;

public class RWSkipList<E> extends SkipList<E> {
    private final ReentrantReadWriteLock sync = new ReentrantReadWriteLock() ;
    private final Lock rsync = sync.readLock() ;
    private final Lock wsync = sync.writeLock() ;
    
    @Override
    public int size() {
        int ret ;
        rsync.lock() ; try {
            ret = super.size();
        } finally { rsync.unlock() ;}
        return ret ;
    }

    @Override
    public void clear() {
        wsync.lock() ; try {
            super.clear() ;
        } finally { wsync.unlock() ;}
        
    }

    @Override
    public boolean add(E element) {
        wsync.lock() ; try {
            super.add( element ) ;
        } finally { wsync.unlock() ;}
        return true;
    }

    @Override
    public void add(int index, E element) {
        wsync.lock() ; try {
            super.add( index,element ) ;
        } finally { wsync.unlock() ;}
    }

    @Override
    public E remove(int index) {
        E ret = null ;
        wsync.lock() ; try {
            ret = super.remove( index ) ;
        } finally { wsync.unlock() ;}
        return ret ;
    }

    @Override
    public boolean remove(Object o) {
        boolean ret = false ;
        wsync.lock() ; try {
            ret = super.remove( o ) ;
        } finally { wsync.unlock() ;}
        return ret ;
    }

    @Override
    public E get(int index) {
        E ret = null ;
        rsync.lock() ; try {
            ret = super.get( index ) ;
        } finally { rsync.unlock() ;}
        return ret ;
    }

    @Override
    public E set(int index, E value) {
        E ret = null ;
        wsync.lock() ; try {
            ret = super.set( index,value ) ;
        } finally { wsync.unlock() ;}
        return ret ;
    }

    @Override
    public int indexOf(Object o) {
        int ret = -1 ;
        rsync.lock() ; try {
            ret = super.indexOf( o ) ;
        } finally { rsync.unlock() ;}
        return ret ;
    }

    @Override
    public int lastIndexOf(Object o) {
        int ret = -1 ;
        rsync.lock() ; try {
            ret = super.lastIndexOf( o ) ;
        } finally { rsync.unlock() ;}
        return ret ;
    }

    @Override
    public Iterator<E> iterator() {
        return new RWSkipListIterator<E>( this,this.toArray() ) ;
    }
    
    @Override
    public Object[] toArray() {
        Object[] ret = null ;
        rsync.lock() ; try {
            int len = super.size();
            if( len <= 0 ) {
                ret = new Object[ 0 ] ;
            }
            else {
                ret = new Object[ len ] ;
                for( int i = 0 ; i < len ; i ++ ) {
                    ret[ i ] = super.get( i ) ;
                }
            }
        } finally { rsync.unlock() ;}
        return ret ;
    }
    
    @Override
    public Object clone() {
        RWSkipList<E> ret = null ;
        rsync.lock() ; try {
            ret = new RWSkipList<E>() ;
            int len = super.size() ;
            for( int i = 0 ; i < len ; i ++ ) {
                ret.add( super.get( i ) ) ;
            }
        } finally { rsync.unlock() ;}
        return ret ;
    }
    
    private static class RWSkipListIterator<E> implements Iterator {
        private E[] arrays = null ;
        private RWSkipList lst = null ;
        private int count = 0 ;
        private boolean nextFlag = false ;
        private RWSkipListIterator() {
        }
        public RWSkipListIterator( RWSkipList lst,Object[] arys ) {
            this.lst = lst ;
            this.arrays = (E[])arys ;
            this.count = 0 ;
        }
        public boolean hasNext() {
            return ( count < arrays.length ) ;
        }
        public E next() {
            if( count < arrays.length ) {
                nextFlag = true ;
                int c = count ;
                count ++ ;
                return arrays[ c ] ;
            }
            throw new NoSuchElementException() ;
        }
        public void remove() {
            if( count < arrays.length ) {
                if( nextFlag == false ) {
                    throw new IllegalStateException() ;
                }
                nextFlag = false ;
                lst.remove( (E)arrays[ count ] ) ;
            }
        }
    }
}
