package org.pnuts.util;

import java.io.* ;
import java.util.concurrent.atomic.* ;

public class XorRandom implements Serializable {
    private static final long serialVersionUID = 746227844186191007L;
    private static final AtomicLong seedUniquifier = new AtomicLong( 8682522807148012L );
    private int a = 123456789 ;
    private int b = 362436069 ;
    private int c = 521288629 ;
    private int d = 88675123 ;
    
    private long nano8bit() {
        return ( System.nanoTime() & 0x00000000000000ffL ) ;
    }
    
    public XorRandom() {
        this( seedUniquifier.incrementAndGet() ) ;
    }
    
    public XorRandom( long seed ) {
        this.setSeed( seed ) ;
    }
    
    public final void setSeed(long seed) {
        seed = ( seed * nano8bit() ) & 0x00ffffffffffffffL ;
        d = (int)( seed & 0x00000000ffffffffL ) ;
        nextInt() ;
        d = ( d ^ (int)( ( seed & 0xffffffff00000000L ) >> 32L ) ) ;
        nextInt() ;
    }
    
    public final int nextInt() {
        int t,r ;
        t=a;r=t; t<<=11; t^=r;   r=t;  r>>=8;  t^=r;  r=b;a=r;
        r=c;b=r; r=d;c=r; t^=r;   r>>=19; r^=t; d=r;  return r;
    }
    
    public void nextBytes(byte[] binary) {
        int rnd ;
        int cnt = 0 ;
        int len = binary.length ;
        while( true ) {
            if( cnt + 4 > len ) {
                int last = len - cnt ;
                switch( last ) {
                    case 0 : break ;
                    case 1 :
                        rnd = nextInt() ;
                        binary[ cnt ] = ( byte )( rnd & 0x000000ff ) ;
                        break ;
                    case 2 :
                        rnd = nextInt() ;
                        binary[ cnt ] = ( byte )( rnd & 0x000000ff ) ;
                        cnt ++ ;
                        binary[ cnt ] = ( byte )( ( rnd & 0x0000ff00 ) >> 8 ) ;
                        break ;
                    case 3 :
                        rnd = nextInt() ;
                        binary[ cnt ] = ( byte )( rnd & 0x000000ff ) ;
                        cnt ++ ;
                        binary[ cnt ] = ( byte )( ( rnd & 0x0000ff00 ) >> 8 ) ;
                        cnt ++ ;
                        binary[ cnt ] = ( byte )( ( rnd & 0x00ff0000 ) >> 16 ) ;
                        break ;
                }
                break ;
            }
            else {
                rnd = nextInt() ;
                binary[ cnt ] = ( byte )( rnd & 0x000000ff ) ;
                cnt ++ ;
                binary[ cnt ] = ( byte )( ( rnd & 0x0000ff00 ) >> 8 ) ;
                cnt ++ ;
                binary[ cnt ] = ( byte )( ( rnd & 0x00ff0000 ) >> 16 ) ;
                cnt ++ ;
                binary[ cnt ] = ( byte )( ( rnd & 0xff000000 ) >> 24 ) ;
                cnt ++ ;
            }
        }
    }
    
    public int nextInt(int n) {
        if (n <= 0) {
            throw new IllegalArgumentException("argument is illegal.");
        }
        if ((n & -n) == n)
            return (int)((n * (long)( nextInt() & 0x7fffffff )) >> 31);
        int bits, val;
        do {
            bits = ( nextInt() & 0x7fffffff ) ;
            val = bits % n;
        } while (bits - val + (n-1) < 0);
        return val;
    }
    
    public long nextLong() {
        return ((long)(nextInt()) << 32) + nextInt();
    }
    
    public boolean nextBoolean() {
        return ( nextInt() & 0x00000001 ) != 0;
    }
    
    public float nextFloat() {
        return ( nextInt() & 0x00ffffff ) / ((float)(1 << 24));
    }
    
    public double nextDouble() {
        return (((long)( nextInt() & 0x03ffffff ) << 27) +
            ( nextInt() & 0x07ffffff )) / (double)(1L << 53);
    }
}

