/*
 * @(#)DefaultFunctionSerializer.java 1.2 05/06/21
 *
 * Copyright (c) 1997-2005 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "LICENSE.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.io.*;
import java.util.*;
import pnuts.lang.*;
import pnuts.compiler.Compiler;
import org.pnuts.lang.*;

import org.pnuts.util.* ;

class DefaultFunctionSerializer implements Runtime.FunctionSerializer {

    public void serialize(PnutsFunction pnutsFunction, ObjectOutputStream s)
        throws IOException
    {
        s.writeUTF(pnutsFunction.pkg.getName());
        s.writeInt(1);
        Function f = pnutsFunction.func ;
        SimpleNode node = null;
        if (f != null){
            node = f.node;
            if (node != null){     // the function was written in Pnuts
                s.writeObject(f);
            } else {               // the function was written in Java
                node = f.getNode();
                if (node != null){
                    s.writeObject(node);  // nodes was deserialized properly
                } else {
                    s.writeObject(f.unparse(null)); // nodes was reconstracted from the script
                }
                f.writeAttributes(s);
            }
        } else {
            s.writeObject(null);
        }
    }

    public void deserialize(PnutsFunction pnutsFunction, ObjectInputStream s)
        throws IOException, ClassNotFoundException
    {
        String pkgName = s.readUTF();
        Context threadContext = Runtime.getThreadContext();
        int n = s.readInt();
        boolean[] compile = new boolean[n];
        boolean compileString = false;
        boolean compileNode = false;
        StringBuilder sbuf = new StringBuilder();
        List nodes = new SkipList();
        Package pkg = null;
        Object f = s.readObject();
        if (f instanceof Function){  // AST interpreter
            pnutsFunction.func = (Function)f;
        } else if (f instanceof SimpleNode){
            SimpleNode ss = new SimpleNode(PnutsParserTreeConstants.JJTSTARTSET);
            SimpleNode node = (SimpleNode)f;
            Function func = new Function();
            pnutsFunction.func = func;
            compile[0] = true;
            compileNode = true;
            func.node = node.jjtGetChild(1);
            func.readAttributes(s);
            func.setPackage(Package.find(func.pkgName, threadContext));
            NodeUtil.setPackage(func.pkgName, ss);
            String[] imports = func.importEnv.list();
            for (int j = 0; j < imports.length; j++){
                NodeUtil.addImportNode(imports[j], ss);
            }
            NodeUtil.addFunction(node, ss);
            nodes.add(ss);
        } else if (f instanceof String){
            String unparsed = (String)f;
            try {
                Function func = new Function();
                func.readAttributes(s);
                func.setPackage(Package.find(func.pkgName, threadContext));
                sbuf.append("package(\"");
                sbuf.append(func.pkgName);
                sbuf.append("\")\n");
                String[] imports = func.importEnv.list();
                for (int j = 0; j < imports.length; j++){
                    sbuf.append("import " + imports[j] + "\n");
                }
                sbuf.append((String)f);
                sbuf.append("\n");
                PnutsParser parser = Pnuts.getParser(new StringReader(unparsed));
                SimpleNode node = parser.FunctionStatement(null);
                func.node = node.jjtGetChild(1);
                func.function = pnutsFunction;
                pnutsFunction.func = func;
                compile[0] = true;
                compileString = true;
            } catch (ParseException pe){
                pe.printStackTrace();
            }
        }
        pnutsFunction.pkg = Package.find(pkgName, threadContext);
        Compiler compiler = null;
        if (compileString || compileNode){
            compiler = new pnuts.compiler.Compiler();
            pkg = new Package(null, null);
        }
        if (compileString){
            Context c2 = (threadContext != null) ? (Context)threadContext.clone() : new Context();
            c2.setCurrentPackage(pkg);
            Pnuts compiled = compiler.compile(sbuf.toString(), c2);
            PnutsFunction pf = (PnutsFunction)compiled.run(c2);
            pf.pkg = Package.find(pkgName, threadContext);
            {
                Function _f = pnutsFunction.func ;
                if (_f != null && compile[0]){
                    Function f2 = pf.func;
                    f2.moduleList = _f.moduleList;
                    f2.config = _f.config;
                    Package p = Package.getPackage(_f.pkg.getName(), c2);
                    for (Enumeration e = _f.pkg.bindings(); e.hasMoreElements();){
                        Binding binding = (Binding)e.nextElement();
                        p.set(binding.getName(), binding.get());
                    }
                    f2.pkg = p;
                    f2.pkgName = _f.pkgName;
                    f2.file = null;
                    pnutsFunction.func = f2;
                }
            }
        }
        if (compileNode){
            Context c2 = (threadContext != null) ? (Context)threadContext.clone() : new Context();
            if (pkg != null){
                c2.setCurrentPackage(pkg);
            }
            for (int j = 0, sz = nodes.size(); j < sz; j++){
                SimpleNode ss = (SimpleNode)nodes.get(j);
                Pnuts compiled = compiler.compile(new P(ss), c2);
                PnutsFunction pf = (PnutsFunction)compiled.run(c2);
                {
                    Function _f = pnutsFunction.func ;
                    if (_f != null && compile[0]){
                        Function f2 = pf.func ;
                        if (f2 == null){
                            continue;
                        }
                        f2.moduleList = _f.moduleList;
                        f2.config = _f.config;
                        f2.pkg = _f.pkg;
                        f2.pkgName = _f.pkgName;
                        f2.file = _f.file;
                        pnutsFunction.func = f2;
                    }
                }
            }
        }
    }

    static class P extends Pnuts {
        public P(SimpleNode ss){
            this.startNodes = ss;
        }
    }
}
