/*
 * @(#)ImportEnv.java 1.3 05/05/25
 * 
 * Copyright (c) 1997-2005 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * See the file "LICENSE.txt" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.*;
import gnu.trove.*;
import java.util.concurrent.*;
import java.lang.reflect.Modifier;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

import org.pnuts.util.MemoryCache;
import org.pnuts.lang.CallableMethod;

import org.pnuts.util.* ;
import org.maachang.atomic.*;
import org.maachang.reflect.*;

class ImportEnv implements Serializable, Cloneable {

    private final static boolean DEBUG = false;

    static final long serialVersionUID = -755390125835215465L;

    /**
     * インポートオブジェクトを管理.
     */
    private Map<String,Import> imports = new ConcurrentHashMap<String,Import>();
    
    /**
     * インポート対象パッケージ＋クラス名を管理.
     */
    private Map<String,String> map = new ConcurrentHashMap<String,String>() ;
    
    /**
     * パッケージ登録確認用.
     */
    private Map<String,Boolean> pkgMap = new ConcurrentHashMap<String,Boolean>() ;


    /** import宣言が、パッケージ名+アスタリスクで登録された場合に呼び出される **/
    void addPackage(String pkgname) {
        if( pkgname == null || ( pkgname = pkgname.trim() ).length() <= 0 ) {
            return ;
        }
        if( !pkgMap.containsKey( pkgname ) ) {
            List<String> list = null ;
            try {
                list = FastPackageCache.getList( pkgname ) ;
            } catch( Exception e ) {
                throw new PnutsException( e ) ;
            }
            if( list != null && list.size() > 0 ) {
                int len = list.size() ;
                String n ;
                Map<String,String> m = map ;
                for( int i = 0 ; i < len ; i ++ ) {
                    n = list.get( i ) ;
                    m.put( n,pkgname ) ;
                }
                pkgMap.put( pkgname,true ) ;
            }
            else {
                throw new PnutsException(
                    "指定パッケージ " + pkgname + " はパッケージ名として存在しません" ) ;
            }
        }
        Import imp = (Import) imports.get(pkgname) ;
        if (imp == null) {
            imp = getImport(pkgname) ;
        }
    }

    /** Importオブジェクトを取得. **/
    private Import getImport(String name) {
        Import imp = imports.get(name);
        if (imp == null) {
            imp = new Import(name);
            imports.put( name,imp ) ;
        }
        return imp;
    }

    /** import宣言が、パッケージ名＋クラス名で登録された場合に呼び出される. **/
    void addClass(String className) {
        int idx = className.lastIndexOf('.');
        if( idx == -1 ) {
            return ;
        }
        String objectName = className.substring( idx + 1 ) ;
        String pkgName = className.substring( 0,idx ) ;
        if( objectName == null || ( objectName = objectName.trim() ).length() <= 0 ||
            pkgName == null || ( pkgName = pkgName.trim() ).length() <= 0 ) {
            throw new PnutsException( "クラス名 " + className + " は不正です" ) ;
        }
        map.put( objectName,pkgName ) ;
    }

    
    /**
     * よく利用する略称クラス名群.
     */
    private static final Map<String,Class> defClass = new HashMap<String,Class>() ;
    static {
        defClass.put( "StrBuf",org.maachang.seabass.mod.object.StrBuf.class ) ;
        defClass.put( "JSAdapter",org.maachang.seabass.mod.object.JSAdapter.class ) ;
        defClass.put( "JDate",org.maachang.JDate.class ) ;
        defClass.put( "EasyDate",org.maachang.EasyDate.class ) ;
        defClass.put( "Boolean",java.lang.Boolean.class ) ;
        defClass.put( "Byte",java.lang.Byte.class ) ;
        defClass.put( "Character",java.lang.Character.class ) ;
        defClass.put( "Short",java.lang.Short.class ) ;
        defClass.put( "Integer",java.lang.Integer.class ) ;
        defClass.put( "Long",java.lang.Long.class ) ;
        defClass.put( "Float",java.lang.Float.class ) ;
        defClass.put( "Double",java.lang.Double.class ) ;
        defClass.put( "BigInteger",java.math.BigInteger.class ) ;
        defClass.put( "BigDecimal",java.math.BigDecimal.class ) ;
        defClass.put( "String",java.lang.String.class ) ;
        defClass.put( "Date",java.util.Date.class ) ;
        defClass.put( "StringBuffer",java.lang.StringBuilder.class ) ;
        defClass.put( "StringBuilder",java.lang.StringBuilder.class ) ;
        defClass.put( "Map",java.util.Map.class ) ;
        defClass.put( "List",java.util.List.class ) ;
        defClass.put( "Exception",java.lang.Exception.class ) ;
        defClass.put( "IllegalArgumentException",java.lang.IllegalArgumentException.class ) ;
        defClass.put( "IOException",java.io.IOException.class ) ;
        defClass.put( "Error",java.lang.Error.class ) ;
        defClass.put( "Throwable",java.lang.Throwable.class ) ;
    }
    
    /** デフォルトクラスに追加 **/
    void putDefault( String name,Class c ) {
        if( name != null && c != null ) {
            defClass.put( name,c ) ;
        }
    }
    
    /** デフォルトクラスに追加 **/
    void putDefault( String name,String clazz ) {
        if( name != null && clazz != null ) {
            try {
                defClass.put( name,SearchJavaInfo.getClass( clazz ) ) ;
            } catch( Exception e ) {
            }
        }
    }
    
    Object get(String sym, Context context) {
        if( sym == null || ( sym = sym.trim() ).length() <= 0 ) {
            return null ;
        }
        Object ret = defClass.get( sym ) ;
        if( ret == null ) {
            ret = _get( sym,context ) ;
        }
        return ret ;
    }
    
    /** パッケージ指定された内容から検索. **/
    Object _get(String sym, Context context) {
        String pkg = map.get( sym ) ;
        if( pkg == null ) {
            return null ;
        }
        Import imp = getImport( pkg ) ;
        if( imp != null ) {
            return imp.get( sym,context ) ;
        }
        return null ;
    }

    /** 登録パッケージ一覧を取得.**/
    String[] list() {
        List<String> list = new ArrayList<String>() ;
        Iterator it = map.keySet().iterator() ;
        String s ;
        while( it.hasNext() ) {
            s = (String)it.next() ;
            list.add( map.get( s ) + "." + s ) ;
        }
        int len = list.size() ;
        String[] ret = new String[ len ] ;
        for( int i = 0 ; i < len ; i ++ ) {
            ret[ i ] = list.get( i ) ;
        }
        return ret ;
    }

    /** static import */
    void addStaticMembers(String name, boolean wildcard, Context context) {
        /** static importは非対応 **/
        throw new IllegalStateException( "static importは非対応です" ) ;
    }

    /** オブジェクトリセット. **/
    void reset() {
        Iterator it = imports.keySet().iterator() ;
        String key ;
        while( it.hasNext() ) {
            key = (String)it.next() ;
            Import imp = (Import)imports.get( key ) ;
            if( imp != null ) {
                imp.reset() ;
            }
        }
    }

    /** オブジェクトクローン **/
    public Object clone() {
        try {
            ImportEnv c = (ImportEnv) super.clone();
            c.imports = new ConcurrentHashMap<String,Import>( imports ) ;
            c.map = new ConcurrentHashMap<String,String>( map ) ;
            c.pkgMap = new ConcurrentHashMap<String,Boolean>( pkgMap ) ;
            c.reset();
            return c;
        } catch (Throwable t) {
            throw new InternalError();
        }
    }

    private void readObject(ObjectInputStream s)
        throws IOException, ClassNotFoundException {
        s.defaultReadObject();
    }
}
